
/***************************************************************************/
/*           HERE DEFINE THE PROJECT SPECIFIC PUBLIC MACROS                */
/*    These are only in effect in a setting that doesn't use configure     */
/***************************************************************************/

/* Version number of project */
#define CGL_VERSION "0.57.3"

/* Major Version number of project */
#define CGL_VERSION_MAJOR 0

/* Minor Version number of project */
#define CGL_VERSION_MINOR 57

/* Release Version number of project */
#define CGL_VERSION_RELEASE 3
