/**
  @example graph_display/graph_display.cpp
  @brief This example gives a minimalistic approach of importing a graph and displaying it with
 Tulip OpenGL rendering engine

  In this example, we are making a standalone program that will load a graph file passed as argument
 to the program and display it into a new window.

  The code contains a step-by-step explanation of the process, please refer to classes documentation
 for more information about their usage.
 **/
