/**
  @example interactor_plugin/interactor_plugin.cpp
  @brief This example gives a minimalistic approach of interaction with a graph

  In this example, we are making an interactor plugin for Node link diagram view that will activate
 zoom and pan and display information when you click on a node/edge.

  The code contains a step-by-step explanation of the process, please refer to classes documentation
 for more information about their usage.
 **/
