/**
 * CMakeHelpers library, distributed under MIT license
 *
 * Copyright (C) 2015, by halex2005
 *
 * Report bugs and download new versions at https://github.com/halex2005/CMakeHelpers
 */

#include "@VERSION_HEADER@"
#include <windows.h>

#if defined(APP_HAS_ICON)
IDI_ICON1               ICON                    PRODUCT_ICON
#endif

LANGUAGE LANG_ENGLISH, SUBLANG_DEFAULT

VS_VERSION_INFO VERSIONINFO
    FILEVERSION FILE_VERSION_RESOURCE
    PRODUCTVERSION PRODUCT_VERSION_RESOURCE
    FILEFLAGSMASK 0x3fL
#if defined(_DEBUG) || !defined(NDEBUG)
    FILEFLAGS 0x1L
#else
    FILEFLAGS 0x0L
#endif
    FILEOS 0x4L
    FILETYPE 0x1L
    FILESUBTYPE 0x0L
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904E4"
        BEGIN
            VALUE "Comments", PRODUCT_COMMENTS
            VALUE "CompanyName", PRODUCT_COMPANY_NAME
            VALUE "FileDescription", PRODUCT_FILE_DESCRIPTION
            VALUE "FileVersion", FILE_VERSION_RESOURCE_STR
            VALUE "InternalName", PRODUCT_INTERNAL_NAME
            VALUE "LegalCopyright", PRODUCT_COMPANY_COPYRIGHT
            VALUE "OriginalFilename", PRODUCT_ORIGINAL_FILENAME
            VALUE "ProductName", PRODUCT_BUNDLE
            VALUE "ProductVersion", PRODUCT_VERSION_RESOURCE_STR
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x409, 1200
    END
END
