/**
 *
 * This file is part of Tulip (http://tulip.labri.fr)
 *
 * Authors: David Auber and the Tulip development Team
 * from LaBRI, University of Bordeaux
 *
 * Tulip is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Tulip is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 */
#include <tulip/StlFunctions.h>
#include <tulip/DoubleProperty.h>

bool tlp::LessByMetric::operator()(tlp::node n1, tlp::node n2) {
  return (metric->getNodeValue(n1) < metric->getNodeValue(n2));
}
bool tlp::LessByMetric::operator()(tlp::edge e1, tlp::edge e2) {
  return (metric->getEdgeValue(e1) < metric->getEdgeValue(e2));
}
