/**
 * @example clique_import/cliqueimport.cpp
 * @brief This example shows a simple graph import plug-in.
 *
 * It details the different functions and possibilities of a Graph import plug-in.
 *
 * @file clique_import/cliqueimport.h
 */
