/**
 *
 * This file is part of Tulip (http://tulip.labri.fr)
 *
 * Authors: David Auber and the Tulip development Team
 * from LaBRI, University of Bordeaux
 *
 * Tulip is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Tulip is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 */

#ifndef DIFFUSIONRATEFUNCTIONSIMPLE_H_
#define DIFFUSIONRATEFUNCTIONSIMPLE_H_

#include "DiffusionRateFunction.h"

class DiffusionRateFunctionSimple : public DiffusionRateFunction {
public:
  DiffusionRateFunctionSimple(TimeDecreasingFunction *timeFunction, unsigned int neighborhood);
  ~DiffusionRateFunctionSimple() override;
  double computeSpaceRate(unsigned int distance, unsigned int currentIteration,
                          unsigned int maxIteration, unsigned int inputSampleSize) override;

protected:
  unsigned int neighborhoodMax;
};

#endif /* DIFFUSIONRATEFUNCTIONSIMPLE_H_ */
