/*
 *
 * This file is part of Tulip (http://tulip.labri.fr)
 *
 * Authors: David Auber and the Tulip development Team
 * from LaBRI, University of Bordeaux
 *
 * Tulip is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Tulip is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 */

///@cond DOXYGEN_HIDDEN

#ifndef TULIP_PLUGIN_HEADERS_H
#define TULIP_PLUGIN_HEADERS_H

#include <tulip/DoubleProperty.h>
#include <tulip/LayoutProperty.h>
#include <tulip/BooleanProperty.h>
#include <tulip/StringProperty.h>
#include <tulip/Algorithm.h>
#include <tulip/PropertyAlgorithm.h>
#include <tulip/ImportModule.h>
#include <tulip/ExportModule.h>
#include <tulip/SizeProperty.h>
#include <tulip/ColorProperty.h>
#include <tulip/IntegerProperty.h>
#include <tulip/GraphMeasure.h>
#include <tulip/GraphTools.h>
#include <tulip/AcyclicTest.h>
#include <tulip/BiconnectedTest.h>
#include <tulip/ConnectedTest.h>
#include <tulip/PlanarityTest.h>
#include <tulip/SimpleTest.h>
#include <tulip/TreeTest.h>
#include <tulip/StringCollection.h>
#include <tulip/TlpTools.h>
#include <tulip/StaticProperty.h>
#include <tulip/PluginLister.h>

#endif // TULIP_PLUGIN_HEADERS_H

///@endcond
