// Warning: do not update this file !!!
// It was automatically generated by utils/scripts/generate_fa_cpp_init_code.sh
// from Font Awesome icons version 5.12.1

std::string TulipFontAwesome::getVersion() {
  return "5.12.1";
}

static void initIconCodePoints() {
  addIconCodePoint("fa-500px", 0xf26e);
  addIconCodePoint("fa-accessible-icon", 0xf368);
  addIconCodePoint("fa-accusoft", 0xf369);
  addIconCodePoint("fa-acquisitions-incorporated", 0xf6af);
  addIconCodePoint("fa-ad", 0xf641);
  addIconCodePoint("fa-address-book", 0xf2b9);
  addIconCodePoint("fa-address-card", 0xf2bb);
  addIconCodePoint("fa-adjust", 0xf042);
  addIconCodePoint("fa-adn", 0xf170);
  addIconCodePoint("fa-adobe", 0xf778);
  addIconCodePoint("fa-adversal", 0xf36a);
  addIconCodePoint("fa-affiliatetheme", 0xf36b);
  addIconCodePoint("fa-air-freshener", 0xf5d0);
  addIconCodePoint("fa-airbnb", 0xf834);
  addIconCodePoint("fa-algolia", 0xf36c);
  addIconCodePoint("fa-align-center", 0xf037);
  addIconCodePoint("fa-align-justify", 0xf039);
  addIconCodePoint("fa-align-left", 0xf036);
  addIconCodePoint("fa-align-right", 0xf038);
  addIconCodePoint("fa-alipay", 0xf642);
  addIconCodePoint("fa-allergies", 0xf461);
  addIconCodePoint("fa-amazon", 0xf270);
  addIconCodePoint("fa-amazon-pay", 0xf42c);
  addIconCodePoint("fa-ambulance", 0xf0f9);
  addIconCodePoint("fa-american-sign-language-interpreting", 0xf2a3);
  addIconCodePoint("fa-amilia", 0xf36d);
  addIconCodePoint("fa-anchor", 0xf13d);
  addIconCodePoint("fa-android", 0xf17b);
  addIconCodePoint("fa-angellist", 0xf209);
  addIconCodePoint("fa-angle-double-down", 0xf103);
  addIconCodePoint("fa-angle-double-left", 0xf100);
  addIconCodePoint("fa-angle-double-right", 0xf101);
  addIconCodePoint("fa-angle-double-up", 0xf102);
  addIconCodePoint("fa-angle-down", 0xf107);
  addIconCodePoint("fa-angle-left", 0xf104);
  addIconCodePoint("fa-angle-right", 0xf105);
  addIconCodePoint("fa-angle-up", 0xf106);
  addIconCodePoint("fa-angry", 0xf556);
  addIconCodePoint("fa-angrycreative", 0xf36e);
  addIconCodePoint("fa-angular", 0xf420);
  addIconCodePoint("fa-ankh", 0xf644);
  addIconCodePoint("fa-app-store", 0xf36f);
  addIconCodePoint("fa-app-store-ios", 0xf370);
  addIconCodePoint("fa-apper", 0xf371);
  addIconCodePoint("fa-apple", 0xf179);
  addIconCodePoint("fa-apple-alt", 0xf5d1);
  addIconCodePoint("fa-apple-pay", 0xf415);
  addIconCodePoint("fa-archive", 0xf187);
  addIconCodePoint("fa-archway", 0xf557);
  addIconCodePoint("fa-arrow-alt-circle-down", 0xf358);
  addIconCodePoint("fa-arrow-alt-circle-left", 0xf359);
  addIconCodePoint("fa-arrow-alt-circle-right", 0xf35a);
  addIconCodePoint("fa-arrow-alt-circle-up", 0xf35b);
  addIconCodePoint("fa-arrow-circle-down", 0xf0ab);
  addIconCodePoint("fa-arrow-circle-left", 0xf0a8);
  addIconCodePoint("fa-arrow-circle-right", 0xf0a9);
  addIconCodePoint("fa-arrow-circle-up", 0xf0aa);
  addIconCodePoint("fa-arrow-down", 0xf063);
  addIconCodePoint("fa-arrow-left", 0xf060);
  addIconCodePoint("fa-arrow-right", 0xf061);
  addIconCodePoint("fa-arrow-up", 0xf062);
  addIconCodePoint("fa-arrows-alt", 0xf0b2);
  addIconCodePoint("fa-arrows-alt-h", 0xf337);
  addIconCodePoint("fa-arrows-alt-v", 0xf338);
  addIconCodePoint("fa-artstation", 0xf77a);
  addIconCodePoint("fa-assistive-listening-systems", 0xf2a2);
  addIconCodePoint("fa-asterisk", 0xf069);
  addIconCodePoint("fa-asymmetrik", 0xf372);
  addIconCodePoint("fa-at", 0xf1fa);
  addIconCodePoint("fa-atlas", 0xf558);
  addIconCodePoint("fa-atlassian", 0xf77b);
  addIconCodePoint("fa-atom", 0xf5d2);
  addIconCodePoint("fa-audible", 0xf373);
  addIconCodePoint("fa-audio-description", 0xf29e);
  addIconCodePoint("fa-autoprefixer", 0xf41c);
  addIconCodePoint("fa-avianex", 0xf374);
  addIconCodePoint("fa-aviato", 0xf421);
  addIconCodePoint("fa-award", 0xf559);
  addIconCodePoint("fa-aws", 0xf375);
  addIconCodePoint("fa-baby", 0xf77c);
  addIconCodePoint("fa-baby-carriage", 0xf77d);
  addIconCodePoint("fa-backspace", 0xf55a);
  addIconCodePoint("fa-backward", 0xf04a);
  addIconCodePoint("fa-bacon", 0xf7e5);
  addIconCodePoint("fa-bahai", 0xf666);
  addIconCodePoint("fa-balance-scale", 0xf24e);
  addIconCodePoint("fa-balance-scale-left", 0xf515);
  addIconCodePoint("fa-balance-scale-right", 0xf516);
  addIconCodePoint("fa-ban", 0xf05e);
  addIconCodePoint("fa-band-aid", 0xf462);
  addIconCodePoint("fa-bandcamp", 0xf2d5);
  addIconCodePoint("fa-barcode", 0xf02a);
  addIconCodePoint("fa-bars", 0xf0c9);
  addIconCodePoint("fa-baseball-ball", 0xf433);
  addIconCodePoint("fa-basketball-ball", 0xf434);
  addIconCodePoint("fa-bath", 0xf2cd);
  addIconCodePoint("fa-battery-empty", 0xf244);
  addIconCodePoint("fa-battery-full", 0xf240);
  addIconCodePoint("fa-battery-half", 0xf242);
  addIconCodePoint("fa-battery-quarter", 0xf243);
  addIconCodePoint("fa-battery-three-quarters", 0xf241);
  addIconCodePoint("fa-battle-net", 0xf835);
  addIconCodePoint("fa-bed", 0xf236);
  addIconCodePoint("fa-beer", 0xf0fc);
  addIconCodePoint("fa-behance", 0xf1b4);
  addIconCodePoint("fa-behance-square", 0xf1b5);
  addIconCodePoint("fa-bell", 0xf0f3);
  addIconCodePoint("fa-bell-slash", 0xf1f6);
  addIconCodePoint("fa-bezier-curve", 0xf55b);
  addIconCodePoint("fa-bible", 0xf647);
  addIconCodePoint("fa-bicycle", 0xf206);
  addIconCodePoint("fa-biking", 0xf84a);
  addIconCodePoint("fa-bimobject", 0xf378);
  addIconCodePoint("fa-binoculars", 0xf1e5);
  addIconCodePoint("fa-biohazard", 0xf780);
  addIconCodePoint("fa-birthday-cake", 0xf1fd);
  addIconCodePoint("fa-bitbucket", 0xf171);
  addIconCodePoint("fa-bitcoin", 0xf379);
  addIconCodePoint("fa-bity", 0xf37a);
  addIconCodePoint("fa-black-tie", 0xf27e);
  addIconCodePoint("fa-blackberry", 0xf37b);
  addIconCodePoint("fa-blender", 0xf517);
  addIconCodePoint("fa-blender-phone", 0xf6b6);
  addIconCodePoint("fa-blind", 0xf29d);
  addIconCodePoint("fa-blog", 0xf781);
  addIconCodePoint("fa-blogger", 0xf37c);
  addIconCodePoint("fa-blogger-b", 0xf37d);
  addIconCodePoint("fa-bluetooth", 0xf293);
  addIconCodePoint("fa-bluetooth-b", 0xf294);
  addIconCodePoint("fa-bold", 0xf032);
  addIconCodePoint("fa-bolt", 0xf0e7);
  addIconCodePoint("fa-bomb", 0xf1e2);
  addIconCodePoint("fa-bone", 0xf5d7);
  addIconCodePoint("fa-bong", 0xf55c);
  addIconCodePoint("fa-book", 0xf02d);
  addIconCodePoint("fa-book-dead", 0xf6b7);
  addIconCodePoint("fa-book-medical", 0xf7e6);
  addIconCodePoint("fa-book-open", 0xf518);
  addIconCodePoint("fa-book-reader", 0xf5da);
  addIconCodePoint("fa-bookmark", 0xf02e);
  addIconCodePoint("fa-bootstrap", 0xf836);
  addIconCodePoint("fa-border-all", 0xf84c);
  addIconCodePoint("fa-border-none", 0xf850);
  addIconCodePoint("fa-border-style", 0xf853);
  addIconCodePoint("fa-bowling-ball", 0xf436);
  addIconCodePoint("fa-box", 0xf466);
  addIconCodePoint("fa-box-open", 0xf49e);
  addIconCodePoint("fa-boxes", 0xf468);
  addIconCodePoint("fa-braille", 0xf2a1);
  addIconCodePoint("fa-brain", 0xf5dc);
  addIconCodePoint("fa-bread-slice", 0xf7ec);
  addIconCodePoint("fa-briefcase", 0xf0b1);
  addIconCodePoint("fa-briefcase-medical", 0xf469);
  addIconCodePoint("fa-broadcast-tower", 0xf519);
  addIconCodePoint("fa-broom", 0xf51a);
  addIconCodePoint("fa-brush", 0xf55d);
  addIconCodePoint("fa-btc", 0xf15a);
  addIconCodePoint("fa-buffer", 0xf837);
  addIconCodePoint("fa-bug", 0xf188);
  addIconCodePoint("fa-building", 0xf1ad);
  addIconCodePoint("fa-bullhorn", 0xf0a1);
  addIconCodePoint("fa-bullseye", 0xf140);
  addIconCodePoint("fa-burn", 0xf46a);
  addIconCodePoint("fa-buromobelexperte", 0xf37f);
  addIconCodePoint("fa-bus", 0xf207);
  addIconCodePoint("fa-bus-alt", 0xf55e);
  addIconCodePoint("fa-business-time", 0xf64a);
  addIconCodePoint("fa-buy-n-large", 0xf8a6);
  addIconCodePoint("fa-buysellads", 0xf20d);
  addIconCodePoint("fa-calculator", 0xf1ec);
  addIconCodePoint("fa-calendar", 0xf133);
  addIconCodePoint("fa-calendar-alt", 0xf073);
  addIconCodePoint("fa-calendar-check", 0xf274);
  addIconCodePoint("fa-calendar-day", 0xf783);
  addIconCodePoint("fa-calendar-minus", 0xf272);
  addIconCodePoint("fa-calendar-plus", 0xf271);
  addIconCodePoint("fa-calendar-times", 0xf273);
  addIconCodePoint("fa-calendar-week", 0xf784);
  addIconCodePoint("fa-camera", 0xf030);
  addIconCodePoint("fa-camera-retro", 0xf083);
  addIconCodePoint("fa-campground", 0xf6bb);
  addIconCodePoint("fa-canadian-maple-leaf", 0xf785);
  addIconCodePoint("fa-candy-cane", 0xf786);
  addIconCodePoint("fa-cannabis", 0xf55f);
  addIconCodePoint("fa-capsules", 0xf46b);
  addIconCodePoint("fa-car", 0xf1b9);
  addIconCodePoint("fa-car-alt", 0xf5de);
  addIconCodePoint("fa-car-battery", 0xf5df);
  addIconCodePoint("fa-car-crash", 0xf5e1);
  addIconCodePoint("fa-car-side", 0xf5e4);
  addIconCodePoint("fa-caravan", 0xf8ff);
  addIconCodePoint("fa-caret-down", 0xf0d7);
  addIconCodePoint("fa-caret-left", 0xf0d9);
  addIconCodePoint("fa-caret-right", 0xf0da);
  addIconCodePoint("fa-caret-square-down", 0xf150);
  addIconCodePoint("fa-caret-square-left", 0xf191);
  addIconCodePoint("fa-caret-square-right", 0xf152);
  addIconCodePoint("fa-caret-square-up", 0xf151);
  addIconCodePoint("fa-caret-up", 0xf0d8);
  addIconCodePoint("fa-carrot", 0xf787);
  addIconCodePoint("fa-cart-arrow-down", 0xf218);
  addIconCodePoint("fa-cart-plus", 0xf217);
  addIconCodePoint("fa-cash-register", 0xf788);
  addIconCodePoint("fa-cat", 0xf6be);
  addIconCodePoint("fa-cc-amazon-pay", 0xf42d);
  addIconCodePoint("fa-cc-amex", 0xf1f3);
  addIconCodePoint("fa-cc-apple-pay", 0xf416);
  addIconCodePoint("fa-cc-diners-club", 0xf24c);
  addIconCodePoint("fa-cc-discover", 0xf1f2);
  addIconCodePoint("fa-cc-jcb", 0xf24b);
  addIconCodePoint("fa-cc-mastercard", 0xf1f1);
  addIconCodePoint("fa-cc-paypal", 0xf1f4);
  addIconCodePoint("fa-cc-stripe", 0xf1f5);
  addIconCodePoint("fa-cc-visa", 0xf1f0);
  addIconCodePoint("fa-centercode", 0xf380);
  addIconCodePoint("fa-centos", 0xf789);
  addIconCodePoint("fa-certificate", 0xf0a3);
  addIconCodePoint("fa-chair", 0xf6c0);
  addIconCodePoint("fa-chalkboard", 0xf51b);
  addIconCodePoint("fa-chalkboard-teacher", 0xf51c);
  addIconCodePoint("fa-charging-station", 0xf5e7);
  addIconCodePoint("fa-chart-area", 0xf1fe);
  addIconCodePoint("fa-chart-bar", 0xf080);
  addIconCodePoint("fa-chart-line", 0xf201);
  addIconCodePoint("fa-chart-pie", 0xf200);
  addIconCodePoint("fa-check", 0xf00c);
  addIconCodePoint("fa-check-circle", 0xf058);
  addIconCodePoint("fa-check-double", 0xf560);
  addIconCodePoint("fa-check-square", 0xf14a);
  addIconCodePoint("fa-cheese", 0xf7ef);
  addIconCodePoint("fa-chess", 0xf439);
  addIconCodePoint("fa-chess-bishop", 0xf43a);
  addIconCodePoint("fa-chess-board", 0xf43c);
  addIconCodePoint("fa-chess-king", 0xf43f);
  addIconCodePoint("fa-chess-knight", 0xf441);
  addIconCodePoint("fa-chess-pawn", 0xf443);
  addIconCodePoint("fa-chess-queen", 0xf445);
  addIconCodePoint("fa-chess-rook", 0xf447);
  addIconCodePoint("fa-chevron-circle-down", 0xf13a);
  addIconCodePoint("fa-chevron-circle-left", 0xf137);
  addIconCodePoint("fa-chevron-circle-right", 0xf138);
  addIconCodePoint("fa-chevron-circle-up", 0xf139);
  addIconCodePoint("fa-chevron-down", 0xf078);
  addIconCodePoint("fa-chevron-left", 0xf053);
  addIconCodePoint("fa-chevron-right", 0xf054);
  addIconCodePoint("fa-chevron-up", 0xf077);
  addIconCodePoint("fa-child", 0xf1ae);
  addIconCodePoint("fa-chrome", 0xf268);
  addIconCodePoint("fa-chromecast", 0xf838);
  addIconCodePoint("fa-church", 0xf51d);
  addIconCodePoint("fa-circle", 0xf111);
  addIconCodePoint("fa-circle-notch", 0xf1ce);
  addIconCodePoint("fa-city", 0xf64f);
  addIconCodePoint("fa-clinic-medical", 0xf7f2);
  addIconCodePoint("fa-clipboard", 0xf328);
  addIconCodePoint("fa-clipboard-check", 0xf46c);
  addIconCodePoint("fa-clipboard-list", 0xf46d);
  addIconCodePoint("fa-clock", 0xf017);
  addIconCodePoint("fa-clone", 0xf24d);
  addIconCodePoint("fa-closed-captioning", 0xf20a);
  addIconCodePoint("fa-cloud", 0xf0c2);
  addIconCodePoint("fa-cloud-download-alt", 0xf381);
  addIconCodePoint("fa-cloud-meatball", 0xf73b);
  addIconCodePoint("fa-cloud-moon", 0xf6c3);
  addIconCodePoint("fa-cloud-moon-rain", 0xf73c);
  addIconCodePoint("fa-cloud-rain", 0xf73d);
  addIconCodePoint("fa-cloud-showers-heavy", 0xf740);
  addIconCodePoint("fa-cloud-sun", 0xf6c4);
  addIconCodePoint("fa-cloud-sun-rain", 0xf743);
  addIconCodePoint("fa-cloud-upload-alt", 0xf382);
  addIconCodePoint("fa-cloudscale", 0xf383);
  addIconCodePoint("fa-cloudsmith", 0xf384);
  addIconCodePoint("fa-cloudversify", 0xf385);
  addIconCodePoint("fa-cocktail", 0xf561);
  addIconCodePoint("fa-code", 0xf121);
  addIconCodePoint("fa-code-branch", 0xf126);
  addIconCodePoint("fa-codepen", 0xf1cb);
  addIconCodePoint("fa-codiepie", 0xf284);
  addIconCodePoint("fa-coffee", 0xf0f4);
  addIconCodePoint("fa-cog", 0xf013);
  addIconCodePoint("fa-cogs", 0xf085);
  addIconCodePoint("fa-coins", 0xf51e);
  addIconCodePoint("fa-columns", 0xf0db);
  addIconCodePoint("fa-comment", 0xf075);
  addIconCodePoint("fa-comment-alt", 0xf27a);
  addIconCodePoint("fa-comment-dollar", 0xf651);
  addIconCodePoint("fa-comment-dots", 0xf4ad);
  addIconCodePoint("fa-comment-medical", 0xf7f5);
  addIconCodePoint("fa-comment-slash", 0xf4b3);
  addIconCodePoint("fa-comments", 0xf086);
  addIconCodePoint("fa-comments-dollar", 0xf653);
  addIconCodePoint("fa-compact-disc", 0xf51f);
  addIconCodePoint("fa-compass", 0xf14e);
  addIconCodePoint("fa-compress", 0xf066);
  addIconCodePoint("fa-compress-alt", 0xf422);
  addIconCodePoint("fa-compress-arrows-alt", 0xf78c);
  addIconCodePoint("fa-concierge-bell", 0xf562);
  addIconCodePoint("fa-confluence", 0xf78d);
  addIconCodePoint("fa-connectdevelop", 0xf20e);
  addIconCodePoint("fa-contao", 0xf26d);
  addIconCodePoint("fa-cookie", 0xf563);
  addIconCodePoint("fa-cookie-bite", 0xf564);
  addIconCodePoint("fa-copy", 0xf0c5);
  addIconCodePoint("fa-copyright", 0xf1f9);
  addIconCodePoint("fa-cotton-bureau", 0xf89e);
  addIconCodePoint("fa-couch", 0xf4b8);
  addIconCodePoint("fa-cpanel", 0xf388);
  addIconCodePoint("fa-creative-commons", 0xf25e);
  addIconCodePoint("fa-creative-commons-by", 0xf4e7);
  addIconCodePoint("fa-creative-commons-nc", 0xf4e8);
  addIconCodePoint("fa-creative-commons-nc-eu", 0xf4e9);
  addIconCodePoint("fa-creative-commons-nc-jp", 0xf4ea);
  addIconCodePoint("fa-creative-commons-nd", 0xf4eb);
  addIconCodePoint("fa-creative-commons-pd", 0xf4ec);
  addIconCodePoint("fa-creative-commons-pd-alt", 0xf4ed);
  addIconCodePoint("fa-creative-commons-remix", 0xf4ee);
  addIconCodePoint("fa-creative-commons-sa", 0xf4ef);
  addIconCodePoint("fa-creative-commons-sampling", 0xf4f0);
  addIconCodePoint("fa-creative-commons-sampling-plus", 0xf4f1);
  addIconCodePoint("fa-creative-commons-share", 0xf4f2);
  addIconCodePoint("fa-creative-commons-zero", 0xf4f3);
  addIconCodePoint("fa-credit-card", 0xf09d);
  addIconCodePoint("fa-critical-role", 0xf6c9);
  addIconCodePoint("fa-crop", 0xf125);
  addIconCodePoint("fa-crop-alt", 0xf565);
  addIconCodePoint("fa-cross", 0xf654);
  addIconCodePoint("fa-crosshairs", 0xf05b);
  addIconCodePoint("fa-crow", 0xf520);
  addIconCodePoint("fa-crown", 0xf521);
  addIconCodePoint("fa-crutch", 0xf7f7);
  addIconCodePoint("fa-css3", 0xf13c);
  addIconCodePoint("fa-css3-alt", 0xf38b);
  addIconCodePoint("fa-cube", 0xf1b2);
  addIconCodePoint("fa-cubes", 0xf1b3);
  addIconCodePoint("fa-cut", 0xf0c4);
  addIconCodePoint("fa-cuttlefish", 0xf38c);
  addIconCodePoint("fa-d-and-d", 0xf38d);
  addIconCodePoint("fa-d-and-d-beyond", 0xf6ca);
  addIconCodePoint("fa-dailymotion", 0xf952);
  addIconCodePoint("fa-dashcube", 0xf210);
  addIconCodePoint("fa-database", 0xf1c0);
  addIconCodePoint("fa-deaf", 0xf2a4);
  addIconCodePoint("fa-delicious", 0xf1a5);
  addIconCodePoint("fa-democrat", 0xf747);
  addIconCodePoint("fa-deploydog", 0xf38e);
  addIconCodePoint("fa-deskpro", 0xf38f);
  addIconCodePoint("fa-desktop", 0xf108);
  addIconCodePoint("fa-dev", 0xf6cc);
  addIconCodePoint("fa-deviantart", 0xf1bd);
  addIconCodePoint("fa-dharmachakra", 0xf655);
  addIconCodePoint("fa-dhl", 0xf790);
  addIconCodePoint("fa-diagnoses", 0xf470);
  addIconCodePoint("fa-diaspora", 0xf791);
  addIconCodePoint("fa-dice", 0xf522);
  addIconCodePoint("fa-dice-d20", 0xf6cf);
  addIconCodePoint("fa-dice-d6", 0xf6d1);
  addIconCodePoint("fa-dice-five", 0xf523);
  addIconCodePoint("fa-dice-four", 0xf524);
  addIconCodePoint("fa-dice-one", 0xf525);
  addIconCodePoint("fa-dice-six", 0xf526);
  addIconCodePoint("fa-dice-three", 0xf527);
  addIconCodePoint("fa-dice-two", 0xf528);
  addIconCodePoint("fa-digg", 0xf1a6);
  addIconCodePoint("fa-digital-ocean", 0xf391);
  addIconCodePoint("fa-digital-tachograph", 0xf566);
  addIconCodePoint("fa-directions", 0xf5eb);
  addIconCodePoint("fa-discord", 0xf392);
  addIconCodePoint("fa-discourse", 0xf393);
  addIconCodePoint("fa-divide", 0xf529);
  addIconCodePoint("fa-dizzy", 0xf567);
  addIconCodePoint("fa-dna", 0xf471);
  addIconCodePoint("fa-dochub", 0xf394);
  addIconCodePoint("fa-docker", 0xf395);
  addIconCodePoint("fa-dog", 0xf6d3);
  addIconCodePoint("fa-dollar-sign", 0xf155);
  addIconCodePoint("fa-dolly", 0xf472);
  addIconCodePoint("fa-dolly-flatbed", 0xf474);
  addIconCodePoint("fa-donate", 0xf4b9);
  addIconCodePoint("fa-door-closed", 0xf52a);
  addIconCodePoint("fa-door-open", 0xf52b);
  addIconCodePoint("fa-dot-circle", 0xf192);
  addIconCodePoint("fa-dove", 0xf4ba);
  addIconCodePoint("fa-download", 0xf019);
  addIconCodePoint("fa-draft2digital", 0xf396);
  addIconCodePoint("fa-drafting-compass", 0xf568);
  addIconCodePoint("fa-dragon", 0xf6d5);
  addIconCodePoint("fa-draw-polygon", 0xf5ee);
  addIconCodePoint("fa-dribbble", 0xf17d);
  addIconCodePoint("fa-dribbble-square", 0xf397);
  addIconCodePoint("fa-dropbox", 0xf16b);
  addIconCodePoint("fa-drum", 0xf569);
  addIconCodePoint("fa-drum-steelpan", 0xf56a);
  addIconCodePoint("fa-drumstick-bite", 0xf6d7);
  addIconCodePoint("fa-drupal", 0xf1a9);
  addIconCodePoint("fa-dumbbell", 0xf44b);
  addIconCodePoint("fa-dumpster", 0xf793);
  addIconCodePoint("fa-dumpster-fire", 0xf794);
  addIconCodePoint("fa-dungeon", 0xf6d9);
  addIconCodePoint("fa-dyalog", 0xf399);
  addIconCodePoint("fa-earlybirds", 0xf39a);
  addIconCodePoint("fa-ebay", 0xf4f4);
  addIconCodePoint("fa-edge", 0xf282);
  addIconCodePoint("fa-edit", 0xf044);
  addIconCodePoint("fa-egg", 0xf7fb);
  addIconCodePoint("fa-eject", 0xf052);
  addIconCodePoint("fa-elementor", 0xf430);
  addIconCodePoint("fa-ellipsis-h", 0xf141);
  addIconCodePoint("fa-ellipsis-v", 0xf142);
  addIconCodePoint("fa-ello", 0xf5f1);
  addIconCodePoint("fa-ember", 0xf423);
  addIconCodePoint("fa-empire", 0xf1d1);
  addIconCodePoint("fa-envelope", 0xf0e0);
  addIconCodePoint("fa-envelope-open", 0xf2b6);
  addIconCodePoint("fa-envelope-open-text", 0xf658);
  addIconCodePoint("fa-envelope-square", 0xf199);
  addIconCodePoint("fa-envira", 0xf299);
  addIconCodePoint("fa-equals", 0xf52c);
  addIconCodePoint("fa-eraser", 0xf12d);
  addIconCodePoint("fa-erlang", 0xf39d);
  addIconCodePoint("fa-ethereum", 0xf42e);
  addIconCodePoint("fa-ethernet", 0xf796);
  addIconCodePoint("fa-etsy", 0xf2d7);
  addIconCodePoint("fa-euro-sign", 0xf153);
  addIconCodePoint("fa-evernote", 0xf839);
  addIconCodePoint("fa-exchange-alt", 0xf362);
  addIconCodePoint("fa-exclamation", 0xf12a);
  addIconCodePoint("fa-exclamation-circle", 0xf06a);
  addIconCodePoint("fa-exclamation-triangle", 0xf071);
  addIconCodePoint("fa-expand", 0xf065);
  addIconCodePoint("fa-expand-alt", 0xf424);
  addIconCodePoint("fa-expand-arrows-alt", 0xf31e);
  addIconCodePoint("fa-expeditedssl", 0xf23e);
  addIconCodePoint("fa-external-link-alt", 0xf35d);
  addIconCodePoint("fa-external-link-square-alt", 0xf360);
  addIconCodePoint("fa-eye", 0xf06e);
  addIconCodePoint("fa-eye-dropper", 0xf1fb);
  addIconCodePoint("fa-eye-slash", 0xf070);
  addIconCodePoint("fa-facebook", 0xf09a);
  addIconCodePoint("fa-facebook-f", 0xf39e);
  addIconCodePoint("fa-facebook-messenger", 0xf39f);
  addIconCodePoint("fa-facebook-square", 0xf082);
  addIconCodePoint("fa-fan", 0xf863);
  addIconCodePoint("fa-fantasy-flight-games", 0xf6dc);
  addIconCodePoint("fa-fast-backward", 0xf049);
  addIconCodePoint("fa-fast-forward", 0xf050);
  addIconCodePoint("fa-fax", 0xf1ac);
  addIconCodePoint("fa-feather", 0xf52d);
  addIconCodePoint("fa-feather-alt", 0xf56b);
  addIconCodePoint("fa-fedex", 0xf797);
  addIconCodePoint("fa-fedora", 0xf798);
  addIconCodePoint("fa-female", 0xf182);
  addIconCodePoint("fa-fighter-jet", 0xf0fb);
  addIconCodePoint("fa-figma", 0xf799);
  addIconCodePoint("fa-file", 0xf15b);
  addIconCodePoint("fa-file-alt", 0xf15c);
  addIconCodePoint("fa-file-archive", 0xf1c6);
  addIconCodePoint("fa-file-audio", 0xf1c7);
  addIconCodePoint("fa-file-code", 0xf1c9);
  addIconCodePoint("fa-file-contract", 0xf56c);
  addIconCodePoint("fa-file-csv", 0xf6dd);
  addIconCodePoint("fa-file-download", 0xf56d);
  addIconCodePoint("fa-file-excel", 0xf1c3);
  addIconCodePoint("fa-file-export", 0xf56e);
  addIconCodePoint("fa-file-image", 0xf1c5);
  addIconCodePoint("fa-file-import", 0xf56f);
  addIconCodePoint("fa-file-invoice", 0xf570);
  addIconCodePoint("fa-file-invoice-dollar", 0xf571);
  addIconCodePoint("fa-file-medical", 0xf477);
  addIconCodePoint("fa-file-medical-alt", 0xf478);
  addIconCodePoint("fa-file-pdf", 0xf1c1);
  addIconCodePoint("fa-file-powerpoint", 0xf1c4);
  addIconCodePoint("fa-file-prescription", 0xf572);
  addIconCodePoint("fa-file-signature", 0xf573);
  addIconCodePoint("fa-file-upload", 0xf574);
  addIconCodePoint("fa-file-video", 0xf1c8);
  addIconCodePoint("fa-file-word", 0xf1c2);
  addIconCodePoint("fa-fill", 0xf575);
  addIconCodePoint("fa-fill-drip", 0xf576);
  addIconCodePoint("fa-film", 0xf008);
  addIconCodePoint("fa-filter", 0xf0b0);
  addIconCodePoint("fa-fingerprint", 0xf577);
  addIconCodePoint("fa-fire", 0xf06d);
  addIconCodePoint("fa-fire-alt", 0xf7e4);
  addIconCodePoint("fa-fire-extinguisher", 0xf134);
  addIconCodePoint("fa-firefox", 0xf269);
  addIconCodePoint("fa-firefox-browser", 0xf907);
  addIconCodePoint("fa-first-aid", 0xf479);
  addIconCodePoint("fa-first-order", 0xf2b0);
  addIconCodePoint("fa-first-order-alt", 0xf50a);
  addIconCodePoint("fa-firstdraft", 0xf3a1);
  addIconCodePoint("fa-fish", 0xf578);
  addIconCodePoint("fa-fist-raised", 0xf6de);
  addIconCodePoint("fa-flag", 0xf024);
  addIconCodePoint("fa-flag-checkered", 0xf11e);
  addIconCodePoint("fa-flag-usa", 0xf74d);
  addIconCodePoint("fa-flask", 0xf0c3);
  addIconCodePoint("fa-flickr", 0xf16e);
  addIconCodePoint("fa-flipboard", 0xf44d);
  addIconCodePoint("fa-flushed", 0xf579);
  addIconCodePoint("fa-fly", 0xf417);
  addIconCodePoint("fa-folder", 0xf07b);
  addIconCodePoint("fa-folder-minus", 0xf65d);
  addIconCodePoint("fa-folder-open", 0xf07c);
  addIconCodePoint("fa-folder-plus", 0xf65e);
  addIconCodePoint("fa-font", 0xf031);
  addIconCodePoint("fa-font-awesome", 0xf2b4);
  addIconCodePoint("fa-font-awesome-alt", 0xf35c);
  addIconCodePoint("fa-font-awesome-flag", 0xf425);
  // fa-font-awesome-logo-full cannot be found
  // addIconCodePoint("fa-font-awesome-logo-full", 0xf4e6);
  addIconCodePoint("fa-fonticons", 0xf280);
  addIconCodePoint("fa-fonticons-fi", 0xf3a2);
  addIconCodePoint("fa-football-ball", 0xf44e);
  addIconCodePoint("fa-fort-awesome", 0xf286);
  addIconCodePoint("fa-fort-awesome-alt", 0xf3a3);
  addIconCodePoint("fa-forumbee", 0xf211);
  addIconCodePoint("fa-forward", 0xf04e);
  addIconCodePoint("fa-foursquare", 0xf180);
  addIconCodePoint("fa-free-code-camp", 0xf2c5);
  addIconCodePoint("fa-freebsd", 0xf3a4);
  addIconCodePoint("fa-frog", 0xf52e);
  addIconCodePoint("fa-frown", 0xf119);
  addIconCodePoint("fa-frown-open", 0xf57a);
  addIconCodePoint("fa-fulcrum", 0xf50b);
  addIconCodePoint("fa-funnel-dollar", 0xf662);
  addIconCodePoint("fa-futbol", 0xf1e3);
  addIconCodePoint("fa-galactic-republic", 0xf50c);
  addIconCodePoint("fa-galactic-senate", 0xf50d);
  addIconCodePoint("fa-gamepad", 0xf11b);
  addIconCodePoint("fa-gas-pump", 0xf52f);
  addIconCodePoint("fa-gavel", 0xf0e3);
  addIconCodePoint("fa-gem", 0xf3a5);
  addIconCodePoint("fa-genderless", 0xf22d);
  addIconCodePoint("fa-get-pocket", 0xf265);
  addIconCodePoint("fa-gg", 0xf260);
  addIconCodePoint("fa-gg-circle", 0xf261);
  addIconCodePoint("fa-ghost", 0xf6e2);
  addIconCodePoint("fa-gift", 0xf06b);
  addIconCodePoint("fa-gifts", 0xf79c);
  addIconCodePoint("fa-git", 0xf1d3);
  addIconCodePoint("fa-git-alt", 0xf841);
  addIconCodePoint("fa-git-square", 0xf1d2);
  addIconCodePoint("fa-github", 0xf09b);
  addIconCodePoint("fa-github-alt", 0xf113);
  addIconCodePoint("fa-github-square", 0xf092);
  addIconCodePoint("fa-gitkraken", 0xf3a6);
  addIconCodePoint("fa-gitlab", 0xf296);
  addIconCodePoint("fa-gitter", 0xf426);
  addIconCodePoint("fa-glass-cheers", 0xf79f);
  addIconCodePoint("fa-glass-martini", 0xf000);
  addIconCodePoint("fa-glass-martini-alt", 0xf57b);
  addIconCodePoint("fa-glass-whiskey", 0xf7a0);
  addIconCodePoint("fa-glasses", 0xf530);
  addIconCodePoint("fa-glide", 0xf2a5);
  addIconCodePoint("fa-glide-g", 0xf2a6);
  addIconCodePoint("fa-globe", 0xf0ac);
  addIconCodePoint("fa-globe-africa", 0xf57c);
  addIconCodePoint("fa-globe-americas", 0xf57d);
  addIconCodePoint("fa-globe-asia", 0xf57e);
  addIconCodePoint("fa-globe-europe", 0xf7a2);
  addIconCodePoint("fa-gofore", 0xf3a7);
  addIconCodePoint("fa-golf-ball", 0xf450);
  addIconCodePoint("fa-goodreads", 0xf3a8);
  addIconCodePoint("fa-goodreads-g", 0xf3a9);
  addIconCodePoint("fa-google", 0xf1a0);
  addIconCodePoint("fa-google-drive", 0xf3aa);
  addIconCodePoint("fa-google-play", 0xf3ab);
  addIconCodePoint("fa-google-plus", 0xf2b3);
  addIconCodePoint("fa-google-plus-g", 0xf0d5);
  addIconCodePoint("fa-google-plus-square", 0xf0d4);
  addIconCodePoint("fa-google-wallet", 0xf1ee);
  addIconCodePoint("fa-gopuram", 0xf664);
  addIconCodePoint("fa-graduation-cap", 0xf19d);
  addIconCodePoint("fa-gratipay", 0xf184);
  addIconCodePoint("fa-grav", 0xf2d6);
  addIconCodePoint("fa-greater-than", 0xf531);
  addIconCodePoint("fa-greater-than-equal", 0xf532);
  addIconCodePoint("fa-grimace", 0xf57f);
  addIconCodePoint("fa-grin", 0xf580);
  addIconCodePoint("fa-grin-alt", 0xf581);
  addIconCodePoint("fa-grin-beam", 0xf582);
  addIconCodePoint("fa-grin-beam-sweat", 0xf583);
  addIconCodePoint("fa-grin-hearts", 0xf584);
  addIconCodePoint("fa-grin-squint", 0xf585);
  addIconCodePoint("fa-grin-squint-tears", 0xf586);
  addIconCodePoint("fa-grin-stars", 0xf587);
  addIconCodePoint("fa-grin-tears", 0xf588);
  addIconCodePoint("fa-grin-tongue", 0xf589);
  addIconCodePoint("fa-grin-tongue-squint", 0xf58a);
  addIconCodePoint("fa-grin-tongue-wink", 0xf58b);
  addIconCodePoint("fa-grin-wink", 0xf58c);
  addIconCodePoint("fa-grip-horizontal", 0xf58d);
  addIconCodePoint("fa-grip-lines", 0xf7a4);
  addIconCodePoint("fa-grip-lines-vertical", 0xf7a5);
  addIconCodePoint("fa-grip-vertical", 0xf58e);
  addIconCodePoint("fa-gripfire", 0xf3ac);
  addIconCodePoint("fa-grunt", 0xf3ad);
  addIconCodePoint("fa-guitar", 0xf7a6);
  addIconCodePoint("fa-gulp", 0xf3ae);
  addIconCodePoint("fa-h-square", 0xf0fd);
  addIconCodePoint("fa-hacker-news", 0xf1d4);
  addIconCodePoint("fa-hacker-news-square", 0xf3af);
  addIconCodePoint("fa-hackerrank", 0xf5f7);
  addIconCodePoint("fa-hamburger", 0xf805);
  addIconCodePoint("fa-hammer", 0xf6e3);
  addIconCodePoint("fa-hamsa", 0xf665);
  addIconCodePoint("fa-hand-holding", 0xf4bd);
  addIconCodePoint("fa-hand-holding-heart", 0xf4be);
  addIconCodePoint("fa-hand-holding-usd", 0xf4c0);
  addIconCodePoint("fa-hand-lizard", 0xf258);
  addIconCodePoint("fa-hand-middle-finger", 0xf806);
  addIconCodePoint("fa-hand-paper", 0xf256);
  addIconCodePoint("fa-hand-peace", 0xf25b);
  addIconCodePoint("fa-hand-point-down", 0xf0a7);
  addIconCodePoint("fa-hand-point-left", 0xf0a5);
  addIconCodePoint("fa-hand-point-right", 0xf0a4);
  addIconCodePoint("fa-hand-point-up", 0xf0a6);
  addIconCodePoint("fa-hand-pointer", 0xf25a);
  addIconCodePoint("fa-hand-rock", 0xf255);
  addIconCodePoint("fa-hand-scissors", 0xf257);
  addIconCodePoint("fa-hand-spock", 0xf259);
  addIconCodePoint("fa-hands", 0xf4c2);
  addIconCodePoint("fa-hands-helping", 0xf4c4);
  addIconCodePoint("fa-handshake", 0xf2b5);
  addIconCodePoint("fa-hanukiah", 0xf6e6);
  addIconCodePoint("fa-hard-hat", 0xf807);
  addIconCodePoint("fa-hashtag", 0xf292);
  addIconCodePoint("fa-hat-cowboy", 0xf8c0);
  addIconCodePoint("fa-hat-cowboy-side", 0xf8c1);
  addIconCodePoint("fa-hat-wizard", 0xf6e8);
  addIconCodePoint("fa-hdd", 0xf0a0);
  addIconCodePoint("fa-heading", 0xf1dc);
  addIconCodePoint("fa-headphones", 0xf025);
  addIconCodePoint("fa-headphones-alt", 0xf58f);
  addIconCodePoint("fa-headset", 0xf590);
  addIconCodePoint("fa-heart", 0xf004);
  addIconCodePoint("fa-heart-broken", 0xf7a9);
  addIconCodePoint("fa-heartbeat", 0xf21e);
  addIconCodePoint("fa-helicopter", 0xf533);
  addIconCodePoint("fa-highlighter", 0xf591);
  addIconCodePoint("fa-hiking", 0xf6ec);
  addIconCodePoint("fa-hippo", 0xf6ed);
  addIconCodePoint("fa-hips", 0xf452);
  addIconCodePoint("fa-hire-a-helper", 0xf3b0);
  addIconCodePoint("fa-history", 0xf1da);
  addIconCodePoint("fa-hockey-puck", 0xf453);
  addIconCodePoint("fa-holly-berry", 0xf7aa);
  addIconCodePoint("fa-home", 0xf015);
  addIconCodePoint("fa-hooli", 0xf427);
  addIconCodePoint("fa-hornbill", 0xf592);
  addIconCodePoint("fa-horse", 0xf6f0);
  addIconCodePoint("fa-horse-head", 0xf7ab);
  addIconCodePoint("fa-hospital", 0xf0f8);
  addIconCodePoint("fa-hospital-alt", 0xf47d);
  addIconCodePoint("fa-hospital-symbol", 0xf47e);
  addIconCodePoint("fa-hot-tub", 0xf593);
  addIconCodePoint("fa-hotdog", 0xf80f);
  addIconCodePoint("fa-hotel", 0xf594);
  addIconCodePoint("fa-hotjar", 0xf3b1);
  addIconCodePoint("fa-hourglass", 0xf254);
  addIconCodePoint("fa-hourglass-end", 0xf253);
  addIconCodePoint("fa-hourglass-half", 0xf252);
  addIconCodePoint("fa-hourglass-start", 0xf251);
  addIconCodePoint("fa-house-damage", 0xf6f1);
  addIconCodePoint("fa-houzz", 0xf27c);
  addIconCodePoint("fa-hryvnia", 0xf6f2);
  addIconCodePoint("fa-html5", 0xf13b);
  addIconCodePoint("fa-hubspot", 0xf3b2);
  addIconCodePoint("fa-i-cursor", 0xf246);
  addIconCodePoint("fa-ice-cream", 0xf810);
  addIconCodePoint("fa-icicles", 0xf7ad);
  addIconCodePoint("fa-icons", 0xf86d);
  addIconCodePoint("fa-id-badge", 0xf2c1);
  addIconCodePoint("fa-id-card", 0xf2c2);
  addIconCodePoint("fa-id-card-alt", 0xf47f);
  addIconCodePoint("fa-ideal", 0xf913);
  addIconCodePoint("fa-igloo", 0xf7ae);
  addIconCodePoint("fa-image", 0xf03e);
  addIconCodePoint("fa-images", 0xf302);
  addIconCodePoint("fa-imdb", 0xf2d8);
  addIconCodePoint("fa-inbox", 0xf01c);
  addIconCodePoint("fa-indent", 0xf03c);
  addIconCodePoint("fa-industry", 0xf275);
  addIconCodePoint("fa-infinity", 0xf534);
  addIconCodePoint("fa-info", 0xf129);
  addIconCodePoint("fa-info-circle", 0xf05a);
  addIconCodePoint("fa-instagram", 0xf16d);
  addIconCodePoint("fa-instagram-square", 0xf955);
  addIconCodePoint("fa-intercom", 0xf7af);
  addIconCodePoint("fa-internet-explorer", 0xf26b);
  addIconCodePoint("fa-invision", 0xf7b0);
  addIconCodePoint("fa-ioxhost", 0xf208);
  addIconCodePoint("fa-italic", 0xf033);
  addIconCodePoint("fa-itch-io", 0xf83a);
  addIconCodePoint("fa-itunes", 0xf3b4);
  addIconCodePoint("fa-itunes-note", 0xf3b5);
  addIconCodePoint("fa-java", 0xf4e4);
  addIconCodePoint("fa-jedi", 0xf669);
  addIconCodePoint("fa-jedi-order", 0xf50e);
  addIconCodePoint("fa-jenkins", 0xf3b6);
  addIconCodePoint("fa-jira", 0xf7b1);
  addIconCodePoint("fa-joget", 0xf3b7);
  addIconCodePoint("fa-joint", 0xf595);
  addIconCodePoint("fa-joomla", 0xf1aa);
  addIconCodePoint("fa-journal-whills", 0xf66a);
  addIconCodePoint("fa-js", 0xf3b8);
  addIconCodePoint("fa-js-square", 0xf3b9);
  addIconCodePoint("fa-jsfiddle", 0xf1cc);
  addIconCodePoint("fa-kaaba", 0xf66b);
  addIconCodePoint("fa-kaggle", 0xf5fa);
  addIconCodePoint("fa-key", 0xf084);
  addIconCodePoint("fa-keybase", 0xf4f5);
  addIconCodePoint("fa-keyboard", 0xf11c);
  addIconCodePoint("fa-keycdn", 0xf3ba);
  addIconCodePoint("fa-khanda", 0xf66d);
  addIconCodePoint("fa-kickstarter", 0xf3bb);
  addIconCodePoint("fa-kickstarter-k", 0xf3bc);
  addIconCodePoint("fa-kiss", 0xf596);
  addIconCodePoint("fa-kiss-beam", 0xf597);
  addIconCodePoint("fa-kiss-wink-heart", 0xf598);
  addIconCodePoint("fa-kiwi-bird", 0xf535);
  addIconCodePoint("fa-korvue", 0xf42f);
  addIconCodePoint("fa-landmark", 0xf66f);
  addIconCodePoint("fa-language", 0xf1ab);
  addIconCodePoint("fa-laptop", 0xf109);
  addIconCodePoint("fa-laptop-code", 0xf5fc);
  addIconCodePoint("fa-laptop-medical", 0xf812);
  addIconCodePoint("fa-laravel", 0xf3bd);
  addIconCodePoint("fa-lastfm", 0xf202);
  addIconCodePoint("fa-lastfm-square", 0xf203);
  addIconCodePoint("fa-laugh", 0xf599);
  addIconCodePoint("fa-laugh-beam", 0xf59a);
  addIconCodePoint("fa-laugh-squint", 0xf59b);
  addIconCodePoint("fa-laugh-wink", 0xf59c);
  addIconCodePoint("fa-layer-group", 0xf5fd);
  addIconCodePoint("fa-leaf", 0xf06c);
  addIconCodePoint("fa-leanpub", 0xf212);
  addIconCodePoint("fa-lemon", 0xf094);
  addIconCodePoint("fa-less", 0xf41d);
  addIconCodePoint("fa-less-than", 0xf536);
  addIconCodePoint("fa-less-than-equal", 0xf537);
  addIconCodePoint("fa-level-down-alt", 0xf3be);
  addIconCodePoint("fa-level-up-alt", 0xf3bf);
  addIconCodePoint("fa-life-ring", 0xf1cd);
  addIconCodePoint("fa-lightbulb", 0xf0eb);
  addIconCodePoint("fa-line", 0xf3c0);
  addIconCodePoint("fa-link", 0xf0c1);
  addIconCodePoint("fa-linkedin", 0xf08c);
  addIconCodePoint("fa-linkedin-in", 0xf0e1);
  addIconCodePoint("fa-linode", 0xf2b8);
  addIconCodePoint("fa-linux", 0xf17c);
  addIconCodePoint("fa-lira-sign", 0xf195);
  addIconCodePoint("fa-list", 0xf03a);
  addIconCodePoint("fa-list-alt", 0xf022);
  addIconCodePoint("fa-list-ol", 0xf0cb);
  addIconCodePoint("fa-list-ul", 0xf0ca);
  addIconCodePoint("fa-location-arrow", 0xf124);
  addIconCodePoint("fa-lock", 0xf023);
  addIconCodePoint("fa-lock-open", 0xf3c1);
  addIconCodePoint("fa-long-arrow-alt-down", 0xf309);
  addIconCodePoint("fa-long-arrow-alt-left", 0xf30a);
  addIconCodePoint("fa-long-arrow-alt-right", 0xf30b);
  addIconCodePoint("fa-long-arrow-alt-up", 0xf30c);
  addIconCodePoint("fa-low-vision", 0xf2a8);
  addIconCodePoint("fa-luggage-cart", 0xf59d);
  addIconCodePoint("fa-lyft", 0xf3c3);
  addIconCodePoint("fa-magento", 0xf3c4);
  addIconCodePoint("fa-magic", 0xf0d0);
  addIconCodePoint("fa-magnet", 0xf076);
  addIconCodePoint("fa-mail-bulk", 0xf674);
  addIconCodePoint("fa-mailchimp", 0xf59e);
  addIconCodePoint("fa-male", 0xf183);
  addIconCodePoint("fa-mandalorian", 0xf50f);
  addIconCodePoint("fa-map", 0xf279);
  addIconCodePoint("fa-map-marked", 0xf59f);
  addIconCodePoint("fa-map-marked-alt", 0xf5a0);
  addIconCodePoint("fa-map-marker", 0xf041);
  addIconCodePoint("fa-map-marker-alt", 0xf3c5);
  addIconCodePoint("fa-map-pin", 0xf276);
  addIconCodePoint("fa-map-signs", 0xf277);
  addIconCodePoint("fa-markdown", 0xf60f);
  addIconCodePoint("fa-marker", 0xf5a1);
  addIconCodePoint("fa-mars", 0xf222);
  addIconCodePoint("fa-mars-double", 0xf227);
  addIconCodePoint("fa-mars-stroke", 0xf229);
  addIconCodePoint("fa-mars-stroke-h", 0xf22b);
  addIconCodePoint("fa-mars-stroke-v", 0xf22a);
  addIconCodePoint("fa-mask", 0xf6fa);
  addIconCodePoint("fa-mastodon", 0xf4f6);
  addIconCodePoint("fa-maxcdn", 0xf136);
  addIconCodePoint("fa-mdb", 0xf8ca);
  addIconCodePoint("fa-medal", 0xf5a2);
  addIconCodePoint("fa-medapps", 0xf3c6);
  addIconCodePoint("fa-medium", 0xf23a);
  addIconCodePoint("fa-medium-m", 0xf3c7);
  addIconCodePoint("fa-medkit", 0xf0fa);
  addIconCodePoint("fa-medrt", 0xf3c8);
  addIconCodePoint("fa-meetup", 0xf2e0);
  addIconCodePoint("fa-megaport", 0xf5a3);
  addIconCodePoint("fa-meh", 0xf11a);
  addIconCodePoint("fa-meh-blank", 0xf5a4);
  addIconCodePoint("fa-meh-rolling-eyes", 0xf5a5);
  addIconCodePoint("fa-memory", 0xf538);
  addIconCodePoint("fa-mendeley", 0xf7b3);
  addIconCodePoint("fa-menorah", 0xf676);
  addIconCodePoint("fa-mercury", 0xf223);
  addIconCodePoint("fa-meteor", 0xf753);
  addIconCodePoint("fa-microblog", 0xf91a);
  addIconCodePoint("fa-microchip", 0xf2db);
  addIconCodePoint("fa-microphone", 0xf130);
  addIconCodePoint("fa-microphone-alt", 0xf3c9);
  addIconCodePoint("fa-microphone-alt-slash", 0xf539);
  addIconCodePoint("fa-microphone-slash", 0xf131);
  addIconCodePoint("fa-microscope", 0xf610);
  addIconCodePoint("fa-microsoft", 0xf3ca);
  addIconCodePoint("fa-minus", 0xf068);
  addIconCodePoint("fa-minus-circle", 0xf056);
  addIconCodePoint("fa-minus-square", 0xf146);
  addIconCodePoint("fa-mitten", 0xf7b5);
  addIconCodePoint("fa-mix", 0xf3cb);
  addIconCodePoint("fa-mixcloud", 0xf289);
  addIconCodePoint("fa-mixer", 0xf956);
  addIconCodePoint("fa-mizuni", 0xf3cc);
  addIconCodePoint("fa-mobile", 0xf10b);
  addIconCodePoint("fa-mobile-alt", 0xf3cd);
  addIconCodePoint("fa-modx", 0xf285);
  addIconCodePoint("fa-monero", 0xf3d0);
  addIconCodePoint("fa-money-bill", 0xf0d6);
  addIconCodePoint("fa-money-bill-alt", 0xf3d1);
  addIconCodePoint("fa-money-bill-wave", 0xf53a);
  addIconCodePoint("fa-money-bill-wave-alt", 0xf53b);
  addIconCodePoint("fa-money-check", 0xf53c);
  addIconCodePoint("fa-money-check-alt", 0xf53d);
  addIconCodePoint("fa-monument", 0xf5a6);
  addIconCodePoint("fa-moon", 0xf186);
  addIconCodePoint("fa-mortar-pestle", 0xf5a7);
  addIconCodePoint("fa-mosque", 0xf678);
  addIconCodePoint("fa-motorcycle", 0xf21c);
  addIconCodePoint("fa-mountain", 0xf6fc);
  addIconCodePoint("fa-mouse", 0xf8cc);
  addIconCodePoint("fa-mouse-pointer", 0xf245);
  addIconCodePoint("fa-mug-hot", 0xf7b6);
  addIconCodePoint("fa-music", 0xf001);
  addIconCodePoint("fa-napster", 0xf3d2);
  addIconCodePoint("fa-neos", 0xf612);
  addIconCodePoint("fa-network-wired", 0xf6ff);
  addIconCodePoint("fa-neuter", 0xf22c);
  addIconCodePoint("fa-newspaper", 0xf1ea);
  addIconCodePoint("fa-nimblr", 0xf5a8);
  addIconCodePoint("fa-node", 0xf419);
  addIconCodePoint("fa-node-js", 0xf3d3);
  addIconCodePoint("fa-not-equal", 0xf53e);
  addIconCodePoint("fa-notes-medical", 0xf481);
  addIconCodePoint("fa-npm", 0xf3d4);
  addIconCodePoint("fa-ns8", 0xf3d5);
  addIconCodePoint("fa-nutritionix", 0xf3d6);
  addIconCodePoint("fa-object-group", 0xf247);
  addIconCodePoint("fa-object-ungroup", 0xf248);
  addIconCodePoint("fa-odnoklassniki", 0xf263);
  addIconCodePoint("fa-odnoklassniki-square", 0xf264);
  addIconCodePoint("fa-oil-can", 0xf613);
  addIconCodePoint("fa-old-republic", 0xf510);
  addIconCodePoint("fa-om", 0xf679);
  addIconCodePoint("fa-opencart", 0xf23d);
  addIconCodePoint("fa-openid", 0xf19b);
  addIconCodePoint("fa-opera", 0xf26a);
  addIconCodePoint("fa-optin-monster", 0xf23c);
  addIconCodePoint("fa-orcid", 0xf8d2);
  addIconCodePoint("fa-osi", 0xf41a);
  addIconCodePoint("fa-otter", 0xf700);
  addIconCodePoint("fa-outdent", 0xf03b);
  addIconCodePoint("fa-page4", 0xf3d7);
  addIconCodePoint("fa-pagelines", 0xf18c);
  addIconCodePoint("fa-pager", 0xf815);
  addIconCodePoint("fa-paint-brush", 0xf1fc);
  addIconCodePoint("fa-paint-roller", 0xf5aa);
  addIconCodePoint("fa-palette", 0xf53f);
  addIconCodePoint("fa-palfed", 0xf3d8);
  addIconCodePoint("fa-pallet", 0xf482);
  addIconCodePoint("fa-paper-plane", 0xf1d8);
  addIconCodePoint("fa-paperclip", 0xf0c6);
  addIconCodePoint("fa-parachute-box", 0xf4cd);
  addIconCodePoint("fa-paragraph", 0xf1dd);
  addIconCodePoint("fa-parking", 0xf540);
  addIconCodePoint("fa-passport", 0xf5ab);
  addIconCodePoint("fa-pastafarianism", 0xf67b);
  addIconCodePoint("fa-paste", 0xf0ea);
  addIconCodePoint("fa-patreon", 0xf3d9);
  addIconCodePoint("fa-pause", 0xf04c);
  addIconCodePoint("fa-pause-circle", 0xf28b);
  addIconCodePoint("fa-paw", 0xf1b0);
  addIconCodePoint("fa-paypal", 0xf1ed);
  addIconCodePoint("fa-peace", 0xf67c);
  addIconCodePoint("fa-pen", 0xf304);
  addIconCodePoint("fa-pen-alt", 0xf305);
  addIconCodePoint("fa-pen-fancy", 0xf5ac);
  addIconCodePoint("fa-pen-nib", 0xf5ad);
  addIconCodePoint("fa-pen-square", 0xf14b);
  addIconCodePoint("fa-pencil-alt", 0xf303);
  addIconCodePoint("fa-pencil-ruler", 0xf5ae);
  addIconCodePoint("fa-penny-arcade", 0xf704);
  addIconCodePoint("fa-people-carry", 0xf4ce);
  addIconCodePoint("fa-pepper-hot", 0xf816);
  addIconCodePoint("fa-percent", 0xf295);
  addIconCodePoint("fa-percentage", 0xf541);
  addIconCodePoint("fa-periscope", 0xf3da);
  addIconCodePoint("fa-person-booth", 0xf756);
  addIconCodePoint("fa-phabricator", 0xf3db);
  addIconCodePoint("fa-phoenix-framework", 0xf3dc);
  addIconCodePoint("fa-phoenix-squadron", 0xf511);
  addIconCodePoint("fa-phone", 0xf095);
  addIconCodePoint("fa-phone-alt", 0xf879);
  addIconCodePoint("fa-phone-slash", 0xf3dd);
  addIconCodePoint("fa-phone-square", 0xf098);
  addIconCodePoint("fa-phone-square-alt", 0xf87b);
  addIconCodePoint("fa-phone-volume", 0xf2a0);
  addIconCodePoint("fa-photo-video", 0xf87c);
  addIconCodePoint("fa-php", 0xf457);
  addIconCodePoint("fa-pied-piper", 0xf2ae);
  addIconCodePoint("fa-pied-piper-alt", 0xf1a8);
  addIconCodePoint("fa-pied-piper-hat", 0xf4e5);
  addIconCodePoint("fa-pied-piper-pp", 0xf1a7);
  addIconCodePoint("fa-pied-piper-square", 0xf91e);
  addIconCodePoint("fa-piggy-bank", 0xf4d3);
  addIconCodePoint("fa-pills", 0xf484);
  addIconCodePoint("fa-pinterest", 0xf0d2);
  addIconCodePoint("fa-pinterest-p", 0xf231);
  addIconCodePoint("fa-pinterest-square", 0xf0d3);
  addIconCodePoint("fa-pizza-slice", 0xf818);
  addIconCodePoint("fa-place-of-worship", 0xf67f);
  addIconCodePoint("fa-plane", 0xf072);
  addIconCodePoint("fa-plane-arrival", 0xf5af);
  addIconCodePoint("fa-plane-departure", 0xf5b0);
  addIconCodePoint("fa-play", 0xf04b);
  addIconCodePoint("fa-play-circle", 0xf144);
  addIconCodePoint("fa-playstation", 0xf3df);
  addIconCodePoint("fa-plug", 0xf1e6);
  addIconCodePoint("fa-plus", 0xf067);
  addIconCodePoint("fa-plus-circle", 0xf055);
  addIconCodePoint("fa-plus-square", 0xf0fe);
  addIconCodePoint("fa-podcast", 0xf2ce);
  addIconCodePoint("fa-poll", 0xf681);
  addIconCodePoint("fa-poll-h", 0xf682);
  addIconCodePoint("fa-poo", 0xf2fe);
  addIconCodePoint("fa-poo-storm", 0xf75a);
  addIconCodePoint("fa-poop", 0xf619);
  addIconCodePoint("fa-portrait", 0xf3e0);
  addIconCodePoint("fa-pound-sign", 0xf154);
  addIconCodePoint("fa-power-off", 0xf011);
  addIconCodePoint("fa-pray", 0xf683);
  addIconCodePoint("fa-praying-hands", 0xf684);
  addIconCodePoint("fa-prescription", 0xf5b1);
  addIconCodePoint("fa-prescription-bottle", 0xf485);
  addIconCodePoint("fa-prescription-bottle-alt", 0xf486);
  addIconCodePoint("fa-print", 0xf02f);
  addIconCodePoint("fa-procedures", 0xf487);
  addIconCodePoint("fa-product-hunt", 0xf288);
  addIconCodePoint("fa-project-diagram", 0xf542);
  addIconCodePoint("fa-pushed", 0xf3e1);
  addIconCodePoint("fa-puzzle-piece", 0xf12e);
  addIconCodePoint("fa-python", 0xf3e2);
  addIconCodePoint("fa-qq", 0xf1d6);
  addIconCodePoint("fa-qrcode", 0xf029);
  addIconCodePoint("fa-question", 0xf128);
  addIconCodePoint("fa-question-circle", 0xf059);
  addIconCodePoint("fa-quidditch", 0xf458);
  addIconCodePoint("fa-quinscape", 0xf459);
  addIconCodePoint("fa-quora", 0xf2c4);
  addIconCodePoint("fa-quote-left", 0xf10d);
  addIconCodePoint("fa-quote-right", 0xf10e);
  addIconCodePoint("fa-quran", 0xf687);
  addIconCodePoint("fa-r-project", 0xf4f7);
  addIconCodePoint("fa-radiation", 0xf7b9);
  addIconCodePoint("fa-radiation-alt", 0xf7ba);
  addIconCodePoint("fa-rainbow", 0xf75b);
  addIconCodePoint("fa-random", 0xf074);
  addIconCodePoint("fa-raspberry-pi", 0xf7bb);
  addIconCodePoint("fa-ravelry", 0xf2d9);
  addIconCodePoint("fa-react", 0xf41b);
  addIconCodePoint("fa-reacteurope", 0xf75d);
  addIconCodePoint("fa-readme", 0xf4d5);
  addIconCodePoint("fa-rebel", 0xf1d0);
  addIconCodePoint("fa-receipt", 0xf543);
  addIconCodePoint("fa-record-vinyl", 0xf8d9);
  addIconCodePoint("fa-recycle", 0xf1b8);
  addIconCodePoint("fa-red-river", 0xf3e3);
  addIconCodePoint("fa-reddit", 0xf1a1);
  addIconCodePoint("fa-reddit-alien", 0xf281);
  addIconCodePoint("fa-reddit-square", 0xf1a2);
  addIconCodePoint("fa-redhat", 0xf7bc);
  addIconCodePoint("fa-redo", 0xf01e);
  addIconCodePoint("fa-redo-alt", 0xf2f9);
  addIconCodePoint("fa-registered", 0xf25d);
  addIconCodePoint("fa-remove-format", 0xf87d);
  addIconCodePoint("fa-renren", 0xf18b);
  addIconCodePoint("fa-reply", 0xf3e5);
  addIconCodePoint("fa-reply-all", 0xf122);
  addIconCodePoint("fa-replyd", 0xf3e6);
  addIconCodePoint("fa-republican", 0xf75e);
  addIconCodePoint("fa-researchgate", 0xf4f8);
  addIconCodePoint("fa-resolving", 0xf3e7);
  addIconCodePoint("fa-restroom", 0xf7bd);
  addIconCodePoint("fa-retweet", 0xf079);
  addIconCodePoint("fa-rev", 0xf5b2);
  addIconCodePoint("fa-ribbon", 0xf4d6);
  addIconCodePoint("fa-ring", 0xf70b);
  addIconCodePoint("fa-road", 0xf018);
  addIconCodePoint("fa-robot", 0xf544);
  addIconCodePoint("fa-rocket", 0xf135);
  addIconCodePoint("fa-rocketchat", 0xf3e8);
  addIconCodePoint("fa-rockrms", 0xf3e9);
  addIconCodePoint("fa-route", 0xf4d7);
  addIconCodePoint("fa-rss", 0xf09e);
  addIconCodePoint("fa-rss-square", 0xf143);
  addIconCodePoint("fa-ruble-sign", 0xf158);
  addIconCodePoint("fa-ruler", 0xf545);
  addIconCodePoint("fa-ruler-combined", 0xf546);
  addIconCodePoint("fa-ruler-horizontal", 0xf547);
  addIconCodePoint("fa-ruler-vertical", 0xf548);
  addIconCodePoint("fa-running", 0xf70c);
  addIconCodePoint("fa-rupee-sign", 0xf156);
  addIconCodePoint("fa-sad-cry", 0xf5b3);
  addIconCodePoint("fa-sad-tear", 0xf5b4);
  addIconCodePoint("fa-safari", 0xf267);
  addIconCodePoint("fa-salesforce", 0xf83b);
  addIconCodePoint("fa-sass", 0xf41e);
  addIconCodePoint("fa-satellite", 0xf7bf);
  addIconCodePoint("fa-satellite-dish", 0xf7c0);
  addIconCodePoint("fa-save", 0xf0c7);
  addIconCodePoint("fa-schlix", 0xf3ea);
  addIconCodePoint("fa-school", 0xf549);
  addIconCodePoint("fa-screwdriver", 0xf54a);
  addIconCodePoint("fa-scribd", 0xf28a);
  addIconCodePoint("fa-scroll", 0xf70e);
  addIconCodePoint("fa-sd-card", 0xf7c2);
  addIconCodePoint("fa-search", 0xf002);
  addIconCodePoint("fa-search-dollar", 0xf688);
  addIconCodePoint("fa-search-location", 0xf689);
  addIconCodePoint("fa-search-minus", 0xf010);
  addIconCodePoint("fa-search-plus", 0xf00e);
  addIconCodePoint("fa-searchengin", 0xf3eb);
  addIconCodePoint("fa-seedling", 0xf4d8);
  addIconCodePoint("fa-sellcast", 0xf2da);
  addIconCodePoint("fa-sellsy", 0xf213);
  addIconCodePoint("fa-server", 0xf233);
  addIconCodePoint("fa-servicestack", 0xf3ec);
  addIconCodePoint("fa-shapes", 0xf61f);
  addIconCodePoint("fa-share", 0xf064);
  addIconCodePoint("fa-share-alt", 0xf1e0);
  addIconCodePoint("fa-share-alt-square", 0xf1e1);
  addIconCodePoint("fa-share-square", 0xf14d);
  addIconCodePoint("fa-shekel-sign", 0xf20b);
  addIconCodePoint("fa-shield-alt", 0xf3ed);
  addIconCodePoint("fa-ship", 0xf21a);
  addIconCodePoint("fa-shipping-fast", 0xf48b);
  addIconCodePoint("fa-shirtsinbulk", 0xf214);
  addIconCodePoint("fa-shoe-prints", 0xf54b);
  addIconCodePoint("fa-shopify", 0xf957);
  addIconCodePoint("fa-shopping-bag", 0xf290);
  addIconCodePoint("fa-shopping-basket", 0xf291);
  addIconCodePoint("fa-shopping-cart", 0xf07a);
  addIconCodePoint("fa-shopware", 0xf5b5);
  addIconCodePoint("fa-shower", 0xf2cc);
  addIconCodePoint("fa-shuttle-van", 0xf5b6);
  addIconCodePoint("fa-sign", 0xf4d9);
  addIconCodePoint("fa-sign-in-alt", 0xf2f6);
  addIconCodePoint("fa-sign-language", 0xf2a7);
  addIconCodePoint("fa-sign-out-alt", 0xf2f5);
  addIconCodePoint("fa-signal", 0xf012);
  addIconCodePoint("fa-signature", 0xf5b7);
  addIconCodePoint("fa-sim-card", 0xf7c4);
  addIconCodePoint("fa-simplybuilt", 0xf215);
  addIconCodePoint("fa-sistrix", 0xf3ee);
  addIconCodePoint("fa-sitemap", 0xf0e8);
  addIconCodePoint("fa-sith", 0xf512);
  addIconCodePoint("fa-skating", 0xf7c5);
  addIconCodePoint("fa-sketch", 0xf7c6);
  addIconCodePoint("fa-skiing", 0xf7c9);
  addIconCodePoint("fa-skiing-nordic", 0xf7ca);
  addIconCodePoint("fa-skull", 0xf54c);
  addIconCodePoint("fa-skull-crossbones", 0xf714);
  addIconCodePoint("fa-skyatlas", 0xf216);
  addIconCodePoint("fa-skype", 0xf17e);
  addIconCodePoint("fa-slack", 0xf198);
  addIconCodePoint("fa-slack-hash", 0xf3ef);
  addIconCodePoint("fa-slash", 0xf715);
  addIconCodePoint("fa-sleigh", 0xf7cc);
  addIconCodePoint("fa-sliders-h", 0xf1de);
  addIconCodePoint("fa-slideshare", 0xf1e7);
  addIconCodePoint("fa-smile", 0xf118);
  addIconCodePoint("fa-smile-beam", 0xf5b8);
  addIconCodePoint("fa-smile-wink", 0xf4da);
  addIconCodePoint("fa-smog", 0xf75f);
  addIconCodePoint("fa-smoking", 0xf48d);
  addIconCodePoint("fa-smoking-ban", 0xf54d);
  addIconCodePoint("fa-sms", 0xf7cd);
  addIconCodePoint("fa-snapchat", 0xf2ab);
  addIconCodePoint("fa-snapchat-ghost", 0xf2ac);
  addIconCodePoint("fa-snapchat-square", 0xf2ad);
  addIconCodePoint("fa-snowboarding", 0xf7ce);
  addIconCodePoint("fa-snowflake", 0xf2dc);
  addIconCodePoint("fa-snowman", 0xf7d0);
  addIconCodePoint("fa-snowplow", 0xf7d2);
  addIconCodePoint("fa-socks", 0xf696);
  addIconCodePoint("fa-solar-panel", 0xf5ba);
  addIconCodePoint("fa-sort", 0xf0dc);
  addIconCodePoint("fa-sort-alpha-down", 0xf15d);
  addIconCodePoint("fa-sort-alpha-down-alt", 0xf881);
  addIconCodePoint("fa-sort-alpha-up", 0xf15e);
  addIconCodePoint("fa-sort-alpha-up-alt", 0xf882);
  addIconCodePoint("fa-sort-amount-down", 0xf160);
  addIconCodePoint("fa-sort-amount-down-alt", 0xf884);
  addIconCodePoint("fa-sort-amount-up", 0xf161);
  addIconCodePoint("fa-sort-amount-up-alt", 0xf885);
  addIconCodePoint("fa-sort-down", 0xf0dd);
  addIconCodePoint("fa-sort-numeric-down", 0xf162);
  addIconCodePoint("fa-sort-numeric-down-alt", 0xf886);
  addIconCodePoint("fa-sort-numeric-up", 0xf163);
  addIconCodePoint("fa-sort-numeric-up-alt", 0xf887);
  addIconCodePoint("fa-sort-up", 0xf0de);
  addIconCodePoint("fa-soundcloud", 0xf1be);
  addIconCodePoint("fa-sourcetree", 0xf7d3);
  addIconCodePoint("fa-spa", 0xf5bb);
  addIconCodePoint("fa-space-shuttle", 0xf197);
  addIconCodePoint("fa-speakap", 0xf3f3);
  addIconCodePoint("fa-speaker-deck", 0xf83c);
  addIconCodePoint("fa-spell-check", 0xf891);
  addIconCodePoint("fa-spider", 0xf717);
  addIconCodePoint("fa-spinner", 0xf110);
  addIconCodePoint("fa-splotch", 0xf5bc);
  addIconCodePoint("fa-spotify", 0xf1bc);
  addIconCodePoint("fa-spray-can", 0xf5bd);
  addIconCodePoint("fa-square", 0xf0c8);
  addIconCodePoint("fa-square-full", 0xf45c);
  addIconCodePoint("fa-square-root-alt", 0xf698);
  addIconCodePoint("fa-squarespace", 0xf5be);
  addIconCodePoint("fa-stack-exchange", 0xf18d);
  addIconCodePoint("fa-stack-overflow", 0xf16c);
  addIconCodePoint("fa-stackpath", 0xf842);
  addIconCodePoint("fa-stamp", 0xf5bf);
  addIconCodePoint("fa-star", 0xf005);
  addIconCodePoint("fa-star-and-crescent", 0xf699);
  addIconCodePoint("fa-star-half", 0xf089);
  addIconCodePoint("fa-star-half-alt", 0xf5c0);
  addIconCodePoint("fa-star-of-david", 0xf69a);
  addIconCodePoint("fa-star-of-life", 0xf621);
  addIconCodePoint("fa-staylinked", 0xf3f5);
  addIconCodePoint("fa-steam", 0xf1b6);
  addIconCodePoint("fa-steam-square", 0xf1b7);
  addIconCodePoint("fa-steam-symbol", 0xf3f6);
  addIconCodePoint("fa-step-backward", 0xf048);
  addIconCodePoint("fa-step-forward", 0xf051);
  addIconCodePoint("fa-stethoscope", 0xf0f1);
  addIconCodePoint("fa-sticker-mule", 0xf3f7);
  addIconCodePoint("fa-sticky-note", 0xf249);
  addIconCodePoint("fa-stop", 0xf04d);
  addIconCodePoint("fa-stop-circle", 0xf28d);
  addIconCodePoint("fa-stopwatch", 0xf2f2);
  addIconCodePoint("fa-store", 0xf54e);
  addIconCodePoint("fa-store-alt", 0xf54f);
  addIconCodePoint("fa-strava", 0xf428);
  addIconCodePoint("fa-stream", 0xf550);
  addIconCodePoint("fa-street-view", 0xf21d);
  addIconCodePoint("fa-strikethrough", 0xf0cc);
  addIconCodePoint("fa-stripe", 0xf429);
  addIconCodePoint("fa-stripe-s", 0xf42a);
  addIconCodePoint("fa-stroopwafel", 0xf551);
  addIconCodePoint("fa-studiovinari", 0xf3f8);
  addIconCodePoint("fa-stumbleupon", 0xf1a4);
  addIconCodePoint("fa-stumbleupon-circle", 0xf1a3);
  addIconCodePoint("fa-subscript", 0xf12c);
  addIconCodePoint("fa-subway", 0xf239);
  addIconCodePoint("fa-suitcase", 0xf0f2);
  addIconCodePoint("fa-suitcase-rolling", 0xf5c1);
  addIconCodePoint("fa-sun", 0xf185);
  addIconCodePoint("fa-superpowers", 0xf2dd);
  addIconCodePoint("fa-superscript", 0xf12b);
  addIconCodePoint("fa-supple", 0xf3f9);
  addIconCodePoint("fa-surprise", 0xf5c2);
  addIconCodePoint("fa-suse", 0xf7d6);
  addIconCodePoint("fa-swatchbook", 0xf5c3);
  addIconCodePoint("fa-swift", 0xf8e1);
  addIconCodePoint("fa-swimmer", 0xf5c4);
  addIconCodePoint("fa-swimming-pool", 0xf5c5);
  addIconCodePoint("fa-symfony", 0xf83d);
  addIconCodePoint("fa-synagogue", 0xf69b);
  addIconCodePoint("fa-sync", 0xf021);
  addIconCodePoint("fa-sync-alt", 0xf2f1);
  addIconCodePoint("fa-syringe", 0xf48e);
  addIconCodePoint("fa-table", 0xf0ce);
  addIconCodePoint("fa-table-tennis", 0xf45d);
  addIconCodePoint("fa-tablet", 0xf10a);
  addIconCodePoint("fa-tablet-alt", 0xf3fa);
  addIconCodePoint("fa-tablets", 0xf490);
  addIconCodePoint("fa-tachometer-alt", 0xf3fd);
  addIconCodePoint("fa-tag", 0xf02b);
  addIconCodePoint("fa-tags", 0xf02c);
  addIconCodePoint("fa-tape", 0xf4db);
  addIconCodePoint("fa-tasks", 0xf0ae);
  addIconCodePoint("fa-taxi", 0xf1ba);
  addIconCodePoint("fa-teamspeak", 0xf4f9);
  addIconCodePoint("fa-teeth", 0xf62e);
  addIconCodePoint("fa-teeth-open", 0xf62f);
  addIconCodePoint("fa-telegram", 0xf2c6);
  addIconCodePoint("fa-telegram-plane", 0xf3fe);
  addIconCodePoint("fa-temperature-high", 0xf769);
  addIconCodePoint("fa-temperature-low", 0xf76b);
  addIconCodePoint("fa-tencent-weibo", 0xf1d5);
  addIconCodePoint("fa-tenge", 0xf7d7);
  addIconCodePoint("fa-terminal", 0xf120);
  addIconCodePoint("fa-text-height", 0xf034);
  addIconCodePoint("fa-text-width", 0xf035);
  addIconCodePoint("fa-th", 0xf00a);
  addIconCodePoint("fa-th-large", 0xf009);
  addIconCodePoint("fa-th-list", 0xf00b);
  addIconCodePoint("fa-the-red-yeti", 0xf69d);
  addIconCodePoint("fa-theater-masks", 0xf630);
  addIconCodePoint("fa-themeco", 0xf5c6);
  addIconCodePoint("fa-themeisle", 0xf2b2);
  addIconCodePoint("fa-thermometer", 0xf491);
  addIconCodePoint("fa-thermometer-empty", 0xf2cb);
  addIconCodePoint("fa-thermometer-full", 0xf2c7);
  addIconCodePoint("fa-thermometer-half", 0xf2c9);
  addIconCodePoint("fa-thermometer-quarter", 0xf2ca);
  addIconCodePoint("fa-thermometer-three-quarters", 0xf2c8);
  addIconCodePoint("fa-think-peaks", 0xf731);
  addIconCodePoint("fa-thumbs-down", 0xf165);
  addIconCodePoint("fa-thumbs-up", 0xf164);
  addIconCodePoint("fa-thumbtack", 0xf08d);
  addIconCodePoint("fa-ticket-alt", 0xf3ff);
  addIconCodePoint("fa-times", 0xf00d);
  addIconCodePoint("fa-times-circle", 0xf057);
  addIconCodePoint("fa-tint", 0xf043);
  addIconCodePoint("fa-tint-slash", 0xf5c7);
  addIconCodePoint("fa-tired", 0xf5c8);
  addIconCodePoint("fa-toggle-off", 0xf204);
  addIconCodePoint("fa-toggle-on", 0xf205);
  addIconCodePoint("fa-toilet", 0xf7d8);
  addIconCodePoint("fa-toilet-paper", 0xf71e);
  addIconCodePoint("fa-toolbox", 0xf552);
  addIconCodePoint("fa-tools", 0xf7d9);
  addIconCodePoint("fa-tooth", 0xf5c9);
  addIconCodePoint("fa-torah", 0xf6a0);
  addIconCodePoint("fa-torii-gate", 0xf6a1);
  addIconCodePoint("fa-tractor", 0xf722);
  addIconCodePoint("fa-trade-federation", 0xf513);
  addIconCodePoint("fa-trademark", 0xf25c);
  addIconCodePoint("fa-traffic-light", 0xf637);
  addIconCodePoint("fa-trailer", 0xf941);
  addIconCodePoint("fa-train", 0xf238);
  addIconCodePoint("fa-tram", 0xf7da);
  addIconCodePoint("fa-transgender", 0xf224);
  addIconCodePoint("fa-transgender-alt", 0xf225);
  addIconCodePoint("fa-trash", 0xf1f8);
  addIconCodePoint("fa-trash-alt", 0xf2ed);
  addIconCodePoint("fa-trash-restore", 0xf829);
  addIconCodePoint("fa-trash-restore-alt", 0xf82a);
  addIconCodePoint("fa-tree", 0xf1bb);
  addIconCodePoint("fa-trello", 0xf181);
  addIconCodePoint("fa-tripadvisor", 0xf262);
  addIconCodePoint("fa-trophy", 0xf091);
  addIconCodePoint("fa-truck", 0xf0d1);
  addIconCodePoint("fa-truck-loading", 0xf4de);
  addIconCodePoint("fa-truck-monster", 0xf63b);
  addIconCodePoint("fa-truck-moving", 0xf4df);
  addIconCodePoint("fa-truck-pickup", 0xf63c);
  addIconCodePoint("fa-tshirt", 0xf553);
  addIconCodePoint("fa-tty", 0xf1e4);
  addIconCodePoint("fa-tumblr", 0xf173);
  addIconCodePoint("fa-tumblr-square", 0xf174);
  addIconCodePoint("fa-tv", 0xf26c);
  addIconCodePoint("fa-twitch", 0xf1e8);
  addIconCodePoint("fa-twitter", 0xf099);
  addIconCodePoint("fa-twitter-square", 0xf081);
  addIconCodePoint("fa-typo3", 0xf42b);
  addIconCodePoint("fa-uber", 0xf402);
  addIconCodePoint("fa-ubuntu", 0xf7df);
  addIconCodePoint("fa-uikit", 0xf403);
  addIconCodePoint("fa-umbraco", 0xf8e8);
  addIconCodePoint("fa-umbrella", 0xf0e9);
  addIconCodePoint("fa-umbrella-beach", 0xf5ca);
  addIconCodePoint("fa-underline", 0xf0cd);
  addIconCodePoint("fa-undo", 0xf0e2);
  addIconCodePoint("fa-undo-alt", 0xf2ea);
  addIconCodePoint("fa-uniregistry", 0xf404);
  addIconCodePoint("fa-unity", 0xf949);
  addIconCodePoint("fa-universal-access", 0xf29a);
  addIconCodePoint("fa-university", 0xf19c);
  addIconCodePoint("fa-unlink", 0xf127);
  addIconCodePoint("fa-unlock", 0xf09c);
  addIconCodePoint("fa-unlock-alt", 0xf13e);
  addIconCodePoint("fa-untappd", 0xf405);
  addIconCodePoint("fa-upload", 0xf093);
  addIconCodePoint("fa-ups", 0xf7e0);
  addIconCodePoint("fa-usb", 0xf287);
  addIconCodePoint("fa-user", 0xf007);
  addIconCodePoint("fa-user-alt", 0xf406);
  addIconCodePoint("fa-user-alt-slash", 0xf4fa);
  addIconCodePoint("fa-user-astronaut", 0xf4fb);
  addIconCodePoint("fa-user-check", 0xf4fc);
  addIconCodePoint("fa-user-circle", 0xf2bd);
  addIconCodePoint("fa-user-clock", 0xf4fd);
  addIconCodePoint("fa-user-cog", 0xf4fe);
  addIconCodePoint("fa-user-edit", 0xf4ff);
  addIconCodePoint("fa-user-friends", 0xf500);
  addIconCodePoint("fa-user-graduate", 0xf501);
  addIconCodePoint("fa-user-injured", 0xf728);
  addIconCodePoint("fa-user-lock", 0xf502);
  addIconCodePoint("fa-user-md", 0xf0f0);
  addIconCodePoint("fa-user-minus", 0xf503);
  addIconCodePoint("fa-user-ninja", 0xf504);
  addIconCodePoint("fa-user-nurse", 0xf82f);
  addIconCodePoint("fa-user-plus", 0xf234);
  addIconCodePoint("fa-user-secret", 0xf21b);
  addIconCodePoint("fa-user-shield", 0xf505);
  addIconCodePoint("fa-user-slash", 0xf506);
  addIconCodePoint("fa-user-tag", 0xf507);
  addIconCodePoint("fa-user-tie", 0xf508);
  addIconCodePoint("fa-user-times", 0xf235);
  addIconCodePoint("fa-users", 0xf0c0);
  addIconCodePoint("fa-users-cog", 0xf509);
  addIconCodePoint("fa-usps", 0xf7e1);
  addIconCodePoint("fa-ussunnah", 0xf407);
  addIconCodePoint("fa-utensil-spoon", 0xf2e5);
  addIconCodePoint("fa-utensils", 0xf2e7);
  addIconCodePoint("fa-vaadin", 0xf408);
  addIconCodePoint("fa-vector-square", 0xf5cb);
  addIconCodePoint("fa-venus", 0xf221);
  addIconCodePoint("fa-venus-double", 0xf226);
  addIconCodePoint("fa-venus-mars", 0xf228);
  addIconCodePoint("fa-viacoin", 0xf237);
  addIconCodePoint("fa-viadeo", 0xf2a9);
  addIconCodePoint("fa-viadeo-square", 0xf2aa);
  addIconCodePoint("fa-vial", 0xf492);
  addIconCodePoint("fa-vials", 0xf493);
  addIconCodePoint("fa-viber", 0xf409);
  addIconCodePoint("fa-video", 0xf03d);
  addIconCodePoint("fa-video-slash", 0xf4e2);
  addIconCodePoint("fa-vihara", 0xf6a7);
  addIconCodePoint("fa-vimeo", 0xf40a);
  addIconCodePoint("fa-vimeo-square", 0xf194);
  addIconCodePoint("fa-vimeo-v", 0xf27d);
  addIconCodePoint("fa-vine", 0xf1ca);
  addIconCodePoint("fa-vk", 0xf189);
  addIconCodePoint("fa-vnv", 0xf40b);
  addIconCodePoint("fa-voicemail", 0xf897);
  addIconCodePoint("fa-volleyball-ball", 0xf45f);
  addIconCodePoint("fa-volume-down", 0xf027);
  addIconCodePoint("fa-volume-mute", 0xf6a9);
  addIconCodePoint("fa-volume-off", 0xf026);
  addIconCodePoint("fa-volume-up", 0xf028);
  addIconCodePoint("fa-vote-yea", 0xf772);
  addIconCodePoint("fa-vr-cardboard", 0xf729);
  addIconCodePoint("fa-vuejs", 0xf41f);
  addIconCodePoint("fa-walking", 0xf554);
  addIconCodePoint("fa-wallet", 0xf555);
  addIconCodePoint("fa-warehouse", 0xf494);
  addIconCodePoint("fa-water", 0xf773);
  addIconCodePoint("fa-wave-square", 0xf83e);
  addIconCodePoint("fa-waze", 0xf83f);
  addIconCodePoint("fa-weebly", 0xf5cc);
  addIconCodePoint("fa-weibo", 0xf18a);
  addIconCodePoint("fa-weight", 0xf496);
  addIconCodePoint("fa-weight-hanging", 0xf5cd);
  addIconCodePoint("fa-weixin", 0xf1d7);
  addIconCodePoint("fa-whatsapp", 0xf232);
  addIconCodePoint("fa-whatsapp-square", 0xf40c);
  addIconCodePoint("fa-wheelchair", 0xf193);
  addIconCodePoint("fa-whmcs", 0xf40d);
  addIconCodePoint("fa-wifi", 0xf1eb);
  addIconCodePoint("fa-wikipedia-w", 0xf266);
  addIconCodePoint("fa-wind", 0xf72e);
  addIconCodePoint("fa-window-close", 0xf410);
  addIconCodePoint("fa-window-maximize", 0xf2d0);
  addIconCodePoint("fa-window-minimize", 0xf2d1);
  addIconCodePoint("fa-window-restore", 0xf2d2);
  addIconCodePoint("fa-windows", 0xf17a);
  addIconCodePoint("fa-wine-bottle", 0xf72f);
  addIconCodePoint("fa-wine-glass", 0xf4e3);
  addIconCodePoint("fa-wine-glass-alt", 0xf5ce);
  addIconCodePoint("fa-wix", 0xf5cf);
  addIconCodePoint("fa-wizards-of-the-coast", 0xf730);
  addIconCodePoint("fa-wolf-pack-battalion", 0xf514);
  addIconCodePoint("fa-won-sign", 0xf159);
  addIconCodePoint("fa-wordpress", 0xf19a);
  addIconCodePoint("fa-wordpress-simple", 0xf411);
  addIconCodePoint("fa-wpbeginner", 0xf297);
  addIconCodePoint("fa-wpexplorer", 0xf2de);
  addIconCodePoint("fa-wpforms", 0xf298);
  addIconCodePoint("fa-wpressr", 0xf3e4);
  addIconCodePoint("fa-wrench", 0xf0ad);
  addIconCodePoint("fa-x-ray", 0xf497);
  addIconCodePoint("fa-xbox", 0xf412);
  addIconCodePoint("fa-xing", 0xf168);
  addIconCodePoint("fa-xing-square", 0xf169);
  addIconCodePoint("fa-y-combinator", 0xf23b);
  addIconCodePoint("fa-yahoo", 0xf19e);
  addIconCodePoint("fa-yammer", 0xf840);
  addIconCodePoint("fa-yandex", 0xf413);
  addIconCodePoint("fa-yandex-international", 0xf414);
  addIconCodePoint("fa-yarn", 0xf7e3);
  addIconCodePoint("fa-yelp", 0xf1e9);
  addIconCodePoint("fa-yen-sign", 0xf157);
  addIconCodePoint("fa-yin-yang", 0xf6ad);
  addIconCodePoint("fa-yoast", 0xf2b1);
  addIconCodePoint("fa-youtube", 0xf167);
  addIconCodePoint("fa-youtube-square", 0xf431);
  addIconCodePoint("fa-zhihu", 0xf63f);

  clearFtFaces();

  iconsNames.reserve(iconCodePoint.size());

  for (const auto &it : iconCodePoint) {
    iconsNames.emplace_back(it.first);
  }
}
