// Warning: do not update this file !!!
// It was automatically generated by utils/scripts/generate_md_cpp_init_code.sh
// from Material Design icons version 4.9.95

std::string TulipMaterialDesignIcons::getVersion() {
  return "4.9.95";
}

static void initIconCodePoints() {
  addIconCodePoint("md-ab-testing", 0xF001C);
  addIconCodePoint("md-abjad-arabic", 0xF0353);
  addIconCodePoint("md-abjad-hebrew", 0xF0354);
  addIconCodePoint("md-abugida-devanagari", 0xF0355);
  addIconCodePoint("md-abugida-thai", 0xF0356);
  addIconCodePoint("md-access-point", 0xF002);
  addIconCodePoint("md-access-point-network", 0xF003);
  addIconCodePoint("md-access-point-network-off", 0xFBBD);
  addIconCodePoint("md-account", 0xF004);
  addIconCodePoint("md-account-alert", 0xF005);
  addIconCodePoint("md-account-alert-outline", 0xFB2C);
  addIconCodePoint("md-account-arrow-left", 0xFB2D);
  addIconCodePoint("md-account-arrow-left-outline", 0xFB2E);
  addIconCodePoint("md-account-arrow-right", 0xFB2F);
  addIconCodePoint("md-account-arrow-right-outline", 0xFB30);
  addIconCodePoint("md-account-badge", 0xFD83);
  addIconCodePoint("md-account-badge-alert", 0xFD84);
  addIconCodePoint("md-account-badge-alert-outline", 0xFD85);
  addIconCodePoint("md-account-badge-horizontal", 0xFDF0);
  addIconCodePoint("md-account-badge-horizontal-outline", 0xFDF1);
  addIconCodePoint("md-account-badge-outline", 0xFD86);
  addIconCodePoint("md-account-box", 0xF006);
  addIconCodePoint("md-account-box-multiple", 0xF933);
  addIconCodePoint("md-account-box-multiple-outline", 0xF002C);
  addIconCodePoint("md-account-box-outline", 0xF007);
  addIconCodePoint("md-account-cancel", 0xF030A);
  addIconCodePoint("md-account-cancel-outline", 0xF030B);
  addIconCodePoint("md-account-card-details", 0xF5D2);
  addIconCodePoint("md-account-card-details-outline", 0xFD87);
  addIconCodePoint("md-account-cash", 0xF00C2);
  addIconCodePoint("md-account-cash-outline", 0xF00C3);
  addIconCodePoint("md-account-check", 0xF008);
  addIconCodePoint("md-account-check-outline", 0xFBBE);
  addIconCodePoint("md-account-child", 0xFA88);
  addIconCodePoint("md-account-child-circle", 0xFA89);
  addIconCodePoint("md-account-child-outline", 0xF00F3);
  addIconCodePoint("md-account-circle", 0xF009);
  addIconCodePoint("md-account-circle-outline", 0xFB31);
  addIconCodePoint("md-account-clock", 0xFB32);
  addIconCodePoint("md-account-clock-outline", 0xFB33);
  addIconCodePoint("md-account-cog", 0xF039B);
  addIconCodePoint("md-account-cog-outline", 0xF039C);
  addIconCodePoint("md-account-convert", 0xF00A);
  addIconCodePoint("md-account-convert-outline", 0xF032C);
  addIconCodePoint("md-account-details", 0xF631);
  addIconCodePoint("md-account-details-outline", 0xF039D);
  addIconCodePoint("md-account-edit", 0xF6BB);
  addIconCodePoint("md-account-edit-outline", 0xF001D);
  addIconCodePoint("md-account-group", 0xF848);
  addIconCodePoint("md-account-group-outline", 0xFB34);
  addIconCodePoint("md-account-heart", 0xF898);
  addIconCodePoint("md-account-heart-outline", 0xFBBF);
  addIconCodePoint("md-account-key", 0xF00B);
  addIconCodePoint("md-account-key-outline", 0xFBC0);
  addIconCodePoint("md-account-lock", 0xF0189);
  addIconCodePoint("md-account-lock-outline", 0xF018A);
  addIconCodePoint("md-account-minus", 0xF00D);
  addIconCodePoint("md-account-minus-outline", 0xFAEB);
  addIconCodePoint("md-account-multiple", 0xF00E);
  addIconCodePoint("md-account-multiple-check", 0xF8C4);
  addIconCodePoint("md-account-multiple-check-outline", 0xF0229);
  addIconCodePoint("md-account-multiple-minus", 0xF5D3);
  addIconCodePoint("md-account-multiple-minus-outline", 0xFBC1);
  addIconCodePoint("md-account-multiple-outline", 0xF00F);
  addIconCodePoint("md-account-multiple-plus", 0xF010);
  addIconCodePoint("md-account-multiple-plus-outline", 0xF7FF);
  addIconCodePoint("md-account-multiple-remove", 0xF0235);
  addIconCodePoint("md-account-multiple-remove-outline", 0xF0236);
  addIconCodePoint("md-account-network", 0xF011);
  addIconCodePoint("md-account-network-outline", 0xFBC2);
  addIconCodePoint("md-account-off", 0xF012);
  addIconCodePoint("md-account-off-outline", 0xFBC3);
  addIconCodePoint("md-account-outline", 0xF013);
  addIconCodePoint("md-account-plus", 0xF014);
  addIconCodePoint("md-account-plus-outline", 0xF800);
  addIconCodePoint("md-account-question", 0xFB35);
  addIconCodePoint("md-account-question-outline", 0xFB36);
  addIconCodePoint("md-account-remove", 0xF015);
  addIconCodePoint("md-account-remove-outline", 0xFAEC);
  addIconCodePoint("md-account-search", 0xF016);
  addIconCodePoint("md-account-search-outline", 0xF934);
  addIconCodePoint("md-account-settings", 0xF630);
  addIconCodePoint("md-account-settings-outline", 0xF00F4);
  addIconCodePoint("md-account-star", 0xF017);
  addIconCodePoint("md-account-star-outline", 0xFBC4);
  addIconCodePoint("md-account-supervisor", 0xFA8A);
  addIconCodePoint("md-account-supervisor-circle", 0xFA8B);
  addIconCodePoint("md-account-supervisor-outline", 0xF0158);
  addIconCodePoint("md-account-switch", 0xF019);
  addIconCodePoint("md-account-tie", 0xFCBF);
  addIconCodePoint("md-account-tie-outline", 0xF00F5);
  addIconCodePoint("md-account-tie-voice", 0xF0333);
  addIconCodePoint("md-account-tie-voice-off", 0xF0335);
  addIconCodePoint("md-account-tie-voice-off-outline", 0xF0336);
  addIconCodePoint("md-account-tie-voice-outline", 0xF0334);
  addIconCodePoint("md-accusoft", 0xF849);
  addIconCodePoint("md-adjust", 0xF01A);
  addIconCodePoint("md-adobe", 0xF935);
  addIconCodePoint("md-adobe-acrobat", 0xFFBD);
  addIconCodePoint("md-air-conditioner", 0xF01B);
  addIconCodePoint("md-air-filter", 0xFD1F);
  addIconCodePoint("md-air-horn", 0xFD88);
  addIconCodePoint("md-air-humidifier", 0xF00C4);
  addIconCodePoint("md-air-purifier", 0xFD20);
  addIconCodePoint("md-airbag", 0xFBC5);
  addIconCodePoint("md-airballoon", 0xF01C);
  addIconCodePoint("md-airballoon-outline", 0xF002D);
  addIconCodePoint("md-airplane", 0xF01D);
  addIconCodePoint("md-airplane-landing", 0xF5D4);
  addIconCodePoint("md-airplane-off", 0xF01E);
  addIconCodePoint("md-airplane-takeoff", 0xF5D5);
  addIconCodePoint("md-airplay", 0xF01F);
  addIconCodePoint("md-airport", 0xF84A);
  addIconCodePoint("md-alarm", 0xF020);
  addIconCodePoint("md-alarm-bell", 0xF78D);
  addIconCodePoint("md-alarm-check", 0xF021);
  addIconCodePoint("md-alarm-light", 0xF78E);
  addIconCodePoint("md-alarm-light-outline", 0xFBC6);
  addIconCodePoint("md-alarm-multiple", 0xF022);
  addIconCodePoint("md-alarm-note", 0xFE8E);
  addIconCodePoint("md-alarm-note-off", 0xFE8F);
  addIconCodePoint("md-alarm-off", 0xF023);
  addIconCodePoint("md-alarm-plus", 0xF024);
  addIconCodePoint("md-alarm-snooze", 0xF68D);
  addIconCodePoint("md-album", 0xF025);
  addIconCodePoint("md-alert", 0xF026);
  addIconCodePoint("md-alert-box", 0xF027);
  addIconCodePoint("md-alert-box-outline", 0xFCC0);
  addIconCodePoint("md-alert-circle", 0xF028);
  addIconCodePoint("md-alert-circle-check", 0xF0218);
  addIconCodePoint("md-alert-circle-check-outline", 0xF0219);
  addIconCodePoint("md-alert-circle-outline", 0xF5D6);
  addIconCodePoint("md-alert-decagram", 0xF6BC);
  addIconCodePoint("md-alert-decagram-outline", 0xFCC1);
  addIconCodePoint("md-alert-octagon", 0xF029);
  addIconCodePoint("md-alert-octagon-outline", 0xFCC2);
  addIconCodePoint("md-alert-octagram", 0xF766);
  addIconCodePoint("md-alert-octagram-outline", 0xFCC3);
  addIconCodePoint("md-alert-outline", 0xF02A);
  addIconCodePoint("md-alert-rhombus", 0xF01F9);
  addIconCodePoint("md-alert-rhombus-outline", 0xF01FA);
  addIconCodePoint("md-alien", 0xF899);
  addIconCodePoint("md-alien-outline", 0xF00F6);
  addIconCodePoint("md-align-horizontal-center", 0xF01EE);
  addIconCodePoint("md-align-horizontal-left", 0xF01ED);
  addIconCodePoint("md-align-horizontal-right", 0xF01EF);
  addIconCodePoint("md-align-vertical-bottom", 0xF01F0);
  addIconCodePoint("md-align-vertical-center", 0xF01F1);
  addIconCodePoint("md-align-vertical-top", 0xF01F2);
  addIconCodePoint("md-all-inclusive", 0xF6BD);
  addIconCodePoint("md-allergy", 0xF0283);
  addIconCodePoint("md-alpha", 0xF02B);
  addIconCodePoint("md-alpha-a", 0x0041);
  addIconCodePoint("md-alpha-a-box", 0xFAED);
  addIconCodePoint("md-alpha-a-box-outline", 0xFBC7);
  addIconCodePoint("md-alpha-a-circle", 0xFBC8);
  addIconCodePoint("md-alpha-a-circle-outline", 0xFBC9);
  addIconCodePoint("md-alpha-b", 0x0042);
  addIconCodePoint("md-alpha-b-box", 0xFAEE);
  addIconCodePoint("md-alpha-b-box-outline", 0xFBCA);
  addIconCodePoint("md-alpha-b-circle", 0xFBCB);
  addIconCodePoint("md-alpha-b-circle-outline", 0xFBCC);
  addIconCodePoint("md-alpha-c", 0x0043);
  addIconCodePoint("md-alpha-c-box", 0xFAEF);
  addIconCodePoint("md-alpha-c-box-outline", 0xFBCD);
  addIconCodePoint("md-alpha-c-circle", 0xFBCE);
  addIconCodePoint("md-alpha-c-circle-outline", 0xFBCF);
  addIconCodePoint("md-alpha-d", 0x0044);
  addIconCodePoint("md-alpha-d-box", 0xFAF0);
  addIconCodePoint("md-alpha-d-box-outline", 0xFBD0);
  addIconCodePoint("md-alpha-d-circle", 0xFBD1);
  addIconCodePoint("md-alpha-d-circle-outline", 0xFBD2);
  addIconCodePoint("md-alpha-e", 0x0045);
  addIconCodePoint("md-alpha-e-box", 0xFAF1);
  addIconCodePoint("md-alpha-e-box-outline", 0xFBD3);
  addIconCodePoint("md-alpha-e-circle", 0xFBD4);
  addIconCodePoint("md-alpha-e-circle-outline", 0xFBD5);
  addIconCodePoint("md-alpha-f", 0x0046);
  addIconCodePoint("md-alpha-f-box", 0xFAF2);
  addIconCodePoint("md-alpha-f-box-outline", 0xFBD6);
  addIconCodePoint("md-alpha-f-circle", 0xFBD7);
  addIconCodePoint("md-alpha-f-circle-outline", 0xFBD8);
  addIconCodePoint("md-alpha-g", 0x0047);
  addIconCodePoint("md-alpha-g-box", 0xFAF3);
  addIconCodePoint("md-alpha-g-box-outline", 0xFBD9);
  addIconCodePoint("md-alpha-g-circle", 0xFBDA);
  addIconCodePoint("md-alpha-g-circle-outline", 0xFBDB);
  addIconCodePoint("md-alpha-h", 0x0048);
  addIconCodePoint("md-alpha-h-box", 0xFAF4);
  addIconCodePoint("md-alpha-h-box-outline", 0xFBDC);
  addIconCodePoint("md-alpha-h-circle", 0xFBDD);
  addIconCodePoint("md-alpha-h-circle-outline", 0xFBDE);
  addIconCodePoint("md-alpha-i", 0x0049);
  addIconCodePoint("md-alpha-i-box", 0xFAF5);
  addIconCodePoint("md-alpha-i-box-outline", 0xFBDF);
  addIconCodePoint("md-alpha-i-circle", 0xFBE0);
  addIconCodePoint("md-alpha-i-circle-outline", 0xFBE1);
  addIconCodePoint("md-alpha-j", 0x004A);
  addIconCodePoint("md-alpha-j-box", 0xFAF6);
  addIconCodePoint("md-alpha-j-box-outline", 0xFBE2);
  addIconCodePoint("md-alpha-j-circle", 0xFBE3);
  addIconCodePoint("md-alpha-j-circle-outline", 0xFBE4);
  addIconCodePoint("md-alpha-k", 0x004B);
  addIconCodePoint("md-alpha-k-box", 0xFAF7);
  addIconCodePoint("md-alpha-k-box-outline", 0xFBE5);
  addIconCodePoint("md-alpha-k-circle", 0xFBE6);
  addIconCodePoint("md-alpha-k-circle-outline", 0xFBE7);
  addIconCodePoint("md-alpha-l", 0x004C);
  addIconCodePoint("md-alpha-l-box", 0xFAF8);
  addIconCodePoint("md-alpha-l-box-outline", 0xFBE8);
  addIconCodePoint("md-alpha-l-circle", 0xFBE9);
  addIconCodePoint("md-alpha-l-circle-outline", 0xFBEA);
  addIconCodePoint("md-alpha-m", 0x004D);
  addIconCodePoint("md-alpha-m-box", 0xFAF9);
  addIconCodePoint("md-alpha-m-box-outline", 0xFBEB);
  addIconCodePoint("md-alpha-m-circle", 0xFBEC);
  addIconCodePoint("md-alpha-m-circle-outline", 0xFBED);
  addIconCodePoint("md-alpha-n", 0x004E);
  addIconCodePoint("md-alpha-n-box", 0xFAFA);
  addIconCodePoint("md-alpha-n-box-outline", 0xFBEE);
  addIconCodePoint("md-alpha-n-circle", 0xFBEF);
  addIconCodePoint("md-alpha-n-circle-outline", 0xFBF0);
  addIconCodePoint("md-alpha-o", 0x004F);
  addIconCodePoint("md-alpha-o-box", 0xFAFB);
  addIconCodePoint("md-alpha-o-box-outline", 0xFBF1);
  addIconCodePoint("md-alpha-o-circle", 0xFBF2);
  addIconCodePoint("md-alpha-o-circle-outline", 0xFBF3);
  addIconCodePoint("md-alpha-p", 0x0050);
  addIconCodePoint("md-alpha-p-box", 0xFAFC);
  addIconCodePoint("md-alpha-p-box-outline", 0xFBF4);
  addIconCodePoint("md-alpha-p-circle", 0xFBF5);
  addIconCodePoint("md-alpha-p-circle-outline", 0xFBF6);
  addIconCodePoint("md-alpha-q", 0x0051);
  addIconCodePoint("md-alpha-q-box", 0xFAFD);
  addIconCodePoint("md-alpha-q-box-outline", 0xFBF7);
  addIconCodePoint("md-alpha-q-circle", 0xFBF8);
  addIconCodePoint("md-alpha-q-circle-outline", 0xFBF9);
  addIconCodePoint("md-alpha-r", 0x0052);
  addIconCodePoint("md-alpha-r-box", 0xFAFE);
  addIconCodePoint("md-alpha-r-box-outline", 0xFBFA);
  addIconCodePoint("md-alpha-r-circle", 0xFBFB);
  addIconCodePoint("md-alpha-r-circle-outline", 0xFBFC);
  addIconCodePoint("md-alpha-s", 0x0053);
  addIconCodePoint("md-alpha-s-box", 0xFAFF);
  addIconCodePoint("md-alpha-s-box-outline", 0xFBFD);
  addIconCodePoint("md-alpha-s-circle", 0xFBFE);
  addIconCodePoint("md-alpha-s-circle-outline", 0xFBFF);
  addIconCodePoint("md-alpha-t", 0x0054);
  addIconCodePoint("md-alpha-t-box", 0xFB00);
  addIconCodePoint("md-alpha-t-box-outline", 0xFC00);
  addIconCodePoint("md-alpha-t-circle", 0xFC01);
  addIconCodePoint("md-alpha-t-circle-outline", 0xFC02);
  addIconCodePoint("md-alpha-u", 0x0055);
  addIconCodePoint("md-alpha-u-box", 0xFB01);
  addIconCodePoint("md-alpha-u-box-outline", 0xFC03);
  addIconCodePoint("md-alpha-u-circle", 0xFC04);
  addIconCodePoint("md-alpha-u-circle-outline", 0xFC05);
  addIconCodePoint("md-alpha-v", 0x0056);
  addIconCodePoint("md-alpha-v-box", 0xFB02);
  addIconCodePoint("md-alpha-v-box-outline", 0xFC06);
  addIconCodePoint("md-alpha-v-circle", 0xFC07);
  addIconCodePoint("md-alpha-v-circle-outline", 0xFC08);
  addIconCodePoint("md-alpha-w", 0x0057);
  addIconCodePoint("md-alpha-w-box", 0xFB03);
  addIconCodePoint("md-alpha-w-box-outline", 0xFC09);
  addIconCodePoint("md-alpha-w-circle", 0xFC0A);
  addIconCodePoint("md-alpha-w-circle-outline", 0xFC0B);
  addIconCodePoint("md-alpha-x", 0x0058);
  addIconCodePoint("md-alpha-x-box", 0xFB04);
  addIconCodePoint("md-alpha-x-box-outline", 0xFC0C);
  addIconCodePoint("md-alpha-x-circle", 0xFC0D);
  addIconCodePoint("md-alpha-x-circle-outline", 0xFC0E);
  addIconCodePoint("md-alpha-y", 0x0059);
  addIconCodePoint("md-alpha-y-box", 0xFB05);
  addIconCodePoint("md-alpha-y-box-outline", 0xFC0F);
  addIconCodePoint("md-alpha-y-circle", 0xFC10);
  addIconCodePoint("md-alpha-y-circle-outline", 0xFC11);
  addIconCodePoint("md-alpha-z", 0x005A);
  addIconCodePoint("md-alpha-z-box", 0xFB06);
  addIconCodePoint("md-alpha-z-box-outline", 0xFC12);
  addIconCodePoint("md-alpha-z-circle", 0xFC13);
  addIconCodePoint("md-alpha-z-circle-outline", 0xFC14);
  addIconCodePoint("md-alphabet-aurebesh", 0xF0357);
  addIconCodePoint("md-alphabet-cyrillic", 0xF0358);
  addIconCodePoint("md-alphabet-greek", 0xF0359);
  addIconCodePoint("md-alphabet-latin", 0xF035A);
  addIconCodePoint("md-alphabet-piqad", 0xF035B);
  addIconCodePoint("md-alphabet-tengwar", 0xF0362);
  addIconCodePoint("md-alphabetical", 0xF02C);
  addIconCodePoint("md-alphabetical-off", 0xF002E);
  addIconCodePoint("md-alphabetical-variant", 0xF002F);
  addIconCodePoint("md-alphabetical-variant-off", 0xF0030);
  addIconCodePoint("md-altimeter", 0xF5D7);
  addIconCodePoint("md-amazon", 0xF02D);
  addIconCodePoint("md-amazon-alexa", 0xF8C5);
  addIconCodePoint("md-amazon-drive", 0xF02E);
  addIconCodePoint("md-ambulance", 0xF02F);
  addIconCodePoint("md-ammunition", 0xFCC4);
  addIconCodePoint("md-ampersand", 0xFA8C);
  addIconCodePoint("md-amplifier", 0xF030);
  addIconCodePoint("md-amplifier-off", 0xF01E0);
  addIconCodePoint("md-anchor", 0xF031);
  addIconCodePoint("md-android", 0xF032);
  addIconCodePoint("md-android-auto", 0xFA8D);
  addIconCodePoint("md-android-debug-bridge", 0xF033);
  addIconCodePoint("md-android-head", 0xF78F);
  addIconCodePoint("md-android-messages", 0xFD21);
  addIconCodePoint("md-android-studio", 0xF034);
  addIconCodePoint("md-angle-acute", 0xF936);
  addIconCodePoint("md-angle-obtuse", 0xF937);
  addIconCodePoint("md-angle-right", 0xF938);
  addIconCodePoint("md-angular", 0xF6B1);
  addIconCodePoint("md-angularjs", 0xF6BE);
  addIconCodePoint("md-animation", 0xF5D8);
  addIconCodePoint("md-animation-outline", 0xFA8E);
  addIconCodePoint("md-animation-play", 0xF939);
  addIconCodePoint("md-animation-play-outline", 0xFA8F);
  addIconCodePoint("md-ansible", 0xF00C5);
  addIconCodePoint("md-antenna", 0xF0144);
  addIconCodePoint("md-anvil", 0xF89A);
  addIconCodePoint("md-apache-kafka", 0xF0031);
  addIconCodePoint("md-api", 0xF00C6);
  addIconCodePoint("md-api-off", 0xF0282);
  addIconCodePoint("md-apple", 0xF035);
  addIconCodePoint("md-apple-finder", 0xF036);
  addIconCodePoint("md-apple-icloud", 0xF038);
  addIconCodePoint("md-apple-ios", 0xF037);
  addIconCodePoint("md-apple-keyboard-caps", 0xF632);
  addIconCodePoint("md-apple-keyboard-command", 0xF633);
  addIconCodePoint("md-apple-keyboard-control", 0xF634);
  addIconCodePoint("md-apple-keyboard-option", 0xF635);
  addIconCodePoint("md-apple-keyboard-shift", 0xF636);
  addIconCodePoint("md-apple-safari", 0xF039);
  addIconCodePoint("md-application", 0xF614);
  addIconCodePoint("md-application-export", 0xFD89);
  addIconCodePoint("md-application-import", 0xFD8A);
  addIconCodePoint("md-approximately-equal", 0xFFBE);
  addIconCodePoint("md-approximately-equal-box", 0xFFBF);
  addIconCodePoint("md-apps", 0xF03B);
  addIconCodePoint("md-apps-box", 0xFD22);
  addIconCodePoint("md-arch", 0xF8C6);
  addIconCodePoint("md-archive", 0xF03C);
  addIconCodePoint("md-archive-arrow-down", 0xF0284);
  addIconCodePoint("md-archive-arrow-down-outline", 0xF0285);
  addIconCodePoint("md-archive-arrow-up", 0xF0286);
  addIconCodePoint("md-archive-arrow-up-outline", 0xF0287);
  addIconCodePoint("md-archive-outline", 0xF0239);
  addIconCodePoint("md-arm-flex", 0xF008F);
  addIconCodePoint("md-arm-flex-outline", 0xF0090);
  addIconCodePoint("md-arrange-bring-forward", 0xF03D);
  addIconCodePoint("md-arrange-bring-to-front", 0xF03E);
  addIconCodePoint("md-arrange-send-backward", 0xF03F);
  addIconCodePoint("md-arrange-send-to-back", 0xF040);
  addIconCodePoint("md-arrow-all", 0xF041);
  addIconCodePoint("md-arrow-bottom-left", 0xF042);
  addIconCodePoint("md-arrow-bottom-left-bold-outline", 0xF9B6);
  addIconCodePoint("md-arrow-bottom-left-thick", 0xF9B7);
  addIconCodePoint("md-arrow-bottom-right", 0xF043);
  addIconCodePoint("md-arrow-bottom-right-bold-outline", 0xF9B8);
  addIconCodePoint("md-arrow-bottom-right-thick", 0xF9B9);
  addIconCodePoint("md-arrow-collapse", 0xF615);
  addIconCodePoint("md-arrow-collapse-all", 0xF044);
  addIconCodePoint("md-arrow-collapse-down", 0xF791);
  addIconCodePoint("md-arrow-collapse-horizontal", 0xF84B);
  addIconCodePoint("md-arrow-collapse-left", 0xF792);
  addIconCodePoint("md-arrow-collapse-right", 0xF793);
  addIconCodePoint("md-arrow-collapse-up", 0xF794);
  addIconCodePoint("md-arrow-collapse-vertical", 0xF84C);
  addIconCodePoint("md-arrow-decision", 0xF9BA);
  addIconCodePoint("md-arrow-decision-auto", 0xF9BB);
  addIconCodePoint("md-arrow-decision-auto-outline", 0xF9BC);
  addIconCodePoint("md-arrow-decision-outline", 0xF9BD);
  addIconCodePoint("md-arrow-down", 0xF045);
  addIconCodePoint("md-arrow-down-bold", 0xF72D);
  addIconCodePoint("md-arrow-down-bold-box", 0xF72E);
  addIconCodePoint("md-arrow-down-bold-box-outline", 0xF72F);
  addIconCodePoint("md-arrow-down-bold-circle", 0xF047);
  addIconCodePoint("md-arrow-down-bold-circle-outline", 0xF048);
  addIconCodePoint("md-arrow-down-bold-hexagon-outline", 0xF049);
  addIconCodePoint("md-arrow-down-bold-outline", 0xF9BE);
  addIconCodePoint("md-arrow-down-box", 0xF6BF);
  addIconCodePoint("md-arrow-down-circle", 0xFCB7);
  addIconCodePoint("md-arrow-down-circle-outline", 0xFCB8);
  addIconCodePoint("md-arrow-down-drop-circle", 0xF04A);
  addIconCodePoint("md-arrow-down-drop-circle-outline", 0xF04B);
  addIconCodePoint("md-arrow-down-thick", 0xF046);
  addIconCodePoint("md-arrow-expand", 0xF616);
  addIconCodePoint("md-arrow-expand-all", 0xF04C);
  addIconCodePoint("md-arrow-expand-down", 0xF795);
  addIconCodePoint("md-arrow-expand-horizontal", 0xF84D);
  addIconCodePoint("md-arrow-expand-left", 0xF796);
  addIconCodePoint("md-arrow-expand-right", 0xF797);
  addIconCodePoint("md-arrow-expand-up", 0xF798);
  addIconCodePoint("md-arrow-expand-vertical", 0xF84E);
  addIconCodePoint("md-arrow-horizontal-lock", 0xF0186);
  addIconCodePoint("md-arrow-left", 0xF04D);
  addIconCodePoint("md-arrow-left-bold", 0xF730);
  addIconCodePoint("md-arrow-left-bold-box", 0xF731);
  addIconCodePoint("md-arrow-left-bold-box-outline", 0xF732);
  addIconCodePoint("md-arrow-left-bold-circle", 0xF04F);
  addIconCodePoint("md-arrow-left-bold-circle-outline", 0xF050);
  addIconCodePoint("md-arrow-left-bold-hexagon-outline", 0xF051);
  addIconCodePoint("md-arrow-left-bold-outline", 0xF9BF);
  addIconCodePoint("md-arrow-left-box", 0xF6C0);
  addIconCodePoint("md-arrow-left-circle", 0xFCB9);
  addIconCodePoint("md-arrow-left-circle-outline", 0xFCBA);
  addIconCodePoint("md-arrow-left-drop-circle", 0xF052);
  addIconCodePoint("md-arrow-left-drop-circle-outline", 0xF053);
  addIconCodePoint("md-arrow-left-right", 0xFE90);
  addIconCodePoint("md-arrow-left-right-bold", 0xFE91);
  addIconCodePoint("md-arrow-left-right-bold-outline", 0xF9C0);
  addIconCodePoint("md-arrow-left-thick", 0xF04E);
  addIconCodePoint("md-arrow-right", 0xF054);
  addIconCodePoint("md-arrow-right-bold", 0xF733);
  addIconCodePoint("md-arrow-right-bold-box", 0xF734);
  addIconCodePoint("md-arrow-right-bold-box-outline", 0xF735);
  addIconCodePoint("md-arrow-right-bold-circle", 0xF056);
  addIconCodePoint("md-arrow-right-bold-circle-outline", 0xF057);
  addIconCodePoint("md-arrow-right-bold-hexagon-outline", 0xF058);
  addIconCodePoint("md-arrow-right-bold-outline", 0xF9C1);
  addIconCodePoint("md-arrow-right-box", 0xF6C1);
  addIconCodePoint("md-arrow-right-circle", 0xFCBB);
  addIconCodePoint("md-arrow-right-circle-outline", 0xFCBC);
  addIconCodePoint("md-arrow-right-drop-circle", 0xF059);
  addIconCodePoint("md-arrow-right-drop-circle-outline", 0xF05A);
  addIconCodePoint("md-arrow-right-thick", 0xF055);
  addIconCodePoint("md-arrow-split-horizontal", 0xF93A);
  addIconCodePoint("md-arrow-split-vertical", 0xF93B);
  addIconCodePoint("md-arrow-top-left", 0xF05B);
  addIconCodePoint("md-arrow-top-left-bold-outline", 0xF9C2);
  addIconCodePoint("md-arrow-top-left-bottom-right", 0xFE92);
  addIconCodePoint("md-arrow-top-left-bottom-right-bold", 0xFE93);
  addIconCodePoint("md-arrow-top-left-thick", 0xF9C3);
  addIconCodePoint("md-arrow-top-right", 0xF05C);
  addIconCodePoint("md-arrow-top-right-bold-outline", 0xF9C4);
  addIconCodePoint("md-arrow-top-right-bottom-left", 0xFE94);
  addIconCodePoint("md-arrow-top-right-bottom-left-bold", 0xFE95);
  addIconCodePoint("md-arrow-top-right-thick", 0xF9C5);
  addIconCodePoint("md-arrow-up", 0xF05D);
  addIconCodePoint("md-arrow-up-bold", 0xF736);
  addIconCodePoint("md-arrow-up-bold-box", 0xF737);
  addIconCodePoint("md-arrow-up-bold-box-outline", 0xF738);
  addIconCodePoint("md-arrow-up-bold-circle", 0xF05F);
  addIconCodePoint("md-arrow-up-bold-circle-outline", 0xF060);
  addIconCodePoint("md-arrow-up-bold-hexagon-outline", 0xF061);
  addIconCodePoint("md-arrow-up-bold-outline", 0xF9C6);
  addIconCodePoint("md-arrow-up-box", 0xF6C2);
  addIconCodePoint("md-arrow-up-circle", 0xFCBD);
  addIconCodePoint("md-arrow-up-circle-outline", 0xFCBE);
  addIconCodePoint("md-arrow-up-down", 0xFE96);
  addIconCodePoint("md-arrow-up-down-bold", 0xFE97);
  addIconCodePoint("md-arrow-up-down-bold-outline", 0xF9C7);
  addIconCodePoint("md-arrow-up-drop-circle", 0xF062);
  addIconCodePoint("md-arrow-up-drop-circle-outline", 0xF063);
  addIconCodePoint("md-arrow-up-thick", 0xF05E);
  addIconCodePoint("md-arrow-vertical-lock", 0xF0187);
  addIconCodePoint("md-artist", 0xF802);
  addIconCodePoint("md-artist-outline", 0xFCC5);
  addIconCodePoint("md-artstation", 0xFB37);
  addIconCodePoint("md-aspect-ratio", 0xFA23);
  addIconCodePoint("md-assistant", 0xF064);
  addIconCodePoint("md-asterisk", 0xF6C3);
  addIconCodePoint("md-at", 0xF065);
  addIconCodePoint("md-atlassian", 0xF803);
  addIconCodePoint("md-atm", 0xFD23);
  addIconCodePoint("md-atom", 0xF767);
  addIconCodePoint("md-atom-variant", 0xFE98);
  addIconCodePoint("md-attachment", 0xF066);
  addIconCodePoint("md-audio-video", 0xF93C);
  addIconCodePoint("md-audio-video-off", 0xF01E1);
  addIconCodePoint("md-audiobook", 0xF067);
  addIconCodePoint("md-augmented-reality", 0xF84F);
  addIconCodePoint("md-auto-download", 0xF03A9);
  addIconCodePoint("md-auto-fix", 0xF068);
  addIconCodePoint("md-auto-upload", 0xF069);
  addIconCodePoint("md-autorenew", 0xF06A);
  addIconCodePoint("md-av-timer", 0xF06B);
  addIconCodePoint("md-aws", 0xFDF2);
  addIconCodePoint("md-axe", 0xF8C7);
  addIconCodePoint("md-axis", 0xFD24);
  addIconCodePoint("md-axis-arrow", 0xFD25);
  addIconCodePoint("md-axis-arrow-lock", 0xFD26);
  addIconCodePoint("md-axis-lock", 0xFD27);
  addIconCodePoint("md-axis-x-arrow", 0xFD28);
  addIconCodePoint("md-axis-x-arrow-lock", 0xFD29);
  addIconCodePoint("md-axis-x-rotate-clockwise", 0xFD2A);
  addIconCodePoint("md-axis-x-rotate-counterclockwise", 0xFD2B);
  addIconCodePoint("md-axis-x-y-arrow-lock", 0xFD2C);
  addIconCodePoint("md-axis-y-arrow", 0xFD2D);
  addIconCodePoint("md-axis-y-arrow-lock", 0xFD2E);
  addIconCodePoint("md-axis-y-rotate-clockwise", 0xFD2F);
  addIconCodePoint("md-axis-y-rotate-counterclockwise", 0xFD30);
  addIconCodePoint("md-axis-z-arrow", 0xFD31);
  addIconCodePoint("md-axis-z-arrow-lock", 0xFD32);
  addIconCodePoint("md-axis-z-rotate-clockwise", 0xFD33);
  addIconCodePoint("md-axis-z-rotate-counterclockwise", 0xFD34);
  addIconCodePoint("md-azure", 0xF804);
  addIconCodePoint("md-azure-devops", 0xF0091);
  addIconCodePoint("md-babel", 0xFA24);
  addIconCodePoint("md-baby", 0xF06C);
  addIconCodePoint("md-baby-bottle", 0xFF56);
  addIconCodePoint("md-baby-bottle-outline", 0xFF57);
  addIconCodePoint("md-baby-carriage", 0xF68E);
  addIconCodePoint("md-baby-carriage-off", 0xFFC0);
  addIconCodePoint("md-baby-face", 0xFE99);
  addIconCodePoint("md-baby-face-outline", 0xFE9A);
  addIconCodePoint("md-backburger", 0xF06D);
  addIconCodePoint("md-backspace", 0xF06E);
  addIconCodePoint("md-backspace-outline", 0xFB38);
  addIconCodePoint("md-backspace-reverse", 0xFE9B);
  addIconCodePoint("md-backspace-reverse-outline", 0xFE9C);
  addIconCodePoint("md-backup-restore", 0xF06F);
  addIconCodePoint("md-bacteria", 0xFEF2);
  addIconCodePoint("md-bacteria-outline", 0xFEF3);
  addIconCodePoint("md-badminton", 0xF850);
  addIconCodePoint("md-bag-carry-on", 0xFF58);
  addIconCodePoint("md-bag-carry-on-check", 0xFD41);
  addIconCodePoint("md-bag-carry-on-off", 0xFF59);
  addIconCodePoint("md-bag-checked", 0xFF5A);
  addIconCodePoint("md-bag-personal", 0xFDF3);
  addIconCodePoint("md-bag-personal-off", 0xFDF4);
  addIconCodePoint("md-bag-personal-off-outline", 0xFDF5);
  addIconCodePoint("md-bag-personal-outline", 0xFDF6);
  addIconCodePoint("md-baguette", 0xFF5B);
  addIconCodePoint("md-balloon", 0xFA25);
  addIconCodePoint("md-ballot", 0xF9C8);
  addIconCodePoint("md-ballot-outline", 0xF9C9);
  addIconCodePoint("md-ballot-recount", 0xFC15);
  addIconCodePoint("md-ballot-recount-outline", 0xFC16);
  addIconCodePoint("md-bandage", 0xFD8B);
  addIconCodePoint("md-bandcamp", 0xF674);
  addIconCodePoint("md-bank", 0xF070);
  addIconCodePoint("md-bank-minus", 0xFD8C);
  addIconCodePoint("md-bank-outline", 0xFE9D);
  addIconCodePoint("md-bank-plus", 0xFD8D);
  addIconCodePoint("md-bank-remove", 0xFD8E);
  addIconCodePoint("md-bank-transfer", 0xFA26);
  addIconCodePoint("md-bank-transfer-in", 0xFA27);
  addIconCodePoint("md-bank-transfer-out", 0xFA28);
  addIconCodePoint("md-barcode", 0xF071);
  addIconCodePoint("md-barcode-off", 0xF0261);
  addIconCodePoint("md-barcode-scan", 0xF072);
  addIconCodePoint("md-barley", 0xF073);
  addIconCodePoint("md-barley-off", 0xFB39);
  addIconCodePoint("md-barn", 0xFB3A);
  addIconCodePoint("md-barrel", 0xF074);
  addIconCodePoint("md-baseball", 0xF851);
  addIconCodePoint("md-baseball-bat", 0xF852);
  addIconCodePoint("md-basecamp", 0xF075);
  addIconCodePoint("md-bash", 0xF01AE);
  addIconCodePoint("md-basket", 0xF076);
  addIconCodePoint("md-basket-fill", 0xF077);
  addIconCodePoint("md-basket-outline", 0xF01AC);
  addIconCodePoint("md-basket-unfill", 0xF078);
  addIconCodePoint("md-basketball", 0xF805);
  addIconCodePoint("md-basketball-hoop", 0xFC17);
  addIconCodePoint("md-basketball-hoop-outline", 0xFC18);
  addIconCodePoint("md-bat", 0xFB3B);
  addIconCodePoint("md-battery", 0xF079);
  addIconCodePoint("md-battery-10", 0xF07A);
  addIconCodePoint("md-battery-10-bluetooth", 0xF93D);
  addIconCodePoint("md-battery-20", 0xF07B);
  addIconCodePoint("md-battery-20-bluetooth", 0xF93E);
  addIconCodePoint("md-battery-30", 0xF07C);
  addIconCodePoint("md-battery-30-bluetooth", 0xF93F);
  addIconCodePoint("md-battery-40", 0xF07D);
  addIconCodePoint("md-battery-40-bluetooth", 0xF940);
  addIconCodePoint("md-battery-50", 0xF07E);
  addIconCodePoint("md-battery-50-bluetooth", 0xF941);
  addIconCodePoint("md-battery-60", 0xF07F);
  addIconCodePoint("md-battery-60-bluetooth", 0xF942);
  addIconCodePoint("md-battery-70", 0xF080);
  addIconCodePoint("md-battery-70-bluetooth", 0xF943);
  addIconCodePoint("md-battery-80", 0xF081);
  addIconCodePoint("md-battery-80-bluetooth", 0xF944);
  addIconCodePoint("md-battery-90", 0xF082);
  addIconCodePoint("md-battery-90-bluetooth", 0xF945);
  addIconCodePoint("md-battery-alert", 0xF083);
  addIconCodePoint("md-battery-alert-bluetooth", 0xF946);
  addIconCodePoint("md-battery-alert-variant", 0xF00F7);
  addIconCodePoint("md-battery-alert-variant-outline", 0xF00F8);
  addIconCodePoint("md-battery-bluetooth", 0xF947);
  addIconCodePoint("md-battery-bluetooth-variant", 0xF948);
  addIconCodePoint("md-battery-charging", 0xF084);
  addIconCodePoint("md-battery-charging-10", 0xF89B);
  addIconCodePoint("md-battery-charging-100", 0xF085);
  addIconCodePoint("md-battery-charging-20", 0xF086);
  addIconCodePoint("md-battery-charging-30", 0xF087);
  addIconCodePoint("md-battery-charging-40", 0xF088);
  addIconCodePoint("md-battery-charging-50", 0xF89C);
  addIconCodePoint("md-battery-charging-60", 0xF089);
  addIconCodePoint("md-battery-charging-70", 0xF89D);
  addIconCodePoint("md-battery-charging-80", 0xF08A);
  addIconCodePoint("md-battery-charging-90", 0xF08B);
  addIconCodePoint("md-battery-charging-high", 0xF02D1);
  addIconCodePoint("md-battery-charging-low", 0xF02CF);
  addIconCodePoint("md-battery-charging-medium", 0xF02D0);
  addIconCodePoint("md-battery-charging-outline", 0xF89E);
  addIconCodePoint("md-battery-charging-wireless", 0xF806);
  addIconCodePoint("md-battery-charging-wireless-10", 0xF807);
  addIconCodePoint("md-battery-charging-wireless-20", 0xF808);
  addIconCodePoint("md-battery-charging-wireless-30", 0xF809);
  addIconCodePoint("md-battery-charging-wireless-40", 0xF80A);
  addIconCodePoint("md-battery-charging-wireless-50", 0xF80B);
  addIconCodePoint("md-battery-charging-wireless-60", 0xF80C);
  addIconCodePoint("md-battery-charging-wireless-70", 0xF80D);
  addIconCodePoint("md-battery-charging-wireless-80", 0xF80E);
  addIconCodePoint("md-battery-charging-wireless-90", 0xF80F);
  addIconCodePoint("md-battery-charging-wireless-alert", 0xF810);
  addIconCodePoint("md-battery-charging-wireless-outline", 0xF811);
  addIconCodePoint("md-battery-heart", 0xF023A);
  addIconCodePoint("md-battery-heart-outline", 0xF023B);
  addIconCodePoint("md-battery-heart-variant", 0xF023C);
  addIconCodePoint("md-battery-high", 0xF02CE);
  addIconCodePoint("md-battery-low", 0xF02CC);
  addIconCodePoint("md-battery-medium", 0xF02CD);
  addIconCodePoint("md-battery-minus", 0xF08C);
  addIconCodePoint("md-battery-negative", 0xF08D);
  addIconCodePoint("md-battery-off", 0xF0288);
  addIconCodePoint("md-battery-off-outline", 0xF0289);
  addIconCodePoint("md-battery-outline", 0xF08E);
  addIconCodePoint("md-battery-plus", 0xF08F);
  addIconCodePoint("md-battery-positive", 0xF090);
  addIconCodePoint("md-battery-unknown", 0xF091);
  addIconCodePoint("md-battery-unknown-bluetooth", 0xF949);
  addIconCodePoint("md-battlenet", 0xFB3C);
  addIconCodePoint("md-beach", 0xF092);
  addIconCodePoint("md-beaker", 0xFCC6);
  addIconCodePoint("md-beaker-alert", 0xF0254);
  addIconCodePoint("md-beaker-alert-outline", 0xF0255);
  addIconCodePoint("md-beaker-check", 0xF0256);
  addIconCodePoint("md-beaker-check-outline", 0xF0257);
  addIconCodePoint("md-beaker-minus", 0xF0258);
  addIconCodePoint("md-beaker-minus-outline", 0xF0259);
  addIconCodePoint("md-beaker-outline", 0xF68F);
  addIconCodePoint("md-beaker-plus", 0xF025A);
  addIconCodePoint("md-beaker-plus-outline", 0xF025B);
  addIconCodePoint("md-beaker-question", 0xF025C);
  addIconCodePoint("md-beaker-question-outline", 0xF025D);
  addIconCodePoint("md-beaker-remove", 0xF025E);
  addIconCodePoint("md-beaker-remove-outline", 0xF025F);
  addIconCodePoint("md-beats", 0xF097);
  addIconCodePoint("md-bed-double", 0xF0092);
  addIconCodePoint("md-bed-double-outline", 0xF0093);
  addIconCodePoint("md-bed-empty", 0xF89F);
  addIconCodePoint("md-bed-king", 0xF0094);
  addIconCodePoint("md-bed-king-outline", 0xF0095);
  addIconCodePoint("md-bed-queen", 0xF0096);
  addIconCodePoint("md-bed-queen-outline", 0xF0097);
  addIconCodePoint("md-bed-single", 0xF0098);
  addIconCodePoint("md-bed-single-outline", 0xF0099);
  addIconCodePoint("md-bee", 0xFFC1);
  addIconCodePoint("md-bee-flower", 0xFFC2);
  addIconCodePoint("md-beehive-outline", 0xF00F9);
  addIconCodePoint("md-beer", 0xF098);
  addIconCodePoint("md-beer-outline", 0xF0337);
  addIconCodePoint("md-behance", 0xF099);
  addIconCodePoint("md-bell", 0xF09A);
  addIconCodePoint("md-bell-alert", 0xFD35);
  addIconCodePoint("md-bell-alert-outline", 0xFE9E);
  addIconCodePoint("md-bell-check", 0xF0210);
  addIconCodePoint("md-bell-check-outline", 0xF0211);
  addIconCodePoint("md-bell-circle", 0xFD36);
  addIconCodePoint("md-bell-circle-outline", 0xFD37);
  addIconCodePoint("md-bell-off", 0xF09B);
  addIconCodePoint("md-bell-off-outline", 0xFA90);
  addIconCodePoint("md-bell-outline", 0xF09C);
  addIconCodePoint("md-bell-plus", 0xF09D);
  addIconCodePoint("md-bell-plus-outline", 0xFA91);
  addIconCodePoint("md-bell-ring", 0xF09E);
  addIconCodePoint("md-bell-ring-outline", 0xF09F);
  addIconCodePoint("md-bell-sleep", 0xF0A0);
  addIconCodePoint("md-bell-sleep-outline", 0xFA92);
  addIconCodePoint("md-beta", 0xF0A1);
  addIconCodePoint("md-betamax", 0xF9CA);
  addIconCodePoint("md-biathlon", 0xFDF7);
  addIconCodePoint("md-bible", 0xF0A2);
  addIconCodePoint("md-bicycle", 0xF00C7);
  addIconCodePoint("md-bicycle-basket", 0xF0260);
  addIconCodePoint("md-bike", 0xF0A3);
  addIconCodePoint("md-bike-fast", 0xF014A);
  addIconCodePoint("md-billboard", 0xF0032);
  addIconCodePoint("md-billiards", 0xFB3D);
  addIconCodePoint("md-billiards-rack", 0xFB3E);
  addIconCodePoint("md-bing", 0xF0A4);
  addIconCodePoint("md-binoculars", 0xF0A5);
  addIconCodePoint("md-bio", 0xF0A6);
  addIconCodePoint("md-biohazard", 0xF0A7);
  addIconCodePoint("md-bitbucket", 0xF0A8);
  addIconCodePoint("md-bitcoin", 0xF812);
  addIconCodePoint("md-black-mesa", 0xF0A9);
  addIconCodePoint("md-blackberry", 0xF0AA);
  addIconCodePoint("md-blender", 0xFCC7);
  addIconCodePoint("md-blender-software", 0xF0AB);
  addIconCodePoint("md-blinds", 0xF0AC);
  addIconCodePoint("md-blinds-open", 0xF0033);
  addIconCodePoint("md-block-helper", 0xF0AD);
  addIconCodePoint("md-blogger", 0xF0AE);
  addIconCodePoint("md-blood-bag", 0xFCC8);
  addIconCodePoint("md-bluetooth", 0xF0AF);
  addIconCodePoint("md-bluetooth-audio", 0xF0B0);
  addIconCodePoint("md-bluetooth-connect", 0xF0B1);
  addIconCodePoint("md-bluetooth-off", 0xF0B2);
  addIconCodePoint("md-bluetooth-settings", 0xF0B3);
  addIconCodePoint("md-bluetooth-transfer", 0xF0B4);
  addIconCodePoint("md-blur", 0xF0B5);
  addIconCodePoint("md-blur-linear", 0xF0B6);
  addIconCodePoint("md-blur-off", 0xF0B7);
  addIconCodePoint("md-blur-radial", 0xF0B8);
  addIconCodePoint("md-bolnisi-cross", 0xFCC9);
  addIconCodePoint("md-bolt", 0xFD8F);
  addIconCodePoint("md-bomb", 0xF690);
  addIconCodePoint("md-bomb-off", 0xF6C4);
  addIconCodePoint("md-bone", 0xF0B9);
  addIconCodePoint("md-book", 0xF0BA);
  addIconCodePoint("md-book-information-variant", 0xF009A);
  addIconCodePoint("md-book-lock", 0xF799);
  addIconCodePoint("md-book-lock-open", 0xF79A);
  addIconCodePoint("md-book-minus", 0xF5D9);
  addIconCodePoint("md-book-minus-multiple", 0xFA93);
  addIconCodePoint("md-book-multiple", 0xF0BB);
  addIconCodePoint("md-book-open", 0xF0BD);
  addIconCodePoint("md-book-open-outline", 0xFB3F);
  addIconCodePoint("md-book-open-page-variant", 0xF5DA);
  addIconCodePoint("md-book-open-variant", 0xF0BE);
  addIconCodePoint("md-book-outline", 0xFB40);
  addIconCodePoint("md-book-play", 0xFE9F);
  addIconCodePoint("md-book-play-outline", 0xFEA0);
  addIconCodePoint("md-book-plus", 0xF5DB);
  addIconCodePoint("md-book-plus-multiple", 0xFA94);
  addIconCodePoint("md-book-remove", 0xFA96);
  addIconCodePoint("md-book-remove-multiple", 0xFA95);
  addIconCodePoint("md-book-search", 0xFEA1);
  addIconCodePoint("md-book-search-outline", 0xFEA2);
  addIconCodePoint("md-book-variant", 0xF0BF);
  addIconCodePoint("md-book-variant-multiple", 0xF0BC);
  addIconCodePoint("md-bookmark", 0xF0C0);
  addIconCodePoint("md-bookmark-check", 0xF0C1);
  addIconCodePoint("md-bookmark-check-outline", 0xF03A6);
  addIconCodePoint("md-bookmark-minus", 0xF9CB);
  addIconCodePoint("md-bookmark-minus-outline", 0xF9CC);
  addIconCodePoint("md-bookmark-multiple", 0xFDF8);
  addIconCodePoint("md-bookmark-multiple-outline", 0xFDF9);
  addIconCodePoint("md-bookmark-music", 0xF0C2);
  addIconCodePoint("md-bookmark-music-outline", 0xF03A4);
  addIconCodePoint("md-bookmark-off", 0xF9CD);
  addIconCodePoint("md-bookmark-off-outline", 0xF9CE);
  addIconCodePoint("md-bookmark-outline", 0xF0C3);
  addIconCodePoint("md-bookmark-plus", 0xF0C5);
  addIconCodePoint("md-bookmark-plus-outline", 0xF0C4);
  addIconCodePoint("md-bookmark-remove", 0xF0C6);
  addIconCodePoint("md-bookmark-remove-outline", 0xF03A5);
  addIconCodePoint("md-bookshelf", 0xF028A);
  addIconCodePoint("md-boom-gate", 0xFEA3);
  addIconCodePoint("md-boom-gate-alert", 0xFEA4);
  addIconCodePoint("md-boom-gate-alert-outline", 0xFEA5);
  addIconCodePoint("md-boom-gate-down", 0xFEA6);
  addIconCodePoint("md-boom-gate-down-outline", 0xFEA7);
  addIconCodePoint("md-boom-gate-outline", 0xFEA8);
  addIconCodePoint("md-boom-gate-up", 0xFEA9);
  addIconCodePoint("md-boom-gate-up-outline", 0xFEAA);
  addIconCodePoint("md-boombox", 0xF5DC);
  addIconCodePoint("md-boomerang", 0xF00FA);
  addIconCodePoint("md-bootstrap", 0xF6C5);
  addIconCodePoint("md-border-all", 0xF0C7);
  addIconCodePoint("md-border-all-variant", 0xF8A0);
  addIconCodePoint("md-border-bottom", 0xF0C8);
  addIconCodePoint("md-border-bottom-variant", 0xF8A1);
  addIconCodePoint("md-border-color", 0xF0C9);
  addIconCodePoint("md-border-horizontal", 0xF0CA);
  addIconCodePoint("md-border-inside", 0xF0CB);
  addIconCodePoint("md-border-left", 0xF0CC);
  addIconCodePoint("md-border-left-variant", 0xF8A2);
  addIconCodePoint("md-border-none", 0xF0CD);
  addIconCodePoint("md-border-none-variant", 0xF8A3);
  addIconCodePoint("md-border-outside", 0xF0CE);
  addIconCodePoint("md-border-right", 0xF0CF);
  addIconCodePoint("md-border-right-variant", 0xF8A4);
  addIconCodePoint("md-border-style", 0xF0D0);
  addIconCodePoint("md-border-top", 0xF0D1);
  addIconCodePoint("md-border-top-variant", 0xF8A5);
  addIconCodePoint("md-border-vertical", 0xF0D2);
  addIconCodePoint("md-bottle-soda", 0xF009B);
  addIconCodePoint("md-bottle-soda-classic", 0xF009C);
  addIconCodePoint("md-bottle-soda-classic-outline", 0xF038E);
  addIconCodePoint("md-bottle-soda-outline", 0xF009D);
  addIconCodePoint("md-bottle-tonic", 0xF0159);
  addIconCodePoint("md-bottle-tonic-outline", 0xF015A);
  addIconCodePoint("md-bottle-tonic-plus", 0xF015B);
  addIconCodePoint("md-bottle-tonic-plus-outline", 0xF015C);
  addIconCodePoint("md-bottle-tonic-skull", 0xF015D);
  addIconCodePoint("md-bottle-tonic-skull-outline", 0xF015E);
  addIconCodePoint("md-bottle-wine", 0xF853);
  addIconCodePoint("md-bottle-wine-outline", 0xF033B);
  addIconCodePoint("md-bow-tie", 0xF677);
  addIconCodePoint("md-bowl", 0xF617);
  addIconCodePoint("md-bowling", 0xF0D3);
  addIconCodePoint("md-box", 0xF0D4);
  addIconCodePoint("md-box-cutter", 0xF0D5);
  addIconCodePoint("md-box-shadow", 0xF637);
  addIconCodePoint("md-boxing-glove", 0xFB41);
  addIconCodePoint("md-braille", 0xF9CF);
  addIconCodePoint("md-brain", 0xF9D0);
  addIconCodePoint("md-bread-slice", 0xFCCA);
  addIconCodePoint("md-bread-slice-outline", 0xFCCB);
  addIconCodePoint("md-bridge", 0xF618);
  addIconCodePoint("md-briefcase", 0xF0D6);
  addIconCodePoint("md-briefcase-account", 0xFCCC);
  addIconCodePoint("md-briefcase-account-outline", 0xFCCD);
  addIconCodePoint("md-briefcase-check", 0xF0D7);
  addIconCodePoint("md-briefcase-check-outline", 0xF0349);
  addIconCodePoint("md-briefcase-clock", 0xF00FB);
  addIconCodePoint("md-briefcase-clock-outline", 0xF00FC);
  addIconCodePoint("md-briefcase-download", 0xF0D8);
  addIconCodePoint("md-briefcase-download-outline", 0xFC19);
  addIconCodePoint("md-briefcase-edit", 0xFA97);
  addIconCodePoint("md-briefcase-edit-outline", 0xFC1A);
  addIconCodePoint("md-briefcase-minus", 0xFA29);
  addIconCodePoint("md-briefcase-minus-outline", 0xFC1B);
  addIconCodePoint("md-briefcase-outline", 0xF813);
  addIconCodePoint("md-briefcase-plus", 0xFA2A);
  addIconCodePoint("md-briefcase-plus-outline", 0xFC1C);
  addIconCodePoint("md-briefcase-remove", 0xFA2B);
  addIconCodePoint("md-briefcase-remove-outline", 0xFC1D);
  addIconCodePoint("md-briefcase-search", 0xFA2C);
  addIconCodePoint("md-briefcase-search-outline", 0xFC1E);
  addIconCodePoint("md-briefcase-upload", 0xF0D9);
  addIconCodePoint("md-briefcase-upload-outline", 0xFC1F);
  addIconCodePoint("md-brightness-1", 0xF0DA);
  addIconCodePoint("md-brightness-2", 0xF0DB);
  addIconCodePoint("md-brightness-3", 0xF0DC);
  addIconCodePoint("md-brightness-4", 0xF0DD);
  addIconCodePoint("md-brightness-5", 0xF0DE);
  addIconCodePoint("md-brightness-6", 0xF0DF);
  addIconCodePoint("md-brightness-7", 0xF0E0);
  addIconCodePoint("md-brightness-auto", 0xF0E1);
  addIconCodePoint("md-brightness-percent", 0xFCCE);
  addIconCodePoint("md-broom", 0xF0E2);
  addIconCodePoint("md-brush", 0xF0E3);
  addIconCodePoint("md-buddhism", 0xF94A);
  addIconCodePoint("md-buffer", 0xF619);
  addIconCodePoint("md-bug", 0xF0E4);
  addIconCodePoint("md-bug-check", 0xFA2D);
  addIconCodePoint("md-bug-check-outline", 0xFA2E);
  addIconCodePoint("md-bug-outline", 0xFA2F);
  addIconCodePoint("md-bugle", 0xFD90);
  addIconCodePoint("md-bulldozer", 0xFB07);
  addIconCodePoint("md-bullet", 0xFCCF);
  addIconCodePoint("md-bulletin-board", 0xF0E5);
  addIconCodePoint("md-bullhorn", 0xF0E6);
  addIconCodePoint("md-bullhorn-outline", 0xFB08);
  addIconCodePoint("md-bullseye", 0xF5DD);
  addIconCodePoint("md-bullseye-arrow", 0xF8C8);
  addIconCodePoint("md-bulma", 0xF0312);
  addIconCodePoint("md-bunk-bed", 0xF032D);
  addIconCodePoint("md-bus", 0xF0E7);
  addIconCodePoint("md-bus-alert", 0xFA98);
  addIconCodePoint("md-bus-articulated-end", 0xF79B);
  addIconCodePoint("md-bus-articulated-front", 0xF79C);
  addIconCodePoint("md-bus-clock", 0xF8C9);
  addIconCodePoint("md-bus-double-decker", 0xF79D);
  addIconCodePoint("md-bus-marker", 0xF023D);
  addIconCodePoint("md-bus-multiple", 0xFF5C);
  addIconCodePoint("md-bus-school", 0xF79E);
  addIconCodePoint("md-bus-side", 0xF79F);
  addIconCodePoint("md-bus-stop", 0xF0034);
  addIconCodePoint("md-bus-stop-covered", 0xF0035);
  addIconCodePoint("md-bus-stop-uncovered", 0xF0036);
  addIconCodePoint("md-cached", 0xF0E8);
  addIconCodePoint("md-cactus", 0xFD91);
  addIconCodePoint("md-cake", 0xF0E9);
  addIconCodePoint("md-cake-layered", 0xF0EA);
  addIconCodePoint("md-cake-variant", 0xF0EB);
  addIconCodePoint("md-calculator", 0xF0EC);
  addIconCodePoint("md-calculator-variant", 0xFA99);
  addIconCodePoint("md-calendar", 0xF0ED);
  addIconCodePoint("md-calendar-account", 0xFEF4);
  addIconCodePoint("md-calendar-account-outline", 0xFEF5);
  addIconCodePoint("md-calendar-alert", 0xFA30);
  addIconCodePoint("md-calendar-arrow-left", 0xF015F);
  addIconCodePoint("md-calendar-arrow-right", 0xF0160);
  addIconCodePoint("md-calendar-blank", 0xF0EE);
  addIconCodePoint("md-calendar-blank-multiple", 0xF009E);
  addIconCodePoint("md-calendar-blank-outline", 0xFB42);
  addIconCodePoint("md-calendar-check", 0xF0EF);
  addIconCodePoint("md-calendar-check-outline", 0xFC20);
  addIconCodePoint("md-calendar-clock", 0xF0F0);
  addIconCodePoint("md-calendar-edit", 0xF8A6);
  addIconCodePoint("md-calendar-export", 0xFB09);
  addIconCodePoint("md-calendar-heart", 0xF9D1);
  addIconCodePoint("md-calendar-import", 0xFB0A);
  addIconCodePoint("md-calendar-minus", 0xFD38);
  addIconCodePoint("md-calendar-month", 0xFDFA);
  addIconCodePoint("md-calendar-month-outline", 0xFDFB);
  addIconCodePoint("md-calendar-multiple", 0xF0F1);
  addIconCodePoint("md-calendar-multiple-check", 0xF0F2);
  addIconCodePoint("md-calendar-multiselect", 0xFA31);
  addIconCodePoint("md-calendar-outline", 0xFB43);
  addIconCodePoint("md-calendar-plus", 0xF0F3);
  addIconCodePoint("md-calendar-question", 0xF691);
  addIconCodePoint("md-calendar-range", 0xF678);
  addIconCodePoint("md-calendar-range-outline", 0xFB44);
  addIconCodePoint("md-calendar-remove", 0xF0F4);
  addIconCodePoint("md-calendar-remove-outline", 0xFC21);
  addIconCodePoint("md-calendar-repeat", 0xFEAB);
  addIconCodePoint("md-calendar-repeat-outline", 0xFEAC);
  addIconCodePoint("md-calendar-search", 0xF94B);
  addIconCodePoint("md-calendar-star", 0xF9D2);
  addIconCodePoint("md-calendar-text", 0xF0F5);
  addIconCodePoint("md-calendar-text-outline", 0xFC22);
  addIconCodePoint("md-calendar-today", 0xF0F6);
  addIconCodePoint("md-calendar-week", 0xFA32);
  addIconCodePoint("md-calendar-week-begin", 0xFA33);
  addIconCodePoint("md-calendar-weekend", 0xFEF6);
  addIconCodePoint("md-calendar-weekend-outline", 0xFEF7);
  addIconCodePoint("md-call-made", 0xF0F7);
  addIconCodePoint("md-call-merge", 0xF0F8);
  addIconCodePoint("md-call-missed", 0xF0F9);
  addIconCodePoint("md-call-received", 0xF0FA);
  addIconCodePoint("md-call-split", 0xF0FB);
  addIconCodePoint("md-camcorder", 0xF0FC);
  addIconCodePoint("md-camcorder-box", 0xF0FD);
  addIconCodePoint("md-camcorder-box-off", 0xF0FE);
  addIconCodePoint("md-camcorder-off", 0xF0FF);
  addIconCodePoint("md-camera", 0xF100);
  addIconCodePoint("md-camera-account", 0xF8CA);
  addIconCodePoint("md-camera-burst", 0xF692);
  addIconCodePoint("md-camera-control", 0xFB45);
  addIconCodePoint("md-camera-enhance", 0xF101);
  addIconCodePoint("md-camera-enhance-outline", 0xFB46);
  addIconCodePoint("md-camera-front", 0xF102);
  addIconCodePoint("md-camera-front-variant", 0xF103);
  addIconCodePoint("md-camera-gopro", 0xF7A0);
  addIconCodePoint("md-camera-image", 0xF8CB);
  addIconCodePoint("md-camera-iris", 0xF104);
  addIconCodePoint("md-camera-metering-center", 0xF7A1);
  addIconCodePoint("md-camera-metering-matrix", 0xF7A2);
  addIconCodePoint("md-camera-metering-partial", 0xF7A3);
  addIconCodePoint("md-camera-metering-spot", 0xF7A4);
  addIconCodePoint("md-camera-off", 0xF5DF);
  addIconCodePoint("md-camera-outline", 0xFD39);
  addIconCodePoint("md-camera-party-mode", 0xF105);
  addIconCodePoint("md-camera-plus", 0xFEF8);
  addIconCodePoint("md-camera-plus-outline", 0xFEF9);
  addIconCodePoint("md-camera-rear", 0xF106);
  addIconCodePoint("md-camera-rear-variant", 0xF107);
  addIconCodePoint("md-camera-retake", 0xFDFC);
  addIconCodePoint("md-camera-retake-outline", 0xFDFD);
  addIconCodePoint("md-camera-switch", 0xF108);
  addIconCodePoint("md-camera-timer", 0xF109);
  addIconCodePoint("md-camera-wireless", 0xFD92);
  addIconCodePoint("md-camera-wireless-outline", 0xFD93);
  addIconCodePoint("md-campfire", 0xFEFA);
  addIconCodePoint("md-cancel", 0xF739);
  addIconCodePoint("md-candle", 0xF5E2);
  addIconCodePoint("md-candycane", 0xF10A);
  addIconCodePoint("md-cannabis", 0xF7A5);
  addIconCodePoint("md-caps-lock", 0xFA9A);
  addIconCodePoint("md-car", 0xF10B);
  addIconCodePoint("md-car-2-plus", 0xF0037);
  addIconCodePoint("md-car-3-plus", 0xF0038);
  addIconCodePoint("md-car-back", 0xFDFE);
  addIconCodePoint("md-car-battery", 0xF10C);
  addIconCodePoint("md-car-brake-abs", 0xFC23);
  addIconCodePoint("md-car-brake-alert", 0xFC24);
  addIconCodePoint("md-car-brake-hold", 0xFD3A);
  addIconCodePoint("md-car-brake-parking", 0xFD3B);
  addIconCodePoint("md-car-brake-retarder", 0xF0039);
  addIconCodePoint("md-car-child-seat", 0xFFC3);
  addIconCodePoint("md-car-clutch", 0xF003A);
  addIconCodePoint("md-car-connected", 0xF10D);
  addIconCodePoint("md-car-convertible", 0xF7A6);
  addIconCodePoint("md-car-coolant-level", 0xF003B);
  addIconCodePoint("md-car-cruise-control", 0xFD3C);
  addIconCodePoint("md-car-defrost-front", 0xFD3D);
  addIconCodePoint("md-car-defrost-rear", 0xFD3E);
  addIconCodePoint("md-car-door", 0xFB47);
  addIconCodePoint("md-car-door-lock", 0xF00C8);
  addIconCodePoint("md-car-electric", 0xFB48);
  addIconCodePoint("md-car-esp", 0xFC25);
  addIconCodePoint("md-car-estate", 0xF7A7);
  addIconCodePoint("md-car-hatchback", 0xF7A8);
  addIconCodePoint("md-car-info", 0xF01E9);
  addIconCodePoint("md-car-key", 0xFB49);
  addIconCodePoint("md-car-light-dimmed", 0xFC26);
  addIconCodePoint("md-car-light-fog", 0xFC27);
  addIconCodePoint("md-car-light-high", 0xFC28);
  addIconCodePoint("md-car-limousine", 0xF8CC);
  addIconCodePoint("md-car-multiple", 0xFB4A);
  addIconCodePoint("md-car-off", 0xFDFF);
  addIconCodePoint("md-car-parking-lights", 0xFD3F);
  addIconCodePoint("md-car-pickup", 0xF7A9);
  addIconCodePoint("md-car-seat", 0xFFC4);
  addIconCodePoint("md-car-seat-cooler", 0xFFC5);
  addIconCodePoint("md-car-seat-heater", 0xFFC6);
  addIconCodePoint("md-car-shift-pattern", 0xFF5D);
  addIconCodePoint("md-car-side", 0xF7AA);
  addIconCodePoint("md-car-sports", 0xF7AB);
  addIconCodePoint("md-car-tire-alert", 0xFC29);
  addIconCodePoint("md-car-traction-control", 0xFD40);
  addIconCodePoint("md-car-turbocharger", 0xF003C);
  addIconCodePoint("md-car-wash", 0xF10E);
  addIconCodePoint("md-car-windshield", 0xF003D);
  addIconCodePoint("md-car-windshield-outline", 0xF003E);
  addIconCodePoint("md-caravan", 0xF7AC);
  addIconCodePoint("md-card", 0xFB4B);
  addIconCodePoint("md-card-bulleted", 0xFB4C);
  addIconCodePoint("md-card-bulleted-off", 0xFB4D);
  addIconCodePoint("md-card-bulleted-off-outline", 0xFB4E);
  addIconCodePoint("md-card-bulleted-outline", 0xFB4F);
  addIconCodePoint("md-card-bulleted-settings", 0xFB50);
  addIconCodePoint("md-card-bulleted-settings-outline", 0xFB51);
  addIconCodePoint("md-card-outline", 0xFB52);
  addIconCodePoint("md-card-plus", 0xF022A);
  addIconCodePoint("md-card-plus-outline", 0xF022B);
  addIconCodePoint("md-card-search", 0xF009F);
  addIconCodePoint("md-card-search-outline", 0xF00A0);
  addIconCodePoint("md-card-text", 0xFB53);
  addIconCodePoint("md-card-text-outline", 0xFB54);
  addIconCodePoint("md-cards", 0xF638);
  addIconCodePoint("md-cards-club", 0xF8CD);
  addIconCodePoint("md-cards-diamond", 0xF8CE);
  addIconCodePoint("md-cards-diamond-outline", 0xF003F);
  addIconCodePoint("md-cards-heart", 0xF8CF);
  addIconCodePoint("md-cards-outline", 0xF639);
  addIconCodePoint("md-cards-playing-outline", 0xF63A);
  addIconCodePoint("md-cards-spade", 0xF8D0);
  addIconCodePoint("md-cards-variant", 0xF6C6);
  addIconCodePoint("md-carrot", 0xF10F);
  addIconCodePoint("md-cart", 0xF110);
  addIconCodePoint("md-cart-arrow-down", 0xFD42);
  addIconCodePoint("md-cart-arrow-right", 0xFC2A);
  addIconCodePoint("md-cart-arrow-up", 0xFD43);
  addIconCodePoint("md-cart-minus", 0xFD44);
  addIconCodePoint("md-cart-off", 0xF66B);
  addIconCodePoint("md-cart-outline", 0xF111);
  addIconCodePoint("md-cart-plus", 0xF112);
  addIconCodePoint("md-cart-remove", 0xFD45);
  addIconCodePoint("md-case-sensitive-alt", 0xF113);
  addIconCodePoint("md-cash", 0xF114);
  addIconCodePoint("md-cash-100", 0xF115);
  addIconCodePoint("md-cash-marker", 0xFD94);
  addIconCodePoint("md-cash-minus", 0xF028B);
  addIconCodePoint("md-cash-multiple", 0xF116);
  addIconCodePoint("md-cash-plus", 0xF028C);
  addIconCodePoint("md-cash-refund", 0xFA9B);
  addIconCodePoint("md-cash-register", 0xFCD0);
  addIconCodePoint("md-cash-remove", 0xF028D);
  addIconCodePoint("md-cash-usd", 0xF01A1);
  addIconCodePoint("md-cash-usd-outline", 0xF117);
  addIconCodePoint("md-cassette", 0xF9D3);
  addIconCodePoint("md-cast", 0xF118);
  addIconCodePoint("md-cast-audio", 0xF0040);
  addIconCodePoint("md-cast-connected", 0xF119);
  addIconCodePoint("md-cast-education", 0xFE6D);
  addIconCodePoint("md-cast-off", 0xF789);
  addIconCodePoint("md-castle", 0xF11A);
  addIconCodePoint("md-cat", 0xF11B);
  addIconCodePoint("md-cctv", 0xF7AD);
  addIconCodePoint("md-ceiling-light", 0xF768);
  addIconCodePoint("md-cellphone", 0xF11C);
  addIconCodePoint("md-cellphone-android", 0xF11D);
  addIconCodePoint("md-cellphone-arrow-down", 0xF9D4);
  addIconCodePoint("md-cellphone-basic", 0xF11E);
  addIconCodePoint("md-cellphone-dock", 0xF11F);
  addIconCodePoint("md-cellphone-erase", 0xF94C);
  addIconCodePoint("md-cellphone-information", 0xFF5E);
  addIconCodePoint("md-cellphone-iphone", 0xF120);
  addIconCodePoint("md-cellphone-key", 0xF94D);
  addIconCodePoint("md-cellphone-link", 0xF121);
  addIconCodePoint("md-cellphone-link-off", 0xF122);
  addIconCodePoint("md-cellphone-lock", 0xF94E);
  addIconCodePoint("md-cellphone-message", 0xF8D2);
  addIconCodePoint("md-cellphone-message-off", 0xF00FD);
  addIconCodePoint("md-cellphone-nfc", 0xFEAD);
  addIconCodePoint("md-cellphone-nfc-off", 0xF0303);
  addIconCodePoint("md-cellphone-off", 0xF94F);
  addIconCodePoint("md-cellphone-play", 0xF0041);
  addIconCodePoint("md-cellphone-screenshot", 0xFA34);
  addIconCodePoint("md-cellphone-settings", 0xF123);
  addIconCodePoint("md-cellphone-settings-variant", 0xF950);
  addIconCodePoint("md-cellphone-sound", 0xF951);
  addIconCodePoint("md-cellphone-text", 0xF8D1);
  addIconCodePoint("md-cellphone-wireless", 0xF814);
  addIconCodePoint("md-celtic-cross", 0xFCD1);
  addIconCodePoint("md-centos", 0xF0145);
  addIconCodePoint("md-certificate", 0xF124);
  addIconCodePoint("md-certificate-outline", 0xF01B3);
  addIconCodePoint("md-chair-rolling", 0xFFBA);
  addIconCodePoint("md-chair-school", 0xF125);
  addIconCodePoint("md-charity", 0xFC2B);
  addIconCodePoint("md-chart-arc", 0xF126);
  addIconCodePoint("md-chart-areaspline", 0xF127);
  addIconCodePoint("md-chart-areaspline-variant", 0xFEAE);
  addIconCodePoint("md-chart-bar", 0xF128);
  addIconCodePoint("md-chart-bar-stacked", 0xF769);
  addIconCodePoint("md-chart-bell-curve", 0xFC2C);
  addIconCodePoint("md-chart-bell-curve-cumulative", 0xFFC7);
  addIconCodePoint("md-chart-bubble", 0xF5E3);
  addIconCodePoint("md-chart-donut", 0xF7AE);
  addIconCodePoint("md-chart-donut-variant", 0xF7AF);
  addIconCodePoint("md-chart-gantt", 0xF66C);
  addIconCodePoint("md-chart-histogram", 0xF129);
  addIconCodePoint("md-chart-line", 0xF12A);
  addIconCodePoint("md-chart-line-stacked", 0xF76A);
  addIconCodePoint("md-chart-line-variant", 0xF7B0);
  addIconCodePoint("md-chart-multiline", 0xF8D3);
  addIconCodePoint("md-chart-multiple", 0xF023E);
  addIconCodePoint("md-chart-pie", 0xF12B);
  addIconCodePoint("md-chart-ppf", 0xF03AB);
  addIconCodePoint("md-chart-scatter-plot", 0xFEAF);
  addIconCodePoint("md-chart-scatter-plot-hexbin", 0xF66D);
  addIconCodePoint("md-chart-snakey", 0xF020A);
  addIconCodePoint("md-chart-snakey-variant", 0xF020B);
  addIconCodePoint("md-chart-timeline", 0xF66E);
  addIconCodePoint("md-chart-timeline-variant", 0xFEB0);
  addIconCodePoint("md-chart-tree", 0xFEB1);
  addIconCodePoint("md-chat", 0xFB55);
  addIconCodePoint("md-chat-alert", 0xFB56);
  addIconCodePoint("md-chat-alert-outline", 0xF02F4);
  addIconCodePoint("md-chat-outline", 0xFEFB);
  addIconCodePoint("md-chat-processing", 0xFB57);
  addIconCodePoint("md-chat-processing-outline", 0xF02F5);
  addIconCodePoint("md-chat-sleep", 0xF02FC);
  addIconCodePoint("md-chat-sleep-outline", 0xF02FD);
  addIconCodePoint("md-check", 0xF12C);
  addIconCodePoint("md-check-all", 0xF12D);
  addIconCodePoint("md-check-bold", 0xFE6E);
  addIconCodePoint("md-check-box-multiple-outline", 0xFC2D);
  addIconCodePoint("md-check-box-outline", 0xFC2E);
  addIconCodePoint("md-check-circle", 0xF5E0);
  addIconCodePoint("md-check-circle-outline", 0xF5E1);
  addIconCodePoint("md-check-decagram", 0xF790);
  addIconCodePoint("md-check-network", 0xFC2F);
  addIconCodePoint("md-check-network-outline", 0xFC30);
  addIconCodePoint("md-check-outline", 0xF854);
  addIconCodePoint("md-check-underline", 0xFE70);
  addIconCodePoint("md-check-underline-circle", 0xFE71);
  addIconCodePoint("md-check-underline-circle-outline", 0xFE72);
  addIconCodePoint("md-checkbook", 0xFA9C);
  addIconCodePoint("md-checkbox-blank", 0xF12E);
  addIconCodePoint("md-checkbox-blank-circle", 0xF12F);
  addIconCodePoint("md-checkbox-blank-circle-outline", 0xF130);
  addIconCodePoint("md-checkbox-blank-off", 0xF0317);
  addIconCodePoint("md-checkbox-blank-off-outline", 0xF0318);
  addIconCodePoint("md-checkbox-blank-outline", 0xF131);
  addIconCodePoint("md-checkbox-intermediate", 0xF855);
  addIconCodePoint("md-checkbox-marked", 0xF132);
  addIconCodePoint("md-checkbox-marked-circle", 0xF133);
  addIconCodePoint("md-checkbox-marked-circle-outline", 0xF134);
  addIconCodePoint("md-checkbox-marked-outline", 0xF135);
  addIconCodePoint("md-checkbox-multiple-blank", 0xF136);
  addIconCodePoint("md-checkbox-multiple-blank-circle", 0xF63B);
  addIconCodePoint("md-checkbox-multiple-blank-circle-outline", 0xF63C);
  addIconCodePoint("md-checkbox-multiple-blank-outline", 0xF137);
  addIconCodePoint("md-checkbox-multiple-marked", 0xF138);
  addIconCodePoint("md-checkbox-multiple-marked-circle", 0xF63D);
  addIconCodePoint("md-checkbox-multiple-marked-circle-outline", 0xF63E);
  addIconCodePoint("md-checkbox-multiple-marked-outline", 0xF139);
  addIconCodePoint("md-checkerboard", 0xF13A);
  addIconCodePoint("md-checkerboard-minus", 0xF022D);
  addIconCodePoint("md-checkerboard-plus", 0xF022C);
  addIconCodePoint("md-checkerboard-remove", 0xF022E);
  addIconCodePoint("md-cheese", 0xF02E4);
  addIconCodePoint("md-chef-hat", 0xFB58);
  addIconCodePoint("md-chemical-weapon", 0xF13B);
  addIconCodePoint("md-chess-bishop", 0xF85B);
  addIconCodePoint("md-chess-king", 0xF856);
  addIconCodePoint("md-chess-knight", 0xF857);
  addIconCodePoint("md-chess-pawn", 0xF858);
  addIconCodePoint("md-chess-queen", 0xF859);
  addIconCodePoint("md-chess-rook", 0xF85A);
  addIconCodePoint("md-chevron-double-down", 0xF13C);
  addIconCodePoint("md-chevron-double-left", 0xF13D);
  addIconCodePoint("md-chevron-double-right", 0xF13E);
  addIconCodePoint("md-chevron-double-up", 0xF13F);
  addIconCodePoint("md-chevron-down", 0xF140);
  addIconCodePoint("md-chevron-down-box", 0xF9D5);
  addIconCodePoint("md-chevron-down-box-outline", 0xF9D6);
  addIconCodePoint("md-chevron-down-circle", 0xFB0B);
  addIconCodePoint("md-chevron-down-circle-outline", 0xFB0C);
  addIconCodePoint("md-chevron-left", 0xF141);
  addIconCodePoint("md-chevron-left-box", 0xF9D7);
  addIconCodePoint("md-chevron-left-box-outline", 0xF9D8);
  addIconCodePoint("md-chevron-left-circle", 0xFB0D);
  addIconCodePoint("md-chevron-left-circle-outline", 0xFB0E);
  addIconCodePoint("md-chevron-right", 0xF142);
  addIconCodePoint("md-chevron-right-box", 0xF9D9);
  addIconCodePoint("md-chevron-right-box-outline", 0xF9DA);
  addIconCodePoint("md-chevron-right-circle", 0xFB0F);
  addIconCodePoint("md-chevron-right-circle-outline", 0xFB10);
  addIconCodePoint("md-chevron-triple-down", 0xFD95);
  addIconCodePoint("md-chevron-triple-left", 0xFD96);
  addIconCodePoint("md-chevron-triple-right", 0xFD97);
  addIconCodePoint("md-chevron-triple-up", 0xFD98);
  addIconCodePoint("md-chevron-up", 0xF143);
  addIconCodePoint("md-chevron-up-box", 0xF9DB);
  addIconCodePoint("md-chevron-up-box-outline", 0xF9DC);
  addIconCodePoint("md-chevron-up-circle", 0xFB11);
  addIconCodePoint("md-chevron-up-circle-outline", 0xFB12);
  addIconCodePoint("md-chili-hot", 0xF7B1);
  addIconCodePoint("md-chili-medium", 0xF7B2);
  addIconCodePoint("md-chili-mild", 0xF7B3);
  addIconCodePoint("md-chip", 0xF61A);
  addIconCodePoint("md-christianity", 0xF952);
  addIconCodePoint("md-christianity-outline", 0xFCD2);
  addIconCodePoint("md-church", 0xF144);
  addIconCodePoint("md-cigar", 0xF01B4);
  addIconCodePoint("md-circle", 0xF764);
  addIconCodePoint("md-circle-double", 0xFEB2);
  addIconCodePoint("md-circle-edit-outline", 0xF8D4);
  addIconCodePoint("md-circle-expand", 0xFEB3);
  addIconCodePoint("md-circle-medium", 0xF9DD);
  addIconCodePoint("md-circle-off-outline", 0xF00FE);
  addIconCodePoint("md-circle-outline", 0xF765);
  addIconCodePoint("md-circle-slice-1", 0xFA9D);
  addIconCodePoint("md-circle-slice-2", 0xFA9E);
  addIconCodePoint("md-circle-slice-3", 0xFA9F);
  addIconCodePoint("md-circle-slice-4", 0xFAA0);
  addIconCodePoint("md-circle-slice-5", 0xFAA1);
  addIconCodePoint("md-circle-slice-6", 0xFAA2);
  addIconCodePoint("md-circle-slice-7", 0xFAA3);
  addIconCodePoint("md-circle-slice-8", 0xFAA4);
  addIconCodePoint("md-circle-small", 0xF9DE);
  addIconCodePoint("md-circular-saw", 0xFE73);
  addIconCodePoint("md-cisco-webex", 0xF145);
  addIconCodePoint("md-city", 0xF146);
  addIconCodePoint("md-city-variant", 0xFA35);
  addIconCodePoint("md-city-variant-outline", 0xFA36);
  addIconCodePoint("md-clipboard", 0xF147);
  addIconCodePoint("md-clipboard-account", 0xF148);
  addIconCodePoint("md-clipboard-account-outline", 0xFC31);
  addIconCodePoint("md-clipboard-alert", 0xF149);
  addIconCodePoint("md-clipboard-alert-outline", 0xFCD3);
  addIconCodePoint("md-clipboard-arrow-down", 0xF14A);
  addIconCodePoint("md-clipboard-arrow-down-outline", 0xFC32);
  addIconCodePoint("md-clipboard-arrow-left", 0xF14B);
  addIconCodePoint("md-clipboard-arrow-left-outline", 0xFCD4);
  addIconCodePoint("md-clipboard-arrow-right", 0xFCD5);
  addIconCodePoint("md-clipboard-arrow-right-outline", 0xFCD6);
  addIconCodePoint("md-clipboard-arrow-up", 0xFC33);
  addIconCodePoint("md-clipboard-arrow-up-outline", 0xFC34);
  addIconCodePoint("md-clipboard-check", 0xF14C);
  addIconCodePoint("md-clipboard-check-multiple", 0xF028E);
  addIconCodePoint("md-clipboard-check-multiple-outline", 0xF028F);
  addIconCodePoint("md-clipboard-check-outline", 0xF8A7);
  addIconCodePoint("md-clipboard-file", 0xF0290);
  addIconCodePoint("md-clipboard-file-outline", 0xF0291);
  addIconCodePoint("md-clipboard-flow", 0xF6C7);
  addIconCodePoint("md-clipboard-flow-outline", 0xF0142);
  addIconCodePoint("md-clipboard-list", 0xF00FF);
  addIconCodePoint("md-clipboard-list-outline", 0xF0100);
  addIconCodePoint("md-clipboard-multiple", 0xF0292);
  addIconCodePoint("md-clipboard-multiple-outline", 0xF0293);
  addIconCodePoint("md-clipboard-outline", 0xF14D);
  addIconCodePoint("md-clipboard-play", 0xFC35);
  addIconCodePoint("md-clipboard-play-multiple", 0xF0294);
  addIconCodePoint("md-clipboard-play-multiple-outline", 0xF0295);
  addIconCodePoint("md-clipboard-play-outline", 0xFC36);
  addIconCodePoint("md-clipboard-plus", 0xF750);
  addIconCodePoint("md-clipboard-plus-outline", 0xF034A);
  addIconCodePoint("md-clipboard-pulse", 0xF85C);
  addIconCodePoint("md-clipboard-pulse-outline", 0xF85D);
  addIconCodePoint("md-clipboard-text", 0xF14E);
  addIconCodePoint("md-clipboard-text-multiple", 0xF0296);
  addIconCodePoint("md-clipboard-text-multiple-outline", 0xF0297);
  addIconCodePoint("md-clipboard-text-outline", 0xFA37);
  addIconCodePoint("md-clipboard-text-play", 0xFC37);
  addIconCodePoint("md-clipboard-text-play-outline", 0xFC38);
  addIconCodePoint("md-clippy", 0xF14F);
  addIconCodePoint("md-clock", 0xF953);
  addIconCodePoint("md-clock-alert", 0xF954);
  addIconCodePoint("md-clock-alert-outline", 0xF5CE);
  addIconCodePoint("md-clock-check", 0xFFC8);
  addIconCodePoint("md-clock-check-outline", 0xFFC9);
  addIconCodePoint("md-clock-digital", 0xFEB4);
  addIconCodePoint("md-clock-end", 0xF151);
  addIconCodePoint("md-clock-fast", 0xF152);
  addIconCodePoint("md-clock-in", 0xF153);
  addIconCodePoint("md-clock-out", 0xF154);
  addIconCodePoint("md-clock-outline", 0xF150);
  addIconCodePoint("md-clock-start", 0xF155);
  addIconCodePoint("md-close", 0xF156);
  addIconCodePoint("md-close-box", 0xF157);
  addIconCodePoint("md-close-box-multiple", 0xFC39);
  addIconCodePoint("md-close-box-multiple-outline", 0xFC3A);
  addIconCodePoint("md-close-box-outline", 0xF158);
  addIconCodePoint("md-close-circle", 0xF159);
  addIconCodePoint("md-close-circle-outline", 0xF15A);
  addIconCodePoint("md-close-network", 0xF15B);
  addIconCodePoint("md-close-network-outline", 0xFC3B);
  addIconCodePoint("md-close-octagon", 0xF15C);
  addIconCodePoint("md-close-octagon-outline", 0xF15D);
  addIconCodePoint("md-close-outline", 0xF6C8);
  addIconCodePoint("md-closed-caption", 0xF15E);
  addIconCodePoint("md-closed-caption-outline", 0xFD99);
  addIconCodePoint("md-cloud", 0xF15F);
  addIconCodePoint("md-cloud-alert", 0xF9DF);
  addIconCodePoint("md-cloud-braces", 0xF7B4);
  addIconCodePoint("md-cloud-check", 0xF160);
  addIconCodePoint("md-cloud-check-outline", 0xF02F7);
  addIconCodePoint("md-cloud-circle", 0xF161);
  addIconCodePoint("md-cloud-download", 0xF162);
  addIconCodePoint("md-cloud-download-outline", 0xFB59);
  addIconCodePoint("md-cloud-lock", 0xF021C);
  addIconCodePoint("md-cloud-lock-outline", 0xF021D);
  addIconCodePoint("md-cloud-off-outline", 0xF164);
  addIconCodePoint("md-cloud-outline", 0xF163);
  addIconCodePoint("md-cloud-print", 0xF165);
  addIconCodePoint("md-cloud-print-outline", 0xF166);
  addIconCodePoint("md-cloud-question", 0xFA38);
  addIconCodePoint("md-cloud-search", 0xF955);
  addIconCodePoint("md-cloud-search-outline", 0xF956);
  addIconCodePoint("md-cloud-sync", 0xF63F);
  addIconCodePoint("md-cloud-sync-outline", 0xF0301);
  addIconCodePoint("md-cloud-tags", 0xF7B5);
  addIconCodePoint("md-cloud-upload", 0xF167);
  addIconCodePoint("md-cloud-upload-outline", 0xFB5A);
  addIconCodePoint("md-clover", 0xF815);
  addIconCodePoint("md-coach-lamp", 0xF0042);
  addIconCodePoint("md-coat-rack", 0xF00C9);
  addIconCodePoint("md-code-array", 0xF168);
  addIconCodePoint("md-code-braces", 0xF169);
  addIconCodePoint("md-code-braces-box", 0xF0101);
  addIconCodePoint("md-code-brackets", 0xF16A);
  addIconCodePoint("md-code-equal", 0xF16B);
  addIconCodePoint("md-code-greater-than", 0xF16C);
  addIconCodePoint("md-code-greater-than-or-equal", 0xF16D);
  addIconCodePoint("md-code-less-than", 0xF16E);
  addIconCodePoint("md-code-less-than-or-equal", 0xF16F);
  addIconCodePoint("md-code-not-equal", 0xF170);
  addIconCodePoint("md-code-not-equal-variant", 0xF171);
  addIconCodePoint("md-code-parentheses", 0xF172);
  addIconCodePoint("md-code-parentheses-box", 0xF0102);
  addIconCodePoint("md-code-string", 0xF173);
  addIconCodePoint("md-code-tags", 0xF174);
  addIconCodePoint("md-code-tags-check", 0xF693);
  addIconCodePoint("md-codepen", 0xF175);
  addIconCodePoint("md-coffee", 0xF176);
  addIconCodePoint("md-coffee-maker", 0xF00CA);
  addIconCodePoint("md-coffee-off", 0xFFCA);
  addIconCodePoint("md-coffee-off-outline", 0xFFCB);
  addIconCodePoint("md-coffee-outline", 0xF6C9);
  addIconCodePoint("md-coffee-to-go", 0xF177);
  addIconCodePoint("md-coffee-to-go-outline", 0xF0339);
  addIconCodePoint("md-coffin", 0xFB5B);
  addIconCodePoint("md-cog-clockwise", 0xF0208);
  addIconCodePoint("md-cog-counterclockwise", 0xF0209);
  addIconCodePoint("md-cogs", 0xF8D5);
  addIconCodePoint("md-coin", 0xF0196);
  addIconCodePoint("md-coin-outline", 0xF178);
  addIconCodePoint("md-coins", 0xF694);
  addIconCodePoint("md-collage", 0xF640);
  addIconCodePoint("md-collapse-all", 0xFAA5);
  addIconCodePoint("md-collapse-all-outline", 0xFAA6);
  addIconCodePoint("md-color-helper", 0xF179);
  addIconCodePoint("md-comma", 0xFE74);
  addIconCodePoint("md-comma-box", 0xFE75);
  addIconCodePoint("md-comma-box-outline", 0xFE76);
  addIconCodePoint("md-comma-circle", 0xFE77);
  addIconCodePoint("md-comma-circle-outline", 0xFE78);
  addIconCodePoint("md-comment", 0xF17A);
  addIconCodePoint("md-comment-account", 0xF17B);
  addIconCodePoint("md-comment-account-outline", 0xF17C);
  addIconCodePoint("md-comment-alert", 0xF17D);
  addIconCodePoint("md-comment-alert-outline", 0xF17E);
  addIconCodePoint("md-comment-arrow-left", 0xF9E0);
  addIconCodePoint("md-comment-arrow-left-outline", 0xF9E1);
  addIconCodePoint("md-comment-arrow-right", 0xF9E2);
  addIconCodePoint("md-comment-arrow-right-outline", 0xF9E3);
  addIconCodePoint("md-comment-check", 0xF17F);
  addIconCodePoint("md-comment-check-outline", 0xF180);
  addIconCodePoint("md-comment-edit", 0xF01EA);
  addIconCodePoint("md-comment-edit-outline", 0xF02EF);
  addIconCodePoint("md-comment-eye", 0xFA39);
  addIconCodePoint("md-comment-eye-outline", 0xFA3A);
  addIconCodePoint("md-comment-multiple", 0xF85E);
  addIconCodePoint("md-comment-multiple-outline", 0xF181);
  addIconCodePoint("md-comment-outline", 0xF182);
  addIconCodePoint("md-comment-plus", 0xF9E4);
  addIconCodePoint("md-comment-plus-outline", 0xF183);
  addIconCodePoint("md-comment-processing", 0xF184);
  addIconCodePoint("md-comment-processing-outline", 0xF185);
  addIconCodePoint("md-comment-question", 0xF816);
  addIconCodePoint("md-comment-question-outline", 0xF186);
  addIconCodePoint("md-comment-quote", 0xF0043);
  addIconCodePoint("md-comment-quote-outline", 0xF0044);
  addIconCodePoint("md-comment-remove", 0xF5DE);
  addIconCodePoint("md-comment-remove-outline", 0xF187);
  addIconCodePoint("md-comment-search", 0xFA3B);
  addIconCodePoint("md-comment-search-outline", 0xFA3C);
  addIconCodePoint("md-comment-text", 0xF188);
  addIconCodePoint("md-comment-text-multiple", 0xF85F);
  addIconCodePoint("md-comment-text-multiple-outline", 0xF860);
  addIconCodePoint("md-comment-text-outline", 0xF189);
  addIconCodePoint("md-compare", 0xF18A);
  addIconCodePoint("md-compass", 0xF18B);
  addIconCodePoint("md-compass-off", 0xFB5C);
  addIconCodePoint("md-compass-off-outline", 0xFB5D);
  addIconCodePoint("md-compass-outline", 0xF18C);
  addIconCodePoint("md-compass-rose", 0xF03AD);
  addIconCodePoint("md-concourse-ci", 0xF00CB);
  addIconCodePoint("md-console", 0xF18D);
  addIconCodePoint("md-console-line", 0xF7B6);
  addIconCodePoint("md-console-network", 0xF8A8);
  addIconCodePoint("md-console-network-outline", 0xFC3C);
  addIconCodePoint("md-consolidate", 0xF0103);
  addIconCodePoint("md-contact-mail", 0xF18E);
  addIconCodePoint("md-contact-mail-outline", 0xFEB5);
  addIconCodePoint("md-contact-phone", 0xFEB6);
  addIconCodePoint("md-contact-phone-outline", 0xFEB7);
  addIconCodePoint("md-contactless-payment", 0xFD46);
  addIconCodePoint("md-contacts", 0xF6CA);
  addIconCodePoint("md-contain", 0xFA3D);
  addIconCodePoint("md-contain-end", 0xFA3E);
  addIconCodePoint("md-contain-start", 0xFA3F);
  addIconCodePoint("md-content-copy", 0xF18F);
  addIconCodePoint("md-content-cut", 0xF190);
  addIconCodePoint("md-content-duplicate", 0xF191);
  addIconCodePoint("md-content-paste", 0xF192);
  addIconCodePoint("md-content-save", 0xF193);
  addIconCodePoint("md-content-save-alert", 0xFF5F);
  addIconCodePoint("md-content-save-alert-outline", 0xFF60);
  addIconCodePoint("md-content-save-all", 0xF194);
  addIconCodePoint("md-content-save-all-outline", 0xFF61);
  addIconCodePoint("md-content-save-edit", 0xFCD7);
  addIconCodePoint("md-content-save-edit-outline", 0xFCD8);
  addIconCodePoint("md-content-save-move", 0xFE79);
  addIconCodePoint("md-content-save-move-outline", 0xFE7A);
  addIconCodePoint("md-content-save-outline", 0xF817);
  addIconCodePoint("md-content-save-settings", 0xF61B);
  addIconCodePoint("md-content-save-settings-outline", 0xFB13);
  addIconCodePoint("md-contrast", 0xF195);
  addIconCodePoint("md-contrast-box", 0xF196);
  addIconCodePoint("md-contrast-circle", 0xF197);
  addIconCodePoint("md-controller-classic", 0xFB5E);
  addIconCodePoint("md-controller-classic-outline", 0xFB5F);
  addIconCodePoint("md-cookie", 0xF198);
  addIconCodePoint("md-coolant-temperature", 0xF3C8);
  addIconCodePoint("md-copyright", 0xF5E6);
  addIconCodePoint("md-cordova", 0xF957);
  addIconCodePoint("md-corn", 0xF7B7);
  addIconCodePoint("md-counter", 0xF199);
  addIconCodePoint("md-cow", 0xF19A);
  addIconCodePoint("md-cowboy", 0xFEB8);
  addIconCodePoint("md-cpu-32-bit", 0xFEFC);
  addIconCodePoint("md-cpu-64-bit", 0xFEFD);
  addIconCodePoint("md-crane", 0xF861);
  addIconCodePoint("md-creation", 0xF1C9);
  addIconCodePoint("md-creative-commons", 0xFD47);
  addIconCodePoint("md-credit-card", 0xF0010);
  addIconCodePoint("md-credit-card-clock", 0xFEFE);
  addIconCodePoint("md-credit-card-clock-outline", 0xFFBC);
  addIconCodePoint("md-credit-card-marker", 0xF6A7);
  addIconCodePoint("md-credit-card-marker-outline", 0xFD9A);
  addIconCodePoint("md-credit-card-minus", 0xFFCC);
  addIconCodePoint("md-credit-card-minus-outline", 0xFFCD);
  addIconCodePoint("md-credit-card-multiple", 0xF0011);
  addIconCodePoint("md-credit-card-multiple-outline", 0xF19C);
  addIconCodePoint("md-credit-card-off", 0xF0012);
  addIconCodePoint("md-credit-card-off-outline", 0xF5E4);
  addIconCodePoint("md-credit-card-outline", 0xF19B);
  addIconCodePoint("md-credit-card-plus", 0xF0013);
  addIconCodePoint("md-credit-card-plus-outline", 0xF675);
  addIconCodePoint("md-credit-card-refund", 0xF0014);
  addIconCodePoint("md-credit-card-refund-outline", 0xFAA7);
  addIconCodePoint("md-credit-card-remove", 0xFFCE);
  addIconCodePoint("md-credit-card-remove-outline", 0xFFCF);
  addIconCodePoint("md-credit-card-scan", 0xF0015);
  addIconCodePoint("md-credit-card-scan-outline", 0xF19D);
  addIconCodePoint("md-credit-card-settings", 0xF0016);
  addIconCodePoint("md-credit-card-settings-outline", 0xF8D6);
  addIconCodePoint("md-credit-card-wireless", 0xF801);
  addIconCodePoint("md-credit-card-wireless-outline", 0xFD48);
  addIconCodePoint("md-cricket", 0xFD49);
  addIconCodePoint("md-crop", 0xF19E);
  addIconCodePoint("md-crop-free", 0xF19F);
  addIconCodePoint("md-crop-landscape", 0xF1A0);
  addIconCodePoint("md-crop-portrait", 0xF1A1);
  addIconCodePoint("md-crop-rotate", 0xF695);
  addIconCodePoint("md-crop-square", 0xF1A2);
  addIconCodePoint("md-crosshairs", 0xF1A3);
  addIconCodePoint("md-crosshairs-gps", 0xF1A4);
  addIconCodePoint("md-crosshairs-off", 0xFF62);
  addIconCodePoint("md-crosshairs-question", 0xF0161);
  addIconCodePoint("md-crown", 0xF1A5);
  addIconCodePoint("md-crown-outline", 0xF01FB);
  addIconCodePoint("md-cryengine", 0xF958);
  addIconCodePoint("md-crystal-ball", 0xFB14);
  addIconCodePoint("md-cube", 0xF1A6);
  addIconCodePoint("md-cube-outline", 0xF1A7);
  addIconCodePoint("md-cube-scan", 0xFB60);
  addIconCodePoint("md-cube-send", 0xF1A8);
  addIconCodePoint("md-cube-unfolded", 0xF1A9);
  addIconCodePoint("md-cup", 0xF1AA);
  addIconCodePoint("md-cup-off", 0xF5E5);
  addIconCodePoint("md-cup-off-outline", 0xF03A8);
  addIconCodePoint("md-cup-outline", 0xF033A);
  addIconCodePoint("md-cup-water", 0xF1AB);
  addIconCodePoint("md-cupboard", 0xFF63);
  addIconCodePoint("md-cupboard-outline", 0xFF64);
  addIconCodePoint("md-cupcake", 0xF959);
  addIconCodePoint("md-curling", 0xF862);
  addIconCodePoint("md-currency-bdt", 0xF863);
  addIconCodePoint("md-currency-brl", 0xFB61);
  addIconCodePoint("md-currency-btc", 0xF1AC);
  addIconCodePoint("md-currency-cny", 0xF7B9);
  addIconCodePoint("md-currency-eth", 0xF7BA);
  addIconCodePoint("md-currency-eur", 0xF1AD);
  addIconCodePoint("md-currency-eur-off", 0xF0340);
  addIconCodePoint("md-currency-gbp", 0xF1AE);
  addIconCodePoint("md-currency-ils", 0xFC3D);
  addIconCodePoint("md-currency-inr", 0xF1AF);
  addIconCodePoint("md-currency-jpy", 0xF7BB);
  addIconCodePoint("md-currency-krw", 0xF7BC);
  addIconCodePoint("md-currency-kzt", 0xF864);
  addIconCodePoint("md-currency-ngn", 0xF1B0);
  addIconCodePoint("md-currency-php", 0xF9E5);
  addIconCodePoint("md-currency-rial", 0xFEB9);
  addIconCodePoint("md-currency-rub", 0xF1B1);
  addIconCodePoint("md-currency-sign", 0xF7BD);
  addIconCodePoint("md-currency-try", 0xF1B2);
  addIconCodePoint("md-currency-twd", 0xF7BE);
  addIconCodePoint("md-currency-usd", 0xF1B3);
  addIconCodePoint("md-currency-usd-off", 0xF679);
  addIconCodePoint("md-current-ac", 0xF95A);
  addIconCodePoint("md-current-dc", 0xF95B);
  addIconCodePoint("md-cursor-default", 0xF1B4);
  addIconCodePoint("md-cursor-default-click", 0xFCD9);
  addIconCodePoint("md-cursor-default-click-outline", 0xFCDA);
  addIconCodePoint("md-cursor-default-gesture", 0xF0152);
  addIconCodePoint("md-cursor-default-gesture-outline", 0xF0153);
  addIconCodePoint("md-cursor-default-outline", 0xF1B5);
  addIconCodePoint("md-cursor-move", 0xF1B6);
  addIconCodePoint("md-cursor-pointer", 0xF1B7);
  addIconCodePoint("md-cursor-text", 0xF5E7);
  addIconCodePoint("md-database", 0xF1B8);
  addIconCodePoint("md-database-check", 0xFAA8);
  addIconCodePoint("md-database-edit", 0xFB62);
  addIconCodePoint("md-database-export", 0xF95D);
  addIconCodePoint("md-database-import", 0xF95C);
  addIconCodePoint("md-database-lock", 0xFAA9);
  addIconCodePoint("md-database-marker", 0xF0321);
  addIconCodePoint("md-database-minus", 0xF1B9);
  addIconCodePoint("md-database-plus", 0xF1BA);
  addIconCodePoint("md-database-refresh", 0xFCDB);
  addIconCodePoint("md-database-remove", 0xFCDC);
  addIconCodePoint("md-database-search", 0xF865);
  addIconCodePoint("md-database-settings", 0xFCDD);
  addIconCodePoint("md-death-star", 0xF8D7);
  addIconCodePoint("md-death-star-variant", 0xF8D8);
  addIconCodePoint("md-deathly-hallows", 0xFB63);
  addIconCodePoint("md-debian", 0xF8D9);
  addIconCodePoint("md-debug-step-into", 0xF1BB);
  addIconCodePoint("md-debug-step-out", 0xF1BC);
  addIconCodePoint("md-debug-step-over", 0xF1BD);
  addIconCodePoint("md-decagram", 0xF76B);
  addIconCodePoint("md-decagram-outline", 0xF76C);
  addIconCodePoint("md-decimal", 0xF00CC);
  addIconCodePoint("md-decimal-comma", 0xF00CD);
  addIconCodePoint("md-decimal-comma-decrease", 0xF00CE);
  addIconCodePoint("md-decimal-comma-increase", 0xF00CF);
  addIconCodePoint("md-decimal-decrease", 0xF1BE);
  addIconCodePoint("md-decimal-increase", 0xF1BF);
  addIconCodePoint("md-delete", 0xF1C0);
  addIconCodePoint("md-delete-alert", 0xF00D0);
  addIconCodePoint("md-delete-alert-outline", 0xF00D1);
  addIconCodePoint("md-delete-circle", 0xF682);
  addIconCodePoint("md-delete-circle-outline", 0xFB64);
  addIconCodePoint("md-delete-empty", 0xF6CB);
  addIconCodePoint("md-delete-empty-outline", 0xFEBA);
  addIconCodePoint("md-delete-forever", 0xF5E8);
  addIconCodePoint("md-delete-forever-outline", 0xFB65);
  addIconCodePoint("md-delete-off", 0xF00D2);
  addIconCodePoint("md-delete-off-outline", 0xF00D3);
  addIconCodePoint("md-delete-outline", 0xF9E6);
  addIconCodePoint("md-delete-restore", 0xF818);
  addIconCodePoint("md-delete-sweep", 0xF5E9);
  addIconCodePoint("md-delete-sweep-outline", 0xFC3E);
  addIconCodePoint("md-delete-variant", 0xF1C1);
  addIconCodePoint("md-delta", 0xF1C2);
  addIconCodePoint("md-desk", 0xF0264);
  addIconCodePoint("md-desk-lamp", 0xF95E);
  addIconCodePoint("md-deskphone", 0xF1C3);
  addIconCodePoint("md-desktop-classic", 0xF7BF);
  addIconCodePoint("md-desktop-mac", 0xF1C4);
  addIconCodePoint("md-desktop-mac-dashboard", 0xF9E7);
  addIconCodePoint("md-desktop-tower", 0xF1C5);
  addIconCodePoint("md-desktop-tower-monitor", 0xFAAA);
  addIconCodePoint("md-details", 0xF1C6);
  addIconCodePoint("md-dev-to", 0xFD4A);
  addIconCodePoint("md-developer-board", 0xF696);
  addIconCodePoint("md-deviantart", 0xF1C7);
  addIconCodePoint("md-devices", 0xFFD0);
  addIconCodePoint("md-diabetes", 0xF0151);
  addIconCodePoint("md-dialpad", 0xF61C);
  addIconCodePoint("md-diameter", 0xFC3F);
  addIconCodePoint("md-diameter-outline", 0xFC40);
  addIconCodePoint("md-diameter-variant", 0xFC41);
  addIconCodePoint("md-diamond", 0xFB66);
  addIconCodePoint("md-diamond-outline", 0xFB67);
  addIconCodePoint("md-diamond-stone", 0xF1C8);
  addIconCodePoint("md-dice-1", 0xF1CA);
  addIconCodePoint("md-dice-1-outline", 0xF0175);
  addIconCodePoint("md-dice-2", 0xF1CB);
  addIconCodePoint("md-dice-2-outline", 0xF0176);
  addIconCodePoint("md-dice-3", 0xF1CC);
  addIconCodePoint("md-dice-3-outline", 0xF0177);
  addIconCodePoint("md-dice-4", 0xF1CD);
  addIconCodePoint("md-dice-4-outline", 0xF0178);
  addIconCodePoint("md-dice-5", 0xF1CE);
  addIconCodePoint("md-dice-5-outline", 0xF0179);
  addIconCodePoint("md-dice-6", 0xF1CF);
  addIconCodePoint("md-dice-6-outline", 0xF017A);
  addIconCodePoint("md-dice-d10", 0xF017E);
  addIconCodePoint("md-dice-d10-outline", 0xF76E);
  addIconCodePoint("md-dice-d12", 0xF017F);
  addIconCodePoint("md-dice-d12-outline", 0xF866);
  addIconCodePoint("md-dice-d20", 0xF0180);
  addIconCodePoint("md-dice-d20-outline", 0xF5EA);
  addIconCodePoint("md-dice-d4", 0xF017B);
  addIconCodePoint("md-dice-d4-outline", 0xF5EB);
  addIconCodePoint("md-dice-d6", 0xF017C);
  addIconCodePoint("md-dice-d6-outline", 0xF5EC);
  addIconCodePoint("md-dice-d8", 0xF017D);
  addIconCodePoint("md-dice-d8-outline", 0xF5ED);
  addIconCodePoint("md-dice-multiple", 0xF76D);
  addIconCodePoint("md-dice-multiple-outline", 0xF0181);
  addIconCodePoint("md-dictionary", 0xF61D);
  addIconCodePoint("md-digital-ocean", 0xF0262);
  addIconCodePoint("md-dip-switch", 0xF7C0);
  addIconCodePoint("md-directions", 0xF1D0);
  addIconCodePoint("md-directions-fork", 0xF641);
  addIconCodePoint("md-disc", 0xF5EE);
  addIconCodePoint("md-disc-alert", 0xF1D1);
  addIconCodePoint("md-disc-player", 0xF95F);
  addIconCodePoint("md-discord", 0xF66F);
  addIconCodePoint("md-dishwasher", 0xFAAB);
  addIconCodePoint("md-dishwasher-alert", 0xF01E3);
  addIconCodePoint("md-dishwasher-off", 0xF01E4);
  addIconCodePoint("md-disqus", 0xF1D2);
  addIconCodePoint("md-disqus-outline", 0xF1D3);
  addIconCodePoint("md-distribute-horizontal-center", 0xF01F4);
  addIconCodePoint("md-distribute-horizontal-left", 0xF01F3);
  addIconCodePoint("md-distribute-horizontal-right", 0xF01F5);
  addIconCodePoint("md-distribute-vertical-bottom", 0xF01F6);
  addIconCodePoint("md-distribute-vertical-center", 0xF01F7);
  addIconCodePoint("md-distribute-vertical-top", 0xF01F8);
  addIconCodePoint("md-diving-flippers", 0xFD9B);
  addIconCodePoint("md-diving-helmet", 0xFD9C);
  addIconCodePoint("md-diving-scuba", 0xFD9D);
  addIconCodePoint("md-diving-scuba-flag", 0xFD9E);
  addIconCodePoint("md-diving-scuba-tank", 0xFD9F);
  addIconCodePoint("md-diving-scuba-tank-multiple", 0xFDA0);
  addIconCodePoint("md-diving-snorkel", 0xFDA1);
  addIconCodePoint("md-division", 0xF1D4);
  addIconCodePoint("md-division-box", 0xF1D5);
  addIconCodePoint("md-dlna", 0xFA40);
  addIconCodePoint("md-dna", 0xF683);
  addIconCodePoint("md-dns", 0xF1D6);
  addIconCodePoint("md-dns-outline", 0xFB68);
  addIconCodePoint("md-do-not-disturb", 0xF697);
  addIconCodePoint("md-do-not-disturb-off", 0xF698);
  addIconCodePoint("md-dock-bottom", 0xF00D4);
  addIconCodePoint("md-dock-left", 0xF00D5);
  addIconCodePoint("md-dock-right", 0xF00D6);
  addIconCodePoint("md-dock-window", 0xF00D7);
  addIconCodePoint("md-docker", 0xF867);
  addIconCodePoint("md-doctor", 0xFA41);
  addIconCodePoint("md-dog", 0xFA42);
  addIconCodePoint("md-dog-service", 0xFAAC);
  addIconCodePoint("md-dog-side", 0xFA43);
  addIconCodePoint("md-dolby", 0xF6B2);
  addIconCodePoint("md-dolly", 0xFEBB);
  addIconCodePoint("md-domain", 0xF1D7);
  addIconCodePoint("md-domain-off", 0xFD4B);
  addIconCodePoint("md-domain-plus", 0xF00D8);
  addIconCodePoint("md-domain-remove", 0xF00D9);
  addIconCodePoint("md-domino-mask", 0xF0045);
  addIconCodePoint("md-donkey", 0xF7C1);
  addIconCodePoint("md-door", 0xF819);
  addIconCodePoint("md-door-closed", 0xF81A);
  addIconCodePoint("md-door-closed-lock", 0xF00DA);
  addIconCodePoint("md-door-open", 0xF81B);
  addIconCodePoint("md-doorbell", 0xF0311);
  addIconCodePoint("md-doorbell-video", 0xF868);
  addIconCodePoint("md-dot-net", 0xFAAD);
  addIconCodePoint("md-dots-horizontal", 0xF1D8);
  addIconCodePoint("md-dots-horizontal-circle", 0xF7C2);
  addIconCodePoint("md-dots-horizontal-circle-outline", 0xFB69);
  addIconCodePoint("md-dots-vertical", 0xF1D9);
  addIconCodePoint("md-dots-vertical-circle", 0xF7C3);
  addIconCodePoint("md-dots-vertical-circle-outline", 0xFB6A);
  addIconCodePoint("md-douban", 0xF699);
  addIconCodePoint("md-download", 0xF1DA);
  addIconCodePoint("md-download-lock", 0xF034B);
  addIconCodePoint("md-download-lock-outline", 0xF034C);
  addIconCodePoint("md-download-multiple", 0xF9E8);
  addIconCodePoint("md-download-network", 0xF6F3);
  addIconCodePoint("md-download-network-outline", 0xFC42);
  addIconCodePoint("md-download-off", 0xF00DB);
  addIconCodePoint("md-download-off-outline", 0xF00DC);
  addIconCodePoint("md-download-outline", 0xFB6B);
  addIconCodePoint("md-drag", 0xF1DB);
  addIconCodePoint("md-drag-horizontal", 0xF1DC);
  addIconCodePoint("md-drag-horizontal-variant", 0xF031B);
  addIconCodePoint("md-drag-variant", 0xFB6C);
  addIconCodePoint("md-drag-vertical", 0xF1DD);
  addIconCodePoint("md-drag-vertical-variant", 0xF031C);
  addIconCodePoint("md-drama-masks", 0xFCDE);
  addIconCodePoint("md-draw", 0xFF66);
  addIconCodePoint("md-drawing", 0xF1DE);
  addIconCodePoint("md-drawing-box", 0xF1DF);
  addIconCodePoint("md-dresser", 0xFF67);
  addIconCodePoint("md-dresser-outline", 0xFF68);
  addIconCodePoint("md-dribbble", 0xF1E0);
  addIconCodePoint("md-dribbble-box", 0xF1E1);
  addIconCodePoint("md-drone", 0xF1E2);
  addIconCodePoint("md-dropbox", 0xF1E3);
  addIconCodePoint("md-drupal", 0xF1E4);
  addIconCodePoint("md-duck", 0xF1E5);
  addIconCodePoint("md-dumbbell", 0xF1E6);
  addIconCodePoint("md-dump-truck", 0xFC43);
  addIconCodePoint("md-ear-hearing", 0xF7C4);
  addIconCodePoint("md-ear-hearing-off", 0xFA44);
  addIconCodePoint("md-earth", 0xF1E7);
  addIconCodePoint("md-earth-arrow-right", 0xF033C);
  addIconCodePoint("md-earth-box", 0xF6CC);
  addIconCodePoint("md-earth-box-off", 0xF6CD);
  addIconCodePoint("md-earth-off", 0xF1E8);
  addIconCodePoint("md-edge", 0xF1E9);
  addIconCodePoint("md-edge-legacy", 0xF027B);
  addIconCodePoint("md-egg", 0xFAAE);
  addIconCodePoint("md-egg-easter", 0xFAAF);
  addIconCodePoint("md-eight-track", 0xF9E9);
  addIconCodePoint("md-eject", 0xF1EA);
  addIconCodePoint("md-eject-outline", 0xFB6D);
  addIconCodePoint("md-electric-switch", 0xFEBC);
  addIconCodePoint("md-electric-switch-closed", 0xF0104);
  addIconCodePoint("md-electron-framework", 0xF0046);
  addIconCodePoint("md-elephant", 0xF7C5);
  addIconCodePoint("md-elevation-decline", 0xF1EB);
  addIconCodePoint("md-elevation-rise", 0xF1EC);
  addIconCodePoint("md-elevator", 0xF1ED);
  addIconCodePoint("md-elevator-down", 0xF02ED);
  addIconCodePoint("md-elevator-passenger", 0xF03AC);
  addIconCodePoint("md-elevator-up", 0xF02EC);
  addIconCodePoint("md-ellipse", 0xFEBD);
  addIconCodePoint("md-ellipse-outline", 0xFEBE);
  addIconCodePoint("md-email", 0xF1EE);
  addIconCodePoint("md-email-alert", 0xF6CE);
  addIconCodePoint("md-email-alert-outline", 0xFD1E);
  addIconCodePoint("md-email-box", 0xFCDF);
  addIconCodePoint("md-email-check", 0xFAB0);
  addIconCodePoint("md-email-check-outline", 0xFAB1);
  addIconCodePoint("md-email-edit", 0xFF00);
  addIconCodePoint("md-email-edit-outline", 0xFF01);
  addIconCodePoint("md-email-lock", 0xF1F1);
  addIconCodePoint("md-email-mark-as-unread", 0xFB6E);
  addIconCodePoint("md-email-minus", 0xFF02);
  addIconCodePoint("md-email-minus-outline", 0xFF03);
  addIconCodePoint("md-email-multiple", 0xFF04);
  addIconCodePoint("md-email-multiple-outline", 0xFF05);
  addIconCodePoint("md-email-newsletter", 0xFFD1);
  addIconCodePoint("md-email-open", 0xF1EF);
  addIconCodePoint("md-email-open-multiple", 0xFF06);
  addIconCodePoint("md-email-open-multiple-outline", 0xFF07);
  addIconCodePoint("md-email-open-outline", 0xF5EF);
  addIconCodePoint("md-email-outline", 0xF1F0);
  addIconCodePoint("md-email-plus", 0xF9EA);
  addIconCodePoint("md-email-plus-outline", 0xF9EB);
  addIconCodePoint("md-email-receive", 0xF0105);
  addIconCodePoint("md-email-receive-outline", 0xF0106);
  addIconCodePoint("md-email-search", 0xF960);
  addIconCodePoint("md-email-search-outline", 0xF961);
  addIconCodePoint("md-email-send", 0xF0107);
  addIconCodePoint("md-email-send-outline", 0xF0108);
  addIconCodePoint("md-email-sync", 0xF02F2);
  addIconCodePoint("md-email-sync-outline", 0xF02F3);
  addIconCodePoint("md-email-variant", 0xF5F0);
  addIconCodePoint("md-ember", 0xFB15);
  addIconCodePoint("md-emby", 0xF6B3);
  addIconCodePoint("md-emoticon", 0xFC44);
  addIconCodePoint("md-emoticon-angry", 0xFC45);
  addIconCodePoint("md-emoticon-angry-outline", 0xFC46);
  addIconCodePoint("md-emoticon-confused", 0xF0109);
  addIconCodePoint("md-emoticon-confused-outline", 0xF010A);
  addIconCodePoint("md-emoticon-cool", 0xFC47);
  addIconCodePoint("md-emoticon-cool-outline", 0xF1F3);
  addIconCodePoint("md-emoticon-cry", 0xFC48);
  addIconCodePoint("md-emoticon-cry-outline", 0xFC49);
  addIconCodePoint("md-emoticon-dead", 0xFC4A);
  addIconCodePoint("md-emoticon-dead-outline", 0xF69A);
  addIconCodePoint("md-emoticon-devil", 0xFC4B);
  addIconCodePoint("md-emoticon-devil-outline", 0xF1F4);
  addIconCodePoint("md-emoticon-excited", 0xFC4C);
  addIconCodePoint("md-emoticon-excited-outline", 0xF69B);
  addIconCodePoint("md-emoticon-frown", 0xFF69);
  addIconCodePoint("md-emoticon-frown-outline", 0xFF6A);
  addIconCodePoint("md-emoticon-happy", 0xFC4D);
  addIconCodePoint("md-emoticon-happy-outline", 0xF1F5);
  addIconCodePoint("md-emoticon-kiss", 0xFC4E);
  addIconCodePoint("md-emoticon-kiss-outline", 0xFC4F);
  addIconCodePoint("md-emoticon-lol", 0xF023F);
  addIconCodePoint("md-emoticon-lol-outline", 0xF0240);
  addIconCodePoint("md-emoticon-neutral", 0xFC50);
  addIconCodePoint("md-emoticon-neutral-outline", 0xF1F6);
  addIconCodePoint("md-emoticon-outline", 0xF1F2);
  addIconCodePoint("md-emoticon-poop", 0xF1F7);
  addIconCodePoint("md-emoticon-poop-outline", 0xFC51);
  addIconCodePoint("md-emoticon-sad", 0xFC52);
  addIconCodePoint("md-emoticon-sad-outline", 0xF1F8);
  addIconCodePoint("md-emoticon-tongue", 0xF1F9);
  addIconCodePoint("md-emoticon-tongue-outline", 0xFC53);
  addIconCodePoint("md-emoticon-wink", 0xFC54);
  addIconCodePoint("md-emoticon-wink-outline", 0xFC55);
  addIconCodePoint("md-engine", 0xF1FA);
  addIconCodePoint("md-engine-off", 0xFA45);
  addIconCodePoint("md-engine-off-outline", 0xFA46);
  addIconCodePoint("md-engine-outline", 0xF1FB);
  addIconCodePoint("md-epsilon", 0xF010B);
  addIconCodePoint("md-equal", 0xF1FC);
  addIconCodePoint("md-equal-box", 0xF1FD);
  addIconCodePoint("md-equalizer", 0xFEBF);
  addIconCodePoint("md-equalizer-outline", 0xFEC0);
  addIconCodePoint("md-eraser", 0xF1FE);
  addIconCodePoint("md-eraser-variant", 0xF642);
  addIconCodePoint("md-escalator", 0xF1FF);
  addIconCodePoint("md-escalator-down", 0xF02EB);
  addIconCodePoint("md-escalator-up", 0xF02EA);
  addIconCodePoint("md-eslint", 0xFC56);
  addIconCodePoint("md-et", 0xFAB2);
  addIconCodePoint("md-ethereum", 0xF869);
  addIconCodePoint("md-ethernet", 0xF200);
  addIconCodePoint("md-ethernet-cable", 0xF201);
  addIconCodePoint("md-ethernet-cable-off", 0xF202);
  addIconCodePoint("md-etsy", 0xF203);
  addIconCodePoint("md-ev-station", 0xF5F1);
  addIconCodePoint("md-eventbrite", 0xF7C6);
  addIconCodePoint("md-evernote", 0xF204);
  addIconCodePoint("md-excavator", 0xF0047);
  addIconCodePoint("md-exclamation", 0xF205);
  addIconCodePoint("md-exclamation-thick", 0xF0263);
  addIconCodePoint("md-exit-run", 0xFA47);
  addIconCodePoint("md-exit-to-app", 0xF206);
  addIconCodePoint("md-expand-all", 0xFAB3);
  addIconCodePoint("md-expand-all-outline", 0xFAB4);
  addIconCodePoint("md-expansion-card", 0xF8AD);
  addIconCodePoint("md-expansion-card-variant", 0xFFD2);
  addIconCodePoint("md-exponent", 0xF962);
  addIconCodePoint("md-exponent-box", 0xF963);
  addIconCodePoint("md-export", 0xF207);
  addIconCodePoint("md-export-variant", 0xFB6F);
  addIconCodePoint("md-eye", 0xF208);
  addIconCodePoint("md-eye-check", 0xFCE0);
  addIconCodePoint("md-eye-check-outline", 0xFCE1);
  addIconCodePoint("md-eye-circle", 0xFB70);
  addIconCodePoint("md-eye-circle-outline", 0xFB71);
  addIconCodePoint("md-eye-minus", 0xF0048);
  addIconCodePoint("md-eye-minus-outline", 0xF0049);
  addIconCodePoint("md-eye-off", 0xF209);
  addIconCodePoint("md-eye-off-outline", 0xF6D0);
  addIconCodePoint("md-eye-outline", 0xF6CF);
  addIconCodePoint("md-eye-plus", 0xF86A);
  addIconCodePoint("md-eye-plus-outline", 0xF86B);
  addIconCodePoint("md-eye-settings", 0xF86C);
  addIconCodePoint("md-eye-settings-outline", 0xF86D);
  addIconCodePoint("md-eyedropper", 0xF20A);
  addIconCodePoint("md-eyedropper-variant", 0xF20B);
  addIconCodePoint("md-face", 0xF643);
  addIconCodePoint("md-face-agent", 0xFD4C);
  addIconCodePoint("md-face-outline", 0xFB72);
  addIconCodePoint("md-face-profile", 0xF644);
  addIconCodePoint("md-face-profile-woman", 0xF00A1);
  addIconCodePoint("md-face-recognition", 0xFC57);
  addIconCodePoint("md-face-woman", 0xF00A2);
  addIconCodePoint("md-face-woman-outline", 0xF00A3);
  addIconCodePoint("md-facebook", 0xF20C);
  addIconCodePoint("md-facebook-box", 0xF20D);
  addIconCodePoint("md-facebook-messenger", 0xF20E);
  addIconCodePoint("md-facebook-workplace", 0xFB16);
  addIconCodePoint("md-factory", 0xF20F);
  addIconCodePoint("md-fan", 0xF210);
  addIconCodePoint("md-fan-off", 0xF81C);
  addIconCodePoint("md-fast-forward", 0xF211);
  addIconCodePoint("md-fast-forward-10", 0xFD4D);
  addIconCodePoint("md-fast-forward-30", 0xFCE2);
  addIconCodePoint("md-fast-forward-5", 0xF0223);
  addIconCodePoint("md-fast-forward-outline", 0xF6D1);
  addIconCodePoint("md-fax", 0xF212);
  addIconCodePoint("md-feather", 0xF6D2);
  addIconCodePoint("md-feature-search", 0xFA48);
  addIconCodePoint("md-feature-search-outline", 0xFA49);
  addIconCodePoint("md-fedora", 0xF8DA);
  addIconCodePoint("md-ferris-wheel", 0xFEC1);
  addIconCodePoint("md-ferry", 0xF213);
  addIconCodePoint("md-file", 0xF214);
  addIconCodePoint("md-file-account", 0xF73A);
  addIconCodePoint("md-file-account-outline", 0xF004A);
  addIconCodePoint("md-file-alert", 0xFA4A);
  addIconCodePoint("md-file-alert-outline", 0xFA4B);
  addIconCodePoint("md-file-cabinet", 0xFAB5);
  addIconCodePoint("md-file-cad", 0xFF08);
  addIconCodePoint("md-file-cad-box", 0xFF09);
  addIconCodePoint("md-file-cancel", 0xFDA2);
  addIconCodePoint("md-file-cancel-outline", 0xFDA3);
  addIconCodePoint("md-file-certificate", 0xF01B1);
  addIconCodePoint("md-file-certificate-outline", 0xF01B2);
  addIconCodePoint("md-file-chart", 0xF215);
  addIconCodePoint("md-file-chart-outline", 0xF004B);
  addIconCodePoint("md-file-check", 0xF216);
  addIconCodePoint("md-file-check-outline", 0xFE7B);
  addIconCodePoint("md-file-clock", 0xF030C);
  addIconCodePoint("md-file-clock-outline", 0xF030D);
  addIconCodePoint("md-file-cloud", 0xF217);
  addIconCodePoint("md-file-cloud-outline", 0xF004C);
  addIconCodePoint("md-file-code", 0xF22E);
  addIconCodePoint("md-file-code-outline", 0xF004D);
  addIconCodePoint("md-file-compare", 0xF8A9);
  addIconCodePoint("md-file-delimited", 0xF218);
  addIconCodePoint("md-file-delimited-outline", 0xFEC2);
  addIconCodePoint("md-file-document", 0xF219);
  addIconCodePoint("md-file-document-box", 0xF21A);
  addIconCodePoint("md-file-document-box-check", 0xFEC3);
  addIconCodePoint("md-file-document-box-check-outline", 0xFEC4);
  addIconCodePoint("md-file-document-box-minus", 0xFEC5);
  addIconCodePoint("md-file-document-box-minus-outline", 0xFEC6);
  addIconCodePoint("md-file-document-box-multiple", 0xFAB6);
  addIconCodePoint("md-file-document-box-multiple-outline", 0xFAB7);
  addIconCodePoint("md-file-document-box-outline", 0xF9EC);
  addIconCodePoint("md-file-document-box-plus", 0xFEC7);
  addIconCodePoint("md-file-document-box-plus-outline", 0xFEC8);
  addIconCodePoint("md-file-document-box-remove", 0xFEC9);
  addIconCodePoint("md-file-document-box-remove-outline", 0xFECA);
  addIconCodePoint("md-file-document-box-search", 0xFECB);
  addIconCodePoint("md-file-document-box-search-outline", 0xFECC);
  addIconCodePoint("md-file-document-edit", 0xFDA4);
  addIconCodePoint("md-file-document-edit-outline", 0xFDA5);
  addIconCodePoint("md-file-document-outline", 0xF9ED);
  addIconCodePoint("md-file-download", 0xF964);
  addIconCodePoint("md-file-download-outline", 0xF965);
  addIconCodePoint("md-file-edit", 0xF0212);
  addIconCodePoint("md-file-edit-outline", 0xF0213);
  addIconCodePoint("md-file-excel", 0xF21B);
  addIconCodePoint("md-file-excel-box", 0xF21C);
  addIconCodePoint("md-file-excel-box-outline", 0xF004E);
  addIconCodePoint("md-file-excel-outline", 0xF004F);
  addIconCodePoint("md-file-export", 0xF21D);
  addIconCodePoint("md-file-export-outline", 0xF0050);
  addIconCodePoint("md-file-eye", 0xFDA6);
  addIconCodePoint("md-file-eye-outline", 0xFDA7);
  addIconCodePoint("md-file-find", 0xF21E);
  addIconCodePoint("md-file-find-outline", 0xFB73);
  addIconCodePoint("md-file-hidden", 0xF613);
  addIconCodePoint("md-file-image", 0xF21F);
  addIconCodePoint("md-file-image-outline", 0xFECD);
  addIconCodePoint("md-file-import", 0xF220);
  addIconCodePoint("md-file-import-outline", 0xF0051);
  addIconCodePoint("md-file-key", 0xF01AF);
  addIconCodePoint("md-file-key-outline", 0xF01B0);
  addIconCodePoint("md-file-link", 0xF01A2);
  addIconCodePoint("md-file-link-outline", 0xF01A3);
  addIconCodePoint("md-file-lock", 0xF221);
  addIconCodePoint("md-file-lock-outline", 0xF0052);
  addIconCodePoint("md-file-move", 0xFAB8);
  addIconCodePoint("md-file-move-outline", 0xF0053);
  addIconCodePoint("md-file-multiple", 0xF222);
  addIconCodePoint("md-file-multiple-outline", 0xF0054);
  addIconCodePoint("md-file-music", 0xF223);
  addIconCodePoint("md-file-music-outline", 0xFE7C);
  addIconCodePoint("md-file-outline", 0xF224);
  addIconCodePoint("md-file-pdf", 0xF225);
  addIconCodePoint("md-file-pdf-box", 0xF226);
  addIconCodePoint("md-file-pdf-box-outline", 0xFFD3);
  addIconCodePoint("md-file-pdf-outline", 0xFE7D);
  addIconCodePoint("md-file-percent", 0xF81D);
  addIconCodePoint("md-file-percent-outline", 0xF0055);
  addIconCodePoint("md-file-phone", 0xF01A4);
  addIconCodePoint("md-file-phone-outline", 0xF01A5);
  addIconCodePoint("md-file-plus", 0xF751);
  addIconCodePoint("md-file-plus-outline", 0xFF0A);
  addIconCodePoint("md-file-powerpoint", 0xF227);
  addIconCodePoint("md-file-powerpoint-box", 0xF228);
  addIconCodePoint("md-file-powerpoint-box-outline", 0xF0056);
  addIconCodePoint("md-file-powerpoint-outline", 0xF0057);
  addIconCodePoint("md-file-presentation-box", 0xF229);
  addIconCodePoint("md-file-question", 0xF86E);
  addIconCodePoint("md-file-question-outline", 0xF0058);
  addIconCodePoint("md-file-remove", 0xFB74);
  addIconCodePoint("md-file-remove-outline", 0xF0059);
  addIconCodePoint("md-file-replace", 0xFB17);
  addIconCodePoint("md-file-replace-outline", 0xFB18);
  addIconCodePoint("md-file-restore", 0xF670);
  addIconCodePoint("md-file-restore-outline", 0xF005A);
  addIconCodePoint("md-file-search", 0xFC58);
  addIconCodePoint("md-file-search-outline", 0xFC59);
  addIconCodePoint("md-file-send", 0xF22A);
  addIconCodePoint("md-file-send-outline", 0xF005B);
  addIconCodePoint("md-file-settings", 0xF00A4);
  addIconCodePoint("md-file-settings-outline", 0xF00A5);
  addIconCodePoint("md-file-settings-variant", 0xF00A6);
  addIconCodePoint("md-file-settings-variant-outline", 0xF00A7);
  addIconCodePoint("md-file-star", 0xF005C);
  addIconCodePoint("md-file-star-outline", 0xF005D);
  addIconCodePoint("md-file-swap", 0xFFD4);
  addIconCodePoint("md-file-swap-outline", 0xFFD5);
  addIconCodePoint("md-file-sync", 0xF0241);
  addIconCodePoint("md-file-sync-outline", 0xF0242);
  addIconCodePoint("md-file-table", 0xFC5A);
  addIconCodePoint("md-file-table-box", 0xF010C);
  addIconCodePoint("md-file-table-box-multiple", 0xF010D);
  addIconCodePoint("md-file-table-box-multiple-outline", 0xF010E);
  addIconCodePoint("md-file-table-box-outline", 0xF010F);
  addIconCodePoint("md-file-table-outline", 0xFC5B);
  addIconCodePoint("md-file-tree", 0xF645);
  addIconCodePoint("md-file-undo", 0xF8DB);
  addIconCodePoint("md-file-undo-outline", 0xF005E);
  addIconCodePoint("md-file-upload", 0xFA4C);
  addIconCodePoint("md-file-upload-outline", 0xFA4D);
  addIconCodePoint("md-file-video", 0xF22B);
  addIconCodePoint("md-file-video-outline", 0xFE10);
  addIconCodePoint("md-file-word", 0xF22C);
  addIconCodePoint("md-file-word-box", 0xF22D);
  addIconCodePoint("md-file-word-box-outline", 0xF005F);
  addIconCodePoint("md-file-word-outline", 0xF0060);
  addIconCodePoint("md-film", 0xF22F);
  addIconCodePoint("md-filmstrip", 0xF230);
  addIconCodePoint("md-filmstrip-off", 0xF231);
  addIconCodePoint("md-filter", 0xF232);
  addIconCodePoint("md-filter-menu", 0xF0110);
  addIconCodePoint("md-filter-menu-outline", 0xF0111);
  addIconCodePoint("md-filter-minus", 0xFF0B);
  addIconCodePoint("md-filter-minus-outline", 0xFF0C);
  addIconCodePoint("md-filter-outline", 0xF233);
  addIconCodePoint("md-filter-plus", 0xFF0D);
  addIconCodePoint("md-filter-plus-outline", 0xFF0E);
  addIconCodePoint("md-filter-remove", 0xF234);
  addIconCodePoint("md-filter-remove-outline", 0xF235);
  addIconCodePoint("md-filter-variant", 0xF236);
  addIconCodePoint("md-filter-variant-minus", 0xF013D);
  addIconCodePoint("md-filter-variant-plus", 0xF013E);
  addIconCodePoint("md-filter-variant-remove", 0xF0061);
  addIconCodePoint("md-finance", 0xF81E);
  addIconCodePoint("md-find-replace", 0xF6D3);
  addIconCodePoint("md-fingerprint", 0xF237);
  addIconCodePoint("md-fingerprint-off", 0xFECE);
  addIconCodePoint("md-fire", 0xF238);
  addIconCodePoint("md-fire-extinguisher", 0xFF0F);
  addIconCodePoint("md-fire-hydrant", 0xF0162);
  addIconCodePoint("md-fire-hydrant-alert", 0xF0163);
  addIconCodePoint("md-fire-hydrant-off", 0xF0164);
  addIconCodePoint("md-fire-truck", 0xF8AA);
  addIconCodePoint("md-firebase", 0xF966);
  addIconCodePoint("md-firefox", 0xF239);
  addIconCodePoint("md-fireplace", 0xFE11);
  addIconCodePoint("md-fireplace-off", 0xFE12);
  addIconCodePoint("md-firework", 0xFE13);
  addIconCodePoint("md-fish", 0xF23A);
  addIconCodePoint("md-fishbowl", 0xFF10);
  addIconCodePoint("md-fishbowl-outline", 0xFF11);
  addIconCodePoint("md-fit-to-page", 0xFF12);
  addIconCodePoint("md-fit-to-page-outline", 0xFF13);
  addIconCodePoint("md-flag", 0xF23B);
  addIconCodePoint("md-flag-checkered", 0xF23C);
  addIconCodePoint("md-flag-minus", 0xFB75);
  addIconCodePoint("md-flag-minus-outline", 0xF00DD);
  addIconCodePoint("md-flag-outline", 0xF23D);
  addIconCodePoint("md-flag-plus", 0xFB76);
  addIconCodePoint("md-flag-plus-outline", 0xF00DE);
  addIconCodePoint("md-flag-remove", 0xFB77);
  addIconCodePoint("md-flag-remove-outline", 0xF00DF);
  addIconCodePoint("md-flag-triangle", 0xF23F);
  addIconCodePoint("md-flag-variant", 0xF240);
  addIconCodePoint("md-flag-variant-outline", 0xF23E);
  addIconCodePoint("md-flare", 0xFD4E);
  addIconCodePoint("md-flash", 0xF241);
  addIconCodePoint("md-flash-alert", 0xFF14);
  addIconCodePoint("md-flash-alert-outline", 0xFF15);
  addIconCodePoint("md-flash-auto", 0xF242);
  addIconCodePoint("md-flash-circle", 0xF81F);
  addIconCodePoint("md-flash-off", 0xF243);
  addIconCodePoint("md-flash-outline", 0xF6D4);
  addIconCodePoint("md-flash-red-eye", 0xF67A);
  addIconCodePoint("md-flashlight", 0xF244);
  addIconCodePoint("md-flashlight-off", 0xF245);
  addIconCodePoint("md-flask", 0xF093);
  addIconCodePoint("md-flask-empty", 0xF094);
  addIconCodePoint("md-flask-empty-minus", 0xF0265);
  addIconCodePoint("md-flask-empty-minus-outline", 0xF0266);
  addIconCodePoint("md-flask-empty-outline", 0xF095);
  addIconCodePoint("md-flask-empty-plus", 0xF0267);
  addIconCodePoint("md-flask-empty-plus-outline", 0xF0268);
  addIconCodePoint("md-flask-empty-remove", 0xF0269);
  addIconCodePoint("md-flask-empty-remove-outline", 0xF026A);
  addIconCodePoint("md-flask-minus", 0xF026B);
  addIconCodePoint("md-flask-minus-outline", 0xF026C);
  addIconCodePoint("md-flask-outline", 0xF096);
  addIconCodePoint("md-flask-plus", 0xF026D);
  addIconCodePoint("md-flask-plus-outline", 0xF026E);
  addIconCodePoint("md-flask-remove", 0xF026F);
  addIconCodePoint("md-flask-remove-outline", 0xF0270);
  addIconCodePoint("md-flask-round-bottom", 0xF0276);
  addIconCodePoint("md-flask-round-bottom-empty", 0xF0277);
  addIconCodePoint("md-flask-round-bottom-empty-outline", 0xF0278);
  addIconCodePoint("md-flask-round-bottom-outline", 0xF0279);
  addIconCodePoint("md-flattr", 0xF246);
  addIconCodePoint("md-fleur-de-lis", 0xF032E);
  addIconCodePoint("md-flickr", 0xFCE3);
  addIconCodePoint("md-flip-horizontal", 0xF0112);
  addIconCodePoint("md-flip-to-back", 0xF247);
  addIconCodePoint("md-flip-to-front", 0xF248);
  addIconCodePoint("md-flip-vertical", 0xF0113);
  addIconCodePoint("md-floor-lamp", 0xF8DC);
  addIconCodePoint("md-floor-lamp-dual", 0xF0062);
  addIconCodePoint("md-floor-lamp-variant", 0xF0063);
  addIconCodePoint("md-floor-plan", 0xF820);
  addIconCodePoint("md-floppy", 0xF249);
  addIconCodePoint("md-floppy-variant", 0xF9EE);
  addIconCodePoint("md-flower", 0xF24A);
  addIconCodePoint("md-flower-outline", 0xF9EF);
  addIconCodePoint("md-flower-poppy", 0xFCE4);
  addIconCodePoint("md-flower-tulip", 0xF9F0);
  addIconCodePoint("md-flower-tulip-outline", 0xF9F1);
  addIconCodePoint("md-focus-auto", 0xFF6B);
  addIconCodePoint("md-focus-field", 0xFF6C);
  addIconCodePoint("md-focus-field-horizontal", 0xFF6D);
  addIconCodePoint("md-focus-field-vertical", 0xFF6E);
  addIconCodePoint("md-folder", 0xF24B);
  addIconCodePoint("md-folder-account", 0xF24C);
  addIconCodePoint("md-folder-account-outline", 0xFB78);
  addIconCodePoint("md-folder-alert", 0xFDA8);
  addIconCodePoint("md-folder-alert-outline", 0xFDA9);
  addIconCodePoint("md-folder-clock", 0xFAB9);
  addIconCodePoint("md-folder-clock-outline", 0xFABA);
  addIconCodePoint("md-folder-download", 0xF24D);
  addIconCodePoint("md-folder-download-outline", 0xF0114);
  addIconCodePoint("md-folder-edit", 0xF8DD);
  addIconCodePoint("md-folder-edit-outline", 0xFDAA);
  addIconCodePoint("md-folder-google-drive", 0xF24E);
  addIconCodePoint("md-folder-heart", 0xF0115);
  addIconCodePoint("md-folder-heart-outline", 0xF0116);
  addIconCodePoint("md-folder-home", 0xF00E0);
  addIconCodePoint("md-folder-home-outline", 0xF00E1);
  addIconCodePoint("md-folder-image", 0xF24F);
  addIconCodePoint("md-folder-information", 0xF00E2);
  addIconCodePoint("md-folder-information-outline", 0xF00E3);
  addIconCodePoint("md-folder-key", 0xF8AB);
  addIconCodePoint("md-folder-key-network", 0xF8AC);
  addIconCodePoint("md-folder-key-network-outline", 0xFC5C);
  addIconCodePoint("md-folder-key-outline", 0xF0117);
  addIconCodePoint("md-folder-lock", 0xF250);
  addIconCodePoint("md-folder-lock-open", 0xF251);
  addIconCodePoint("md-folder-marker", 0xF0298);
  addIconCodePoint("md-folder-marker-outline", 0xF0299);
  addIconCodePoint("md-folder-move", 0xF252);
  addIconCodePoint("md-folder-move-outline", 0xF0271);
  addIconCodePoint("md-folder-multiple", 0xF253);
  addIconCodePoint("md-folder-multiple-image", 0xF254);
  addIconCodePoint("md-folder-multiple-outline", 0xF255);
  addIconCodePoint("md-folder-music", 0xF0384);
  addIconCodePoint("md-folder-music-outline", 0xF0385);
  addIconCodePoint("md-folder-network", 0xF86F);
  addIconCodePoint("md-folder-network-outline", 0xFC5D);
  addIconCodePoint("md-folder-open", 0xF76F);
  addIconCodePoint("md-folder-open-outline", 0xFDAB);
  addIconCodePoint("md-folder-outline", 0xF256);
  addIconCodePoint("md-folder-plus", 0xF257);
  addIconCodePoint("md-folder-plus-outline", 0xFB79);
  addIconCodePoint("md-folder-pound", 0xFCE5);
  addIconCodePoint("md-folder-pound-outline", 0xFCE6);
  addIconCodePoint("md-folder-remove", 0xF258);
  addIconCodePoint("md-folder-remove-outline", 0xFB7A);
  addIconCodePoint("md-folder-search", 0xF967);
  addIconCodePoint("md-folder-search-outline", 0xF968);
  addIconCodePoint("md-folder-settings", 0xF00A8);
  addIconCodePoint("md-folder-settings-outline", 0xF00A9);
  addIconCodePoint("md-folder-settings-variant", 0xF00AA);
  addIconCodePoint("md-folder-settings-variant-outline", 0xF00AB);
  addIconCodePoint("md-folder-star", 0xF69C);
  addIconCodePoint("md-folder-star-outline", 0xFB7B);
  addIconCodePoint("md-folder-swap", 0xFFD6);
  addIconCodePoint("md-folder-swap-outline", 0xFFD7);
  addIconCodePoint("md-folder-sync", 0xFCE7);
  addIconCodePoint("md-folder-sync-outline", 0xFCE8);
  addIconCodePoint("md-folder-table", 0xF030E);
  addIconCodePoint("md-folder-table-outline", 0xF030F);
  addIconCodePoint("md-folder-text", 0xFC5E);
  addIconCodePoint("md-folder-text-outline", 0xFC5F);
  addIconCodePoint("md-folder-upload", 0xF259);
  addIconCodePoint("md-folder-upload-outline", 0xF0118);
  addIconCodePoint("md-folder-zip", 0xF6EA);
  addIconCodePoint("md-folder-zip-outline", 0xF7B8);
  addIconCodePoint("md-font-awesome", 0xF03A);
  addIconCodePoint("md-food", 0xF25A);
  addIconCodePoint("md-food-apple", 0xF25B);
  addIconCodePoint("md-food-apple-outline", 0xFC60);
  addIconCodePoint("md-food-croissant", 0xF7C7);
  addIconCodePoint("md-food-fork-drink", 0xF5F2);
  addIconCodePoint("md-food-off", 0xF5F3);
  addIconCodePoint("md-food-variant", 0xF25C);
  addIconCodePoint("md-foot-print", 0xFF6F);
  addIconCodePoint("md-football", 0xF25D);
  addIconCodePoint("md-football-australian", 0xF25E);
  addIconCodePoint("md-football-helmet", 0xF25F);
  addIconCodePoint("md-forklift", 0xF7C8);
  addIconCodePoint("md-format-align-bottom", 0xF752);
  addIconCodePoint("md-format-align-center", 0xF260);
  addIconCodePoint("md-format-align-justify", 0xF261);
  addIconCodePoint("md-format-align-left", 0xF262);
  addIconCodePoint("md-format-align-middle", 0xF753);
  addIconCodePoint("md-format-align-right", 0xF263);
  addIconCodePoint("md-format-align-top", 0xF754);
  addIconCodePoint("md-format-annotation-minus", 0xFABB);
  addIconCodePoint("md-format-annotation-plus", 0xF646);
  addIconCodePoint("md-format-bold", 0xF264);
  addIconCodePoint("md-format-clear", 0xF265);
  addIconCodePoint("md-format-color-fill", 0xF266);
  addIconCodePoint("md-format-color-highlight", 0xFE14);
  addIconCodePoint("md-format-color-marker-cancel", 0xF033E);
  addIconCodePoint("md-format-color-text", 0xF69D);
  addIconCodePoint("md-format-columns", 0xF8DE);
  addIconCodePoint("md-format-float-center", 0xF267);
  addIconCodePoint("md-format-float-left", 0xF268);
  addIconCodePoint("md-format-float-none", 0xF269);
  addIconCodePoint("md-format-float-right", 0xF26A);
  addIconCodePoint("md-format-font", 0xF6D5);
  addIconCodePoint("md-format-font-size-decrease", 0xF9F2);
  addIconCodePoint("md-format-font-size-increase", 0xF9F3);
  addIconCodePoint("md-format-header-1", 0xF26B);
  addIconCodePoint("md-format-header-2", 0xF26C);
  addIconCodePoint("md-format-header-3", 0xF26D);
  addIconCodePoint("md-format-header-4", 0xF26E);
  addIconCodePoint("md-format-header-5", 0xF26F);
  addIconCodePoint("md-format-header-6", 0xF270);
  addIconCodePoint("md-format-header-decrease", 0xF271);
  addIconCodePoint("md-format-header-equal", 0xF272);
  addIconCodePoint("md-format-header-increase", 0xF273);
  addIconCodePoint("md-format-header-pound", 0xF274);
  addIconCodePoint("md-format-horizontal-align-center", 0xF61E);
  addIconCodePoint("md-format-horizontal-align-left", 0xF61F);
  addIconCodePoint("md-format-horizontal-align-right", 0xF620);
  addIconCodePoint("md-format-indent-decrease", 0xF275);
  addIconCodePoint("md-format-indent-increase", 0xF276);
  addIconCodePoint("md-format-italic", 0xF277);
  addIconCodePoint("md-format-letter-case", 0xFB19);
  addIconCodePoint("md-format-letter-case-lower", 0xFB1A);
  addIconCodePoint("md-format-letter-case-upper", 0xFB1B);
  addIconCodePoint("md-format-letter-ends-with", 0xFFD8);
  addIconCodePoint("md-format-letter-matches", 0xFFD9);
  addIconCodePoint("md-format-letter-starts-with", 0xFFDA);
  addIconCodePoint("md-format-line-spacing", 0xF278);
  addIconCodePoint("md-format-line-style", 0xF5C8);
  addIconCodePoint("md-format-line-weight", 0xF5C9);
  addIconCodePoint("md-format-list-bulleted", 0xF279);
  addIconCodePoint("md-format-list-bulleted-square", 0xFDAC);
  addIconCodePoint("md-format-list-bulleted-triangle", 0xFECF);
  addIconCodePoint("md-format-list-bulleted-type", 0xF27A);
  addIconCodePoint("md-format-list-checkbox", 0xF969);
  addIconCodePoint("md-format-list-checks", 0xF755);
  addIconCodePoint("md-format-list-numbered", 0xF27B);
  addIconCodePoint("md-format-list-numbered-rtl", 0xFCE9);
  addIconCodePoint("md-format-list-text", 0xF029A);
  addIconCodePoint("md-format-overline", 0xFED0);
  addIconCodePoint("md-format-page-break", 0xF6D6);
  addIconCodePoint("md-format-paint", 0xF27C);
  addIconCodePoint("md-format-paragraph", 0xF27D);
  addIconCodePoint("md-format-pilcrow", 0xF6D7);
  addIconCodePoint("md-format-quote-close", 0xF27E);
  addIconCodePoint("md-format-quote-close-outline", 0xF01D3);
  addIconCodePoint("md-format-quote-open", 0xF756);
  addIconCodePoint("md-format-quote-open-outline", 0xF01D2);
  addIconCodePoint("md-format-rotate-90", 0xF6A9);
  addIconCodePoint("md-format-section", 0xF69E);
  addIconCodePoint("md-format-size", 0xF27F);
  addIconCodePoint("md-format-strikethrough", 0xF280);
  addIconCodePoint("md-format-strikethrough-variant", 0xF281);
  addIconCodePoint("md-format-subscript", 0xF282);
  addIconCodePoint("md-format-superscript", 0xF283);
  addIconCodePoint("md-format-text", 0xF284);
  addIconCodePoint("md-format-text-rotation-angle-down", 0xFFDB);
  addIconCodePoint("md-format-text-rotation-angle-up", 0xFFDC);
  addIconCodePoint("md-format-text-rotation-down", 0xFD4F);
  addIconCodePoint("md-format-text-rotation-down-vertical", 0xFFDD);
  addIconCodePoint("md-format-text-rotation-none", 0xFD50);
  addIconCodePoint("md-format-text-rotation-up", 0xFFDE);
  addIconCodePoint("md-format-text-rotation-vertical", 0xFFDF);
  addIconCodePoint("md-format-text-variant", 0xFE15);
  addIconCodePoint("md-format-text-wrapping-clip", 0xFCEA);
  addIconCodePoint("md-format-text-wrapping-overflow", 0xFCEB);
  addIconCodePoint("md-format-text-wrapping-wrap", 0xFCEC);
  addIconCodePoint("md-format-textbox", 0xFCED);
  addIconCodePoint("md-format-textdirection-l-to-r", 0xF285);
  addIconCodePoint("md-format-textdirection-r-to-l", 0xF286);
  addIconCodePoint("md-format-title", 0xF5F4);
  addIconCodePoint("md-format-underline", 0xF287);
  addIconCodePoint("md-format-vertical-align-bottom", 0xF621);
  addIconCodePoint("md-format-vertical-align-center", 0xF622);
  addIconCodePoint("md-format-vertical-align-top", 0xF623);
  addIconCodePoint("md-format-wrap-inline", 0xF288);
  addIconCodePoint("md-format-wrap-square", 0xF289);
  addIconCodePoint("md-format-wrap-tight", 0xF28A);
  addIconCodePoint("md-format-wrap-top-bottom", 0xF28B);
  addIconCodePoint("md-forum", 0xF28C);
  addIconCodePoint("md-forum-outline", 0xF821);
  addIconCodePoint("md-forward", 0xF28D);
  addIconCodePoint("md-forwardburger", 0xFD51);
  addIconCodePoint("md-fountain", 0xF96A);
  addIconCodePoint("md-fountain-pen", 0xFCEE);
  addIconCodePoint("md-fountain-pen-tip", 0xFCEF);
  addIconCodePoint("md-foursquare", 0xF28E);
  addIconCodePoint("md-freebsd", 0xF8DF);
  addIconCodePoint("md-frequently-asked-questions", 0xFED1);
  addIconCodePoint("md-fridge", 0xF290);
  addIconCodePoint("md-fridge-alert", 0xF01DC);
  addIconCodePoint("md-fridge-alert-outline", 0xF01DD);
  addIconCodePoint("md-fridge-bottom", 0xF292);
  addIconCodePoint("md-fridge-off", 0xF01DA);
  addIconCodePoint("md-fridge-off-outline", 0xF01DB);
  addIconCodePoint("md-fridge-outline", 0xF28F);
  addIconCodePoint("md-fridge-top", 0xF291);
  addIconCodePoint("md-fruit-cherries", 0xF0064);
  addIconCodePoint("md-fruit-citrus", 0xF0065);
  addIconCodePoint("md-fruit-grapes", 0xF0066);
  addIconCodePoint("md-fruit-grapes-outline", 0xF0067);
  addIconCodePoint("md-fruit-pineapple", 0xF0068);
  addIconCodePoint("md-fruit-watermelon", 0xF0069);
  addIconCodePoint("md-fuel", 0xF7C9);
  addIconCodePoint("md-fullscreen", 0xF293);
  addIconCodePoint("md-fullscreen-exit", 0xF294);
  addIconCodePoint("md-function", 0xF295);
  addIconCodePoint("md-function-variant", 0xF870);
  addIconCodePoint("md-furigana-horizontal", 0xF00AC);
  addIconCodePoint("md-furigana-vertical", 0xF00AD);
  addIconCodePoint("md-fuse", 0xFC61);
  addIconCodePoint("md-fuse-blade", 0xFC62);
  addIconCodePoint("md-gamepad", 0xF296);
  addIconCodePoint("md-gamepad-circle", 0xFE16);
  addIconCodePoint("md-gamepad-circle-down", 0xFE17);
  addIconCodePoint("md-gamepad-circle-left", 0xFE18);
  addIconCodePoint("md-gamepad-circle-outline", 0xFE19);
  addIconCodePoint("md-gamepad-circle-right", 0xFE1A);
  addIconCodePoint("md-gamepad-circle-up", 0xFE1B);
  addIconCodePoint("md-gamepad-down", 0xFE1C);
  addIconCodePoint("md-gamepad-left", 0xFE1D);
  addIconCodePoint("md-gamepad-right", 0xFE1E);
  addIconCodePoint("md-gamepad-round", 0xFE1F);
  addIconCodePoint("md-gamepad-round-down", 0xFE7E);
  addIconCodePoint("md-gamepad-round-left", 0xFE7F);
  addIconCodePoint("md-gamepad-round-outline", 0xFE80);
  addIconCodePoint("md-gamepad-round-right", 0xFE81);
  addIconCodePoint("md-gamepad-round-up", 0xFE82);
  addIconCodePoint("md-gamepad-square", 0xFED2);
  addIconCodePoint("md-gamepad-square-outline", 0xFED3);
  addIconCodePoint("md-gamepad-up", 0xFE83);
  addIconCodePoint("md-gamepad-variant", 0xF297);
  addIconCodePoint("md-gamepad-variant-outline", 0xFED4);
  addIconCodePoint("md-gamma", 0xF0119);
  addIconCodePoint("md-gantry-crane", 0xFDAD);
  addIconCodePoint("md-garage", 0xF6D8);
  addIconCodePoint("md-garage-alert", 0xF871);
  addIconCodePoint("md-garage-alert-variant", 0xF0300);
  addIconCodePoint("md-garage-open", 0xF6D9);
  addIconCodePoint("md-garage-open-variant", 0xF02FF);
  addIconCodePoint("md-garage-variant", 0xF02FE);
  addIconCodePoint("md-gas-cylinder", 0xF647);
  addIconCodePoint("md-gas-station", 0xF298);
  addIconCodePoint("md-gas-station-outline", 0xFED5);
  addIconCodePoint("md-gate", 0xF299);
  addIconCodePoint("md-gate-and", 0xF8E0);
  addIconCodePoint("md-gate-arrow-right", 0xF0194);
  addIconCodePoint("md-gate-nand", 0xF8E1);
  addIconCodePoint("md-gate-nor", 0xF8E2);
  addIconCodePoint("md-gate-not", 0xF8E3);
  addIconCodePoint("md-gate-open", 0xF0195);
  addIconCodePoint("md-gate-or", 0xF8E4);
  addIconCodePoint("md-gate-xnor", 0xF8E5);
  addIconCodePoint("md-gate-xor", 0xF8E6);
  addIconCodePoint("md-gatsby", 0xFE84);
  addIconCodePoint("md-gauge", 0xF29A);
  addIconCodePoint("md-gauge-empty", 0xF872);
  addIconCodePoint("md-gauge-full", 0xF873);
  addIconCodePoint("md-gauge-low", 0xF874);
  addIconCodePoint("md-gavel", 0xF29B);
  addIconCodePoint("md-gender-female", 0xF29C);
  addIconCodePoint("md-gender-male", 0xF29D);
  addIconCodePoint("md-gender-male-female", 0xF29E);
  addIconCodePoint("md-gender-male-female-variant", 0xF016A);
  addIconCodePoint("md-gender-non-binary", 0xF016B);
  addIconCodePoint("md-gender-transgender", 0xF29F);
  addIconCodePoint("md-gentoo", 0xF8E7);
  addIconCodePoint("md-gesture", 0xF7CA);
  addIconCodePoint("md-gesture-double-tap", 0xF73B);
  addIconCodePoint("md-gesture-pinch", 0xFABC);
  addIconCodePoint("md-gesture-spread", 0xFABD);
  addIconCodePoint("md-gesture-swipe", 0xFD52);
  addIconCodePoint("md-gesture-swipe-down", 0xF73C);
  addIconCodePoint("md-gesture-swipe-horizontal", 0xFABE);
  addIconCodePoint("md-gesture-swipe-left", 0xF73D);
  addIconCodePoint("md-gesture-swipe-right", 0xF73E);
  addIconCodePoint("md-gesture-swipe-up", 0xF73F);
  addIconCodePoint("md-gesture-swipe-vertical", 0xFABF);
  addIconCodePoint("md-gesture-tap", 0xF740);
  addIconCodePoint("md-gesture-tap-box", 0xF02D4);
  addIconCodePoint("md-gesture-tap-button", 0xF02D3);
  addIconCodePoint("md-gesture-tap-hold", 0xFD53);
  addIconCodePoint("md-gesture-two-double-tap", 0xF741);
  addIconCodePoint("md-gesture-two-tap", 0xF742);
  addIconCodePoint("md-ghost", 0xF2A0);
  addIconCodePoint("md-ghost-off", 0xF9F4);
  addIconCodePoint("md-gif", 0xFD54);
  addIconCodePoint("md-gift", 0xFE85);
  addIconCodePoint("md-gift-outline", 0xF2A1);
  addIconCodePoint("md-git", 0xF2A2);
  addIconCodePoint("md-github-box", 0xF2A3);
  addIconCodePoint("md-github-circle", 0xF2A4);
  addIconCodePoint("md-github-face", 0xF6DA);
  addIconCodePoint("md-gitlab", 0xFB7C);
  addIconCodePoint("md-glass-cocktail", 0xF356);
  addIconCodePoint("md-glass-flute", 0xF2A5);
  addIconCodePoint("md-glass-mug", 0xF2A6);
  addIconCodePoint("md-glass-mug-variant", 0xF0141);
  addIconCodePoint("md-glass-pint-outline", 0xF0338);
  addIconCodePoint("md-glass-stange", 0xF2A7);
  addIconCodePoint("md-glass-tulip", 0xF2A8);
  addIconCodePoint("md-glass-wine", 0xF875);
  addIconCodePoint("md-glassdoor", 0xF2A9);
  addIconCodePoint("md-glasses", 0xF2AA);
  addIconCodePoint("md-globe-light", 0xF0302);
  addIconCodePoint("md-globe-model", 0xF8E8);
  addIconCodePoint("md-gmail", 0xF2AB);
  addIconCodePoint("md-gnome", 0xF2AC);
  addIconCodePoint("md-go-kart", 0xFD55);
  addIconCodePoint("md-go-kart-track", 0xFD56);
  addIconCodePoint("md-gog", 0xFB7D);
  addIconCodePoint("md-gold", 0xF027A);
  addIconCodePoint("md-golf", 0xF822);
  addIconCodePoint("md-golf-cart", 0xF01CF);
  addIconCodePoint("md-golf-tee", 0xF00AE);
  addIconCodePoint("md-gondola", 0xF685);
  addIconCodePoint("md-goodreads", 0xFD57);
  addIconCodePoint("md-google", 0xF2AD);
  addIconCodePoint("md-google-adwords", 0xFC63);
  addIconCodePoint("md-google-analytics", 0xF7CB);
  addIconCodePoint("md-google-assistant", 0xF7CC);
  addIconCodePoint("md-google-cardboard", 0xF2AE);
  addIconCodePoint("md-google-chrome", 0xF2AF);
  addIconCodePoint("md-google-circles", 0xF2B0);
  addIconCodePoint("md-google-circles-communities", 0xF2B1);
  addIconCodePoint("md-google-circles-extended", 0xF2B2);
  addIconCodePoint("md-google-circles-group", 0xF2B3);
  addIconCodePoint("md-google-classroom", 0xF2C0);
  addIconCodePoint("md-google-cloud", 0xF0221);
  addIconCodePoint("md-google-controller", 0xF2B4);
  addIconCodePoint("md-google-controller-off", 0xF2B5);
  addIconCodePoint("md-google-downasaur", 0xF038D);
  addIconCodePoint("md-google-drive", 0xF2B6);
  addIconCodePoint("md-google-earth", 0xF2B7);
  addIconCodePoint("md-google-fit", 0xF96B);
  addIconCodePoint("md-google-glass", 0xF2B8);
  addIconCodePoint("md-google-hangouts", 0xF2C9);
  addIconCodePoint("md-google-home", 0xF823);
  addIconCodePoint("md-google-keep", 0xF6DB);
  addIconCodePoint("md-google-lens", 0xF9F5);
  addIconCodePoint("md-google-maps", 0xF5F5);
  addIconCodePoint("md-google-my-business", 0xF006A);
  addIconCodePoint("md-google-nearby", 0xF2B9);
  addIconCodePoint("md-google-pages", 0xF2BA);
  addIconCodePoint("md-google-photos", 0xF6DC);
  addIconCodePoint("md-google-physical-web", 0xF2BB);
  addIconCodePoint("md-google-play", 0xF2BC);
  addIconCodePoint("md-google-plus", 0xF2BD);
  addIconCodePoint("md-google-plus-box", 0xF2BE);
  addIconCodePoint("md-google-podcast", 0xFED6);
  addIconCodePoint("md-google-spreadsheet", 0xF9F6);
  addIconCodePoint("md-google-street-view", 0xFC64);
  addIconCodePoint("md-google-translate", 0xF2BF);
  addIconCodePoint("md-gradient", 0xF69F);
  addIconCodePoint("md-grain", 0xFD58);
  addIconCodePoint("md-graph", 0xF006B);
  addIconCodePoint("md-graph-outline", 0xF006C);
  addIconCodePoint("md-graphql", 0xF876);
  addIconCodePoint("md-grave-stone", 0xFB7E);
  addIconCodePoint("md-grease-pencil", 0xF648);
  addIconCodePoint("md-greater-than", 0xF96C);
  addIconCodePoint("md-greater-than-or-equal", 0xF96D);
  addIconCodePoint("md-grid", 0xF2C1);
  addIconCodePoint("md-grid-large", 0xF757);
  addIconCodePoint("md-grid-off", 0xF2C2);
  addIconCodePoint("md-grill", 0xFE86);
  addIconCodePoint("md-grill-outline", 0xF01B5);
  addIconCodePoint("md-group", 0xF2C3);
  addIconCodePoint("md-guitar-acoustic", 0xF770);
  addIconCodePoint("md-guitar-electric", 0xF2C4);
  addIconCodePoint("md-guitar-pick", 0xF2C5);
  addIconCodePoint("md-guitar-pick-outline", 0xF2C6);
  addIconCodePoint("md-guy-fawkes-mask", 0xF824);
  addIconCodePoint("md-hackernews", 0xF624);
  addIconCodePoint("md-hail", 0xFAC0);
  addIconCodePoint("md-hair-dryer", 0xF011A);
  addIconCodePoint("md-hair-dryer-outline", 0xF011B);
  addIconCodePoint("md-halloween", 0xFB7F);
  addIconCodePoint("md-hamburger", 0xF684);
  addIconCodePoint("md-hammer", 0xF8E9);
  addIconCodePoint("md-hammer-screwdriver", 0xF034D);
  addIconCodePoint("md-hammer-wrench", 0xF034E);
  addIconCodePoint("md-hand", 0xFA4E);
  addIconCodePoint("md-hand-heart", 0xF011C);
  addIconCodePoint("md-hand-left", 0xFE87);
  addIconCodePoint("md-hand-okay", 0xFA4F);
  addIconCodePoint("md-hand-peace", 0xFA50);
  addIconCodePoint("md-hand-peace-variant", 0xFA51);
  addIconCodePoint("md-hand-pointing-down", 0xFA52);
  addIconCodePoint("md-hand-pointing-left", 0xFA53);
  addIconCodePoint("md-hand-pointing-right", 0xF2C7);
  addIconCodePoint("md-hand-pointing-up", 0xFA54);
  addIconCodePoint("md-hand-right", 0xFE88);
  addIconCodePoint("md-hand-saw", 0xFE89);
  addIconCodePoint("md-handball", 0xFF70);
  addIconCodePoint("md-handcuffs", 0xF0169);
  addIconCodePoint("md-handshake", 0xF0243);
  addIconCodePoint("md-hanger", 0xF2C8);
  addIconCodePoint("md-hard-hat", 0xF96E);
  addIconCodePoint("md-harddisk", 0xF2CA);
  addIconCodePoint("md-harddisk-plus", 0xF006D);
  addIconCodePoint("md-harddisk-remove", 0xF006E);
  addIconCodePoint("md-hat-fedora", 0xFB80);
  addIconCodePoint("md-hazard-lights", 0xFC65);
  addIconCodePoint("md-hdr", 0xFD59);
  addIconCodePoint("md-hdr-off", 0xFD5A);
  addIconCodePoint("md-head", 0xF0389);
  addIconCodePoint("md-head-alert", 0xF0363);
  addIconCodePoint("md-head-alert-outline", 0xF0364);
  addIconCodePoint("md-head-check", 0xF0365);
  addIconCodePoint("md-head-check-outline", 0xF0366);
  addIconCodePoint("md-head-cog", 0xF0367);
  addIconCodePoint("md-head-cog-outline", 0xF0368);
  addIconCodePoint("md-head-dots-horizontal", 0xF0369);
  addIconCodePoint("md-head-dots-horizontal-outline", 0xF036A);
  addIconCodePoint("md-head-flash", 0xF036B);
  addIconCodePoint("md-head-flash-outline", 0xF036C);
  addIconCodePoint("md-head-heart", 0xF036D);
  addIconCodePoint("md-head-heart-outline", 0xF036E);
  addIconCodePoint("md-head-lightbulb", 0xF036F);
  addIconCodePoint("md-head-lightbulb-outline", 0xF0370);
  addIconCodePoint("md-head-minus", 0xF0371);
  addIconCodePoint("md-head-minus-outline", 0xF0372);
  addIconCodePoint("md-head-outline", 0xF038A);
  addIconCodePoint("md-head-plus", 0xF0373);
  addIconCodePoint("md-head-plus-outline", 0xF0374);
  addIconCodePoint("md-head-question", 0xF0375);
  addIconCodePoint("md-head-question-outline", 0xF0376);
  addIconCodePoint("md-head-remove", 0xF0377);
  addIconCodePoint("md-head-remove-outline", 0xF0378);
  addIconCodePoint("md-head-snowflake", 0xF0379);
  addIconCodePoint("md-head-snowflake-outline", 0xF037A);
  addIconCodePoint("md-head-sync", 0xF037B);
  addIconCodePoint("md-head-sync-outline", 0xF037C);
  addIconCodePoint("md-headphones", 0xF2CB);
  addIconCodePoint("md-headphones-bluetooth", 0xF96F);
  addIconCodePoint("md-headphones-box", 0xF2CC);
  addIconCodePoint("md-headphones-off", 0xF7CD);
  addIconCodePoint("md-headphones-settings", 0xF2CD);
  addIconCodePoint("md-headset", 0xF2CE);
  addIconCodePoint("md-headset-dock", 0xF2CF);
  addIconCodePoint("md-headset-off", 0xF2D0);
  addIconCodePoint("md-heart", 0xF2D1);
  addIconCodePoint("md-heart-box", 0xF2D2);
  addIconCodePoint("md-heart-box-outline", 0xF2D3);
  addIconCodePoint("md-heart-broken", 0xF2D4);
  addIconCodePoint("md-heart-broken-outline", 0xFCF0);
  addIconCodePoint("md-heart-circle", 0xF970);
  addIconCodePoint("md-heart-circle-outline", 0xF971);
  addIconCodePoint("md-heart-flash", 0xFF16);
  addIconCodePoint("md-heart-half", 0xF6DE);
  addIconCodePoint("md-heart-half-full", 0xF6DD);
  addIconCodePoint("md-heart-half-outline", 0xF6DF);
  addIconCodePoint("md-heart-multiple", 0xFA55);
  addIconCodePoint("md-heart-multiple-outline", 0xFA56);
  addIconCodePoint("md-heart-off", 0xF758);
  addIconCodePoint("md-heart-outline", 0xF2D5);
  addIconCodePoint("md-heart-pulse", 0xF5F6);
  addIconCodePoint("md-helicopter", 0xFAC1);
  addIconCodePoint("md-help", 0xF2D6);
  addIconCodePoint("md-help-box", 0xF78A);
  addIconCodePoint("md-help-circle", 0xF2D7);
  addIconCodePoint("md-help-circle-outline", 0xF625);
  addIconCodePoint("md-help-network", 0xF6F4);
  addIconCodePoint("md-help-network-outline", 0xFC66);
  addIconCodePoint("md-help-rhombus", 0xFB81);
  addIconCodePoint("md-help-rhombus-outline", 0xFB82);
  addIconCodePoint("md-hexadecimal", 0xF02D2);
  addIconCodePoint("md-hexagon", 0xF2D8);
  addIconCodePoint("md-hexagon-multiple", 0xF6E0);
  addIconCodePoint("md-hexagon-multiple-outline", 0xF011D);
  addIconCodePoint("md-hexagon-outline", 0xF2D9);
  addIconCodePoint("md-hexagon-slice-1", 0xFAC2);
  addIconCodePoint("md-hexagon-slice-2", 0xFAC3);
  addIconCodePoint("md-hexagon-slice-3", 0xFAC4);
  addIconCodePoint("md-hexagon-slice-4", 0xFAC5);
  addIconCodePoint("md-hexagon-slice-5", 0xFAC6);
  addIconCodePoint("md-hexagon-slice-6", 0xFAC7);
  addIconCodePoint("md-hexagram", 0xFAC8);
  addIconCodePoint("md-hexagram-outline", 0xFAC9);
  addIconCodePoint("md-high-definition", 0xF7CE);
  addIconCodePoint("md-high-definition-box", 0xF877);
  addIconCodePoint("md-highway", 0xF5F7);
  addIconCodePoint("md-hiking", 0xFD5B);
  addIconCodePoint("md-hinduism", 0xF972);
  addIconCodePoint("md-history", 0xF2DA);
  addIconCodePoint("md-hockey-puck", 0xF878);
  addIconCodePoint("md-hockey-sticks", 0xF879);
  addIconCodePoint("md-hololens", 0xF2DB);
  addIconCodePoint("md-home", 0xF2DC);
  addIconCodePoint("md-home-account", 0xF825);
  addIconCodePoint("md-home-alert", 0xF87A);
  addIconCodePoint("md-home-analytics", 0xFED7);
  addIconCodePoint("md-home-assistant", 0xF7CF);
  addIconCodePoint("md-home-automation", 0xF7D0);
  addIconCodePoint("md-home-circle", 0xF7D1);
  addIconCodePoint("md-home-circle-outline", 0xF006F);
  addIconCodePoint("md-home-city", 0xFCF1);
  addIconCodePoint("md-home-city-outline", 0xFCF2);
  addIconCodePoint("md-home-currency-usd", 0xF8AE);
  addIconCodePoint("md-home-edit", 0xF0184);
  addIconCodePoint("md-home-edit-outline", 0xF0185);
  addIconCodePoint("md-home-export-outline", 0xFFB8);
  addIconCodePoint("md-home-flood", 0xFF17);
  addIconCodePoint("md-home-floor-0", 0xFDAE);
  addIconCodePoint("md-home-floor-1", 0xFD5C);
  addIconCodePoint("md-home-floor-2", 0xFD5D);
  addIconCodePoint("md-home-floor-3", 0xFD5E);
  addIconCodePoint("md-home-floor-a", 0xFD5F);
  addIconCodePoint("md-home-floor-b", 0xFD60);
  addIconCodePoint("md-home-floor-g", 0xFD61);
  addIconCodePoint("md-home-floor-l", 0xFD62);
  addIconCodePoint("md-home-floor-negative-1", 0xFDAF);
  addIconCodePoint("md-home-group", 0xFDB0);
  addIconCodePoint("md-home-heart", 0xF826);
  addIconCodePoint("md-home-import-outline", 0xFFB9);
  addIconCodePoint("md-home-lightbulb", 0xF027C);
  addIconCodePoint("md-home-lightbulb-outline", 0xF027D);
  addIconCodePoint("md-home-lock", 0xF8EA);
  addIconCodePoint("md-home-lock-open", 0xF8EB);
  addIconCodePoint("md-home-map-marker", 0xF5F8);
  addIconCodePoint("md-home-minus", 0xF973);
  addIconCodePoint("md-home-modern", 0xF2DD);
  addIconCodePoint("md-home-outline", 0xF6A0);
  addIconCodePoint("md-home-plus", 0xF974);
  addIconCodePoint("md-home-remove", 0xF0272);
  addIconCodePoint("md-home-roof", 0xF0156);
  addIconCodePoint("md-home-thermometer", 0xFF71);
  addIconCodePoint("md-home-thermometer-outline", 0xFF72);
  addIconCodePoint("md-home-variant", 0xF2DE);
  addIconCodePoint("md-home-variant-outline", 0xFB83);
  addIconCodePoint("md-hook", 0xF6E1);
  addIconCodePoint("md-hook-off", 0xF6E2);
  addIconCodePoint("md-hops", 0xF2DF);
  addIconCodePoint("md-horizontal-rotate-clockwise", 0xF011E);
  addIconCodePoint("md-horizontal-rotate-counterclockwise", 0xF011F);
  addIconCodePoint("md-horseshoe", 0xFA57);
  addIconCodePoint("md-hospital", 0xF0017);
  addIconCodePoint("md-hospital-box", 0xF2E0);
  addIconCodePoint("md-hospital-box-outline", 0xF0018);
  addIconCodePoint("md-hospital-building", 0xF2E1);
  addIconCodePoint("md-hospital-marker", 0xF2E2);
  addIconCodePoint("md-hot-tub", 0xF827);
  addIconCodePoint("md-hotel", 0xF2E3);
  addIconCodePoint("md-houzz", 0xF2E4);
  addIconCodePoint("md-houzz-box", 0xF2E5);
  addIconCodePoint("md-hubspot", 0xFCF3);
  addIconCodePoint("md-hulu", 0xF828);
  addIconCodePoint("md-human", 0xF2E6);
  addIconCodePoint("md-human-child", 0xF2E7);
  addIconCodePoint("md-human-female", 0xF649);
  addIconCodePoint("md-human-female-boy", 0xFA58);
  addIconCodePoint("md-human-female-female", 0xFA59);
  addIconCodePoint("md-human-female-girl", 0xFA5A);
  addIconCodePoint("md-human-greeting", 0xF64A);
  addIconCodePoint("md-human-handsdown", 0xF64B);
  addIconCodePoint("md-human-handsup", 0xF64C);
  addIconCodePoint("md-human-male", 0xF64D);
  addIconCodePoint("md-human-male-boy", 0xFA5B);
  addIconCodePoint("md-human-male-female", 0xF2E8);
  addIconCodePoint("md-human-male-girl", 0xFA5C);
  addIconCodePoint("md-human-male-height", 0xFF18);
  addIconCodePoint("md-human-male-height-variant", 0xFF19);
  addIconCodePoint("md-human-male-male", 0xFA5D);
  addIconCodePoint("md-human-pregnant", 0xF5CF);
  addIconCodePoint("md-humble-bundle", 0xF743);
  addIconCodePoint("md-hvac", 0xF037D);
  addIconCodePoint("md-hydraulic-oil-level", 0xF034F);
  addIconCodePoint("md-hydraulic-oil-temperature", 0xF0350);
  addIconCodePoint("md-hydro-power", 0xF0310);
  addIconCodePoint("md-ice-cream", 0xF829);
  addIconCodePoint("md-ice-pop", 0xFF1A);
  addIconCodePoint("md-id-card", 0xFFE0);
  addIconCodePoint("md-identifier", 0xFF1B);
  addIconCodePoint("md-ideogram-cjk", 0xF035C);
  addIconCodePoint("md-ideogram-cjk-variant", 0xF035D);
  addIconCodePoint("md-iframe", 0xFC67);
  addIconCodePoint("md-iframe-array", 0xF0120);
  addIconCodePoint("md-iframe-array-outline", 0xF0121);
  addIconCodePoint("md-iframe-braces", 0xF0122);
  addIconCodePoint("md-iframe-braces-outline", 0xF0123);
  addIconCodePoint("md-iframe-outline", 0xFC68);
  addIconCodePoint("md-iframe-parentheses", 0xF0124);
  addIconCodePoint("md-iframe-parentheses-outline", 0xF0125);
  addIconCodePoint("md-iframe-variable", 0xF0126);
  addIconCodePoint("md-iframe-variable-outline", 0xF0127);
  addIconCodePoint("md-image", 0xF2E9);
  addIconCodePoint("md-image-album", 0xF2EA);
  addIconCodePoint("md-image-area", 0xF2EB);
  addIconCodePoint("md-image-area-close", 0xF2EC);
  addIconCodePoint("md-image-auto-adjust", 0xFFE1);
  addIconCodePoint("md-image-broken", 0xF2ED);
  addIconCodePoint("md-image-broken-variant", 0xF2EE);
  addIconCodePoint("md-image-edit", 0xF020E);
  addIconCodePoint("md-image-edit-outline", 0xF020F);
  addIconCodePoint("md-image-filter", 0xF2EF);
  addIconCodePoint("md-image-filter-black-white", 0xF2F0);
  addIconCodePoint("md-image-filter-center-focus", 0xF2F1);
  addIconCodePoint("md-image-filter-center-focus-strong", 0xFF1C);
  addIconCodePoint("md-image-filter-center-focus-strong-outline", 0xFF1D);
  addIconCodePoint("md-image-filter-center-focus-weak", 0xF2F2);
  addIconCodePoint("md-image-filter-drama", 0xF2F3);
  addIconCodePoint("md-image-filter-frames", 0xF2F4);
  addIconCodePoint("md-image-filter-hdr", 0xF2F5);
  addIconCodePoint("md-image-filter-none", 0xF2F6);
  addIconCodePoint("md-image-filter-tilt-shift", 0xF2F7);
  addIconCodePoint("md-image-filter-vintage", 0xF2F8);
  addIconCodePoint("md-image-frame", 0xFE8A);
  addIconCodePoint("md-image-move", 0xF9F7);
  addIconCodePoint("md-image-multiple", 0xF2F9);
  addIconCodePoint("md-image-off", 0xF82A);
  addIconCodePoint("md-image-off-outline", 0xF01FC);
  addIconCodePoint("md-image-outline", 0xF975);
  addIconCodePoint("md-image-plus", 0xF87B);
  addIconCodePoint("md-image-search", 0xF976);
  addIconCodePoint("md-image-search-outline", 0xF977);
  addIconCodePoint("md-image-size-select-actual", 0xFC69);
  addIconCodePoint("md-image-size-select-large", 0xFC6A);
  addIconCodePoint("md-image-size-select-small", 0xFC6B);
  addIconCodePoint("md-import", 0xF2FA);
  addIconCodePoint("md-inbox", 0xF686);
  addIconCodePoint("md-inbox-arrow-down", 0xF2FB);
  addIconCodePoint("md-inbox-arrow-down-outline", 0xF029B);
  addIconCodePoint("md-inbox-arrow-up", 0xF3D1);
  addIconCodePoint("md-inbox-arrow-up-outline", 0xF029C);
  addIconCodePoint("md-inbox-full", 0xF029D);
  addIconCodePoint("md-inbox-full-outline", 0xF029E);
  addIconCodePoint("md-inbox-multiple", 0xF8AF);
  addIconCodePoint("md-inbox-multiple-outline", 0xFB84);
  addIconCodePoint("md-inbox-outline", 0xF029F);
  addIconCodePoint("md-incognito", 0xF5F9);
  addIconCodePoint("md-infinity", 0xF6E3);
  addIconCodePoint("md-information", 0xF2FC);
  addIconCodePoint("md-information-outline", 0xF2FD);
  addIconCodePoint("md-information-variant", 0xF64E);
  addIconCodePoint("md-instagram", 0xF2FE);
  addIconCodePoint("md-instapaper", 0xF2FF);
  addIconCodePoint("md-instrument-triangle", 0xF0070);
  addIconCodePoint("md-internet-explorer", 0xF300);
  addIconCodePoint("md-invert-colors", 0xF301);
  addIconCodePoint("md-invert-colors-off", 0xFE8B);
  addIconCodePoint("md-iobroker", 0xF0313);
  addIconCodePoint("md-ip", 0xFA5E);
  addIconCodePoint("md-ip-network", 0xFA5F);
  addIconCodePoint("md-ip-network-outline", 0xFC6C);
  addIconCodePoint("md-ipod", 0xFC6D);
  addIconCodePoint("md-islam", 0xF978);
  addIconCodePoint("md-island", 0xF0071);
  addIconCodePoint("md-itunes", 0xF676);
  addIconCodePoint("md-iv-bag", 0xF00E4);
  addIconCodePoint("md-jabber", 0xFDB1);
  addIconCodePoint("md-jeepney", 0xF302);
  addIconCodePoint("md-jellyfish", 0xFF1E);
  addIconCodePoint("md-jellyfish-outline", 0xFF1F);
  addIconCodePoint("md-jira", 0xF303);
  addIconCodePoint("md-jquery", 0xF87C);
  addIconCodePoint("md-jsfiddle", 0xF304);
  addIconCodePoint("md-json", 0xF626);
  addIconCodePoint("md-judaism", 0xF979);
  addIconCodePoint("md-jump-rope", 0xF032A);
  addIconCodePoint("md-kabaddi", 0xFD63);
  addIconCodePoint("md-karate", 0xF82B);
  addIconCodePoint("md-keg", 0xF305);
  addIconCodePoint("md-kettle", 0xF5FA);
  addIconCodePoint("md-kettle-alert", 0xF0342);
  addIconCodePoint("md-kettle-alert-outline", 0xF0343);
  addIconCodePoint("md-kettle-off", 0xF0346);
  addIconCodePoint("md-kettle-off-outline", 0xF0347);
  addIconCodePoint("md-kettle-outline", 0xFF73);
  addIconCodePoint("md-kettle-steam", 0xF0344);
  addIconCodePoint("md-kettle-steam-outline", 0xF0345);
  addIconCodePoint("md-kettlebell", 0xF032B);
  addIconCodePoint("md-key", 0xF306);
  addIconCodePoint("md-key-arrow-right", 0xF033D);
  addIconCodePoint("md-key-change", 0xF307);
  addIconCodePoint("md-key-link", 0xF01CA);
  addIconCodePoint("md-key-minus", 0xF308);
  addIconCodePoint("md-key-outline", 0xFDB2);
  addIconCodePoint("md-key-plus", 0xF309);
  addIconCodePoint("md-key-remove", 0xF30A);
  addIconCodePoint("md-key-star", 0xF01C9);
  addIconCodePoint("md-key-variant", 0xF30B);
  addIconCodePoint("md-key-wireless", 0xFFE2);
  addIconCodePoint("md-keyboard", 0xF30C);
  addIconCodePoint("md-keyboard-backspace", 0xF30D);
  addIconCodePoint("md-keyboard-caps", 0xF30E);
  addIconCodePoint("md-keyboard-close", 0xF30F);
  addIconCodePoint("md-keyboard-esc", 0xF02E2);
  addIconCodePoint("md-keyboard-f1", 0xF02D6);
  addIconCodePoint("md-keyboard-f10", 0xF02DF);
  addIconCodePoint("md-keyboard-f11", 0xF02E0);
  addIconCodePoint("md-keyboard-f12", 0xF02E1);
  addIconCodePoint("md-keyboard-f2", 0xF02D7);
  addIconCodePoint("md-keyboard-f3", 0xF02D8);
  addIconCodePoint("md-keyboard-f4", 0xF02D9);
  addIconCodePoint("md-keyboard-f5", 0xF02DA);
  addIconCodePoint("md-keyboard-f6", 0xF02DB);
  addIconCodePoint("md-keyboard-f7", 0xF02DC);
  addIconCodePoint("md-keyboard-f8", 0xF02DD);
  addIconCodePoint("md-keyboard-f9", 0xF02DE);
  addIconCodePoint("md-keyboard-off", 0xF310);
  addIconCodePoint("md-keyboard-off-outline", 0xFE8C);
  addIconCodePoint("md-keyboard-outline", 0xF97A);
  addIconCodePoint("md-keyboard-return", 0xF311);
  addIconCodePoint("md-keyboard-settings", 0xF9F8);
  addIconCodePoint("md-keyboard-settings-outline", 0xF9F9);
  addIconCodePoint("md-keyboard-space", 0xF0072);
  addIconCodePoint("md-keyboard-tab", 0xF312);
  addIconCodePoint("md-keyboard-variant", 0xF313);
  addIconCodePoint("md-khanda", 0xF0128);
  addIconCodePoint("md-kickstarter", 0xF744);
  addIconCodePoint("md-klingon", 0xF0386);
  addIconCodePoint("md-knife", 0xF9FA);
  addIconCodePoint("md-knife-military", 0xF9FB);
  addIconCodePoint("md-kodi", 0xF314);
  addIconCodePoint("md-kotlin", 0xF0244);
  addIconCodePoint("md-kubernetes", 0xF0129);
  addIconCodePoint("md-label", 0xF315);
  addIconCodePoint("md-label-multiple", 0xF03A0);
  addIconCodePoint("md-label-multiple-outline", 0xF03A1);
  addIconCodePoint("md-label-off", 0xFACA);
  addIconCodePoint("md-label-off-outline", 0xFACB);
  addIconCodePoint("md-label-outline", 0xF316);
  addIconCodePoint("md-label-percent", 0xF0315);
  addIconCodePoint("md-label-percent-outline", 0xF0316);
  addIconCodePoint("md-label-variant", 0xFACC);
  addIconCodePoint("md-label-variant-outline", 0xFACD);
  addIconCodePoint("md-ladybug", 0xF82C);
  addIconCodePoint("md-lambda", 0xF627);
  addIconCodePoint("md-lamp", 0xF6B4);
  addIconCodePoint("md-lan", 0xF317);
  addIconCodePoint("md-lan-check", 0xF02D5);
  addIconCodePoint("md-lan-connect", 0xF318);
  addIconCodePoint("md-lan-disconnect", 0xF319);
  addIconCodePoint("md-lan-pending", 0xF31A);
  addIconCodePoint("md-language-c", 0xF671);
  addIconCodePoint("md-language-cpp", 0xF672);
  addIconCodePoint("md-language-csharp", 0xF31B);
  addIconCodePoint("md-language-css3", 0xF31C);
  addIconCodePoint("md-language-fortran", 0xF0245);
  addIconCodePoint("md-language-go", 0xF7D2);
  addIconCodePoint("md-language-haskell", 0xFC6E);
  addIconCodePoint("md-language-html5", 0xF31D);
  addIconCodePoint("md-language-java", 0xFB1C);
  addIconCodePoint("md-language-javascript", 0xF31E);
  addIconCodePoint("md-language-lua", 0xF8B0);
  addIconCodePoint("md-language-php", 0xF31F);
  addIconCodePoint("md-language-python", 0xF320);
  addIconCodePoint("md-language-python-text", 0xF321);
  addIconCodePoint("md-language-r", 0xF7D3);
  addIconCodePoint("md-language-ruby-on-rails", 0xFACE);
  addIconCodePoint("md-language-swift", 0xF6E4);
  addIconCodePoint("md-language-typescript", 0xF6E5);
  addIconCodePoint("md-laptop", 0xF322);
  addIconCodePoint("md-laptop-chromebook", 0xF323);
  addIconCodePoint("md-laptop-mac", 0xF324);
  addIconCodePoint("md-laptop-off", 0xF6E6);
  addIconCodePoint("md-laptop-windows", 0xF325);
  addIconCodePoint("md-laravel", 0xFACF);
  addIconCodePoint("md-lasso", 0xFF20);
  addIconCodePoint("md-lastfm", 0xF326);
  addIconCodePoint("md-lastpass", 0xF446);
  addIconCodePoint("md-latitude", 0xFF74);
  addIconCodePoint("md-launch", 0xF327);
  addIconCodePoint("md-lava-lamp", 0xF7D4);
  addIconCodePoint("md-layers", 0xF328);
  addIconCodePoint("md-layers-minus", 0xFE8D);
  addIconCodePoint("md-layers-off", 0xF329);
  addIconCodePoint("md-layers-off-outline", 0xF9FC);
  addIconCodePoint("md-layers-outline", 0xF9FD);
  addIconCodePoint("md-layers-plus", 0xFE30);
  addIconCodePoint("md-layers-remove", 0xFE31);
  addIconCodePoint("md-layers-search", 0xF0231);
  addIconCodePoint("md-layers-search-outline", 0xF0232);
  addIconCodePoint("md-layers-triple", 0xFF75);
  addIconCodePoint("md-layers-triple-outline", 0xFF76);
  addIconCodePoint("md-lead-pencil", 0xF64F);
  addIconCodePoint("md-leaf", 0xF32A);
  addIconCodePoint("md-leaf-maple", 0xFC6F);
  addIconCodePoint("md-leaf-maple-off", 0xF0305);
  addIconCodePoint("md-leaf-off", 0xF0304);
  addIconCodePoint("md-leak", 0xFDB3);
  addIconCodePoint("md-leak-off", 0xFDB4);
  addIconCodePoint("md-led-off", 0xF32B);
  addIconCodePoint("md-led-on", 0xF32C);
  addIconCodePoint("md-led-outline", 0xF32D);
  addIconCodePoint("md-led-strip", 0xF7D5);
  addIconCodePoint("md-led-strip-variant", 0xF0073);
  addIconCodePoint("md-led-variant-off", 0xF32E);
  addIconCodePoint("md-led-variant-on", 0xF32F);
  addIconCodePoint("md-led-variant-outline", 0xF330);
  addIconCodePoint("md-leek", 0xF01A8);
  addIconCodePoint("md-less-than", 0xF97B);
  addIconCodePoint("md-less-than-or-equal", 0xF97C);
  addIconCodePoint("md-library", 0xF331);
  addIconCodePoint("md-library-books", 0xF332);
  addIconCodePoint("md-library-movie", 0xFCF4);
  addIconCodePoint("md-library-music", 0xF333);
  addIconCodePoint("md-library-music-outline", 0xFF21);
  addIconCodePoint("md-library-shelves", 0xFB85);
  addIconCodePoint("md-library-video", 0xFCF5);
  addIconCodePoint("md-license", 0xFFE3);
  addIconCodePoint("md-lifebuoy", 0xF87D);
  addIconCodePoint("md-light-switch", 0xF97D);
  addIconCodePoint("md-lightbulb", 0xF335);
  addIconCodePoint("md-lightbulb-cfl", 0xF0233);
  addIconCodePoint("md-lightbulb-cfl-off", 0xF0234);
  addIconCodePoint("md-lightbulb-cfl-spiral", 0xF02A0);
  addIconCodePoint("md-lightbulb-cfl-spiral-off", 0xF02EE);
  addIconCodePoint("md-lightbulb-group", 0xF027E);
  addIconCodePoint("md-lightbulb-group-off", 0xF02F8);
  addIconCodePoint("md-lightbulb-group-off-outline", 0xF02F9);
  addIconCodePoint("md-lightbulb-group-outline", 0xF027F);
  addIconCodePoint("md-lightbulb-multiple", 0xF0280);
  addIconCodePoint("md-lightbulb-multiple-off", 0xF02FA);
  addIconCodePoint("md-lightbulb-multiple-off-outline", 0xF02FB);
  addIconCodePoint("md-lightbulb-multiple-outline", 0xF0281);
  addIconCodePoint("md-lightbulb-off", 0xFE32);
  addIconCodePoint("md-lightbulb-off-outline", 0xFE33);
  addIconCodePoint("md-lightbulb-on", 0xF6E7);
  addIconCodePoint("md-lightbulb-on-outline", 0xF6E8);
  addIconCodePoint("md-lightbulb-outline", 0xF336);
  addIconCodePoint("md-lighthouse", 0xF9FE);
  addIconCodePoint("md-lighthouse-on", 0xF9FF);
  addIconCodePoint("md-link", 0xF337);
  addIconCodePoint("md-link-box", 0xFCF6);
  addIconCodePoint("md-link-box-outline", 0xFCF7);
  addIconCodePoint("md-link-box-variant", 0xFCF8);
  addIconCodePoint("md-link-box-variant-outline", 0xFCF9);
  addIconCodePoint("md-link-lock", 0xF00E5);
  addIconCodePoint("md-link-off", 0xF338);
  addIconCodePoint("md-link-plus", 0xFC70);
  addIconCodePoint("md-link-variant", 0xF339);
  addIconCodePoint("md-link-variant-minus", 0xF012A);
  addIconCodePoint("md-link-variant-off", 0xF33A);
  addIconCodePoint("md-link-variant-plus", 0xF012B);
  addIconCodePoint("md-link-variant-remove", 0xF012C);
  addIconCodePoint("md-linkedin", 0xF33B);
  addIconCodePoint("md-linkedin-box", 0xF33C);
  addIconCodePoint("md-linux", 0xF33D);
  addIconCodePoint("md-linux-mint", 0xF8EC);
  addIconCodePoint("md-litecoin", 0xFA60);
  addIconCodePoint("md-loading", 0xF771);
  addIconCodePoint("md-location-enter", 0xFFE4);
  addIconCodePoint("md-location-exit", 0xFFE5);
  addIconCodePoint("md-lock", 0xF33E);
  addIconCodePoint("md-lock-alert", 0xF8ED);
  addIconCodePoint("md-lock-clock", 0xF97E);
  addIconCodePoint("md-lock-open", 0xF33F);
  addIconCodePoint("md-lock-open-outline", 0xF340);
  addIconCodePoint("md-lock-open-variant", 0xFFE6);
  addIconCodePoint("md-lock-open-variant-outline", 0xFFE7);
  addIconCodePoint("md-lock-outline", 0xF341);
  addIconCodePoint("md-lock-pattern", 0xF6E9);
  addIconCodePoint("md-lock-plus", 0xF5FB);
  addIconCodePoint("md-lock-question", 0xF8EE);
  addIconCodePoint("md-lock-reset", 0xF772);
  addIconCodePoint("md-lock-smart", 0xF8B1);
  addIconCodePoint("md-locker", 0xF7D6);
  addIconCodePoint("md-locker-multiple", 0xF7D7);
  addIconCodePoint("md-login", 0xF342);
  addIconCodePoint("md-login-variant", 0xF5FC);
  addIconCodePoint("md-logout", 0xF343);
  addIconCodePoint("md-logout-variant", 0xF5FD);
  addIconCodePoint("md-longitude", 0xFF77);
  addIconCodePoint("md-looks", 0xF344);
  addIconCodePoint("md-loupe", 0xF345);
  addIconCodePoint("md-lumx", 0xF346);
  addIconCodePoint("md-lungs", 0xF00AF);
  addIconCodePoint("md-lyft", 0xFB1D);
  addIconCodePoint("md-magnet", 0xF347);
  addIconCodePoint("md-magnet-on", 0xF348);
  addIconCodePoint("md-magnify", 0xF349);
  addIconCodePoint("md-magnify-close", 0xF97F);
  addIconCodePoint("md-magnify-minus", 0xF34A);
  addIconCodePoint("md-magnify-minus-cursor", 0xFA61);
  addIconCodePoint("md-magnify-minus-outline", 0xF6EB);
  addIconCodePoint("md-magnify-plus", 0xF34B);
  addIconCodePoint("md-magnify-plus-cursor", 0xFA62);
  addIconCodePoint("md-magnify-plus-outline", 0xF6EC);
  addIconCodePoint("md-magnify-remove-cursor", 0xF0237);
  addIconCodePoint("md-magnify-remove-outline", 0xF0238);
  addIconCodePoint("md-magnify-scan", 0xF02A1);
  addIconCodePoint("md-mail", 0xFED8);
  addIconCodePoint("md-mail-ru", 0xF34C);
  addIconCodePoint("md-mailbox", 0xF6ED);
  addIconCodePoint("md-mailbox-open", 0xFD64);
  addIconCodePoint("md-mailbox-open-outline", 0xFD65);
  addIconCodePoint("md-mailbox-open-up", 0xFD66);
  addIconCodePoint("md-mailbox-open-up-outline", 0xFD67);
  addIconCodePoint("md-mailbox-outline", 0xFD68);
  addIconCodePoint("md-mailbox-up", 0xFD69);
  addIconCodePoint("md-mailbox-up-outline", 0xFD6A);
  addIconCodePoint("md-map", 0xF34D);
  addIconCodePoint("md-map-check", 0xFED9);
  addIconCodePoint("md-map-check-outline", 0xFEDA);
  addIconCodePoint("md-map-clock", 0xFCFA);
  addIconCodePoint("md-map-clock-outline", 0xFCFB);
  addIconCodePoint("md-map-legend", 0xFA00);
  addIconCodePoint("md-map-marker", 0xF34E);
  addIconCodePoint("md-map-marker-alert", 0xFF22);
  addIconCodePoint("md-map-marker-alert-outline", 0xFF23);
  addIconCodePoint("md-map-marker-check", 0xFC71);
  addIconCodePoint("md-map-marker-check-outline", 0xF0326);
  addIconCodePoint("md-map-marker-circle", 0xF34F);
  addIconCodePoint("md-map-marker-distance", 0xF8EF);
  addIconCodePoint("md-map-marker-down", 0xF012D);
  addIconCodePoint("md-map-marker-left", 0xF0306);
  addIconCodePoint("md-map-marker-left-outline", 0xF0308);
  addIconCodePoint("md-map-marker-minus", 0xF650);
  addIconCodePoint("md-map-marker-minus-outline", 0xF0324);
  addIconCodePoint("md-map-marker-multiple", 0xF350);
  addIconCodePoint("md-map-marker-multiple-outline", 0xF02A2);
  addIconCodePoint("md-map-marker-off", 0xF351);
  addIconCodePoint("md-map-marker-off-outline", 0xF0328);
  addIconCodePoint("md-map-marker-outline", 0xF7D8);
  addIconCodePoint("md-map-marker-path", 0xFCFC);
  addIconCodePoint("md-map-marker-plus", 0xF651);
  addIconCodePoint("md-map-marker-plus-outline", 0xF0323);
  addIconCodePoint("md-map-marker-question", 0xFF24);
  addIconCodePoint("md-map-marker-question-outline", 0xFF25);
  addIconCodePoint("md-map-marker-radius", 0xF352);
  addIconCodePoint("md-map-marker-radius-outline", 0xF0327);
  addIconCodePoint("md-map-marker-remove", 0xFF26);
  addIconCodePoint("md-map-marker-remove-outline", 0xF0325);
  addIconCodePoint("md-map-marker-remove-variant", 0xFF27);
  addIconCodePoint("md-map-marker-right", 0xF0307);
  addIconCodePoint("md-map-marker-right-outline", 0xF0309);
  addIconCodePoint("md-map-marker-up", 0xF012E);
  addIconCodePoint("md-map-minus", 0xF980);
  addIconCodePoint("md-map-outline", 0xF981);
  addIconCodePoint("md-map-plus", 0xF982);
  addIconCodePoint("md-map-search", 0xF983);
  addIconCodePoint("md-map-search-outline", 0xF984);
  addIconCodePoint("md-mapbox", 0xFB86);
  addIconCodePoint("md-margin", 0xF353);
  addIconCodePoint("md-markdown", 0xF354);
  addIconCodePoint("md-markdown-outline", 0xFF78);
  addIconCodePoint("md-marker", 0xF652);
  addIconCodePoint("md-marker-cancel", 0xFDB5);
  addIconCodePoint("md-marker-check", 0xF355);
  addIconCodePoint("md-mastodon", 0xFAD0);
  addIconCodePoint("md-mastodon-variant", 0xFAD1);
  addIconCodePoint("md-material-design", 0xF985);
  addIconCodePoint("md-material-ui", 0xF357);
  addIconCodePoint("md-math-compass", 0xF358);
  addIconCodePoint("md-math-cos", 0xFC72);
  addIconCodePoint("md-math-integral", 0xFFE8);
  addIconCodePoint("md-math-integral-box", 0xFFE9);
  addIconCodePoint("md-math-log", 0xF00B0);
  addIconCodePoint("md-math-norm", 0xFFEA);
  addIconCodePoint("md-math-norm-box", 0xFFEB);
  addIconCodePoint("md-math-sin", 0xFC73);
  addIconCodePoint("md-math-tan", 0xFC74);
  addIconCodePoint("md-matrix", 0xF628);
  addIconCodePoint("md-medal", 0xF986);
  addIconCodePoint("md-medal-outline", 0xF0351);
  addIconCodePoint("md-medical-bag", 0xF6EE);
  addIconCodePoint("md-meditation", 0xF01A6);
  addIconCodePoint("md-medium", 0xF35A);
  addIconCodePoint("md-meetup", 0xFAD2);
  addIconCodePoint("md-memory", 0xF35B);
  addIconCodePoint("md-menu", 0xF35C);
  addIconCodePoint("md-menu-down", 0xF35D);
  addIconCodePoint("md-menu-down-outline", 0xF6B5);
  addIconCodePoint("md-menu-left", 0xF35E);
  addIconCodePoint("md-menu-left-outline", 0xFA01);
  addIconCodePoint("md-menu-open", 0xFB87);
  addIconCodePoint("md-menu-right", 0xF35F);
  addIconCodePoint("md-menu-right-outline", 0xFA02);
  addIconCodePoint("md-menu-swap", 0xFA63);
  addIconCodePoint("md-menu-swap-outline", 0xFA64);
  addIconCodePoint("md-menu-up", 0xF360);
  addIconCodePoint("md-menu-up-outline", 0xF6B6);
  addIconCodePoint("md-merge", 0xFF79);
  addIconCodePoint("md-message", 0xF361);
  addIconCodePoint("md-message-alert", 0xF362);
  addIconCodePoint("md-message-alert-outline", 0xFA03);
  addIconCodePoint("md-message-arrow-left", 0xF031D);
  addIconCodePoint("md-message-arrow-left-outline", 0xF031E);
  addIconCodePoint("md-message-arrow-right", 0xF031F);
  addIconCodePoint("md-message-arrow-right-outline", 0xF0320);
  addIconCodePoint("md-message-bulleted", 0xF6A1);
  addIconCodePoint("md-message-bulleted-off", 0xF6A2);
  addIconCodePoint("md-message-draw", 0xF363);
  addIconCodePoint("md-message-image", 0xF364);
  addIconCodePoint("md-message-image-outline", 0xF0197);
  addIconCodePoint("md-message-lock", 0xFFEC);
  addIconCodePoint("md-message-lock-outline", 0xF0198);
  addIconCodePoint("md-message-minus", 0xF0199);
  addIconCodePoint("md-message-minus-outline", 0xF019A);
  addIconCodePoint("md-message-outline", 0xF365);
  addIconCodePoint("md-message-plus", 0xF653);
  addIconCodePoint("md-message-plus-outline", 0xF00E6);
  addIconCodePoint("md-message-processing", 0xF366);
  addIconCodePoint("md-message-processing-outline", 0xF019B);
  addIconCodePoint("md-message-reply", 0xF367);
  addIconCodePoint("md-message-reply-text", 0xF368);
  addIconCodePoint("md-message-settings", 0xF6EF);
  addIconCodePoint("md-message-settings-outline", 0xF019C);
  addIconCodePoint("md-message-settings-variant", 0xF6F0);
  addIconCodePoint("md-message-settings-variant-outline", 0xF019D);
  addIconCodePoint("md-message-text", 0xF369);
  addIconCodePoint("md-message-text-clock", 0xF019E);
  addIconCodePoint("md-message-text-clock-outline", 0xF019F);
  addIconCodePoint("md-message-text-lock", 0xFFED);
  addIconCodePoint("md-message-text-lock-outline", 0xF01A0);
  addIconCodePoint("md-message-text-outline", 0xF36A);
  addIconCodePoint("md-message-video", 0xF36B);
  addIconCodePoint("md-meteor", 0xF629);
  addIconCodePoint("md-metronome", 0xF7D9);
  addIconCodePoint("md-metronome-tick", 0xF7DA);
  addIconCodePoint("md-micro-sd", 0xF7DB);
  addIconCodePoint("md-microphone", 0xF36C);
  addIconCodePoint("md-microphone-minus", 0xF8B2);
  addIconCodePoint("md-microphone-off", 0xF36D);
  addIconCodePoint("md-microphone-outline", 0xF36E);
  addIconCodePoint("md-microphone-plus", 0xF8B3);
  addIconCodePoint("md-microphone-settings", 0xF36F);
  addIconCodePoint("md-microphone-variant", 0xF370);
  addIconCodePoint("md-microphone-variant-off", 0xF371);
  addIconCodePoint("md-microscope", 0xF654);
  addIconCodePoint("md-microsoft", 0xF372);
  addIconCodePoint("md-microsoft-dynamics", 0xF987);
  addIconCodePoint("md-microwave", 0xFC75);
  addIconCodePoint("md-middleware", 0xFF7A);
  addIconCodePoint("md-middleware-outline", 0xFF7B);
  addIconCodePoint("md-midi", 0xF8F0);
  addIconCodePoint("md-midi-port", 0xF8F1);
  addIconCodePoint("md-mine", 0xFDB6);
  addIconCodePoint("md-minecraft", 0xF373);
  addIconCodePoint("md-mini-sd", 0xFA04);
  addIconCodePoint("md-minidisc", 0xFA05);
  addIconCodePoint("md-minus", 0xF374);
  addIconCodePoint("md-minus-box", 0xF375);
  addIconCodePoint("md-minus-box-multiple", 0xF016C);
  addIconCodePoint("md-minus-box-multiple-outline", 0xF016D);
  addIconCodePoint("md-minus-box-outline", 0xF6F1);
  addIconCodePoint("md-minus-circle", 0xF376);
  addIconCodePoint("md-minus-circle-outline", 0xF377);
  addIconCodePoint("md-minus-network", 0xF378);
  addIconCodePoint("md-minus-network-outline", 0xFC76);
  addIconCodePoint("md-mirror", 0xF0228);
  addIconCodePoint("md-mixcloud", 0xF62A);
  addIconCodePoint("md-mixed-martial-arts", 0xFD6B);
  addIconCodePoint("md-mixed-reality", 0xF87E);
  addIconCodePoint("md-mixer", 0xF7DC);
  addIconCodePoint("md-molecule", 0xFB88);
  addIconCodePoint("md-monitor", 0xF379);
  addIconCodePoint("md-monitor-cellphone", 0xF988);
  addIconCodePoint("md-monitor-cellphone-star", 0xF989);
  addIconCodePoint("md-monitor-clean", 0xF012F);
  addIconCodePoint("md-monitor-dashboard", 0xFA06);
  addIconCodePoint("md-monitor-edit", 0xF02F1);
  addIconCodePoint("md-monitor-lock", 0xFDB7);
  addIconCodePoint("md-monitor-multiple", 0xF37A);
  addIconCodePoint("md-monitor-off", 0xFD6C);
  addIconCodePoint("md-monitor-screenshot", 0xFE34);
  addIconCodePoint("md-monitor-speaker", 0xFF7C);
  addIconCodePoint("md-monitor-speaker-off", 0xFF7D);
  addIconCodePoint("md-monitor-star", 0xFDB8);
  addIconCodePoint("md-moon-first-quarter", 0xFF7E);
  addIconCodePoint("md-moon-full", 0xFF7F);
  addIconCodePoint("md-moon-last-quarter", 0xFF80);
  addIconCodePoint("md-moon-new", 0xFF81);
  addIconCodePoint("md-moon-waning-crescent", 0xFF82);
  addIconCodePoint("md-moon-waning-gibbous", 0xFF83);
  addIconCodePoint("md-moon-waxing-crescent", 0xFF84);
  addIconCodePoint("md-moon-waxing-gibbous", 0xFF85);
  addIconCodePoint("md-moped", 0xF00B1);
  addIconCodePoint("md-more", 0xF37B);
  addIconCodePoint("md-mother-heart", 0xF033F);
  addIconCodePoint("md-mother-nurse", 0xFCFD);
  addIconCodePoint("md-motion-sensor", 0xFD6D);
  addIconCodePoint("md-motorbike", 0xF37C);
  addIconCodePoint("md-mouse", 0xF37D);
  addIconCodePoint("md-mouse-bluetooth", 0xF98A);
  addIconCodePoint("md-mouse-off", 0xF37E);
  addIconCodePoint("md-mouse-variant", 0xF37F);
  addIconCodePoint("md-mouse-variant-off", 0xF380);
  addIconCodePoint("md-move-resize", 0xF655);
  addIconCodePoint("md-move-resize-variant", 0xF656);
  addIconCodePoint("md-movie", 0xF381);
  addIconCodePoint("md-movie-edit", 0xF014D);
  addIconCodePoint("md-movie-edit-outline", 0xF014E);
  addIconCodePoint("md-movie-filter", 0xF014F);
  addIconCodePoint("md-movie-filter-outline", 0xF0150);
  addIconCodePoint("md-movie-open", 0xFFEE);
  addIconCodePoint("md-movie-open-outline", 0xFFEF);
  addIconCodePoint("md-movie-outline", 0xFDB9);
  addIconCodePoint("md-movie-roll", 0xF7DD);
  addIconCodePoint("md-movie-search", 0xF01FD);
  addIconCodePoint("md-movie-search-outline", 0xF01FE);
  addIconCodePoint("md-muffin", 0xF98B);
  addIconCodePoint("md-multiplication", 0xF382);
  addIconCodePoint("md-multiplication-box", 0xF383);
  addIconCodePoint("md-mushroom", 0xF7DE);
  addIconCodePoint("md-mushroom-outline", 0xF7DF);
  addIconCodePoint("md-music", 0xF759);
  addIconCodePoint("md-music-accidental-double-flat", 0xFF86);
  addIconCodePoint("md-music-accidental-double-sharp", 0xFF87);
  addIconCodePoint("md-music-accidental-flat", 0xFF88);
  addIconCodePoint("md-music-accidental-natural", 0xFF89);
  addIconCodePoint("md-music-accidental-sharp", 0xFF8A);
  addIconCodePoint("md-music-box", 0xF384);
  addIconCodePoint("md-music-box-outline", 0xF385);
  addIconCodePoint("md-music-circle", 0xF386);
  addIconCodePoint("md-music-circle-outline", 0xFAD3);
  addIconCodePoint("md-music-clef-alto", 0xFF8B);
  addIconCodePoint("md-music-clef-bass", 0xFF8C);
  addIconCodePoint("md-music-clef-treble", 0xFF8D);
  addIconCodePoint("md-music-note", 0xF387);
  addIconCodePoint("md-music-note-bluetooth", 0xF5FE);
  addIconCodePoint("md-music-note-bluetooth-off", 0xF5FF);
  addIconCodePoint("md-music-note-eighth", 0xF388);
  addIconCodePoint("md-music-note-eighth-dotted", 0xFF8E);
  addIconCodePoint("md-music-note-half", 0xF389);
  addIconCodePoint("md-music-note-half-dotted", 0xFF8F);
  addIconCodePoint("md-music-note-off", 0xF38A);
  addIconCodePoint("md-music-note-off-outline", 0xFF90);
  addIconCodePoint("md-music-note-outline", 0xFF91);
  addIconCodePoint("md-music-note-plus", 0xFDBA);
  addIconCodePoint("md-music-note-quarter", 0xF38B);
  addIconCodePoint("md-music-note-quarter-dotted", 0xFF92);
  addIconCodePoint("md-music-note-sixteenth", 0xF38C);
  addIconCodePoint("md-music-note-sixteenth-dotted", 0xFF93);
  addIconCodePoint("md-music-note-whole", 0xF38D);
  addIconCodePoint("md-music-note-whole-dotted", 0xFF94);
  addIconCodePoint("md-music-off", 0xF75A);
  addIconCodePoint("md-music-rest-eighth", 0xFF95);
  addIconCodePoint("md-music-rest-half", 0xFF96);
  addIconCodePoint("md-music-rest-quarter", 0xFF97);
  addIconCodePoint("md-music-rest-sixteenth", 0xFF98);
  addIconCodePoint("md-music-rest-whole", 0xFF99);
  addIconCodePoint("md-nail", 0xFDBB);
  addIconCodePoint("md-nas", 0xF8F2);
  addIconCodePoint("md-nativescript", 0xF87F);
  addIconCodePoint("md-nature", 0xF38E);
  addIconCodePoint("md-nature-people", 0xF38F);
  addIconCodePoint("md-navigation", 0xF390);
  addIconCodePoint("md-near-me", 0xF5CD);
  addIconCodePoint("md-necklace", 0xFF28);
  addIconCodePoint("md-needle", 0xF391);
  addIconCodePoint("md-netflix", 0xF745);
  addIconCodePoint("md-network", 0xF6F2);
  addIconCodePoint("md-network-off", 0xFC77);
  addIconCodePoint("md-network-off-outline", 0xFC78);
  addIconCodePoint("md-network-outline", 0xFC79);
  addIconCodePoint("md-network-router", 0xF00B2);
  addIconCodePoint("md-network-strength-1", 0xF8F3);
  addIconCodePoint("md-network-strength-1-alert", 0xF8F4);
  addIconCodePoint("md-network-strength-2", 0xF8F5);
  addIconCodePoint("md-network-strength-2-alert", 0xF8F6);
  addIconCodePoint("md-network-strength-3", 0xF8F7);
  addIconCodePoint("md-network-strength-3-alert", 0xF8F8);
  addIconCodePoint("md-network-strength-4", 0xF8F9);
  addIconCodePoint("md-network-strength-4-alert", 0xF8FA);
  addIconCodePoint("md-network-strength-off", 0xF8FB);
  addIconCodePoint("md-network-strength-off-outline", 0xF8FC);
  addIconCodePoint("md-network-strength-outline", 0xF8FD);
  addIconCodePoint("md-new-box", 0xF394);
  addIconCodePoint("md-newspaper", 0xF395);
  addIconCodePoint("md-newspaper-minus", 0xFF29);
  addIconCodePoint("md-newspaper-plus", 0xFF2A);
  addIconCodePoint("md-newspaper-variant", 0xF0023);
  addIconCodePoint("md-newspaper-variant-multiple", 0xF0024);
  addIconCodePoint("md-newspaper-variant-multiple-outline", 0xF0025);
  addIconCodePoint("md-newspaper-variant-outline", 0xF0026);
  addIconCodePoint("md-nfc", 0xF396);
  addIconCodePoint("md-nfc-off", 0xFE35);
  addIconCodePoint("md-nfc-search-variant", 0xFE36);
  addIconCodePoint("md-nfc-tap", 0xF397);
  addIconCodePoint("md-nfc-variant", 0xF398);
  addIconCodePoint("md-nfc-variant-off", 0xFE37);
  addIconCodePoint("md-ninja", 0xF773);
  addIconCodePoint("md-nintendo-switch", 0xF7E0);
  addIconCodePoint("md-nix", 0xF0130);
  addIconCodePoint("md-nodejs", 0xF399);
  addIconCodePoint("md-noodles", 0xF01A9);
  addIconCodePoint("md-not-equal", 0xF98C);
  addIconCodePoint("md-not-equal-variant", 0xF98D);
  addIconCodePoint("md-note", 0xF39A);
  addIconCodePoint("md-note-multiple", 0xF6B7);
  addIconCodePoint("md-note-multiple-outline", 0xF6B8);
  addIconCodePoint("md-note-outline", 0xF39B);
  addIconCodePoint("md-note-plus", 0xF39C);
  addIconCodePoint("md-note-plus-outline", 0xF39D);
  addIconCodePoint("md-note-text", 0xF39E);
  addIconCodePoint("md-note-text-outline", 0xF0202);
  addIconCodePoint("md-notebook", 0xF82D);
  addIconCodePoint("md-notebook-multiple", 0xFE38);
  addIconCodePoint("md-notebook-outline", 0xFEDC);
  addIconCodePoint("md-notification-clear-all", 0xF39F);
  addIconCodePoint("md-npm", 0xF6F6);
  addIconCodePoint("md-npm-variant", 0xF98E);
  addIconCodePoint("md-npm-variant-outline", 0xF98F);
  addIconCodePoint("md-nuke", 0xF6A3);
  addIconCodePoint("md-null", 0xF7E1);
  addIconCodePoint("md-numeric", 0xF3A0);
  addIconCodePoint("md-numeric-0", 0x0030);
  addIconCodePoint("md-numeric-0-box", 0xF3A1);
  addIconCodePoint("md-numeric-0-box-multiple", 0xFF2B);
  addIconCodePoint("md-numeric-0-box-multiple-outline", 0xF3A2);
  addIconCodePoint("md-numeric-0-box-outline", 0xF3A3);
  addIconCodePoint("md-numeric-0-circle", 0xFC7A);
  addIconCodePoint("md-numeric-0-circle-outline", 0xFC7B);
  addIconCodePoint("md-numeric-1", 0x0031);
  addIconCodePoint("md-numeric-1-box", 0xF3A4);
  addIconCodePoint("md-numeric-1-box-multiple", 0xFF2C);
  addIconCodePoint("md-numeric-1-box-multiple-outline", 0xF3A5);
  addIconCodePoint("md-numeric-1-box-outline", 0xF3A6);
  addIconCodePoint("md-numeric-1-circle", 0xFC7C);
  addIconCodePoint("md-numeric-1-circle-outline", 0xFC7D);
  addIconCodePoint("md-numeric-10", 0xF000A);
  addIconCodePoint("md-numeric-10-box", 0xFF9A);
  addIconCodePoint("md-numeric-10-box-multiple", 0xF000B);
  addIconCodePoint("md-numeric-10-box-multiple-outline", 0xF000C);
  addIconCodePoint("md-numeric-10-box-outline", 0xFF9B);
  addIconCodePoint("md-numeric-10-circle", 0xF000D);
  addIconCodePoint("md-numeric-10-circle-outline", 0xF000E);
  addIconCodePoint("md-numeric-2", 0x0032);
  addIconCodePoint("md-numeric-2-box", 0xF3A7);
  addIconCodePoint("md-numeric-2-box-multiple", 0xFF2D);
  addIconCodePoint("md-numeric-2-box-multiple-outline", 0xF3A8);
  addIconCodePoint("md-numeric-2-box-outline", 0xF3A9);
  addIconCodePoint("md-numeric-2-circle", 0xFC7E);
  addIconCodePoint("md-numeric-2-circle-outline", 0xFC7F);
  addIconCodePoint("md-numeric-3", 0x0033);
  addIconCodePoint("md-numeric-3-box", 0xF3AA);
  addIconCodePoint("md-numeric-3-box-multiple", 0xFF2E);
  addIconCodePoint("md-numeric-3-box-multiple-outline", 0xF3AB);
  addIconCodePoint("md-numeric-3-box-outline", 0xF3AC);
  addIconCodePoint("md-numeric-3-circle", 0xFC80);
  addIconCodePoint("md-numeric-3-circle-outline", 0xFC81);
  addIconCodePoint("md-numeric-4", 0x0034);
  addIconCodePoint("md-numeric-4-box", 0xF3AD);
  addIconCodePoint("md-numeric-4-box-multiple", 0xFF2F);
  addIconCodePoint("md-numeric-4-box-multiple-outline", 0xF3AE);
  addIconCodePoint("md-numeric-4-box-outline", 0xF3AF);
  addIconCodePoint("md-numeric-4-circle", 0xFC82);
  addIconCodePoint("md-numeric-4-circle-outline", 0xFC83);
  addIconCodePoint("md-numeric-5", 0x0035);
  addIconCodePoint("md-numeric-5-box", 0xF3B0);
  addIconCodePoint("md-numeric-5-box-multiple", 0xFF30);
  addIconCodePoint("md-numeric-5-box-multiple-outline", 0xF3B1);
  addIconCodePoint("md-numeric-5-box-outline", 0xF3B2);
  addIconCodePoint("md-numeric-5-circle", 0xFC84);
  addIconCodePoint("md-numeric-5-circle-outline", 0xFC85);
  addIconCodePoint("md-numeric-6", 0x0036);
  addIconCodePoint("md-numeric-6-box", 0xF3B3);
  addIconCodePoint("md-numeric-6-box-multiple", 0xFF31);
  addIconCodePoint("md-numeric-6-box-multiple-outline", 0xF3B4);
  addIconCodePoint("md-numeric-6-box-outline", 0xF3B5);
  addIconCodePoint("md-numeric-6-circle", 0xFC86);
  addIconCodePoint("md-numeric-6-circle-outline", 0xFC87);
  addIconCodePoint("md-numeric-7", 0x0037);
  addIconCodePoint("md-numeric-7-box", 0xF3B6);
  addIconCodePoint("md-numeric-7-box-multiple", 0xFF32);
  addIconCodePoint("md-numeric-7-box-multiple-outline", 0xF3B7);
  addIconCodePoint("md-numeric-7-box-outline", 0xF3B8);
  addIconCodePoint("md-numeric-7-circle", 0xFC88);
  addIconCodePoint("md-numeric-7-circle-outline", 0xFC89);
  addIconCodePoint("md-numeric-8", 0x0038);
  addIconCodePoint("md-numeric-8-box", 0xF3B9);
  addIconCodePoint("md-numeric-8-box-multiple", 0xFF33);
  addIconCodePoint("md-numeric-8-box-multiple-outline", 0xF3BA);
  addIconCodePoint("md-numeric-8-box-outline", 0xF3BB);
  addIconCodePoint("md-numeric-8-circle", 0xFC8A);
  addIconCodePoint("md-numeric-8-circle-outline", 0xFC8B);
  addIconCodePoint("md-numeric-9", 0x0039);
  addIconCodePoint("md-numeric-9-box", 0xF3BC);
  addIconCodePoint("md-numeric-9-box-multiple", 0xFF34);
  addIconCodePoint("md-numeric-9-box-multiple-outline", 0xF3BD);
  addIconCodePoint("md-numeric-9-box-outline", 0xF3BE);
  addIconCodePoint("md-numeric-9-circle", 0xFC8C);
  addIconCodePoint("md-numeric-9-circle-outline", 0xFC8D);
  addIconCodePoint("md-numeric-9-plus", 0xF000F);
  addIconCodePoint("md-numeric-9-plus-box", 0xF3BF);
  addIconCodePoint("md-numeric-9-plus-box-multiple", 0xFF35);
  addIconCodePoint("md-numeric-9-plus-box-multiple-outline", 0xF3C0);
  addIconCodePoint("md-numeric-9-plus-box-outline", 0xF3C1);
  addIconCodePoint("md-numeric-9-plus-circle", 0xFC8E);
  addIconCodePoint("md-numeric-9-plus-circle-outline", 0xFC8F);
  addIconCodePoint("md-numeric-negative-1", 0xF0074);
  addIconCodePoint("md-nut", 0xF6F7);
  addIconCodePoint("md-nutrition", 0xF3C2);
  addIconCodePoint("md-nuxt", 0xF0131);
  addIconCodePoint("md-oar", 0xF67B);
  addIconCodePoint("md-ocarina", 0xFDBC);
  addIconCodePoint("md-oci", 0xF0314);
  addIconCodePoint("md-ocr", 0xF0165);
  addIconCodePoint("md-octagon", 0xF3C3);
  addIconCodePoint("md-octagon-outline", 0xF3C4);
  addIconCodePoint("md-octagram", 0xF6F8);
  addIconCodePoint("md-octagram-outline", 0xF774);
  addIconCodePoint("md-odnoklassniki", 0xF3C5);
  addIconCodePoint("md-offer", 0xF0246);
  addIconCodePoint("md-office", 0xF3C6);
  addIconCodePoint("md-office-building", 0xF990);
  addIconCodePoint("md-oil", 0xF3C7);
  addIconCodePoint("md-oil-lamp", 0xFF36);
  addIconCodePoint("md-oil-level", 0xF0075);
  addIconCodePoint("md-oil-temperature", 0xF0019);
  addIconCodePoint("md-omega", 0xF3C9);
  addIconCodePoint("md-one-up", 0xFB89);
  addIconCodePoint("md-onedrive", 0xF3CA);
  addIconCodePoint("md-onenote", 0xF746);
  addIconCodePoint("md-onepassword", 0xF880);
  addIconCodePoint("md-opacity", 0xF5CC);
  addIconCodePoint("md-open-in-app", 0xF3CB);
  addIconCodePoint("md-open-in-new", 0xF3CC);
  addIconCodePoint("md-open-source-initiative", 0xFB8A);
  addIconCodePoint("md-openid", 0xF3CD);
  addIconCodePoint("md-opera", 0xF3CE);
  addIconCodePoint("md-orbit", 0xF018);
  addIconCodePoint("md-origin", 0xFB2B);
  addIconCodePoint("md-ornament", 0xF3CF);
  addIconCodePoint("md-ornament-variant", 0xF3D0);
  addIconCodePoint("md-outdoor-lamp", 0xF0076);
  addIconCodePoint("md-outlook", 0xFCFE);
  addIconCodePoint("md-overscan", 0xF0027);
  addIconCodePoint("md-owl", 0xF3D2);
  addIconCodePoint("md-pac-man", 0xFB8B);
  addIconCodePoint("md-package", 0xF3D3);
  addIconCodePoint("md-package-down", 0xF3D4);
  addIconCodePoint("md-package-up", 0xF3D5);
  addIconCodePoint("md-package-variant", 0xF3D6);
  addIconCodePoint("md-package-variant-closed", 0xF3D7);
  addIconCodePoint("md-page-first", 0xF600);
  addIconCodePoint("md-page-last", 0xF601);
  addIconCodePoint("md-page-layout-body", 0xF6F9);
  addIconCodePoint("md-page-layout-footer", 0xF6FA);
  addIconCodePoint("md-page-layout-header", 0xF6FB);
  addIconCodePoint("md-page-layout-header-footer", 0xFF9C);
  addIconCodePoint("md-page-layout-sidebar-left", 0xF6FC);
  addIconCodePoint("md-page-layout-sidebar-right", 0xF6FD);
  addIconCodePoint("md-page-next", 0xFB8C);
  addIconCodePoint("md-page-next-outline", 0xFB8D);
  addIconCodePoint("md-page-previous", 0xFB8E);
  addIconCodePoint("md-page-previous-outline", 0xFB8F);
  addIconCodePoint("md-palette", 0xF3D8);
  addIconCodePoint("md-palette-advanced", 0xF3D9);
  addIconCodePoint("md-palette-outline", 0xFE6C);
  addIconCodePoint("md-palette-swatch", 0xF8B4);
  addIconCodePoint("md-palette-swatch-outline", 0xF0387);
  addIconCodePoint("md-palm-tree", 0xF0077);
  addIconCodePoint("md-pan", 0xFB90);
  addIconCodePoint("md-pan-bottom-left", 0xFB91);
  addIconCodePoint("md-pan-bottom-right", 0xFB92);
  addIconCodePoint("md-pan-down", 0xFB93);
  addIconCodePoint("md-pan-horizontal", 0xFB94);
  addIconCodePoint("md-pan-left", 0xFB95);
  addIconCodePoint("md-pan-right", 0xFB96);
  addIconCodePoint("md-pan-top-left", 0xFB97);
  addIconCodePoint("md-pan-top-right", 0xFB98);
  addIconCodePoint("md-pan-up", 0xFB99);
  addIconCodePoint("md-pan-vertical", 0xFB9A);
  addIconCodePoint("md-panda", 0xF3DA);
  addIconCodePoint("md-pandora", 0xF3DB);
  addIconCodePoint("md-panorama", 0xF3DC);
  addIconCodePoint("md-panorama-fisheye", 0xF3DD);
  addIconCodePoint("md-panorama-horizontal", 0xF3DE);
  addIconCodePoint("md-panorama-vertical", 0xF3DF);
  addIconCodePoint("md-panorama-wide-angle", 0xF3E0);
  addIconCodePoint("md-paper-cut-vertical", 0xF3E1);
  addIconCodePoint("md-paper-roll", 0xF0182);
  addIconCodePoint("md-paper-roll-outline", 0xF0183);
  addIconCodePoint("md-paperclip", 0xF3E2);
  addIconCodePoint("md-parachute", 0xFC90);
  addIconCodePoint("md-parachute-outline", 0xFC91);
  addIconCodePoint("md-parking", 0xF3E3);
  addIconCodePoint("md-party-popper", 0xF0078);
  addIconCodePoint("md-passport", 0xF7E2);
  addIconCodePoint("md-passport-biometric", 0xFDBD);
  addIconCodePoint("md-pasta", 0xF018B);
  addIconCodePoint("md-patio-heater", 0xFF9D);
  addIconCodePoint("md-patreon", 0xF881);
  addIconCodePoint("md-pause", 0xF3E4);
  addIconCodePoint("md-pause-circle", 0xF3E5);
  addIconCodePoint("md-pause-circle-outline", 0xF3E6);
  addIconCodePoint("md-pause-octagon", 0xF3E7);
  addIconCodePoint("md-pause-octagon-outline", 0xF3E8);
  addIconCodePoint("md-paw", 0xF3E9);
  addIconCodePoint("md-paw-off", 0xF657);
  addIconCodePoint("md-paypal", 0xF882);
  addIconCodePoint("md-pdf-box", 0xFE39);
  addIconCodePoint("md-peace", 0xF883);
  addIconCodePoint("md-peanut", 0xF001E);
  addIconCodePoint("md-peanut-off", 0xF001F);
  addIconCodePoint("md-peanut-off-outline", 0xF0021);
  addIconCodePoint("md-peanut-outline", 0xF0020);
  addIconCodePoint("md-pen", 0xF3EA);
  addIconCodePoint("md-pen-lock", 0xFDBE);
  addIconCodePoint("md-pen-minus", 0xFDBF);
  addIconCodePoint("md-pen-off", 0xFDC0);
  addIconCodePoint("md-pen-plus", 0xFDC1);
  addIconCodePoint("md-pen-remove", 0xFDC2);
  addIconCodePoint("md-pencil", 0xF3EB);
  addIconCodePoint("md-pencil-box", 0xF3EC);
  addIconCodePoint("md-pencil-box-multiple", 0xF016F);
  addIconCodePoint("md-pencil-box-multiple-outline", 0xF0170);
  addIconCodePoint("md-pencil-box-outline", 0xF3ED);
  addIconCodePoint("md-pencil-circle", 0xF6FE);
  addIconCodePoint("md-pencil-circle-outline", 0xF775);
  addIconCodePoint("md-pencil-lock", 0xF3EE);
  addIconCodePoint("md-pencil-lock-outline", 0xFDC3);
  addIconCodePoint("md-pencil-minus", 0xFDC4);
  addIconCodePoint("md-pencil-minus-outline", 0xFDC5);
  addIconCodePoint("md-pencil-off", 0xF3EF);
  addIconCodePoint("md-pencil-off-outline", 0xFDC6);
  addIconCodePoint("md-pencil-outline", 0xFC92);
  addIconCodePoint("md-pencil-plus", 0xFDC7);
  addIconCodePoint("md-pencil-plus-outline", 0xFDC8);
  addIconCodePoint("md-pencil-remove", 0xFDC9);
  addIconCodePoint("md-pencil-remove-outline", 0xFDCA);
  addIconCodePoint("md-pencil-ruler", 0xF037E);
  addIconCodePoint("md-penguin", 0xFEDD);
  addIconCodePoint("md-pentagon", 0xF6FF);
  addIconCodePoint("md-pentagon-outline", 0xF700);
  addIconCodePoint("md-percent", 0xF3F0);
  addIconCodePoint("md-percent-outline", 0xF02A3);
  addIconCodePoint("md-periodic-table", 0xF8B5);
  addIconCodePoint("md-periodic-table-co", 0xF0329);
  addIconCodePoint("md-periodic-table-co2", 0xF7E3);
  addIconCodePoint("md-periscope", 0xF747);
  addIconCodePoint("md-perspective-less", 0xFCFF);
  addIconCodePoint("md-perspective-more", 0xFD00);
  addIconCodePoint("md-pharmacy", 0xF3F1);
  addIconCodePoint("md-phone", 0xF3F2);
  addIconCodePoint("md-phone-alert", 0xFF37);
  addIconCodePoint("md-phone-alert-outline", 0xF01B9);
  addIconCodePoint("md-phone-bluetooth", 0xF3F3);
  addIconCodePoint("md-phone-bluetooth-outline", 0xF01BA);
  addIconCodePoint("md-phone-cancel", 0xF00E7);
  addIconCodePoint("md-phone-cancel-outline", 0xF01BB);
  addIconCodePoint("md-phone-check", 0xF01D4);
  addIconCodePoint("md-phone-check-outline", 0xF01D5);
  addIconCodePoint("md-phone-classic", 0xF602);
  addIconCodePoint("md-phone-classic-off", 0xF02A4);
  addIconCodePoint("md-phone-forward", 0xF3F4);
  addIconCodePoint("md-phone-forward-outline", 0xF01BC);
  addIconCodePoint("md-phone-hangup", 0xF3F5);
  addIconCodePoint("md-phone-hangup-outline", 0xF01BD);
  addIconCodePoint("md-phone-in-talk", 0xF3F6);
  addIconCodePoint("md-phone-in-talk-outline", 0xF01AD);
  addIconCodePoint("md-phone-incoming", 0xF3F7);
  addIconCodePoint("md-phone-incoming-outline", 0xF01BE);
  addIconCodePoint("md-phone-lock", 0xF3F8);
  addIconCodePoint("md-phone-lock-outline", 0xF01BF);
  addIconCodePoint("md-phone-log", 0xF3F9);
  addIconCodePoint("md-phone-log-outline", 0xF01C0);
  addIconCodePoint("md-phone-message", 0xF01C1);
  addIconCodePoint("md-phone-message-outline", 0xF01C2);
  addIconCodePoint("md-phone-minus", 0xF658);
  addIconCodePoint("md-phone-minus-outline", 0xF01C3);
  addIconCodePoint("md-phone-missed", 0xF3FA);
  addIconCodePoint("md-phone-missed-outline", 0xF01D0);
  addIconCodePoint("md-phone-off", 0xFDCB);
  addIconCodePoint("md-phone-off-outline", 0xF01D1);
  addIconCodePoint("md-phone-outgoing", 0xF3FB);
  addIconCodePoint("md-phone-outgoing-outline", 0xF01C4);
  addIconCodePoint("md-phone-outline", 0xFDCC);
  addIconCodePoint("md-phone-paused", 0xF3FC);
  addIconCodePoint("md-phone-paused-outline", 0xF01C5);
  addIconCodePoint("md-phone-plus", 0xF659);
  addIconCodePoint("md-phone-plus-outline", 0xF01C6);
  addIconCodePoint("md-phone-return", 0xF82E);
  addIconCodePoint("md-phone-return-outline", 0xF01C7);
  addIconCodePoint("md-phone-ring", 0xF01D6);
  addIconCodePoint("md-phone-ring-outline", 0xF01D7);
  addIconCodePoint("md-phone-rotate-landscape", 0xF884);
  addIconCodePoint("md-phone-rotate-portrait", 0xF885);
  addIconCodePoint("md-phone-settings", 0xF3FD);
  addIconCodePoint("md-phone-settings-outline", 0xF01C8);
  addIconCodePoint("md-phone-voip", 0xF3FE);
  addIconCodePoint("md-pi", 0xF3FF);
  addIconCodePoint("md-pi-box", 0xF400);
  addIconCodePoint("md-pi-hole", 0xFDCD);
  addIconCodePoint("md-piano", 0xF67C);
  addIconCodePoint("md-pickaxe", 0xF8B6);
  addIconCodePoint("md-picture-in-picture-bottom-right", 0xFE3A);
  addIconCodePoint("md-picture-in-picture-bottom-right-outline", 0xFE3B);
  addIconCodePoint("md-picture-in-picture-top-right", 0xFE3C);
  addIconCodePoint("md-picture-in-picture-top-right-outline", 0xFE3D);
  addIconCodePoint("md-pier", 0xF886);
  addIconCodePoint("md-pier-crane", 0xF887);
  addIconCodePoint("md-pig", 0xF401);
  addIconCodePoint("md-pig-variant", 0xF0028);
  addIconCodePoint("md-piggy-bank", 0xF0029);
  addIconCodePoint("md-pill", 0xF402);
  addIconCodePoint("md-pillar", 0xF701);
  addIconCodePoint("md-pin", 0xF403);
  addIconCodePoint("md-pin-off", 0xF404);
  addIconCodePoint("md-pin-off-outline", 0xF92F);
  addIconCodePoint("md-pin-outline", 0xF930);
  addIconCodePoint("md-pine-tree", 0xF405);
  addIconCodePoint("md-pine-tree-box", 0xF406);
  addIconCodePoint("md-pinterest", 0xF407);
  addIconCodePoint("md-pinterest-box", 0xF408);
  addIconCodePoint("md-pinwheel", 0xFAD4);
  addIconCodePoint("md-pinwheel-outline", 0xFAD5);
  addIconCodePoint("md-pipe", 0xF7E4);
  addIconCodePoint("md-pipe-disconnected", 0xF7E5);
  addIconCodePoint("md-pipe-leak", 0xF888);
  addIconCodePoint("md-pipe-wrench", 0xF037F);
  addIconCodePoint("md-pirate", 0xFA07);
  addIconCodePoint("md-pistol", 0xF702);
  addIconCodePoint("md-piston", 0xF889);
  addIconCodePoint("md-pizza", 0xF409);
  addIconCodePoint("md-play", 0xF40A);
  addIconCodePoint("md-play-box", 0xF02A5);
  addIconCodePoint("md-play-box-outline", 0xF40B);
  addIconCodePoint("md-play-circle", 0xF40C);
  addIconCodePoint("md-play-circle-outline", 0xF40D);
  addIconCodePoint("md-play-network", 0xF88A);
  addIconCodePoint("md-play-network-outline", 0xFC93);
  addIconCodePoint("md-play-outline", 0xFF38);
  addIconCodePoint("md-play-pause", 0xF40E);
  addIconCodePoint("md-play-protected-content", 0xF40F);
  addIconCodePoint("md-play-speed", 0xF8FE);
  addIconCodePoint("md-playlist-check", 0xF5C7);
  addIconCodePoint("md-playlist-edit", 0xF8FF);
  addIconCodePoint("md-playlist-minus", 0xF410);
  addIconCodePoint("md-playlist-music", 0xFC94);
  addIconCodePoint("md-playlist-music-outline", 0xFC95);
  addIconCodePoint("md-playlist-play", 0xF411);
  addIconCodePoint("md-playlist-plus", 0xF412);
  addIconCodePoint("md-playlist-remove", 0xF413);
  addIconCodePoint("md-playlist-star", 0xFDCE);
  addIconCodePoint("md-playstation", 0xF414);
  addIconCodePoint("md-plex", 0xF6B9);
  addIconCodePoint("md-plus", 0xF415);
  addIconCodePoint("md-plus-box", 0xF416);
  addIconCodePoint("md-plus-box-multiple", 0xF334);
  addIconCodePoint("md-plus-box-multiple-outline", 0xF016E);
  addIconCodePoint("md-plus-box-outline", 0xF703);
  addIconCodePoint("md-plus-circle", 0xF417);
  addIconCodePoint("md-plus-circle-multiple-outline", 0xF418);
  addIconCodePoint("md-plus-circle-outline", 0xF419);
  addIconCodePoint("md-plus-minus", 0xF991);
  addIconCodePoint("md-plus-minus-box", 0xF992);
  addIconCodePoint("md-plus-network", 0xF41A);
  addIconCodePoint("md-plus-network-outline", 0xFC96);
  addIconCodePoint("md-plus-one", 0xF41B);
  addIconCodePoint("md-plus-outline", 0xF704);
  addIconCodePoint("md-plus-thick", 0xF0217);
  addIconCodePoint("md-pocket", 0xF41C);
  addIconCodePoint("md-podcast", 0xF993);
  addIconCodePoint("md-podium", 0xFD01);
  addIconCodePoint("md-podium-bronze", 0xFD02);
  addIconCodePoint("md-podium-gold", 0xFD03);
  addIconCodePoint("md-podium-silver", 0xFD04);
  addIconCodePoint("md-point-of-sale", 0xFD6E);
  addIconCodePoint("md-pokeball", 0xF41D);
  addIconCodePoint("md-pokemon-go", 0xFA08);
  addIconCodePoint("md-poker-chip", 0xF82F);
  addIconCodePoint("md-polaroid", 0xF41E);
  addIconCodePoint("md-police-badge", 0xF0192);
  addIconCodePoint("md-police-badge-outline", 0xF0193);
  addIconCodePoint("md-poll", 0xF41F);
  addIconCodePoint("md-poll-box", 0xF420);
  addIconCodePoint("md-poll-box-outline", 0xF02A6);
  addIconCodePoint("md-polymer", 0xF421);
  addIconCodePoint("md-pool", 0xF606);
  addIconCodePoint("md-popcorn", 0xF422);
  addIconCodePoint("md-post", 0xF002A);
  addIconCodePoint("md-post-outline", 0xF002B);
  addIconCodePoint("md-postage-stamp", 0xFC97);
  addIconCodePoint("md-pot", 0xF65A);
  addIconCodePoint("md-pot-mix", 0xF65B);
  addIconCodePoint("md-pound", 0xF423);
  addIconCodePoint("md-pound-box", 0xF424);
  addIconCodePoint("md-pound-box-outline", 0xF01AA);
  addIconCodePoint("md-power", 0xF425);
  addIconCodePoint("md-power-cycle", 0xF900);
  addIconCodePoint("md-power-off", 0xF901);
  addIconCodePoint("md-power-on", 0xF902);
  addIconCodePoint("md-power-plug", 0xF6A4);
  addIconCodePoint("md-power-plug-off", 0xF6A5);
  addIconCodePoint("md-power-settings", 0xF426);
  addIconCodePoint("md-power-sleep", 0xF903);
  addIconCodePoint("md-power-socket", 0xF427);
  addIconCodePoint("md-power-socket-au", 0xF904);
  addIconCodePoint("md-power-socket-de", 0xF0132);
  addIconCodePoint("md-power-socket-eu", 0xF7E6);
  addIconCodePoint("md-power-socket-fr", 0xF0133);
  addIconCodePoint("md-power-socket-jp", 0xF0134);
  addIconCodePoint("md-power-socket-uk", 0xF7E7);
  addIconCodePoint("md-power-socket-us", 0xF7E8);
  addIconCodePoint("md-power-standby", 0xF905);
  addIconCodePoint("md-powershell", 0xFA09);
  addIconCodePoint("md-prescription", 0xF705);
  addIconCodePoint("md-presentation", 0xF428);
  addIconCodePoint("md-presentation-play", 0xF429);
  addIconCodePoint("md-printer", 0xF42A);
  addIconCodePoint("md-printer-3d", 0xF42B);
  addIconCodePoint("md-printer-3d-nozzle", 0xFE3E);
  addIconCodePoint("md-printer-3d-nozzle-alert", 0xF01EB);
  addIconCodePoint("md-printer-3d-nozzle-alert-outline", 0xF01EC);
  addIconCodePoint("md-printer-3d-nozzle-outline", 0xFE3F);
  addIconCodePoint("md-printer-alert", 0xF42C);
  addIconCodePoint("md-printer-check", 0xF0171);
  addIconCodePoint("md-printer-off", 0xFE40);
  addIconCodePoint("md-printer-pos", 0xF0079);
  addIconCodePoint("md-printer-settings", 0xF706);
  addIconCodePoint("md-printer-wireless", 0xFA0A);
  addIconCodePoint("md-priority-high", 0xF603);
  addIconCodePoint("md-priority-low", 0xF604);
  addIconCodePoint("md-professional-hexagon", 0xF42D);
  addIconCodePoint("md-progress-alert", 0xFC98);
  addIconCodePoint("md-progress-check", 0xF994);
  addIconCodePoint("md-progress-clock", 0xF995);
  addIconCodePoint("md-progress-close", 0xF0135);
  addIconCodePoint("md-progress-download", 0xF996);
  addIconCodePoint("md-progress-upload", 0xF997);
  addIconCodePoint("md-progress-wrench", 0xFC99);
  addIconCodePoint("md-projector", 0xF42E);
  addIconCodePoint("md-projector-screen", 0xF42F);
  addIconCodePoint("md-propane-tank", 0xF0382);
  addIconCodePoint("md-propane-tank-outline", 0xF0383);
  addIconCodePoint("md-protocol", 0xFFF9);
  addIconCodePoint("md-publish", 0xF6A6);
  addIconCodePoint("md-pulse", 0xF430);
  addIconCodePoint("md-pumpkin", 0xFB9B);
  addIconCodePoint("md-purse", 0xFF39);
  addIconCodePoint("md-purse-outline", 0xFF3A);
  addIconCodePoint("md-puzzle", 0xF431);
  addIconCodePoint("md-puzzle-outline", 0xFA65);
  addIconCodePoint("md-qi", 0xF998);
  addIconCodePoint("md-qqchat", 0xF605);
  addIconCodePoint("md-qrcode", 0xF432);
  addIconCodePoint("md-qrcode-edit", 0xF8B7);
  addIconCodePoint("md-qrcode-minus", 0xF01B7);
  addIconCodePoint("md-qrcode-plus", 0xF01B6);
  addIconCodePoint("md-qrcode-remove", 0xF01B8);
  addIconCodePoint("md-qrcode-scan", 0xF433);
  addIconCodePoint("md-quadcopter", 0xF434);
  addIconCodePoint("md-quality-high", 0xF435);
  addIconCodePoint("md-quality-low", 0xFA0B);
  addIconCodePoint("md-quality-medium", 0xFA0C);
  addIconCodePoint("md-quicktime", 0xF436);
  addIconCodePoint("md-quora", 0xFD05);
  addIconCodePoint("md-rabbit", 0xF906);
  addIconCodePoint("md-racing-helmet", 0xFD6F);
  addIconCodePoint("md-racquetball", 0xFD70);
  addIconCodePoint("md-radar", 0xF437);
  addIconCodePoint("md-radiator", 0xF438);
  addIconCodePoint("md-radiator-disabled", 0xFAD6);
  addIconCodePoint("md-radiator-off", 0xFAD7);
  addIconCodePoint("md-radio", 0xF439);
  addIconCodePoint("md-radio-am", 0xFC9A);
  addIconCodePoint("md-radio-fm", 0xFC9B);
  addIconCodePoint("md-radio-handheld", 0xF43A);
  addIconCodePoint("md-radio-off", 0xF0247);
  addIconCodePoint("md-radio-tower", 0xF43B);
  addIconCodePoint("md-radioactive", 0xF43C);
  addIconCodePoint("md-radioactive-off", 0xFEDE);
  addIconCodePoint("md-radiobox-blank", 0xF43D);
  addIconCodePoint("md-radiobox-marked", 0xF43E);
  addIconCodePoint("md-radius", 0xFC9C);
  addIconCodePoint("md-radius-outline", 0xFC9D);
  addIconCodePoint("md-railroad-light", 0xFF3B);
  addIconCodePoint("md-raspberry-pi", 0xF43F);
  addIconCodePoint("md-ray-end", 0xF440);
  addIconCodePoint("md-ray-end-arrow", 0xF441);
  addIconCodePoint("md-ray-start", 0xF442);
  addIconCodePoint("md-ray-start-arrow", 0xF443);
  addIconCodePoint("md-ray-start-end", 0xF444);
  addIconCodePoint("md-ray-vertex", 0xF445);
  addIconCodePoint("md-react", 0xF707);
  addIconCodePoint("md-read", 0xF447);
  addIconCodePoint("md-receipt", 0xF449);
  addIconCodePoint("md-record", 0xF44A);
  addIconCodePoint("md-record-circle", 0xFEDF);
  addIconCodePoint("md-record-circle-outline", 0xFEE0);
  addIconCodePoint("md-record-player", 0xF999);
  addIconCodePoint("md-record-rec", 0xF44B);
  addIconCodePoint("md-rectangle", 0xFE41);
  addIconCodePoint("md-rectangle-outline", 0xFE42);
  addIconCodePoint("md-recycle", 0xF44C);
  addIconCodePoint("md-reddit", 0xF44D);
  addIconCodePoint("md-redhat", 0xF0146);
  addIconCodePoint("md-redo", 0xF44E);
  addIconCodePoint("md-redo-variant", 0xF44F);
  addIconCodePoint("md-reflect-horizontal", 0xFA0D);
  addIconCodePoint("md-reflect-vertical", 0xFA0E);
  addIconCodePoint("md-refresh", 0xF450);
  addIconCodePoint("md-refresh-circle", 0xF03A2);
  addIconCodePoint("md-regex", 0xF451);
  addIconCodePoint("md-registered-trademark", 0xFA66);
  addIconCodePoint("md-relative-scale", 0xF452);
  addIconCodePoint("md-reload", 0xF453);
  addIconCodePoint("md-reload-alert", 0xF0136);
  addIconCodePoint("md-reminder", 0xF88B);
  addIconCodePoint("md-remote", 0xF454);
  addIconCodePoint("md-remote-desktop", 0xF8B8);
  addIconCodePoint("md-remote-off", 0xFEE1);
  addIconCodePoint("md-remote-tv", 0xFEE2);
  addIconCodePoint("md-remote-tv-off", 0xFEE3);
  addIconCodePoint("md-rename-box", 0xF455);
  addIconCodePoint("md-reorder-horizontal", 0xF687);
  addIconCodePoint("md-reorder-vertical", 0xF688);
  addIconCodePoint("md-repeat", 0xF456);
  addIconCodePoint("md-repeat-off", 0xF457);
  addIconCodePoint("md-repeat-once", 0xF458);
  addIconCodePoint("md-replay", 0xF459);
  addIconCodePoint("md-reply", 0xF45A);
  addIconCodePoint("md-reply-all", 0xF45B);
  addIconCodePoint("md-reply-all-outline", 0xFF3C);
  addIconCodePoint("md-reply-circle", 0xF01D9);
  addIconCodePoint("md-reply-outline", 0xFF3D);
  addIconCodePoint("md-reproduction", 0xF45C);
  addIconCodePoint("md-resistor", 0xFB1F);
  addIconCodePoint("md-resistor-nodes", 0xFB20);
  addIconCodePoint("md-resize", 0xFA67);
  addIconCodePoint("md-resize-bottom-right", 0xF45D);
  addIconCodePoint("md-responsive", 0xF45E);
  addIconCodePoint("md-restart", 0xF708);
  addIconCodePoint("md-restart-alert", 0xF0137);
  addIconCodePoint("md-restart-off", 0xFD71);
  addIconCodePoint("md-restore", 0xF99A);
  addIconCodePoint("md-restore-alert", 0xF0138);
  addIconCodePoint("md-rewind", 0xF45F);
  addIconCodePoint("md-rewind-10", 0xFD06);
  addIconCodePoint("md-rewind-30", 0xFD72);
  addIconCodePoint("md-rewind-5", 0xF0224);
  addIconCodePoint("md-rewind-outline", 0xF709);
  addIconCodePoint("md-rhombus", 0xF70A);
  addIconCodePoint("md-rhombus-medium", 0xFA0F);
  addIconCodePoint("md-rhombus-outline", 0xF70B);
  addIconCodePoint("md-rhombus-split", 0xFA10);
  addIconCodePoint("md-ribbon", 0xF460);
  addIconCodePoint("md-rice", 0xF7E9);
  addIconCodePoint("md-ring", 0xF7EA);
  addIconCodePoint("md-rivet", 0xFE43);
  addIconCodePoint("md-road", 0xF461);
  addIconCodePoint("md-road-variant", 0xF462);
  addIconCodePoint("md-robber", 0xF007A);
  addIconCodePoint("md-robot", 0xF6A8);
  addIconCodePoint("md-robot-industrial", 0xFB21);
  addIconCodePoint("md-robot-mower", 0xF0222);
  addIconCodePoint("md-robot-mower-outline", 0xF021E);
  addIconCodePoint("md-robot-vacuum", 0xF70C);
  addIconCodePoint("md-robot-vacuum-variant", 0xF907);
  addIconCodePoint("md-rocket", 0xF463);
  addIconCodePoint("md-rodent", 0xF0352);
  addIconCodePoint("md-roller-skate", 0xFD07);
  addIconCodePoint("md-rollerblade", 0xFD08);
  addIconCodePoint("md-rollupjs", 0xFB9C);
  addIconCodePoint("md-roman-numeral-1", 0xF00B3);
  addIconCodePoint("md-roman-numeral-10", 0xF00BC);
  addIconCodePoint("md-roman-numeral-2", 0xF00B4);
  addIconCodePoint("md-roman-numeral-3", 0xF00B5);
  addIconCodePoint("md-roman-numeral-4", 0xF00B6);
  addIconCodePoint("md-roman-numeral-5", 0xF00B7);
  addIconCodePoint("md-roman-numeral-6", 0xF00B8);
  addIconCodePoint("md-roman-numeral-7", 0xF00B9);
  addIconCodePoint("md-roman-numeral-8", 0xF00BA);
  addIconCodePoint("md-roman-numeral-9", 0xF00BB);
  addIconCodePoint("md-room-service", 0xF88C);
  addIconCodePoint("md-room-service-outline", 0xFD73);
  addIconCodePoint("md-rotate-3d", 0xFEE4);
  addIconCodePoint("md-rotate-3d-variant", 0xF464);
  addIconCodePoint("md-rotate-left", 0xF465);
  addIconCodePoint("md-rotate-left-variant", 0xF466);
  addIconCodePoint("md-rotate-orbit", 0xFD74);
  addIconCodePoint("md-rotate-right", 0xF467);
  addIconCodePoint("md-rotate-right-variant", 0xF468);
  addIconCodePoint("md-rounded-corner", 0xF607);
  addIconCodePoint("md-router", 0xF020D);
  addIconCodePoint("md-router-wireless", 0xF469);
  addIconCodePoint("md-router-wireless-settings", 0xFA68);
  addIconCodePoint("md-routes", 0xF46A);
  addIconCodePoint("md-routes-clock", 0xF007B);
  addIconCodePoint("md-rowing", 0xF608);
  addIconCodePoint("md-rss", 0xF46B);
  addIconCodePoint("md-rss-box", 0xF46C);
  addIconCodePoint("md-rss-off", 0xFF3E);
  addIconCodePoint("md-ruby", 0xFD09);
  addIconCodePoint("md-rugby", 0xFD75);
  addIconCodePoint("md-ruler", 0xF46D);
  addIconCodePoint("md-ruler-square", 0xFC9E);
  addIconCodePoint("md-ruler-square-compass", 0xFEDB);
  addIconCodePoint("md-run", 0xF70D);
  addIconCodePoint("md-run-fast", 0xF46E);
  addIconCodePoint("md-rv-truck", 0xF01FF);
  addIconCodePoint("md-sack", 0xFD0A);
  addIconCodePoint("md-sack-percent", 0xFD0B);
  addIconCodePoint("md-safe", 0xFA69);
  addIconCodePoint("md-safe-square", 0xF02A7);
  addIconCodePoint("md-safe-square-outline", 0xF02A8);
  addIconCodePoint("md-safety-goggles", 0xFD0C);
  addIconCodePoint("md-sailing", 0xFEE5);
  addIconCodePoint("md-sale", 0xF46F);
  addIconCodePoint("md-salesforce", 0xF88D);
  addIconCodePoint("md-sass", 0xF7EB);
  addIconCodePoint("md-satellite", 0xF470);
  addIconCodePoint("md-satellite-uplink", 0xF908);
  addIconCodePoint("md-satellite-variant", 0xF471);
  addIconCodePoint("md-sausage", 0xF8B9);
  addIconCodePoint("md-saw-blade", 0xFE44);
  addIconCodePoint("md-saxophone", 0xF609);
  addIconCodePoint("md-scale", 0xF472);
  addIconCodePoint("md-scale-balance", 0xF5D1);
  addIconCodePoint("md-scale-bathroom", 0xF473);
  addIconCodePoint("md-scale-off", 0xF007C);
  addIconCodePoint("md-scanner", 0xF6AA);
  addIconCodePoint("md-scanner-off", 0xF909);
  addIconCodePoint("md-scatter-plot", 0xFEE6);
  addIconCodePoint("md-scatter-plot-outline", 0xFEE7);
  addIconCodePoint("md-school", 0xF474);
  addIconCodePoint("md-school-outline", 0xF01AB);
  addIconCodePoint("md-scissors-cutting", 0xFA6A);
  addIconCodePoint("md-scooter", 0xF0214);
  addIconCodePoint("md-scoreboard", 0xF02A9);
  addIconCodePoint("md-scoreboard-outline", 0xF02AA);
  addIconCodePoint("md-screen-rotation", 0xF475);
  addIconCodePoint("md-screen-rotation-lock", 0xF476);
  addIconCodePoint("md-screw-flat-top", 0xFDCF);
  addIconCodePoint("md-screw-lag", 0xFE54);
  addIconCodePoint("md-screw-machine-flat-top", 0xFE55);
  addIconCodePoint("md-screw-machine-round-top", 0xFE56);
  addIconCodePoint("md-screw-round-top", 0xFE57);
  addIconCodePoint("md-screwdriver", 0xF477);
  addIconCodePoint("md-script", 0xFB9D);
  addIconCodePoint("md-script-outline", 0xF478);
  addIconCodePoint("md-script-text", 0xFB9E);
  addIconCodePoint("md-script-text-outline", 0xFB9F);
  addIconCodePoint("md-sd", 0xF479);
  addIconCodePoint("md-seal", 0xF47A);
  addIconCodePoint("md-seal-variant", 0xFFFA);
  addIconCodePoint("md-search-web", 0xF70E);
  addIconCodePoint("md-seat", 0xFC9F);
  addIconCodePoint("md-seat-flat", 0xF47B);
  addIconCodePoint("md-seat-flat-angled", 0xF47C);
  addIconCodePoint("md-seat-individual-suite", 0xF47D);
  addIconCodePoint("md-seat-legroom-extra", 0xF47E);
  addIconCodePoint("md-seat-legroom-normal", 0xF47F);
  addIconCodePoint("md-seat-legroom-reduced", 0xF480);
  addIconCodePoint("md-seat-outline", 0xFCA0);
  addIconCodePoint("md-seat-passenger", 0xF0274);
  addIconCodePoint("md-seat-recline-extra", 0xF481);
  addIconCodePoint("md-seat-recline-normal", 0xF482);
  addIconCodePoint("md-seatbelt", 0xFCA1);
  addIconCodePoint("md-security", 0xF483);
  addIconCodePoint("md-security-network", 0xF484);
  addIconCodePoint("md-seed", 0xFE45);
  addIconCodePoint("md-seed-outline", 0xFE46);
  addIconCodePoint("md-segment", 0xFEE8);
  addIconCodePoint("md-select", 0xF485);
  addIconCodePoint("md-select-all", 0xF486);
  addIconCodePoint("md-select-color", 0xFD0D);
  addIconCodePoint("md-select-compare", 0xFAD8);
  addIconCodePoint("md-select-drag", 0xFA6B);
  addIconCodePoint("md-select-group", 0xFF9F);
  addIconCodePoint("md-select-inverse", 0xF487);
  addIconCodePoint("md-select-marker", 0xF02AB);
  addIconCodePoint("md-select-multiple", 0xF02AC);
  addIconCodePoint("md-select-multiple-marker", 0xF02AD);
  addIconCodePoint("md-select-off", 0xF488);
  addIconCodePoint("md-select-place", 0xFFFB);
  addIconCodePoint("md-select-search", 0xF022F);
  addIconCodePoint("md-selection", 0xF489);
  addIconCodePoint("md-selection-drag", 0xFA6C);
  addIconCodePoint("md-selection-ellipse", 0xFD0E);
  addIconCodePoint("md-selection-ellipse-arrow-inside", 0xFF3F);
  addIconCodePoint("md-selection-marker", 0xF02AE);
  addIconCodePoint("md-selection-multiple-marker", 0xF02AF);
  addIconCodePoint("md-selection-mutliple", 0xF02B0);
  addIconCodePoint("md-selection-off", 0xF776);
  addIconCodePoint("md-selection-search", 0xF0230);
  addIconCodePoint("md-semantic-web", 0xF0341);
  addIconCodePoint("md-send", 0xF48A);
  addIconCodePoint("md-send-check", 0xF018C);
  addIconCodePoint("md-send-check-outline", 0xF018D);
  addIconCodePoint("md-send-circle", 0xFE58);
  addIconCodePoint("md-send-circle-outline", 0xFE59);
  addIconCodePoint("md-send-clock", 0xF018E);
  addIconCodePoint("md-send-clock-outline", 0xF018F);
  addIconCodePoint("md-send-lock", 0xF7EC);
  addIconCodePoint("md-send-lock-outline", 0xF0191);
  addIconCodePoint("md-send-outline", 0xF0190);
  addIconCodePoint("md-serial-port", 0xF65C);
  addIconCodePoint("md-server", 0xF48B);
  addIconCodePoint("md-server-minus", 0xF48C);
  addIconCodePoint("md-server-network", 0xF48D);
  addIconCodePoint("md-server-network-off", 0xF48E);
  addIconCodePoint("md-server-off", 0xF48F);
  addIconCodePoint("md-server-plus", 0xF490);
  addIconCodePoint("md-server-remove", 0xF491);
  addIconCodePoint("md-server-security", 0xF492);
  addIconCodePoint("md-set-all", 0xF777);
  addIconCodePoint("md-set-center", 0xF778);
  addIconCodePoint("md-set-center-right", 0xF779);
  addIconCodePoint("md-set-left", 0xF77A);
  addIconCodePoint("md-set-left-center", 0xF77B);
  addIconCodePoint("md-set-left-right", 0xF77C);
  addIconCodePoint("md-set-none", 0xF77D);
  addIconCodePoint("md-set-right", 0xF77E);
  addIconCodePoint("md-set-top-box", 0xF99E);
  addIconCodePoint("md-settings", 0xF493);
  addIconCodePoint("md-settings-box", 0xF494);
  addIconCodePoint("md-settings-helper", 0xFA6D);
  addIconCodePoint("md-settings-outline", 0xF8BA);
  addIconCodePoint("md-settings-transfer", 0xF007D);
  addIconCodePoint("md-settings-transfer-outline", 0xF007E);
  addIconCodePoint("md-shaker", 0xF0139);
  addIconCodePoint("md-shaker-outline", 0xF013A);
  addIconCodePoint("md-shape", 0xF830);
  addIconCodePoint("md-shape-circle-plus", 0xF65D);
  addIconCodePoint("md-shape-outline", 0xF831);
  addIconCodePoint("md-shape-oval-plus", 0xF0225);
  addIconCodePoint("md-shape-plus", 0xF495);
  addIconCodePoint("md-shape-polygon-plus", 0xF65E);
  addIconCodePoint("md-shape-rectangle-plus", 0xF65F);
  addIconCodePoint("md-shape-square-plus", 0xF660);
  addIconCodePoint("md-share", 0xF496);
  addIconCodePoint("md-share-all", 0xF021F);
  addIconCodePoint("md-share-all-outline", 0xF0220);
  addIconCodePoint("md-share-circle", 0xF01D8);
  addIconCodePoint("md-share-off", 0xFF40);
  addIconCodePoint("md-share-off-outline", 0xFF41);
  addIconCodePoint("md-share-outline", 0xF931);
  addIconCodePoint("md-share-variant", 0xF497);
  addIconCodePoint("md-sheep", 0xFCA2);
  addIconCodePoint("md-shield", 0xF498);
  addIconCodePoint("md-shield-account", 0xF88E);
  addIconCodePoint("md-shield-account-outline", 0xFA11);
  addIconCodePoint("md-shield-airplane", 0xF6BA);
  addIconCodePoint("md-shield-airplane-outline", 0xFCA3);
  addIconCodePoint("md-shield-alert", 0xFEE9);
  addIconCodePoint("md-shield-alert-outline", 0xFEEA);
  addIconCodePoint("md-shield-car", 0xFFA0);
  addIconCodePoint("md-shield-check", 0xF565);
  addIconCodePoint("md-shield-check-outline", 0xFCA4);
  addIconCodePoint("md-shield-cross", 0xFCA5);
  addIconCodePoint("md-shield-cross-outline", 0xFCA6);
  addIconCodePoint("md-shield-edit", 0xF01CB);
  addIconCodePoint("md-shield-edit-outline", 0xF01CC);
  addIconCodePoint("md-shield-half", 0xF038B);
  addIconCodePoint("md-shield-half-full", 0xF77F);
  addIconCodePoint("md-shield-home", 0xF689);
  addIconCodePoint("md-shield-home-outline", 0xFCA7);
  addIconCodePoint("md-shield-key", 0xFBA0);
  addIconCodePoint("md-shield-key-outline", 0xFBA1);
  addIconCodePoint("md-shield-link-variant", 0xFD0F);
  addIconCodePoint("md-shield-link-variant-outline", 0xFD10);
  addIconCodePoint("md-shield-lock", 0xF99C);
  addIconCodePoint("md-shield-lock-outline", 0xFCA8);
  addIconCodePoint("md-shield-off", 0xF99D);
  addIconCodePoint("md-shield-off-outline", 0xF99B);
  addIconCodePoint("md-shield-outline", 0xF499);
  addIconCodePoint("md-shield-plus", 0xFAD9);
  addIconCodePoint("md-shield-plus-outline", 0xFADA);
  addIconCodePoint("md-shield-refresh", 0xF01CD);
  addIconCodePoint("md-shield-refresh-outline", 0xF01CE);
  addIconCodePoint("md-shield-remove", 0xFADB);
  addIconCodePoint("md-shield-remove-outline", 0xFADC);
  addIconCodePoint("md-shield-search", 0xFD76);
  addIconCodePoint("md-shield-star", 0xF0166);
  addIconCodePoint("md-shield-star-outline", 0xF0167);
  addIconCodePoint("md-shield-sun", 0xF007F);
  addIconCodePoint("md-shield-sun-outline", 0xF0080);
  addIconCodePoint("md-ship-wheel", 0xF832);
  addIconCodePoint("md-shoe-formal", 0xFB22);
  addIconCodePoint("md-shoe-heel", 0xFB23);
  addIconCodePoint("md-shoe-print", 0xFE5A);
  addIconCodePoint("md-shopify", 0xFADD);
  addIconCodePoint("md-shopping", 0xF49A);
  addIconCodePoint("md-shopping-music", 0xF49B);
  addIconCodePoint("md-shopping-outline", 0xF0200);
  addIconCodePoint("md-shopping-search", 0xFFA1);
  addIconCodePoint("md-shovel", 0xF70F);
  addIconCodePoint("md-shovel-off", 0xF710);
  addIconCodePoint("md-shower", 0xF99F);
  addIconCodePoint("md-shower-head", 0xF9A0);
  addIconCodePoint("md-shredder", 0xF49C);
  addIconCodePoint("md-shuffle", 0xF49D);
  addIconCodePoint("md-shuffle-disabled", 0xF49E);
  addIconCodePoint("md-shuffle-variant", 0xF49F);
  addIconCodePoint("md-shuriken", 0xF03AA);
  addIconCodePoint("md-sigma", 0xF4A0);
  addIconCodePoint("md-sigma-lower", 0xF62B);
  addIconCodePoint("md-sign-caution", 0xF4A1);
  addIconCodePoint("md-sign-direction", 0xF780);
  addIconCodePoint("md-sign-direction-minus", 0xF0022);
  addIconCodePoint("md-sign-direction-plus", 0xFFFD);
  addIconCodePoint("md-sign-direction-remove", 0xFFFE);
  addIconCodePoint("md-sign-real-estate", 0xF0143);
  addIconCodePoint("md-sign-text", 0xF781);
  addIconCodePoint("md-signal", 0xF4A2);
  addIconCodePoint("md-signal-2g", 0xF711);
  addIconCodePoint("md-signal-3g", 0xF712);
  addIconCodePoint("md-signal-4g", 0xF713);
  addIconCodePoint("md-signal-5g", 0xFA6E);
  addIconCodePoint("md-signal-cellular-1", 0xF8BB);
  addIconCodePoint("md-signal-cellular-2", 0xF8BC);
  addIconCodePoint("md-signal-cellular-3", 0xF8BD);
  addIconCodePoint("md-signal-cellular-outline", 0xF8BE);
  addIconCodePoint("md-signal-distance-variant", 0xFE47);
  addIconCodePoint("md-signal-hspa", 0xF714);
  addIconCodePoint("md-signal-hspa-plus", 0xF715);
  addIconCodePoint("md-signal-off", 0xF782);
  addIconCodePoint("md-signal-variant", 0xF60A);
  addIconCodePoint("md-signature", 0xFE5B);
  addIconCodePoint("md-signature-freehand", 0xFE5C);
  addIconCodePoint("md-signature-image", 0xFE5D);
  addIconCodePoint("md-signature-text", 0xFE5E);
  addIconCodePoint("md-silo", 0xFB24);
  addIconCodePoint("md-silverware", 0xF4A3);
  addIconCodePoint("md-silverware-clean", 0xFFFF);
  addIconCodePoint("md-silverware-fork", 0xF4A4);
  addIconCodePoint("md-silverware-fork-knife", 0xFA6F);
  addIconCodePoint("md-silverware-spoon", 0xF4A5);
  addIconCodePoint("md-silverware-variant", 0xF4A6);
  addIconCodePoint("md-sim", 0xF4A7);
  addIconCodePoint("md-sim-alert", 0xF4A8);
  addIconCodePoint("md-sim-off", 0xF4A9);
  addIconCodePoint("md-simple-icons", 0xF0348);
  addIconCodePoint("md-sina-weibo", 0xFADE);
  addIconCodePoint("md-sitemap", 0xF4AA);
  addIconCodePoint("md-skate", 0xFD11);
  addIconCodePoint("md-skew-less", 0xFD12);
  addIconCodePoint("md-skew-more", 0xFD13);
  addIconCodePoint("md-ski", 0xF032F);
  addIconCodePoint("md-ski-cross-country", 0xF0330);
  addIconCodePoint("md-ski-water", 0xF0331);
  addIconCodePoint("md-skip-backward", 0xF4AB);
  addIconCodePoint("md-skip-backward-outline", 0xFF42);
  addIconCodePoint("md-skip-forward", 0xF4AC);
  addIconCodePoint("md-skip-forward-outline", 0xFF43);
  addIconCodePoint("md-skip-next", 0xF4AD);
  addIconCodePoint("md-skip-next-circle", 0xF661);
  addIconCodePoint("md-skip-next-circle-outline", 0xF662);
  addIconCodePoint("md-skip-next-outline", 0xFF44);
  addIconCodePoint("md-skip-previous", 0xF4AE);
  addIconCodePoint("md-skip-previous-circle", 0xF663);
  addIconCodePoint("md-skip-previous-circle-outline", 0xF664);
  addIconCodePoint("md-skip-previous-outline", 0xFF45);
  addIconCodePoint("md-skull", 0xF68B);
  addIconCodePoint("md-skull-crossbones", 0xFBA2);
  addIconCodePoint("md-skull-crossbones-outline", 0xFBA3);
  addIconCodePoint("md-skull-outline", 0xFBA4);
  addIconCodePoint("md-skype", 0xF4AF);
  addIconCodePoint("md-skype-business", 0xF4B0);
  addIconCodePoint("md-slack", 0xF4B1);
  addIconCodePoint("md-slackware", 0xF90A);
  addIconCodePoint("md-slash-forward", 0xF0000);
  addIconCodePoint("md-slash-forward-box", 0xF0001);
  addIconCodePoint("md-sleep", 0xF4B2);
  addIconCodePoint("md-sleep-off", 0xF4B3);
  addIconCodePoint("md-slope-downhill", 0xFE5F);
  addIconCodePoint("md-slope-uphill", 0xFE60);
  addIconCodePoint("md-slot-machine", 0xF013F);
  addIconCodePoint("md-slot-machine-outline", 0xF0140);
  addIconCodePoint("md-smart-card", 0xF00E8);
  addIconCodePoint("md-smart-card-outline", 0xF00E9);
  addIconCodePoint("md-smart-card-reader", 0xF00EA);
  addIconCodePoint("md-smart-card-reader-outline", 0xF00EB);
  addIconCodePoint("md-smog", 0xFA70);
  addIconCodePoint("md-smoke-detector", 0xF392);
  addIconCodePoint("md-smoking", 0xF4B4);
  addIconCodePoint("md-smoking-off", 0xF4B5);
  addIconCodePoint("md-snapchat", 0xF4B6);
  addIconCodePoint("md-snowboard", 0xF0332);
  addIconCodePoint("md-snowflake", 0xF716);
  addIconCodePoint("md-snowflake-alert", 0xFF46);
  addIconCodePoint("md-snowflake-melt", 0xF02F6);
  addIconCodePoint("md-snowflake-variant", 0xFF47);
  addIconCodePoint("md-snowman", 0xF4B7);
  addIconCodePoint("md-soccer", 0xF4B8);
  addIconCodePoint("md-soccer-field", 0xF833);
  addIconCodePoint("md-sofa", 0xF4B9);
  addIconCodePoint("md-solar-panel", 0xFD77);
  addIconCodePoint("md-solar-panel-large", 0xFD78);
  addIconCodePoint("md-solar-power", 0xFA71);
  addIconCodePoint("md-soldering-iron", 0xF00BD);
  addIconCodePoint("md-solid", 0xF68C);
  addIconCodePoint("md-sort", 0xF4BA);
  addIconCodePoint("md-sort-alphabetical", 0xF4BB);
  addIconCodePoint("md-sort-alphabetical-ascending", 0xF0173);
  addIconCodePoint("md-sort-alphabetical-descending", 0xF0174);
  addIconCodePoint("md-sort-ascending", 0xF4BC);
  addIconCodePoint("md-sort-descending", 0xF4BD);
  addIconCodePoint("md-sort-numeric", 0xF4BE);
  addIconCodePoint("md-sort-variant", 0xF4BF);
  addIconCodePoint("md-sort-variant-lock", 0xFCA9);
  addIconCodePoint("md-sort-variant-lock-open", 0xFCAA);
  addIconCodePoint("md-sort-variant-remove", 0xF0172);
  addIconCodePoint("md-soundcloud", 0xF4C0);
  addIconCodePoint("md-source-branch", 0xF62C);
  addIconCodePoint("md-source-commit", 0xF717);
  addIconCodePoint("md-source-commit-end", 0xF718);
  addIconCodePoint("md-source-commit-end-local", 0xF719);
  addIconCodePoint("md-source-commit-local", 0xF71A);
  addIconCodePoint("md-source-commit-next-local", 0xF71B);
  addIconCodePoint("md-source-commit-start", 0xF71C);
  addIconCodePoint("md-source-commit-start-next-local", 0xF71D);
  addIconCodePoint("md-source-fork", 0xF4C1);
  addIconCodePoint("md-source-merge", 0xF62D);
  addIconCodePoint("md-source-pull", 0xF4C2);
  addIconCodePoint("md-source-repository", 0xFCAB);
  addIconCodePoint("md-source-repository-multiple", 0xFCAC);
  addIconCodePoint("md-soy-sauce", 0xF7ED);
  addIconCodePoint("md-spa", 0xFCAD);
  addIconCodePoint("md-spa-outline", 0xFCAE);
  addIconCodePoint("md-space-invaders", 0xFBA5);
  addIconCodePoint("md-space-station", 0xF03AE);
  addIconCodePoint("md-spade", 0xFE48);
  addIconCodePoint("md-speaker", 0xF4C3);
  addIconCodePoint("md-speaker-bluetooth", 0xF9A1);
  addIconCodePoint("md-speaker-multiple", 0xFD14);
  addIconCodePoint("md-speaker-off", 0xF4C4);
  addIconCodePoint("md-speaker-wireless", 0xF71E);
  addIconCodePoint("md-speedometer", 0xF4C5);
  addIconCodePoint("md-speedometer-medium", 0xFFA2);
  addIconCodePoint("md-speedometer-slow", 0xFFA3);
  addIconCodePoint("md-spellcheck", 0xF4C6);
  addIconCodePoint("md-spider", 0xF0215);
  addIconCodePoint("md-spider-thread", 0xF0216);
  addIconCodePoint("md-spider-web", 0xFBA6);
  addIconCodePoint("md-spotify", 0xF4C7);
  addIconCodePoint("md-spotlight", 0xF4C8);
  addIconCodePoint("md-spotlight-beam", 0xF4C9);
  addIconCodePoint("md-spray", 0xF665);
  addIconCodePoint("md-spray-bottle", 0xFADF);
  addIconCodePoint("md-sprinkler", 0xF0081);
  addIconCodePoint("md-sprinkler-variant", 0xF0082);
  addIconCodePoint("md-sprout", 0xFE49);
  addIconCodePoint("md-sprout-outline", 0xFE4A);
  addIconCodePoint("md-square", 0xF763);
  addIconCodePoint("md-square-edit-outline", 0xF90B);
  addIconCodePoint("md-square-inc", 0xF4CA);
  addIconCodePoint("md-square-inc-cash", 0xF4CB);
  addIconCodePoint("md-square-medium", 0xFA12);
  addIconCodePoint("md-square-medium-outline", 0xFA13);
  addIconCodePoint("md-square-off", 0xF0319);
  addIconCodePoint("md-square-off-outline", 0xF031A);
  addIconCodePoint("md-square-outline", 0xF762);
  addIconCodePoint("md-square-root", 0xF783);
  addIconCodePoint("md-square-root-box", 0xF9A2);
  addIconCodePoint("md-square-small", 0xFA14);
  addIconCodePoint("md-squeegee", 0xFAE0);
  addIconCodePoint("md-ssh", 0xF8BF);
  addIconCodePoint("md-stack-exchange", 0xF60B);
  addIconCodePoint("md-stack-overflow", 0xF4CC);
  addIconCodePoint("md-stackpath", 0xF359);
  addIconCodePoint("md-stadium", 0xF001A);
  addIconCodePoint("md-stadium-variant", 0xF71F);
  addIconCodePoint("md-stairs", 0xF4CD);
  addIconCodePoint("md-stairs-down", 0xF02E9);
  addIconCodePoint("md-stairs-up", 0xF02E8);
  addIconCodePoint("md-stamper", 0xFD15);
  addIconCodePoint("md-standard-definition", 0xF7EE);
  addIconCodePoint("md-star", 0xF4CE);
  addIconCodePoint("md-star-box", 0xFA72);
  addIconCodePoint("md-star-box-multiple", 0xF02B1);
  addIconCodePoint("md-star-box-multiple-outline", 0xF02B2);
  addIconCodePoint("md-star-box-outline", 0xFA73);
  addIconCodePoint("md-star-circle", 0xF4CF);
  addIconCodePoint("md-star-circle-outline", 0xF9A3);
  addIconCodePoint("md-star-face", 0xF9A4);
  addIconCodePoint("md-star-four-points", 0xFAE1);
  addIconCodePoint("md-star-four-points-outline", 0xFAE2);
  addIconCodePoint("md-star-half", 0xF4D0);
  addIconCodePoint("md-star-off", 0xF4D1);
  addIconCodePoint("md-star-outline", 0xF4D2);
  addIconCodePoint("md-star-three-points", 0xFAE3);
  addIconCodePoint("md-star-three-points-outline", 0xFAE4);
  addIconCodePoint("md-state-machine", 0xF021A);
  addIconCodePoint("md-steam", 0xF4D3);
  addIconCodePoint("md-steam-box", 0xF90C);
  addIconCodePoint("md-steering", 0xF4D4);
  addIconCodePoint("md-steering-off", 0xF90D);
  addIconCodePoint("md-step-backward", 0xF4D5);
  addIconCodePoint("md-step-backward-2", 0xF4D6);
  addIconCodePoint("md-step-forward", 0xF4D7);
  addIconCodePoint("md-step-forward-2", 0xF4D8);
  addIconCodePoint("md-stethoscope", 0xF4D9);
  addIconCodePoint("md-sticker", 0xF038F);
  addIconCodePoint("md-sticker-alert", 0xF0390);
  addIconCodePoint("md-sticker-alert-outline", 0xF0391);
  addIconCodePoint("md-sticker-check", 0xF0392);
  addIconCodePoint("md-sticker-check-outline", 0xF0393);
  addIconCodePoint("md-sticker-circle-outline", 0xF5D0);
  addIconCodePoint("md-sticker-emoji", 0xF784);
  addIconCodePoint("md-sticker-minus", 0xF0394);
  addIconCodePoint("md-sticker-minus-outline", 0xF0395);
  addIconCodePoint("md-sticker-outline", 0xF0396);
  addIconCodePoint("md-sticker-plus", 0xF0397);
  addIconCodePoint("md-sticker-plus-outline", 0xF0398);
  addIconCodePoint("md-sticker-remove", 0xF0399);
  addIconCodePoint("md-sticker-remove-outline", 0xF039A);
  addIconCodePoint("md-stocking", 0xF4DA);
  addIconCodePoint("md-stomach", 0xF00BE);
  addIconCodePoint("md-stop", 0xF4DB);
  addIconCodePoint("md-stop-circle", 0xF666);
  addIconCodePoint("md-stop-circle-outline", 0xF667);
  addIconCodePoint("md-store", 0xF4DC);
  addIconCodePoint("md-store-24-hour", 0xF4DD);
  addIconCodePoint("md-store-outline", 0xF038C);
  addIconCodePoint("md-storefront", 0xF00EC);
  addIconCodePoint("md-stove", 0xF4DE);
  addIconCodePoint("md-strategy", 0xF0201);
  addIconCodePoint("md-strava", 0xFB25);
  addIconCodePoint("md-stretch-to-page", 0xFF48);
  addIconCodePoint("md-stretch-to-page-outline", 0xFF49);
  addIconCodePoint("md-string-lights", 0xF02E5);
  addIconCodePoint("md-string-lights-off", 0xF02E6);
  addIconCodePoint("md-subdirectory-arrow-left", 0xF60C);
  addIconCodePoint("md-subdirectory-arrow-right", 0xF60D);
  addIconCodePoint("md-subtitles", 0xFA15);
  addIconCodePoint("md-subtitles-outline", 0xFA16);
  addIconCodePoint("md-subway", 0xF6AB);
  addIconCodePoint("md-subway-alert-variant", 0xFD79);
  addIconCodePoint("md-subway-variant", 0xF4DF);
  addIconCodePoint("md-summit", 0xF785);
  addIconCodePoint("md-sunglasses", 0xF4E0);
  addIconCodePoint("md-surround-sound", 0xF5C5);
  addIconCodePoint("md-surround-sound-2-0", 0xF7EF);
  addIconCodePoint("md-surround-sound-3-1", 0xF7F0);
  addIconCodePoint("md-surround-sound-5-1", 0xF7F1);
  addIconCodePoint("md-surround-sound-7-1", 0xF7F2);
  addIconCodePoint("md-svg", 0xF720);
  addIconCodePoint("md-swap-horizontal", 0xF4E1);
  addIconCodePoint("md-swap-horizontal-bold", 0xFBA9);
  addIconCodePoint("md-swap-horizontal-circle", 0xF0002);
  addIconCodePoint("md-swap-horizontal-circle-outline", 0xF0003);
  addIconCodePoint("md-swap-horizontal-variant", 0xF8C0);
  addIconCodePoint("md-swap-vertical", 0xF4E2);
  addIconCodePoint("md-swap-vertical-bold", 0xFBAA);
  addIconCodePoint("md-swap-vertical-circle", 0xF0004);
  addIconCodePoint("md-swap-vertical-circle-outline", 0xF0005);
  addIconCodePoint("md-swap-vertical-variant", 0xF8C1);
  addIconCodePoint("md-swim", 0xF4E3);
  addIconCodePoint("md-switch", 0xF4E4);
  addIconCodePoint("md-sword", 0xF4E5);
  addIconCodePoint("md-sword-cross", 0xF786);
  addIconCodePoint("md-syllabary-hangul", 0xF035E);
  addIconCodePoint("md-syllabary-hiragana", 0xF035F);
  addIconCodePoint("md-syllabary-katakana", 0xF0360);
  addIconCodePoint("md-syllabary-katakana-half-width", 0xF0361);
  addIconCodePoint("md-symfony", 0xFAE5);
  addIconCodePoint("md-sync", 0xF4E6);
  addIconCodePoint("md-sync-alert", 0xF4E7);
  addIconCodePoint("md-sync-circle", 0xF03A3);
  addIconCodePoint("md-sync-off", 0xF4E8);
  addIconCodePoint("md-tab", 0xF4E9);
  addIconCodePoint("md-tab-minus", 0xFB26);
  addIconCodePoint("md-tab-plus", 0xF75B);
  addIconCodePoint("md-tab-remove", 0xFB27);
  addIconCodePoint("md-tab-unselected", 0xF4EA);
  addIconCodePoint("md-table", 0xF4EB);
  addIconCodePoint("md-table-border", 0xFA17);
  addIconCodePoint("md-table-chair", 0xF0083);
  addIconCodePoint("md-table-column", 0xF834);
  addIconCodePoint("md-table-column-plus-after", 0xF4EC);
  addIconCodePoint("md-table-column-plus-before", 0xF4ED);
  addIconCodePoint("md-table-column-remove", 0xF4EE);
  addIconCodePoint("md-table-column-width", 0xF4EF);
  addIconCodePoint("md-table-edit", 0xF4F0);
  addIconCodePoint("md-table-eye", 0xF00BF);
  addIconCodePoint("md-table-headers-eye", 0xF0248);
  addIconCodePoint("md-table-headers-eye-off", 0xF0249);
  addIconCodePoint("md-table-large", 0xF4F1);
  addIconCodePoint("md-table-large-plus", 0xFFA4);
  addIconCodePoint("md-table-large-remove", 0xFFA5);
  addIconCodePoint("md-table-merge-cells", 0xF9A5);
  addIconCodePoint("md-table-of-contents", 0xF835);
  addIconCodePoint("md-table-plus", 0xFA74);
  addIconCodePoint("md-table-remove", 0xFA75);
  addIconCodePoint("md-table-row", 0xF836);
  addIconCodePoint("md-table-row-height", 0xF4F2);
  addIconCodePoint("md-table-row-plus-after", 0xF4F3);
  addIconCodePoint("md-table-row-plus-before", 0xF4F4);
  addIconCodePoint("md-table-row-remove", 0xF4F5);
  addIconCodePoint("md-table-search", 0xF90E);
  addIconCodePoint("md-table-settings", 0xF837);
  addIconCodePoint("md-table-tennis", 0xFE4B);
  addIconCodePoint("md-tablet", 0xF4F6);
  addIconCodePoint("md-tablet-android", 0xF4F7);
  addIconCodePoint("md-tablet-cellphone", 0xF9A6);
  addIconCodePoint("md-tablet-dashboard", 0xFEEB);
  addIconCodePoint("md-tablet-ipad", 0xF4F8);
  addIconCodePoint("md-taco", 0xF761);
  addIconCodePoint("md-tag", 0xF4F9);
  addIconCodePoint("md-tag-faces", 0xF4FA);
  addIconCodePoint("md-tag-heart", 0xF68A);
  addIconCodePoint("md-tag-heart-outline", 0xFBAB);
  addIconCodePoint("md-tag-minus", 0xF90F);
  addIconCodePoint("md-tag-minus-outline", 0xF024A);
  addIconCodePoint("md-tag-multiple", 0xF4FB);
  addIconCodePoint("md-tag-multiple-outline", 0xF0322);
  addIconCodePoint("md-tag-off", 0xF024B);
  addIconCodePoint("md-tag-off-outline", 0xF024C);
  addIconCodePoint("md-tag-outline", 0xF4FC);
  addIconCodePoint("md-tag-plus", 0xF721);
  addIconCodePoint("md-tag-plus-outline", 0xF024D);
  addIconCodePoint("md-tag-remove", 0xF722);
  addIconCodePoint("md-tag-remove-outline", 0xF024E);
  addIconCodePoint("md-tag-text", 0xF024F);
  addIconCodePoint("md-tag-text-outline", 0xF4FD);
  addIconCodePoint("md-tank", 0xFD16);
  addIconCodePoint("md-tanker-truck", 0xF0006);
  addIconCodePoint("md-tape-measure", 0xFB28);
  addIconCodePoint("md-target", 0xF4FE);
  addIconCodePoint("md-target-account", 0xFBAC);
  addIconCodePoint("md-target-variant", 0xFA76);
  addIconCodePoint("md-taxi", 0xF4FF);
  addIconCodePoint("md-tea", 0xFD7A);
  addIconCodePoint("md-tea-outline", 0xFD7B);
  addIconCodePoint("md-teach", 0xF88F);
  addIconCodePoint("md-teamviewer", 0xF500);
  addIconCodePoint("md-telegram", 0xF501);
  addIconCodePoint("md-telescope", 0xFB29);
  addIconCodePoint("md-television", 0xF502);
  addIconCodePoint("md-television-ambient-light", 0xF0381);
  addIconCodePoint("md-television-box", 0xF838);
  addIconCodePoint("md-television-classic", 0xF7F3);
  addIconCodePoint("md-television-classic-off", 0xF839);
  addIconCodePoint("md-television-clean", 0xF013B);
  addIconCodePoint("md-television-guide", 0xF503);
  addIconCodePoint("md-television-off", 0xF83A);
  addIconCodePoint("md-television-pause", 0xFFA6);
  addIconCodePoint("md-television-play", 0xFEEC);
  addIconCodePoint("md-television-stop", 0xFFA7);
  addIconCodePoint("md-temperature-celsius", 0xF504);
  addIconCodePoint("md-temperature-fahrenheit", 0xF505);
  addIconCodePoint("md-temperature-kelvin", 0xF506);
  addIconCodePoint("md-tennis", 0xFD7C);
  addIconCodePoint("md-tennis-ball", 0xF507);
  addIconCodePoint("md-tent", 0xF508);
  addIconCodePoint("md-terraform", 0xF0084);
  addIconCodePoint("md-terrain", 0xF509);
  addIconCodePoint("md-test-tube", 0xF668);
  addIconCodePoint("md-test-tube-empty", 0xF910);
  addIconCodePoint("md-test-tube-off", 0xF911);
  addIconCodePoint("md-text", 0xF9A7);
  addIconCodePoint("md-text-recognition", 0xF0168);
  addIconCodePoint("md-text-shadow", 0xF669);
  addIconCodePoint("md-text-short", 0xF9A8);
  addIconCodePoint("md-text-subject", 0xF9A9);
  addIconCodePoint("md-text-to-speech", 0xF50A);
  addIconCodePoint("md-text-to-speech-off", 0xF50B);
  addIconCodePoint("md-textarea", 0xF00C0);
  addIconCodePoint("md-textbox", 0xF60E);
  addIconCodePoint("md-textbox-lock", 0xF0388);
  addIconCodePoint("md-textbox-password", 0xF7F4);
  addIconCodePoint("md-texture", 0xF50C);
  addIconCodePoint("md-texture-box", 0xF0007);
  addIconCodePoint("md-theater", 0xF50D);
  addIconCodePoint("md-theme-light-dark", 0xF50E);
  addIconCodePoint("md-thermometer", 0xF50F);
  addIconCodePoint("md-thermometer-alert", 0xFE61);
  addIconCodePoint("md-thermometer-chevron-down", 0xFE62);
  addIconCodePoint("md-thermometer-chevron-up", 0xFE63);
  addIconCodePoint("md-thermometer-high", 0xF00ED);
  addIconCodePoint("md-thermometer-lines", 0xF510);
  addIconCodePoint("md-thermometer-low", 0xF00EE);
  addIconCodePoint("md-thermometer-minus", 0xFE64);
  addIconCodePoint("md-thermometer-plus", 0xFE65);
  addIconCodePoint("md-thermostat", 0xF393);
  addIconCodePoint("md-thermostat-box", 0xF890);
  addIconCodePoint("md-thought-bubble", 0xF7F5);
  addIconCodePoint("md-thought-bubble-outline", 0xF7F6);
  addIconCodePoint("md-thumb-down", 0xF511);
  addIconCodePoint("md-thumb-down-outline", 0xF512);
  addIconCodePoint("md-thumb-up", 0xF513);
  addIconCodePoint("md-thumb-up-outline", 0xF514);
  addIconCodePoint("md-thumbs-up-down", 0xF515);
  addIconCodePoint("md-ticket", 0xF516);
  addIconCodePoint("md-ticket-account", 0xF517);
  addIconCodePoint("md-ticket-confirmation", 0xF518);
  addIconCodePoint("md-ticket-outline", 0xF912);
  addIconCodePoint("md-ticket-percent", 0xF723);
  addIconCodePoint("md-tie", 0xF519);
  addIconCodePoint("md-tilde", 0xF724);
  addIconCodePoint("md-timelapse", 0xF51A);
  addIconCodePoint("md-timeline", 0xFBAD);
  addIconCodePoint("md-timeline-alert", 0xFFB2);
  addIconCodePoint("md-timeline-alert-outline", 0xFFB5);
  addIconCodePoint("md-timeline-clock", 0xF0226);
  addIconCodePoint("md-timeline-clock-outline", 0xF0227);
  addIconCodePoint("md-timeline-help", 0xFFB6);
  addIconCodePoint("md-timeline-help-outline", 0xFFB7);
  addIconCodePoint("md-timeline-outline", 0xFBAE);
  addIconCodePoint("md-timeline-plus", 0xFFB3);
  addIconCodePoint("md-timeline-plus-outline", 0xFFB4);
  addIconCodePoint("md-timeline-text", 0xFBAF);
  addIconCodePoint("md-timeline-text-outline", 0xFBB0);
  addIconCodePoint("md-timer", 0xF51B);
  addIconCodePoint("md-timer-10", 0xF51C);
  addIconCodePoint("md-timer-3", 0xF51D);
  addIconCodePoint("md-timer-off", 0xF51E);
  addIconCodePoint("md-timer-sand", 0xF51F);
  addIconCodePoint("md-timer-sand-empty", 0xF6AC);
  addIconCodePoint("md-timer-sand-full", 0xF78B);
  addIconCodePoint("md-timetable", 0xF520);
  addIconCodePoint("md-toaster", 0xF0085);
  addIconCodePoint("md-toaster-off", 0xF01E2);
  addIconCodePoint("md-toaster-oven", 0xFCAF);
  addIconCodePoint("md-toggle-switch", 0xF521);
  addIconCodePoint("md-toggle-switch-off", 0xF522);
  addIconCodePoint("md-toggle-switch-off-outline", 0xFA18);
  addIconCodePoint("md-toggle-switch-outline", 0xFA19);
  addIconCodePoint("md-toilet", 0xF9AA);
  addIconCodePoint("md-toolbox", 0xF9AB);
  addIconCodePoint("md-toolbox-outline", 0xF9AC);
  addIconCodePoint("md-tools", 0xF0086);
  addIconCodePoint("md-tooltip", 0xF523);
  addIconCodePoint("md-tooltip-account", 0xF00C);
  addIconCodePoint("md-tooltip-edit", 0xF524);
  addIconCodePoint("md-tooltip-edit-outline", 0xF02F0);
  addIconCodePoint("md-tooltip-image", 0xF525);
  addIconCodePoint("md-tooltip-image-outline", 0xFBB1);
  addIconCodePoint("md-tooltip-outline", 0xF526);
  addIconCodePoint("md-tooltip-plus", 0xFBB2);
  addIconCodePoint("md-tooltip-plus-outline", 0xF527);
  addIconCodePoint("md-tooltip-text", 0xF528);
  addIconCodePoint("md-tooltip-text-outline", 0xFBB3);
  addIconCodePoint("md-tooth", 0xF8C2);
  addIconCodePoint("md-tooth-outline", 0xF529);
  addIconCodePoint("md-toothbrush", 0xF0154);
  addIconCodePoint("md-toothbrush-electric", 0xF0157);
  addIconCodePoint("md-toothbrush-paste", 0xF0155);
  addIconCodePoint("md-tor", 0xF52A);
  addIconCodePoint("md-tortoise", 0xFD17);
  addIconCodePoint("md-toslink", 0xF02E3);
  addIconCodePoint("md-tournament", 0xF9AD);
  addIconCodePoint("md-tower-beach", 0xF680);
  addIconCodePoint("md-tower-fire", 0xF681);
  addIconCodePoint("md-towing", 0xF83B);
  addIconCodePoint("md-toy-brick", 0xF02B3);
  addIconCodePoint("md-toy-brick-marker", 0xF02B4);
  addIconCodePoint("md-toy-brick-marker-outline", 0xF02B5);
  addIconCodePoint("md-toy-brick-minus", 0xF02B6);
  addIconCodePoint("md-toy-brick-minus-outline", 0xF02B7);
  addIconCodePoint("md-toy-brick-outline", 0xF02B8);
  addIconCodePoint("md-toy-brick-plus", 0xF02B9);
  addIconCodePoint("md-toy-brick-plus-outline", 0xF02BA);
  addIconCodePoint("md-toy-brick-remove", 0xF02BB);
  addIconCodePoint("md-toy-brick-remove-outline", 0xF02BC);
  addIconCodePoint("md-toy-brick-search", 0xF02BD);
  addIconCodePoint("md-toy-brick-search-outline", 0xF02BE);
  addIconCodePoint("md-track-light", 0xF913);
  addIconCodePoint("md-trackpad", 0xF7F7);
  addIconCodePoint("md-trackpad-lock", 0xF932);
  addIconCodePoint("md-tractor", 0xF891);
  addIconCodePoint("md-trademark", 0xFA77);
  addIconCodePoint("md-traffic-cone", 0xF03A7);
  addIconCodePoint("md-traffic-light", 0xF52B);
  addIconCodePoint("md-train", 0xF52C);
  addIconCodePoint("md-train-car", 0xFBB4);
  addIconCodePoint("md-train-variant", 0xF8C3);
  addIconCodePoint("md-tram", 0xF52D);
  addIconCodePoint("md-tram-side", 0xF0008);
  addIconCodePoint("md-transcribe", 0xF52E);
  addIconCodePoint("md-transcribe-close", 0xF52F);
  addIconCodePoint("md-transfer", 0xF0087);
  addIconCodePoint("md-transfer-down", 0xFD7D);
  addIconCodePoint("md-transfer-left", 0xFD7E);
  addIconCodePoint("md-transfer-right", 0xF530);
  addIconCodePoint("md-transfer-up", 0xFD7F);
  addIconCodePoint("md-transit-connection", 0xFD18);
  addIconCodePoint("md-transit-connection-variant", 0xFD19);
  addIconCodePoint("md-transit-detour", 0xFFA8);
  addIconCodePoint("md-transit-transfer", 0xF6AD);
  addIconCodePoint("md-transition", 0xF914);
  addIconCodePoint("md-transition-masked", 0xF915);
  addIconCodePoint("md-translate", 0xF5CA);
  addIconCodePoint("md-translate-off", 0xFE66);
  addIconCodePoint("md-transmission-tower", 0xFD1A);
  addIconCodePoint("md-trash-can", 0xFA78);
  addIconCodePoint("md-trash-can-outline", 0xFA79);
  addIconCodePoint("md-tray", 0xF02BF);
  addIconCodePoint("md-tray-alert", 0xF02C0);
  addIconCodePoint("md-tray-full", 0xF02C1);
  addIconCodePoint("md-tray-minus", 0xF02C2);
  addIconCodePoint("md-tray-plus", 0xF02C3);
  addIconCodePoint("md-tray-remove", 0xF02C4);
  addIconCodePoint("md-treasure-chest", 0xF725);
  addIconCodePoint("md-tree", 0xF531);
  addIconCodePoint("md-tree-outline", 0xFE4C);
  addIconCodePoint("md-trello", 0xF532);
  addIconCodePoint("md-trending-down", 0xF533);
  addIconCodePoint("md-trending-neutral", 0xF534);
  addIconCodePoint("md-trending-up", 0xF535);
  addIconCodePoint("md-triangle", 0xF536);
  addIconCodePoint("md-triangle-outline", 0xF537);
  addIconCodePoint("md-triforce", 0xFBB5);
  addIconCodePoint("md-trophy", 0xF538);
  addIconCodePoint("md-trophy-award", 0xF539);
  addIconCodePoint("md-trophy-broken", 0xFD80);
  addIconCodePoint("md-trophy-outline", 0xF53A);
  addIconCodePoint("md-trophy-variant", 0xF53B);
  addIconCodePoint("md-trophy-variant-outline", 0xF53C);
  addIconCodePoint("md-truck", 0xF53D);
  addIconCodePoint("md-truck-check", 0xFCB0);
  addIconCodePoint("md-truck-check-outline", 0xF02C5);
  addIconCodePoint("md-truck-delivery", 0xF53E);
  addIconCodePoint("md-truck-delivery-outline", 0xF02C6);
  addIconCodePoint("md-truck-fast", 0xF787);
  addIconCodePoint("md-truck-fast-outline", 0xF02C7);
  addIconCodePoint("md-truck-outline", 0xF02C8);
  addIconCodePoint("md-truck-trailer", 0xF726);
  addIconCodePoint("md-trumpet", 0xF00C1);
  addIconCodePoint("md-tshirt-crew", 0xFA7A);
  addIconCodePoint("md-tshirt-crew-outline", 0xF53F);
  addIconCodePoint("md-tshirt-v", 0xFA7B);
  addIconCodePoint("md-tshirt-v-outline", 0xF540);
  addIconCodePoint("md-tumble-dryer", 0xF916);
  addIconCodePoint("md-tumble-dryer-alert", 0xF01E5);
  addIconCodePoint("md-tumble-dryer-off", 0xF01E6);
  addIconCodePoint("md-tumblr", 0xF541);
  addIconCodePoint("md-tumblr-box", 0xF917);
  addIconCodePoint("md-tumblr-reblog", 0xF542);
  addIconCodePoint("md-tune", 0xF62E);
  addIconCodePoint("md-tune-vertical", 0xF66A);
  addIconCodePoint("md-turnstile", 0xFCB1);
  addIconCodePoint("md-turnstile-outline", 0xFCB2);
  addIconCodePoint("md-turtle", 0xFCB3);
  addIconCodePoint("md-twitch", 0xF543);
  addIconCodePoint("md-twitter", 0xF544);
  addIconCodePoint("md-twitter-box", 0xF545);
  addIconCodePoint("md-twitter-circle", 0xF546);
  addIconCodePoint("md-twitter-retweet", 0xF547);
  addIconCodePoint("md-two-factor-authentication", 0xF9AE);
  addIconCodePoint("md-typewriter", 0xFF4A);
  addIconCodePoint("md-uber", 0xF748);
  addIconCodePoint("md-ubisoft", 0xFBB6);
  addIconCodePoint("md-ubuntu", 0xF548);
  addIconCodePoint("md-ufo", 0xF00EF);
  addIconCodePoint("md-ufo-outline", 0xF00F0);
  addIconCodePoint("md-ultra-high-definition", 0xF7F8);
  addIconCodePoint("md-umbraco", 0xF549);
  addIconCodePoint("md-umbrella", 0xF54A);
  addIconCodePoint("md-umbrella-closed", 0xF9AF);
  addIconCodePoint("md-umbrella-outline", 0xF54B);
  addIconCodePoint("md-undo", 0xF54C);
  addIconCodePoint("md-undo-variant", 0xF54D);
  addIconCodePoint("md-unfold-less-horizontal", 0xF54E);
  addIconCodePoint("md-unfold-less-vertical", 0xF75F);
  addIconCodePoint("md-unfold-more-horizontal", 0xF54F);
  addIconCodePoint("md-unfold-more-vertical", 0xF760);
  addIconCodePoint("md-ungroup", 0xF550);
  addIconCodePoint("md-unicode", 0xFEED);
  addIconCodePoint("md-unity", 0xF6AE);
  addIconCodePoint("md-unreal", 0xF9B0);
  addIconCodePoint("md-untappd", 0xF551);
  addIconCodePoint("md-update", 0xF6AF);
  addIconCodePoint("md-upload", 0xF552);
  addIconCodePoint("md-upload-lock", 0xF039E);
  addIconCodePoint("md-upload-lock-outline", 0xF039F);
  addIconCodePoint("md-upload-multiple", 0xF83C);
  addIconCodePoint("md-upload-network", 0xF6F5);
  addIconCodePoint("md-upload-network-outline", 0xFCB4);
  addIconCodePoint("md-upload-off", 0xF00F1);
  addIconCodePoint("md-upload-off-outline", 0xF00F2);
  addIconCodePoint("md-upload-outline", 0xFE67);
  addIconCodePoint("md-usb", 0xF553);
  addIconCodePoint("md-usb-flash-drive", 0xF02C9);
  addIconCodePoint("md-usb-flash-drive-outline", 0xF02CA);
  addIconCodePoint("md-usb-port", 0xF021B);
  addIconCodePoint("md-valve", 0xF0088);
  addIconCodePoint("md-valve-closed", 0xF0089);
  addIconCodePoint("md-valve-open", 0xF008A);
  addIconCodePoint("md-van-passenger", 0xF7F9);
  addIconCodePoint("md-van-utility", 0xF7FA);
  addIconCodePoint("md-vanish", 0xF7FB);
  addIconCodePoint("md-vanity-light", 0xF020C);
  addIconCodePoint("md-variable", 0xFAE6);
  addIconCodePoint("md-variable-box", 0xF013C);
  addIconCodePoint("md-vector-arrange-above", 0xF554);
  addIconCodePoint("md-vector-arrange-below", 0xF555);
  addIconCodePoint("md-vector-bezier", 0xFAE7);
  addIconCodePoint("md-vector-circle", 0xF556);
  addIconCodePoint("md-vector-circle-variant", 0xF557);
  addIconCodePoint("md-vector-combine", 0xF558);
  addIconCodePoint("md-vector-curve", 0xF559);
  addIconCodePoint("md-vector-difference", 0xF55A);
  addIconCodePoint("md-vector-difference-ab", 0xF55B);
  addIconCodePoint("md-vector-difference-ba", 0xF55C);
  addIconCodePoint("md-vector-ellipse", 0xF892);
  addIconCodePoint("md-vector-intersection", 0xF55D);
  addIconCodePoint("md-vector-line", 0xF55E);
  addIconCodePoint("md-vector-link", 0xF0009);
  addIconCodePoint("md-vector-point", 0xF55F);
  addIconCodePoint("md-vector-polygon", 0xF560);
  addIconCodePoint("md-vector-polyline", 0xF561);
  addIconCodePoint("md-vector-polyline-edit", 0xF0250);
  addIconCodePoint("md-vector-polyline-minus", 0xF0251);
  addIconCodePoint("md-vector-polyline-plus", 0xF0252);
  addIconCodePoint("md-vector-polyline-remove", 0xF0253);
  addIconCodePoint("md-vector-radius", 0xF749);
  addIconCodePoint("md-vector-rectangle", 0xF5C6);
  addIconCodePoint("md-vector-selection", 0xF562);
  addIconCodePoint("md-vector-square", 0xF001);
  addIconCodePoint("md-vector-triangle", 0xF563);
  addIconCodePoint("md-vector-union", 0xF564);
  addIconCodePoint("md-venmo", 0xF578);
  addIconCodePoint("md-vhs", 0xFA1A);
  addIconCodePoint("md-vibrate", 0xF566);
  addIconCodePoint("md-vibrate-off", 0xFCB5);
  addIconCodePoint("md-video", 0xF567);
  addIconCodePoint("md-video-3d", 0xF7FC);
  addIconCodePoint("md-video-3d-variant", 0xFEEE);
  addIconCodePoint("md-video-4k-box", 0xF83D);
  addIconCodePoint("md-video-account", 0xF918);
  addIconCodePoint("md-video-check", 0xF008B);
  addIconCodePoint("md-video-check-outline", 0xF008C);
  addIconCodePoint("md-video-image", 0xF919);
  addIconCodePoint("md-video-input-antenna", 0xF83E);
  addIconCodePoint("md-video-input-component", 0xF83F);
  addIconCodePoint("md-video-input-hdmi", 0xF840);
  addIconCodePoint("md-video-input-scart", 0xFFA9);
  addIconCodePoint("md-video-input-svideo", 0xF841);
  addIconCodePoint("md-video-minus", 0xF9B1);
  addIconCodePoint("md-video-off", 0xF568);
  addIconCodePoint("md-video-off-outline", 0xFBB7);
  addIconCodePoint("md-video-outline", 0xFBB8);
  addIconCodePoint("md-video-plus", 0xF9B2);
  addIconCodePoint("md-video-stabilization", 0xF91A);
  addIconCodePoint("md-video-switch", 0xF569);
  addIconCodePoint("md-video-vintage", 0xFA1B);
  addIconCodePoint("md-video-wireless", 0xFEEF);
  addIconCodePoint("md-video-wireless-outline", 0xFEF0);
  addIconCodePoint("md-view-agenda", 0xF56A);
  addIconCodePoint("md-view-agenda-outline", 0xF0203);
  addIconCodePoint("md-view-array", 0xF56B);
  addIconCodePoint("md-view-carousel", 0xF56C);
  addIconCodePoint("md-view-column", 0xF56D);
  addIconCodePoint("md-view-comfy", 0xFE4D);
  addIconCodePoint("md-view-compact", 0xFE4E);
  addIconCodePoint("md-view-compact-outline", 0xFE4F);
  addIconCodePoint("md-view-dashboard", 0xF56E);
  addIconCodePoint("md-view-dashboard-outline", 0xFA1C);
  addIconCodePoint("md-view-dashboard-variant", 0xF842);
  addIconCodePoint("md-view-day", 0xF56F);
  addIconCodePoint("md-view-grid", 0xF570);
  addIconCodePoint("md-view-grid-outline", 0xF0204);
  addIconCodePoint("md-view-grid-plus", 0xFFAA);
  addIconCodePoint("md-view-grid-plus-outline", 0xF0205);
  addIconCodePoint("md-view-headline", 0xF571);
  addIconCodePoint("md-view-list", 0xF572);
  addIconCodePoint("md-view-module", 0xF573);
  addIconCodePoint("md-view-parallel", 0xF727);
  addIconCodePoint("md-view-quilt", 0xF574);
  addIconCodePoint("md-view-sequential", 0xF728);
  addIconCodePoint("md-view-split-horizontal", 0xFBA7);
  addIconCodePoint("md-view-split-vertical", 0xFBA8);
  addIconCodePoint("md-view-stream", 0xF575);
  addIconCodePoint("md-view-week", 0xF576);
  addIconCodePoint("md-vimeo", 0xF577);
  addIconCodePoint("md-violin", 0xF60F);
  addIconCodePoint("md-virtual-reality", 0xF893);
  addIconCodePoint("md-visual-studio", 0xF610);
  addIconCodePoint("md-visual-studio-code", 0xFA1D);
  addIconCodePoint("md-vk", 0xF579);
  addIconCodePoint("md-vk-box", 0xF57A);
  addIconCodePoint("md-vk-circle", 0xF57B);
  addIconCodePoint("md-vlc", 0xF57C);
  addIconCodePoint("md-voice", 0xF5CB);
  addIconCodePoint("md-voice-off", 0xFEF1);
  addIconCodePoint("md-voicemail", 0xF57D);
  addIconCodePoint("md-volleyball", 0xF9B3);
  addIconCodePoint("md-volume-high", 0xF57E);
  addIconCodePoint("md-volume-low", 0xF57F);
  addIconCodePoint("md-volume-medium", 0xF580);
  addIconCodePoint("md-volume-minus", 0xF75D);
  addIconCodePoint("md-volume-mute", 0xF75E);
  addIconCodePoint("md-volume-off", 0xF581);
  addIconCodePoint("md-volume-plus", 0xF75C);
  addIconCodePoint("md-volume-source", 0xF014B);
  addIconCodePoint("md-volume-variant-off", 0xFE68);
  addIconCodePoint("md-volume-vibrate", 0xF014C);
  addIconCodePoint("md-vote", 0xFA1E);
  addIconCodePoint("md-vote-outline", 0xFA1F);
  addIconCodePoint("md-vpn", 0xF582);
  addIconCodePoint("md-vuejs", 0xF843);
  addIconCodePoint("md-vuetify", 0xFE50);
  addIconCodePoint("md-walk", 0xF583);
  addIconCodePoint("md-wall", 0xF7FD);
  addIconCodePoint("md-wall-sconce", 0xF91B);
  addIconCodePoint("md-wall-sconce-flat", 0xF91C);
  addIconCodePoint("md-wall-sconce-variant", 0xF91D);
  addIconCodePoint("md-wallet", 0xF584);
  addIconCodePoint("md-wallet-giftcard", 0xF585);
  addIconCodePoint("md-wallet-membership", 0xF586);
  addIconCodePoint("md-wallet-outline", 0xFBB9);
  addIconCodePoint("md-wallet-plus", 0xFFAB);
  addIconCodePoint("md-wallet-plus-outline", 0xFFAC);
  addIconCodePoint("md-wallet-travel", 0xF587);
  addIconCodePoint("md-wallpaper", 0xFE69);
  addIconCodePoint("md-wan", 0xF588);
  addIconCodePoint("md-wardrobe", 0xFFAD);
  addIconCodePoint("md-wardrobe-outline", 0xFFAE);
  addIconCodePoint("md-warehouse", 0xFFBB);
  addIconCodePoint("md-washing-machine", 0xF729);
  addIconCodePoint("md-washing-machine-alert", 0xF01E7);
  addIconCodePoint("md-washing-machine-off", 0xF01E8);
  addIconCodePoint("md-watch", 0xF589);
  addIconCodePoint("md-watch-export", 0xF58A);
  addIconCodePoint("md-watch-export-variant", 0xF894);
  addIconCodePoint("md-watch-import", 0xF58B);
  addIconCodePoint("md-watch-import-variant", 0xF895);
  addIconCodePoint("md-watch-variant", 0xF896);
  addIconCodePoint("md-watch-vibrate", 0xF6B0);
  addIconCodePoint("md-watch-vibrate-off", 0xFCB6);
  addIconCodePoint("md-water", 0xF58C);
  addIconCodePoint("md-water-boiler", 0xFFAF);
  addIconCodePoint("md-water-boiler-alert", 0xF01DE);
  addIconCodePoint("md-water-boiler-off", 0xF01DF);
  addIconCodePoint("md-water-off", 0xF58D);
  addIconCodePoint("md-water-outline", 0xFE6A);
  addIconCodePoint("md-water-percent", 0xF58E);
  addIconCodePoint("md-water-polo", 0xF02CB);
  addIconCodePoint("md-water-pump", 0xF58F);
  addIconCodePoint("md-water-pump-off", 0xFFB0);
  addIconCodePoint("md-water-well", 0xF008D);
  addIconCodePoint("md-water-well-outline", 0xF008E);
  addIconCodePoint("md-watermark", 0xF612);
  addIconCodePoint("md-wave", 0xFF4B);
  addIconCodePoint("md-waves", 0xF78C);
  addIconCodePoint("md-waze", 0xFBBA);
  addIconCodePoint("md-weather-cloudy", 0xF590);
  addIconCodePoint("md-weather-cloudy-alert", 0xFF4C);
  addIconCodePoint("md-weather-cloudy-arrow-right", 0xFE51);
  addIconCodePoint("md-weather-fog", 0xF591);
  addIconCodePoint("md-weather-hail", 0xF592);
  addIconCodePoint("md-weather-hazy", 0xFF4D);
  addIconCodePoint("md-weather-hurricane", 0xF897);
  addIconCodePoint("md-weather-lightning", 0xF593);
  addIconCodePoint("md-weather-lightning-rainy", 0xF67D);
  addIconCodePoint("md-weather-night", 0xF594);
  addIconCodePoint("md-weather-night-partly-cloudy", 0xFF4E);
  addIconCodePoint("md-weather-partly-cloudy", 0xF595);
  addIconCodePoint("md-weather-partly-lightning", 0xFF4F);
  addIconCodePoint("md-weather-partly-rainy", 0xFF50);
  addIconCodePoint("md-weather-partly-snowy", 0xFF51);
  addIconCodePoint("md-weather-partly-snowy-rainy", 0xFF52);
  addIconCodePoint("md-weather-pouring", 0xF596);
  addIconCodePoint("md-weather-rainy", 0xF597);
  addIconCodePoint("md-weather-snowy", 0xF598);
  addIconCodePoint("md-weather-snowy-heavy", 0xFF53);
  addIconCodePoint("md-weather-snowy-rainy", 0xF67E);
  addIconCodePoint("md-weather-sunny", 0xF599);
  addIconCodePoint("md-weather-sunny-alert", 0xFF54);
  addIconCodePoint("md-weather-sunset", 0xF59A);
  addIconCodePoint("md-weather-sunset-down", 0xF59B);
  addIconCodePoint("md-weather-sunset-up", 0xF59C);
  addIconCodePoint("md-weather-tornado", 0xFF55);
  addIconCodePoint("md-weather-windy", 0xF59D);
  addIconCodePoint("md-weather-windy-variant", 0xF59E);
  addIconCodePoint("md-web", 0xF59F);
  addIconCodePoint("md-web-box", 0xFFB1);
  addIconCodePoint("md-web-clock", 0xF0275);
  addIconCodePoint("md-webcam", 0xF5A0);
  addIconCodePoint("md-webhook", 0xF62F);
  addIconCodePoint("md-webpack", 0xF72A);
  addIconCodePoint("md-webrtc", 0xF0273);
  addIconCodePoint("md-wechat", 0xF611);
  addIconCodePoint("md-weight", 0xF5A1);
  addIconCodePoint("md-weight-gram", 0xFD1B);
  addIconCodePoint("md-weight-kilogram", 0xF5A2);
  addIconCodePoint("md-weight-lifter", 0xF0188);
  addIconCodePoint("md-weight-pound", 0xF9B4);
  addIconCodePoint("md-whatsapp", 0xF5A3);
  addIconCodePoint("md-wheelchair-accessibility", 0xF5A4);
  addIconCodePoint("md-whistle", 0xF9B5);
  addIconCodePoint("md-whistle-outline", 0xF02E7);
  addIconCodePoint("md-white-balance-auto", 0xF5A5);
  addIconCodePoint("md-white-balance-incandescent", 0xF5A6);
  addIconCodePoint("md-white-balance-iridescent", 0xF5A7);
  addIconCodePoint("md-white-balance-sunny", 0xF5A8);
  addIconCodePoint("md-widgets", 0xF72B);
  addIconCodePoint("md-widgets-outline", 0xF0380);
  addIconCodePoint("md-wifi", 0xF5A9);
  addIconCodePoint("md-wifi-off", 0xF5AA);
  addIconCodePoint("md-wifi-star", 0xFE6B);
  addIconCodePoint("md-wifi-strength-1", 0xF91E);
  addIconCodePoint("md-wifi-strength-1-alert", 0xF91F);
  addIconCodePoint("md-wifi-strength-1-lock", 0xF920);
  addIconCodePoint("md-wifi-strength-2", 0xF921);
  addIconCodePoint("md-wifi-strength-2-alert", 0xF922);
  addIconCodePoint("md-wifi-strength-2-lock", 0xF923);
  addIconCodePoint("md-wifi-strength-3", 0xF924);
  addIconCodePoint("md-wifi-strength-3-alert", 0xF925);
  addIconCodePoint("md-wifi-strength-3-lock", 0xF926);
  addIconCodePoint("md-wifi-strength-4", 0xF927);
  addIconCodePoint("md-wifi-strength-4-alert", 0xF928);
  addIconCodePoint("md-wifi-strength-4-lock", 0xF929);
  addIconCodePoint("md-wifi-strength-alert-outline", 0xF92A);
  addIconCodePoint("md-wifi-strength-lock-outline", 0xF92B);
  addIconCodePoint("md-wifi-strength-off", 0xF92C);
  addIconCodePoint("md-wifi-strength-off-outline", 0xF92D);
  addIconCodePoint("md-wifi-strength-outline", 0xF92E);
  addIconCodePoint("md-wii", 0xF5AB);
  addIconCodePoint("md-wiiu", 0xF72C);
  addIconCodePoint("md-wikipedia", 0xF5AC);
  addIconCodePoint("md-wind-turbine", 0xFD81);
  addIconCodePoint("md-window-close", 0xF5AD);
  addIconCodePoint("md-window-closed", 0xF5AE);
  addIconCodePoint("md-window-closed-variant", 0xF0206);
  addIconCodePoint("md-window-maximize", 0xF5AF);
  addIconCodePoint("md-window-minimize", 0xF5B0);
  addIconCodePoint("md-window-open", 0xF5B1);
  addIconCodePoint("md-window-open-variant", 0xF0207);
  addIconCodePoint("md-window-restore", 0xF5B2);
  addIconCodePoint("md-window-shutter", 0xF0147);
  addIconCodePoint("md-window-shutter-alert", 0xF0148);
  addIconCodePoint("md-window-shutter-open", 0xF0149);
  addIconCodePoint("md-windows", 0xF5B3);
  addIconCodePoint("md-windows-classic", 0xFA20);
  addIconCodePoint("md-wiper", 0xFAE8);
  addIconCodePoint("md-wiper-wash", 0xFD82);
  addIconCodePoint("md-wordpress", 0xF5B4);
  addIconCodePoint("md-worker", 0xF5B5);
  addIconCodePoint("md-wrap", 0xF5B6);
  addIconCodePoint("md-wrap-disabled", 0xFBBB);
  addIconCodePoint("md-wrench", 0xF5B7);
  addIconCodePoint("md-wrench-outline", 0xFBBC);
  addIconCodePoint("md-wunderlist", 0xF5B8);
  addIconCodePoint("md-xamarin", 0xF844);
  addIconCodePoint("md-xamarin-outline", 0xF845);
  addIconCodePoint("md-xaml", 0xF673);
  addIconCodePoint("md-xbox", 0xF5B9);
  addIconCodePoint("md-xbox-controller", 0xF5BA);
  addIconCodePoint("md-xbox-controller-battery-alert", 0xF74A);
  addIconCodePoint("md-xbox-controller-battery-charging", 0xFA21);
  addIconCodePoint("md-xbox-controller-battery-empty", 0xF74B);
  addIconCodePoint("md-xbox-controller-battery-full", 0xF74C);
  addIconCodePoint("md-xbox-controller-battery-low", 0xF74D);
  addIconCodePoint("md-xbox-controller-battery-medium", 0xF74E);
  addIconCodePoint("md-xbox-controller-battery-unknown", 0xF74F);
  addIconCodePoint("md-xbox-controller-menu", 0xFE52);
  addIconCodePoint("md-xbox-controller-off", 0xF5BB);
  addIconCodePoint("md-xbox-controller-view", 0xFE53);
  addIconCodePoint("md-xda", 0xF5BC);
  addIconCodePoint("md-xing", 0xF5BD);
  addIconCodePoint("md-xing-box", 0xF5BE);
  addIconCodePoint("md-xing-circle", 0xF5BF);
  addIconCodePoint("md-xml", 0xF5C0);
  addIconCodePoint("md-xmpp", 0xF7FE);
  addIconCodePoint("md-yahoo", 0xFB2A);
  addIconCodePoint("md-yammer", 0xF788);
  addIconCodePoint("md-yeast", 0xF5C1);
  addIconCodePoint("md-yelp", 0xF5C2);
  addIconCodePoint("md-yin-yang", 0xF67F);
  addIconCodePoint("md-yoga", 0xF01A7);
  addIconCodePoint("md-youtube", 0xF5C3);
  addIconCodePoint("md-youtube-creator-studio", 0xF846);
  addIconCodePoint("md-youtube-gaming", 0xF847);
  addIconCodePoint("md-youtube-subscription", 0xFD1C);
  addIconCodePoint("md-youtube-tv", 0xF448);
  addIconCodePoint("md-z-wave", 0xFAE9);
  addIconCodePoint("md-zend", 0xFAEA);
  addIconCodePoint("md-zigbee", 0xFD1D);
  addIconCodePoint("md-zip-box", 0xF5C4);
  addIconCodePoint("md-zip-box-outline", 0xF001B);
  addIconCodePoint("md-zip-disk", 0xFA22);
  addIconCodePoint("md-zodiac-aquarius", 0xFA7C);
  addIconCodePoint("md-zodiac-aries", 0xFA7D);
  addIconCodePoint("md-zodiac-cancer", 0xFA7E);
  addIconCodePoint("md-zodiac-capricorn", 0xFA7F);
  addIconCodePoint("md-zodiac-gemini", 0xFA80);
  addIconCodePoint("md-zodiac-leo", 0xFA81);
  addIconCodePoint("md-zodiac-libra", 0xFA82);
  addIconCodePoint("md-zodiac-pisces", 0xFA83);
  addIconCodePoint("md-zodiac-sagittarius", 0xFA84);
  addIconCodePoint("md-zodiac-scorpio", 0xFA85);
  addIconCodePoint("md-zodiac-taurus", 0xFA86);
  addIconCodePoint("md-zodiac-virgo", 0xFA87);

  iconsNames.reserve(iconCodePoint.size());

  for (const auto &it : iconCodePoint) {
    iconsNames.emplace_back(it.first);
  }
}
